@@ -25,6 +25,8 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_INVALID, NULL},
 	/* SPObject */
 	{SP_ATTR_ID, "id"},
+	{SP_ATTR_INKSCAPE_COLLECT, "inkscape:collect"},
+	{SP_ATTR_INKSCAPE_LABEL, "inkscape:label"},
 	/* SPItem */
 	{SP_ATTR_TRANSFORM, "transform"},
 	{SP_ATTR_SODIPODI_INSENSITIVE, "sodipodi:insensitive"},
@@ -39,7 +41,6 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_XLINK_SHOW, "xlink:show"},
 	{SP_ATTR_XLINK_ACTUATE, "xlink:actuate"},
 	{SP_ATTR_TARGET, "target"},
-	/* SPGroup */
 	{SP_ATTR_INKSCAPE_GROUPMODE, "inkscape:groupmode"},
 	/* SPRoot */
 	{SP_ATTR_VERSION, "version"},
@@ -52,9 +53,7 @@ static const SPStyleProp props[] = {
 	/* SPNamedView */
 	{SP_ATTR_VIEWONLY, "viewonly"},
 	{SP_ATTR_SHOWGRID, "showgrid"},
-	{SP_ATTR_SNAPTOGRID, "snaptogrid"},
 	{SP_ATTR_SHOWGUIDES, "showguides"},
-	{SP_ATTR_SNAPTOGUIDES, "snaptoguides"},
 	{SP_ATTR_GRIDTOLERANCE, "gridtolerance"},
 	{SP_ATTR_GUIDETOLERANCE, "guidetolerance"},
 	{SP_ATTR_GRIDORIGINX, "gridoriginx"},
@@ -63,6 +62,9 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_GRIDSPACINGY, "gridspacingy"},
 	{SP_ATTR_GRIDCOLOR, "gridcolor"},
 	{SP_ATTR_GRIDOPACITY, "gridopacity"},
+	{SP_ATTR_GRIDEMPCOLOR, "gridempcolor"},
+	{SP_ATTR_GRIDEMPOPACITY, "gridempopacity"},
+	{SP_ATTR_GRIDEMPSPACING, "gridempspacing"},
 	{SP_ATTR_GUIDECOLOR, "guidecolor"},
 	{SP_ATTR_GUIDEOPACITY, "guideopacity"},
 	{SP_ATTR_GUIDEHICOLOR, "guidehicolor"},
@@ -81,6 +83,11 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_INKSCAPE_WINDOW_HEIGHT, "inkscape:window-height"},
 	{SP_ATTR_INKSCAPE_WINDOW_X, "inkscape:window-x"},
 	{SP_ATTR_INKSCAPE_WINDOW_Y, "inkscape:window-y"},
+	{SP_ATTR_INKSCAPE_GRID_BBOX, "inkscape:grid-bbox"},
+	{SP_ATTR_INKSCAPE_GUIDE_BBOX, "inkscape:guide-bbox"},
+	{SP_ATTR_INKSCAPE_GRID_POINTS, "inkscape:grid-points"},
+	{SP_ATTR_INKSCAPE_GUIDE_POINTS, "inkscape:guide-points"},
+	{SP_ATTR_INKSCAPE_CURRENT_LAYER, "inkscape:current-layer"},
 	/* SPGuide */
 	{SP_ATTR_ORIENTATION, "orientation"},
 	{SP_ATTR_POSITION, "position"},
@@ -89,6 +96,9 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_Y, "y"},
 	/* SPPath */
 	{SP_ATTR_D, "d"},
+	/* (Note: XML representation of connectors may change in future.) */
+	{SP_ATTR_CONNECTION_START, "inkscape:connection-start"},
+	{SP_ATTR_CONNECTION_END, "inkscape:connection-end"},
 	/* SPRect */
 	{SP_ATTR_RX, "rx"},
 	{SP_ATTR_RY, "ry"},
@@ -110,6 +120,8 @@ static const SPStyleProp props[] = {
 	{SP_ATTR_SODIPODI_ARG1, "sodipodi:arg1"},
 	{SP_ATTR_SODIPODI_ARG2, "sodipodi:arg2"},
 	{SP_ATTR_INKSCAPE_FLATSIDED, "inkscape:flatsided"},
+	{SP_ATTR_INKSCAPE_ROUNDED, "inkscape:rounded"},
+	{SP_ATTR_INKSCAPE_RANDOMIZED, "inkscape:randomized"},
 	/* SPSpiral */
 	{SP_ATTR_SODIPODI_EXPANSION, "sodipodi:expansion"},
 	{SP_ATTR_SODIPODI_REVOLUTION, "sodipodi:revolution"},
@@ -185,6 +197,16 @@ static const SPStyleProp props[] = {
 	/* XML */
 	{SP_ATTR_XML_SPACE, "xml:space"},
 
+  /* typeset */
+  {SP_ATTR_TEXT_NOMARKUP, "inkscape:srcNoMarkup"},
+  {SP_ATTR_TEXT_PANGOMARKUP, "inkscape:srcPango" },
+  {SP_ATTR_TEXT_INSHAPE, "inkscape:dstShape"},
+  {SP_ATTR_TEXT_ONPATH, "inkscape:dstPath"},
+  {SP_ATTR_TEXT_INBOX,"inkscape:dstBox"},
+  {SP_ATTR_TEXT_INCOLUMN,"inkscape:dstColumn"},
+  {SP_ATTR_TEXT_EXCLUDE,"inkscape:excludeShape"},
+  {SP_ATTR_LAYOUT_OPTIONS,"inkscape:layoutOptions"},
+    
 	/* CSS2 */
 	/* Font */
 	{SP_PROP_FONT, "font"},
