@@ -18,13 +18,12 @@ const unsigned char *sp_attribute_name (unsigned char id);
 
 #define SP_ATTRIBUTE_IS_CSS(k) (((k) >= SP_PROP_FONT) && ((k) <= SP_PROP_WRITING_MODE))
 
-/* This is not nice, but we have to ensure name->id works */
-#define SP_ATTR_FILL SP_PROP_FILL
-
 enum {
 	SP_ATTR_INVALID,
 	/* SPObject */
 	SP_ATTR_ID,
+	SP_ATTR_INKSCAPE_COLLECT,
+	SP_ATTR_INKSCAPE_LABEL,
 	/* SPItem */
 	SP_ATTR_TRANSFORM,
 	SP_ATTR_SODIPODI_INSENSITIVE,
@@ -52,9 +51,7 @@ enum {
 	/* SPNamedView */
 	SP_ATTR_VIEWONLY,
 	SP_ATTR_SHOWGRID,
-	SP_ATTR_SNAPTOGRID,
 	SP_ATTR_SHOWGUIDES,
-	SP_ATTR_SNAPTOGUIDES,
 	SP_ATTR_GRIDTOLERANCE,
 	SP_ATTR_GUIDETOLERANCE,
 	SP_ATTR_GRIDORIGINX,
@@ -63,6 +60,9 @@ enum {
 	SP_ATTR_GRIDSPACINGY,
 	SP_ATTR_GRIDCOLOR,
 	SP_ATTR_GRIDOPACITY,
+	SP_ATTR_GRIDEMPCOLOR,
+	SP_ATTR_GRIDEMPOPACITY,
+	SP_ATTR_GRIDEMPSPACING,
 	SP_ATTR_GUIDECOLOR,
 	SP_ATTR_GUIDEOPACITY,
 	SP_ATTR_GUIDEHICOLOR,
@@ -81,6 +81,11 @@ enum {
 	SP_ATTR_INKSCAPE_WINDOW_HEIGHT,
 	SP_ATTR_INKSCAPE_WINDOW_X,
 	SP_ATTR_INKSCAPE_WINDOW_Y,
+	SP_ATTR_INKSCAPE_GRID_BBOX,
+	SP_ATTR_INKSCAPE_GUIDE_BBOX,
+	SP_ATTR_INKSCAPE_GRID_POINTS,
+	SP_ATTR_INKSCAPE_GUIDE_POINTS,
+	SP_ATTR_INKSCAPE_CURRENT_LAYER,
 	/* SPGuide */
 	SP_ATTR_ORIENTATION,
 	SP_ATTR_POSITION,
@@ -89,6 +94,8 @@ enum {
 	SP_ATTR_Y,
 	/* SPPath */
 	SP_ATTR_D,
+	SP_ATTR_CONNECTION_START,
+	SP_ATTR_CONNECTION_END,
 	/* SPRect */
 	SP_ATTR_RX,
 	SP_ATTR_RY,
@@ -110,6 +117,8 @@ enum {
 	SP_ATTR_SODIPODI_ARG1,
 	SP_ATTR_SODIPODI_ARG2,
 	SP_ATTR_INKSCAPE_FLATSIDED,
+	SP_ATTR_INKSCAPE_ROUNDED,
+	SP_ATTR_INKSCAPE_RANDOMIZED,
 	/* SPSpiral */
 	SP_ATTR_SODIPODI_EXPANSION,
 	SP_ATTR_SODIPODI_REVOLUTION,
@@ -184,6 +193,16 @@ enum {
 
 	/* XML */
 	SP_ATTR_XML_SPACE,
+  
+  /* typeset */
+  SP_ATTR_TEXT_NOMARKUP,
+  SP_ATTR_TEXT_PANGOMARKUP,
+  SP_ATTR_TEXT_INSHAPE,
+  SP_ATTR_TEXT_ONPATH,
+  SP_ATTR_TEXT_INBOX,
+  SP_ATTR_TEXT_INCOLUMN,
+  SP_ATTR_TEXT_EXCLUDE,
+  SP_ATTR_LAYOUT_OPTIONS,
 
 	/* CSS2 */
 	/* Font */
