@@ -11,6 +11,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <gtk/gtksignal.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkspinbutton.h>
@@ -19,8 +22,9 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
-#include "helper/guideline.h"
+#include "display/guideline.h"
 #include "helper/unit-menu.h"
+#include "helper/units.h"
 #include "libnr/nr-matrix-ops.h"
 #include "widgets/icon.h"
 #include "inkscape-private.h"
@@ -31,11 +35,14 @@
 #include "desktop-affine.h"
 #include "desktop-handles.h"
 #include "event-context.h"
+#include "sp-desktop-widget.h"
 #include "sp-metrics.h"
 #include "sp-item.h"
 #include "desktop-events.h"
 #include "helper/sp-intl.h"
 #include "dialogs/dialog-events.h"
+#include "message-context.h"
+#include "xml/repr.h"
 
 static void sp_dt_simple_guide_dialog (SPGuide * guide, SPDesktop * desktop);
 
@@ -97,7 +104,7 @@ sp_dt_ruler_event (GtkWidget *widget, GdkEvent *event, SPDesktopWidget *dtw, gbo
 
 			// explicitly show guidelines; if I draw a guide, I want them on
 			sp_repr_set_boolean (repr, "showguides", TRUE);
-			sp_repr_set_boolean (repr, "snaptoguides", TRUE);
+			sp_repr_set_boolean (repr, "inkscape:guide-bbox", TRUE);
 
 			double const guide_pos_dt = event_dt[ horiz
 							      ? NR::Y
@@ -237,18 +244,21 @@ sp_dt_guide_event (SPCanvasItem * item, GdkEvent * event, gpointer data)
 		}
 	case GDK_ENTER_NOTIFY:
 	{
+		
 		sp_guideline_set_color (SP_GUIDELINE (item), guide->hicolor);
+
 		GString *position_string = SP_PT_TO_METRIC_STRING (guide->position, SP_DEFAULT_METRIC);
 		char *guide_description = sp_guide_description(guide);
-		char *msg = g_strdup_printf(_("%s at %s"), guide_description, position_string->str);
+
+		desktop->guidesMessageContext()->setF(Inkscape::NORMAL_MESSAGE, _("%s at %s"), guide_description, position_string->str);
+
 		g_free(guide_description);
 		g_string_free(position_string, TRUE);
-		sp_view_set_status (SP_VIEW (desktop), msg, FALSE);
        		break;
 	}
 	case GDK_LEAVE_NOTIFY:
 		sp_guideline_set_color (SP_GUIDELINE (item), guide->color);
-		sp_view_set_status (SP_VIEW (desktop), NULL, FALSE);
+		desktop->guidesMessageContext()->clear();
 		break;
 	default:
 		break;
@@ -273,10 +283,14 @@ static void
 guide_dialog_mode_changed (GtkWidget * widget)
 {
   if (mode) {
-    gtk_label_set_text (GTK_LABEL (m), " relative by ");
+    // TRANSLATORS: This string appears when double-clicking on a guide.
+    // This is the distance by which the guide is to be moved.
+    gtk_label_set_text (GTK_LABEL (m), _(" relative by "));
     mode = FALSE;
   } else {
-    gtk_label_set_text (GTK_LABEL (m), " absolute to ");
+    // TRANSLATORS: This string appears when double-clicking on a guide.
+    // This is the target location where the guide is to be moved.
+    gtk_label_set_text (GTK_LABEL (m), _(" absolute to "));
     mode = TRUE;
   }
 }
@@ -289,17 +303,12 @@ guide_dialog_close (GtkWidget * widget, gpointer data)
 
 static void guide_dialog_apply(SPGuide &guide)
 {
-	gdouble distance = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(e));
-	sp_convert_distance_full(&distance,
-				 sp_unit_selector_get_unit(SP_UNIT_SELECTOR(u)),
-				 sp_unit_get_identity(SP_UNIT_ABSOLUTE),
-				 1.0, 1.0);
-	gdouble newpos;
-	if (mode) {
-		newpos = distance;
-	} else {
-		newpos = guide.position + distance;
-	}
+	gdouble const raw_dist = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(e));
+	SPUnit const &unit = *sp_unit_selector_get_unit(SP_UNIT_SELECTOR(u));
+	gdouble const points = sp_convert_distance_full(raw_dist, unit, sp_unit_get_by_id(SP_UNIT_PT), 1.0);
+	gdouble const newpos = ( mode
+				 ? points
+				 : guide.position + points );
 	sp_guide_moveto(guide, newpos, true);
 	sp_document_done(SP_OBJECT_DOCUMENT(&guide));
 }
@@ -350,14 +359,12 @@ guide_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 static void
 sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 {
-	gdouble val = 0;
 	GtkWidget * pix, * b1, * b2, * b3, * b4,* but;
-	const SPUnit *unit;
 
 	if (!GTK_IS_WIDGET (d)) {
 		GtkObject *a;
 		// create dialog
-		d = gtk_dialog_new_with_buttons ("Guideline",
+		d = gtk_dialog_new_with_buttons (_("Guideline"),
 						 NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_STOCK_OK,
@@ -408,7 +415,7 @@ sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 		gtk_signal_connect_while_alive (GTK_OBJECT (but), "clicked", GTK_SIGNAL_FUNC (guide_dialog_mode_changed), 
 						NULL , GTK_OBJECT(but));
 		gtk_widget_show (but);
-		m = gtk_label_new (" absolute to ");
+		m = gtk_label_new (_(" absolute to "));
 		mode = TRUE;
 		gtk_container_add (GTK_CONTAINER (but), m);
 		gtk_widget_show (m);
@@ -447,9 +454,8 @@ sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 		g_free(label);
 	}
 
-	val = oldpos;
-	unit = sp_unit_selector_get_unit (SP_UNIT_SELECTOR (u));
-	sp_convert_distance_full (&val, sp_unit_get_identity (SP_UNIT_ABSOLUTE), unit, 1.0, 1.0);
+	SPUnit const &unit = *sp_unit_selector_get_unit(SP_UNIT_SELECTOR(u));
+	gdouble const val = sp_convert_distance_full(oldpos, sp_unit_get_by_id(SP_UNIT_PT), unit, 1.0);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (e), val);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (e), val);
 	gtk_widget_grab_focus (e);
