@@ -28,8 +28,9 @@
 
 #include "../inkscape-stock.h"
 #include "helper/sp-intl.h"
-#include "helper/window.h"
 #include "helper/unit-menu.h"
+#include "helper/units.h"
+#include "helper/window.h"
 #include "libnr/nr-scale.h"
 #include "libnr/nr-scale-ops.h"
 #include "macros.h"
@@ -147,13 +148,11 @@ static gboolean sp_transformation_dialog_delete(GtkObject *, GdkEvent *, gpointe
 static void
 sp_transformation_dialog_present (unsigned int page)
 {
-    GtkWidget *nbook;
-
     if (!dlg) {
         dlg = sp_transformation_dialog_new ();
     }
 
-    nbook = GTK_WIDGET(g_object_get_data (G_OBJECT (dlg), "notebook"));
+    GtkWidget *nbook = GTK_WIDGET(g_object_get_data (G_OBJECT (dlg), "notebook"));
     gtk_notebook_set_page (GTK_NOTEBOOK (nbook), page);
 
     gtk_widget_show (dlg);
@@ -166,8 +165,6 @@ static void
 sp_transformation_dialog_update_selection ( GObject *dlg, unsigned int page, 
                                             SPSelection *selection )
 {
-    GtkWidget *apply;
-
     switch (page) {
         case SP_TRANSFORMATION_MOVE:
             sp_transformation_move_update (dlg, selection);
@@ -183,8 +180,8 @@ sp_transformation_dialog_update_selection ( GObject *dlg, unsigned int page,
             break;
     }
 
-    apply = GTK_WIDGET(g_object_get_data (dlg, "apply"));
-    if (selection && !sp_selection_is_empty (selection)) {
+    GtkWidget *apply = GTK_WIDGET(g_object_get_data (dlg, "apply"));
+    if (selection && !selection->isEmpty()) {
         gtk_widget_set_sensitive (apply, TRUE);
     } else {
         gtk_widget_set_sensitive (apply, FALSE); 
@@ -213,11 +210,8 @@ sp_transformation_dialog_selection_modified ( Inkscape::Application *inkscape,
                                               unsigned int flags, 
                                               GObject *obj )
 {
-    GObject *notebook;
-    int page;
-
-    notebook = G_OBJECT(g_object_get_data (obj, "notebook"));
-    page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+    GObject *notebook = G_OBJECT(g_object_get_data (obj, "notebook"));
+    int const page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
 
     sp_transformation_dialog_update_selection (obj, page, selection);
 
@@ -231,9 +225,7 @@ sp_transformation_dialog_switch_page ( GtkNotebook *notebook,
                                        guint pagenum, 
                                        GObject *dlg )
 {
-    SPSelection *sel;
-
-    sel = (SP_ACTIVE_DESKTOP) ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL;
+    SPSelection *sel = (SP_ACTIVE_DESKTOP) ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL;
 
     sp_transformation_dialog_update_selection (dlg, pagenum, sel);
 }
@@ -243,14 +235,10 @@ sp_transformation_dialog_switch_page ( GtkNotebook *notebook,
 static GtkWidget *
 sp_transformation_dialog_new (void)
 {
-
     if  (!dlg)
     {
-        GtkWidget *hb, *vb, *nbook, *page, *hs, *bb, *b, *l;
-        SPSelection *sel;
         gchar title[500];
-
-        sp_ui_dialog_title_string (SP_VERB_DIALOG_TRANSFORM, title);
+        sp_ui_dialog_title_string (Inkscape::Verb::get(SP_VERB_DIALOG_TRANSFORM), title);
 
         dlg = sp_window_new (title, TRUE);
         if (x == -1000 || y == -1000) {
@@ -295,29 +283,29 @@ sp_transformation_dialog_new (void)
                              G_CALLBACK (sp_dialog_unhide), dlg);
 
         /* Toplevel hbox */
-        hb = gtk_hbox_new (FALSE, 0);
+        GtkWidget *hb = gtk_hbox_new (FALSE, 0);
         gtk_widget_show (hb);
         gtk_container_add (GTK_CONTAINER (dlg), hb);
 
         /* Toplevel vbox */
-        vb = gtk_vbox_new (FALSE, 4);
+        GtkWidget *vb = gtk_vbox_new (FALSE, 4);
         gtk_widget_show (vb);
         gtk_box_pack_start (GTK_BOX (hb), vb, TRUE, TRUE, 0);
 
         /* Notebook for individual transformations */
-        nbook = gtk_notebook_new ();
+        GtkWidget *nbook = gtk_notebook_new ();
         gtk_widget_show (nbook);
         gtk_box_pack_start (GTK_BOX (vb), nbook, TRUE, TRUE, 0);
         g_object_set_data (G_OBJECT (dlg), "notebook", nbook);
         /* Separator */
-        hs = gtk_hseparator_new ();
+        GtkWidget *hs = gtk_hseparator_new ();
         gtk_widget_show (hs);
         gtk_box_pack_start (GTK_BOX (vb), hs, FALSE, FALSE, 0);
         /* Buttons */
-        bb = gtk_hbox_new (FALSE, 0);
+        GtkWidget *bb = gtk_hbox_new (FALSE, 0);
         gtk_widget_show (bb);
         gtk_box_pack_start (GTK_BOX (vb), bb, FALSE, FALSE, 0);
-        b = gtk_button_new_from_stock (GTK_STOCK_APPLY);
+        GtkWidget *b = gtk_button_new_from_stock (GTK_STOCK_APPLY);
         g_object_set_data (G_OBJECT (dlg), "apply", b);
         gtk_widget_show (b);
         gtk_box_pack_start (GTK_BOX (bb), b, TRUE, TRUE, 0);
@@ -330,9 +318,9 @@ sp_transformation_dialog_new (void)
                            G_CALLBACK (sp_transformation_dialog_close), dlg );
 
         /* Move page */
-        l = gtk_label_new (_("Move"));
+        GtkWidget *l = gtk_label_new (_("Move"));
         gtk_widget_show( l );
-        page = sp_transformation_page_move_new (G_OBJECT (dlg));
+        GtkWidget *page = sp_transformation_page_move_new (G_OBJECT (dlg));
         gtk_widget_show (page);
         gtk_notebook_append_page (GTK_NOTEBOOK (nbook), page, l);
         g_object_set_data (G_OBJECT (dlg), "move", page);
@@ -367,7 +355,7 @@ sp_transformation_dialog_new (void)
                                (sp_transformation_dialog_switch_page), 
                            dlg);
 
-        sel = (SP_ACTIVE_DESKTOP) ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL;
+        SPSelection *sel = (SP_ACTIVE_DESKTOP) ? SP_DT_SELECTION (SP_ACTIVE_DESKTOP) : NULL;
         sp_transformation_dialog_update_selection (G_OBJECT (dlg), 0, sel);
     }
 
@@ -384,7 +372,7 @@ static void sp_transformation_dialog_apply(GObject *, GObject *dlg)
     SPDesktop * const desktop = SP_ACTIVE_DESKTOP;
     g_return_if_fail (desktop != NULL);
     SPSelection * const selection = SP_DT_SELECTION(desktop);
-    g_return_if_fail (!sp_selection_is_empty (selection));
+    g_return_if_fail (!selection->isEmpty());
 
     GtkWidget * const nbookw = GTK_WIDGET(g_object_get_data(dlg, "notebook"));
     int const page = gtk_notebook_get_current_page (GTK_NOTEBOOK (nbookw));
@@ -423,8 +411,9 @@ static void sp_transformation_dialog_close(GObject *, GtkWidget *dlg)
  */
 static void sp_transformation_move_value_changed(GtkAdjustment *, GObject *dlg)
 {
-    if (g_object_get_data (dlg, "update"))
+    if (g_object_get_data (dlg, "update")) {
         return;
+    }
 
     GtkWidget *apply = GTK_WIDGET(g_object_get_data(dlg, "apply"));
     gtk_widget_set_sensitive (apply, TRUE);
@@ -435,42 +424,35 @@ static void sp_transformation_move_value_changed(GtkAdjustment *, GObject *dlg)
 static void
 sp_transformation_move_relative_toggled (GtkToggleButton *tb, GObject *dlg)
 {
-    SPDesktop *desktop;
-    SPSelection *selection;
-    SPUnitSelector *us;
-    GtkAdjustment *ax, *ay;
-    NRRect bbox;
-    float x, y;
-
-    desktop = SP_ACTIVE_DESKTOP;
+    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
     
     if (!desktop)
         return;
         
-    selection = SP_DT_SELECTION (desktop);
+    SPSelection *selection = SP_DT_SELECTION (desktop);
     
-    if (sp_selection_is_empty (selection))
+    if (selection->isEmpty())
         return;
 
     /* Read values from widget */
-    us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
-    ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
-    ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
-    x = sp_unit_selector_get_value_in_points (us, ax);
-    y = sp_unit_selector_get_value_in_points (us, ay);
+    SPUnitSelector *us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
+    GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
+    GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
+    float x = sp_unit_selector_get_value_in_points (us, ax);
+    float y = sp_unit_selector_get_value_in_points (us, ay);
 
-    sp_selection_bbox (selection, &bbox);
+    NR::Rect bbox = selection->bounds();
 
     g_object_set_data (dlg, "update", GUINT_TO_POINTER (TRUE));
 
     if (gtk_toggle_button_get_active (tb)) {
         /* From absolute to relative */
-        sp_unit_selector_set_value_in_points (us, ax, x - bbox.x0);
-        sp_unit_selector_set_value_in_points (us, ay, y - bbox.y0);
+        sp_unit_selector_set_value_in_points (us, ax, x - bbox.min()[NR::X]);
+        sp_unit_selector_set_value_in_points (us, ay, y - bbox.min()[NR::Y]);
     } else {
         /* From relative to absolute */
-        sp_unit_selector_set_value_in_points (us, ax, bbox.x0 + x);
-        sp_unit_selector_set_value_in_points (us, ay, bbox.y0 + y);
+        sp_unit_selector_set_value_in_points (us, ax, bbox.min()[NR::X] + x);
+        sp_unit_selector_set_value_in_points (us, ay, bbox.min()[NR::Y] + y);
     }
 
     g_object_set_data (dlg, "update", GUINT_TO_POINTER (FALSE));
@@ -482,22 +464,19 @@ sp_transformation_move_relative_toggled (GtkToggleButton *tb, GObject *dlg)
 static GtkWidget *
 sp_transformation_page_move_new (GObject *obj)
 {
-    GtkWidget *frame, *vb, *tbl, *lbl, *img, *sb, *us, *cb;
-    GtkAdjustment *adj;
+    GtkWidget *frame = gtk_frame_new (_("Move"));
 
-    frame = gtk_frame_new (_("Move"));
-
-    vb = gtk_vbox_new (FALSE, 0);
+    GtkWidget *vb = gtk_vbox_new (FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
     gtk_container_add (GTK_CONTAINER (frame), vb);
 
-    tbl = gtk_table_new (4, 2, FALSE);
+    GtkWidget *tbl = gtk_table_new (4, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
     gtk_table_set_col_spacings (GTK_TABLE (tbl), 4);
     gtk_box_pack_start (GTK_BOX (vb), tbl, FALSE, FALSE, 0);
 
     /* Unit selector */
-    us = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
+    GtkWidget *us = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
     g_object_set_data (obj, "move_units", us);
     if (SP_ACTIVE_DESKTOP) {
         sp_unit_selector_set_unit (SP_UNIT_SELECTOR (us), 
@@ -505,16 +484,16 @@ sp_transformation_page_move_new (GObject *obj)
     }
     
     /* Horizontal */
-    img = gtk_image_new_from_stock (INKSCAPE_STOCK_ARROWS_HOR, 
+    GtkWidget *img = gtk_image_new_from_stock (INKSCAPE_STOCK_ARROWS_HOR, 
                                     GTK_ICON_SIZE_LARGE_TOOLBAR );
     gtk_table_attach ( GTK_TABLE (tbl), img, 0, 1, 0, 1, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
-    adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
+    GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
     g_object_set_data (obj, "move_position_x", adj);
     sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), adj);
     g_signal_connect ( G_OBJECT (adj), "value_changed", 
                        G_CALLBACK (sp_transformation_move_value_changed), obj);
-    sb = gtk_spin_button_new (adj, 0.1, 2);
+    GtkWidget *sb = gtk_spin_button_new (adj, 0.1, 2);
     gtk_table_attach ( GTK_TABLE (tbl), sb, 1, 2, 0, 1, 
               (GtkAttachOptions)( (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ) ),
               (GtkAttachOptions)( (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ) ),
@@ -537,7 +516,7 @@ sp_transformation_page_move_new (GObject *obj)
                        0, 0 );
                        
     /* Unit selector */
-    lbl = gtk_label_new (_("Units:"));
+    GtkWidget *lbl = gtk_label_new (_("Units:"));
     gtk_misc_set_alignment (GTK_MISC (lbl), 1.0, 0.5);
     gtk_table_attach ( GTK_TABLE (tbl), lbl, 0, 1, 2, 3, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
@@ -546,7 +525,7 @@ sp_transformation_page_move_new (GObject *obj)
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 0, 0);
 
     /* Relative moves */
-    cb = gtk_check_button_new_with_label (_("Relative move"));
+    GtkWidget *cb = gtk_check_button_new_with_label (_("Relative move"));
     g_object_set_data (obj, "move_relative", cb);
     gtk_table_attach ( GTK_TABLE (tbl), cb, 1, 2, 3, 4, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
@@ -565,25 +544,18 @@ sp_transformation_page_move_new (GObject *obj)
 static void
 sp_transformation_move_update (GObject *dlg, SPSelection *selection)
 {
-    GtkWidget *page;
+    GtkWidget *page = GTK_WIDGET(g_object_get_data (dlg, "move"));
 
-    page = GTK_WIDGET(g_object_get_data (dlg, "move"));
-
-    if (selection && !sp_selection_is_empty (selection)) {
-        GtkToggleButton *cb;
-        cb = GTK_TOGGLE_BUTTON(g_object_get_data (dlg, "move_relative"));
+    if (selection && !selection->isEmpty()) {
+        GtkToggleButton *cb = GTK_TOGGLE_BUTTON(g_object_get_data (dlg, "move_relative"));
         
         if (!gtk_toggle_button_get_active (cb)) {
-        
-            GtkAdjustment *ax, *ay;
-            SPUnitSelector *us;
-            NRRect bbox;
-            ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
-            ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
-            us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
-            sp_selection_bbox (selection, &bbox);
-            sp_unit_selector_set_value_in_points (us, ax, bbox.x0);
-            sp_unit_selector_set_value_in_points (us, ay, bbox.y0);
+            GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
+            GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
+            SPUnitSelector *us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
+            NR::Rect bbox = selection->bounds();
+            sp_unit_selector_set_value_in_points (us, ax, bbox.min()[NR::X]);
+            sp_unit_selector_set_value_in_points (us, ay, bbox.min()[NR::Y]);
         }
         gtk_widget_set_sensitive (page, TRUE);
     
@@ -596,30 +568,24 @@ sp_transformation_move_update (GObject *dlg, SPSelection *selection)
 
 static void sp_transformation_move_apply(GObject *dlg, SPSelection *selection)
 {
-    SPUnitSelector *us;
-    GtkAdjustment *ax, *ay;
-    GtkToggleButton *cb;
-    float x, y;
+    SPUnitSelector *us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
+    GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
+    GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
+    float x = sp_unit_selector_get_value_in_points (us, ax);
+    float y = sp_unit_selector_get_value_in_points (us, ay);
 
-    us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "move_units"));
-    ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_x"));
-    ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "move_position_y"));
-    x = sp_unit_selector_get_value_in_points (us, ax);
-    y = sp_unit_selector_get_value_in_points (us, ay);
-
-    cb = GTK_TOGGLE_BUTTON(g_object_get_data (dlg, "move_relative"));
+    GtkToggleButton *cb = GTK_TOGGLE_BUTTON(g_object_get_data (dlg, "move_relative"));
     
     if (gtk_toggle_button_get_active (cb)) {
         sp_selection_move_relative (selection, x, y);
     
     } else {
-        NRRect bbox;
-        sp_selection_bbox (selection, &bbox);
-        sp_selection_move_relative (selection, x - bbox.x0, y - bbox.y0);
+        NR::Rect bbox = selection->bounds();
+        sp_selection_move_relative (selection, x - bbox.min()[NR::X], y - bbox.min()[NR::Y]);
     }
 
     if (selection)
-        sp_document_done ( SP_DT_DOCUMENT (selection->desktop) );
+        sp_document_done ( SP_DT_DOCUMENT (selection->desktop()) );
 }
 
 
@@ -633,56 +599,45 @@ static gboolean sp_transformation_scale_set_unit(SPUnitSelector *,
                                                  SPUnit const *new_units,
                                                  GObject *dlg)
 {
-    SPDesktop *desktop;
-    SPSelection *selection;
-
-    desktop = SP_ACTIVE_DESKTOP;
+    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
     
-    if (!desktop)
+    if (!desktop) {
         return FALSE;
+    }
     
-    selection = SP_DT_SELECTION (desktop);
+    SPSelection *selection = SP_DT_SELECTION (desktop);
     
-    if (sp_selection_is_empty (selection))
+    if (selection->isEmpty())
         return FALSE;
 
     if ((old->base == SP_UNIT_ABSOLUTE) && 
        (new_units->base == SP_UNIT_DIMENSIONLESS)) {
        
-        SPUnitSelector *us;
-        GtkAdjustment *ax, *ay;
-        NRRect bbox;
-        float x, y;
         /* Absolute to percentage */
         g_object_set_data (dlg, "update", GUINT_TO_POINTER (TRUE));
-        us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "scale_units"));
-        ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
-        ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
-        x = sp_units_get_points (ax->value, old);
-        y = sp_units_get_points (ay->value, old);
-        sp_selection_bbox (selection, &bbox);
-        gtk_adjustment_set_value (ax, 100.0 * x / (bbox.x1 - bbox.x0));
-        gtk_adjustment_set_value (ay, 100.0 * y / (bbox.y1 - bbox.y0));
+        GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
+        GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
+        float x = sp_units_get_points (ax->value, old);
+        float y = sp_units_get_points (ay->value, old);
+        NR::Rect bbox = selection->bounds();
+        gtk_adjustment_set_value (ax, 100.0 * x / bbox.extent(NR::X));
+        gtk_adjustment_set_value (ay, 100.0 * y / bbox.extent(NR::Y));
         g_object_set_data (dlg, "update", GUINT_TO_POINTER (FALSE));
         return TRUE;
         
     } else if ((old->base == SP_UNIT_DIMENSIONLESS) && 
               (new_units->base == SP_UNIT_ABSOLUTE)) {
               
-        SPUnitSelector *us;
-        GtkAdjustment *ax, *ay;
-        NRRect bbox;
         /* Percentage to absolute */
         g_object_set_data (dlg, "update", GUINT_TO_POINTER (TRUE));
-        us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "scale_units"));
-        ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
-        ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
-        sp_selection_bbox (selection, &bbox);
+        GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
+        GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
+        NR::Rect bbox = selection->bounds();
         gtk_adjustment_set_value (ax, 
-            sp_points_get_units (0.01 * ax->value * (bbox.x1 - bbox.x0), 
+            sp_points_get_units (0.01 * ax->value * bbox.extent(NR::X),
                                  new_units));
         gtk_adjustment_set_value (ay, 
-            sp_points_get_units (0.01 * ay->value * (bbox.y1 - bbox.y0), 
+            sp_points_get_units (0.01 * ay->value * bbox.extent(NR::Y), 
                                  new_units));
         g_object_set_data (dlg, "update", GUINT_TO_POINTER (FALSE));
         return TRUE;
@@ -695,8 +650,9 @@ static gboolean sp_transformation_scale_set_unit(SPUnitSelector *,
 
 static void sp_transformation_scale_value_changed(GtkAdjustment *, GObject * const dlg)
 {
-    if (g_object_get_data (dlg, "update"))
+    if (g_object_get_data (dlg, "update")) {
         return;
+    }
 
     GtkWidget *apply = GTK_WIDGET(g_object_get_data(dlg, "apply"));
     gtk_widget_set_sensitive (apply, TRUE);
@@ -707,41 +663,38 @@ static void sp_transformation_scale_value_changed(GtkAdjustment *, GObject * con
 static GtkWidget *
 sp_transformation_page_scale_new (GObject *obj)
 {
-    GtkWidget *frame, *vb, *tbl, *lbl, *img, *sb, *us;
-    GtkAdjustment *adj;
+    GtkWidget *frame = gtk_frame_new (_("Scale"));
 
-    frame = gtk_frame_new (_("Scale"));
-
-    vb = gtk_vbox_new (FALSE, 0);
+    GtkWidget *vb = gtk_vbox_new (FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
     gtk_container_add (GTK_CONTAINER (frame), vb);
 
-    tbl = gtk_table_new (4, 2, FALSE);
+    GtkWidget *tbl = gtk_table_new (4, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
     gtk_table_set_col_spacings (GTK_TABLE (tbl), 4);
     gtk_box_pack_start (GTK_BOX (vb), tbl, FALSE, FALSE, 0);
 
     /* Unit selector */
     /* fixme: Default has to be percentage */
-    us = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
+    GtkWidget *us = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
     g_object_set_data (obj, "scale_units", us);
-    sp_unit_selector_add_unit ( SP_UNIT_SELECTOR (us), 
-                                sp_unit_get_by_abbreviation ("%"), 0 );
-    sp_unit_selector_set_unit ( SP_UNIT_SELECTOR (us), 
-                                sp_unit_get_by_abbreviation ("%") );
+    sp_unit_selector_add_unit(SP_UNIT_SELECTOR(us),
+                              &sp_unit_get_by_id(SP_UNIT_PERCENT), 0);
+    sp_unit_selector_set_unit(SP_UNIT_SELECTOR(us),
+                              &sp_unit_get_by_id(SP_UNIT_PERCENT));
     g_signal_connect ( G_OBJECT (us), "set_unit", 
                        G_CALLBACK (sp_transformation_scale_set_unit), obj );
     /* Horizontal */
-    img = gtk_image_new_from_stock ( INKSCAPE_STOCK_SCALE_HOR, 
+    GtkWidget *img = gtk_image_new_from_stock ( INKSCAPE_STOCK_SCALE_HOR, 
                                      GTK_ICON_SIZE_LARGE_TOOLBAR );
     gtk_table_attach ( GTK_TABLE (tbl), img, 0, 1, 0, 1, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
-    adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
+    GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
     g_object_set_data (obj, "scale_dimension_x", adj);
     sp_unit_selector_add_adjustment (SP_UNIT_SELECTOR (us), adj);
     g_signal_connect ( G_OBJECT (adj), "value_changed", 
                        G_CALLBACK (sp_transformation_scale_value_changed), obj );
-    sb = gtk_spin_button_new (adj, 0.1, 2);
+    GtkWidget *sb = gtk_spin_button_new (adj, 0.1, 2);
     gtk_table_attach ( GTK_TABLE (tbl), sb, 1, 2, 0, 1, 
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 0, 0);
@@ -760,7 +713,7 @@ sp_transformation_page_scale_new (GObject *obj)
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 0, 0 );
     /* Unit selector */
-    lbl = gtk_label_new (_("Units:"));
+    GtkWidget *lbl = gtk_label_new (_("Units:"));
     gtk_misc_set_alignment (GTK_MISC (lbl), 1.0, 0.5);
     gtk_table_attach ( GTK_TABLE (tbl), lbl, 0, 1, 2, 3, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
@@ -779,25 +732,18 @@ sp_transformation_page_scale_new (GObject *obj)
 static void
 sp_transformation_scale_update (GObject *dlg, SPSelection *selection)
 {
-    GtkWidget *page;
+    GtkWidget *page = GTK_WIDGET(g_object_get_data (dlg, "scale"));
 
-    page = GTK_WIDGET(g_object_get_data (dlg, "scale"));
-
-    if (selection && !sp_selection_is_empty (selection)) {
-    
-        GtkAdjustment *ax, *ay;
-        SPUnitSelector *us;
-        const SPUnit *unit;
-        NRRect bbox;
-        ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
-        ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
-        us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "scale_units"));
-        sp_selection_bbox (selection, &bbox);
-        unit = sp_unit_selector_get_unit (us);
+    if (selection && !selection->isEmpty()) {
+        GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_x"));
+        GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data (dlg, "scale_dimension_y"));
+        SPUnitSelector *us = SP_UNIT_SELECTOR(g_object_get_data (dlg, "scale_units"));
+        NR::Rect bbox = selection->bounds();
+        const SPUnit *unit = sp_unit_selector_get_unit (us);
         
         if (unit->base == SP_UNIT_ABSOLUTE) {
-            sp_unit_selector_set_value_in_points (us, ax, bbox.x1 - bbox.x0);
-            sp_unit_selector_set_value_in_points (us, ay, bbox.y1 - bbox.y0);
+            sp_unit_selector_set_value_in_points (us, ax, bbox.extent(NR::X));
+            sp_unit_selector_set_value_in_points (us, ay, bbox.extent(NR::Y));
         } else {
             gtk_adjustment_set_value (ax, 100.0);
             gtk_adjustment_set_value (ay, 100.0);
@@ -816,7 +762,7 @@ static void sp_transformation_scale_apply(GObject *dlg, SPSelection *selection)
     GtkAdjustment *ax = GTK_ADJUSTMENT(g_object_get_data(dlg, "scale_dimension_x"));
     GtkAdjustment *ay = GTK_ADJUSTMENT(g_object_get_data(dlg, "scale_dimension_y"));
 
-    NR::Rect const bbox(sp_selection_bbox(selection));
+    NR::Rect const bbox(selection->bounds());
     NR::Point const center(bbox.midpoint());
     SPUnit const *unit = sp_unit_selector_get_unit(us);
 
@@ -833,7 +779,7 @@ static void sp_transformation_scale_apply(GObject *dlg, SPSelection *selection)
     }
 
     if (selection) {
-        sp_document_done(SP_DT_DOCUMENT(selection->desktop));
+        sp_document_done(SP_DT_DOCUMENT(selection->desktop()));
     }
 }
 
@@ -844,8 +790,9 @@ static void sp_transformation_scale_apply(GObject *dlg, SPSelection *selection)
  */
 static void sp_transformation_rotate_value_changed(GtkAdjustment *, GObject * const dlg)
 {
-    if (g_object_get_data (dlg, "update"))
+    if (g_object_get_data (dlg, "update")) {
         return;
+    }
 
     GtkWidget *apply = GTK_WIDGET(g_object_get_data(dlg, "apply"));
     gtk_widget_set_sensitive (apply, TRUE);
@@ -856,33 +803,30 @@ static void sp_transformation_rotate_value_changed(GtkAdjustment *, GObject * co
 static GtkWidget *
 sp_transformation_page_rotate_new (GObject *obj)
 {
-    GtkWidget *frame, *vb, *tbl, *lbl, *img, *sb;
-    GtkAdjustment *adj;
+    GtkWidget *frame = gtk_frame_new (_("Rotate"));
 
-    frame = gtk_frame_new (_("Rotate"));
-
-    vb = gtk_vbox_new (FALSE, 0);
+    GtkWidget *vb = gtk_vbox_new (FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
     gtk_container_add (GTK_CONTAINER (frame), vb);
 
-    tbl = gtk_table_new (1, 3, FALSE);
+    GtkWidget *tbl = gtk_table_new (1, 3, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
     gtk_table_set_col_spacings (GTK_TABLE (tbl), 4);
     gtk_box_pack_start (GTK_BOX (vb), tbl, FALSE, FALSE, 0);
 
-    img = gtk_image_new_from_stock ( INKSCAPE_STOCK_ROTATE_LEFT, 
+    GtkWidget *img = gtk_image_new_from_stock ( INKSCAPE_STOCK_ROTATE_LEFT, 
                                      GTK_ICON_SIZE_LARGE_TOOLBAR );
     gtk_table_attach ( GTK_TABLE (tbl), img, 0, 1, 0, 1, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
-    adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
+    GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new (0.0, -1e6, 1e6, 0.01, 0.1, 0.1);
     g_object_set_data (obj, "rotate_angle", adj);
     g_signal_connect ( G_OBJECT (adj), "value_changed", 
                        G_CALLBACK (sp_transformation_rotate_value_changed), obj);
-    sb = gtk_spin_button_new (adj, 0.1, 2);
+    GtkWidget *sb = gtk_spin_button_new (adj, 0.1, 2);
     gtk_table_attach ( GTK_TABLE (tbl), sb, 1, 2, 0, 1, 
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 0, 0 );
-    lbl = gtk_label_new (_("deg"));
+    GtkWidget *lbl = gtk_label_new (_("deg"));
     gtk_table_attach ( GTK_TABLE (tbl), lbl, 2, 3, 0, 1, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
 
@@ -897,11 +841,9 @@ sp_transformation_page_rotate_new (GObject *obj)
 static void
 sp_transformation_rotate_update (GObject *dlg, SPSelection *selection)
 {
-    GtkWidget *page;
+    GtkWidget *page = GTK_WIDGET(g_object_get_data (dlg, "rotate"));
 
-    page = GTK_WIDGET(g_object_get_data (dlg, "rotate"));
-
-    if (selection && !sp_selection_is_empty (selection)) {
+    if (selection && !selection->isEmpty()) {
         gtk_widget_set_sensitive (page, TRUE);
     } else {
         gtk_widget_set_sensitive (page, FALSE);
@@ -914,14 +856,13 @@ static void sp_transformation_rotate_apply(GObject *dlg, SPSelection *selection)
 {
     GtkAdjustment *a = GTK_ADJUSTMENT(g_object_get_data(dlg, "rotate_angle"));
 
-    NRRect bbox_compat;
-    sp_selection_bbox (selection, &bbox_compat);
-    NR::Rect bbox(bbox_compat);
+    NR::Rect bbox = selection->bounds();
     NR::Point center = bbox.midpoint();
     sp_selection_rotate_relative (selection, center, a->value);
 
-    if (selection)
-        sp_document_done (SP_DT_DOCUMENT (selection->desktop));
+    if (selection) {
+        sp_document_done (SP_DT_DOCUMENT (selection->desktop()));
+    }
 }
 
 static void sp_transformation_skew_apply(GObject *dlg, SPSelection *selection)
