@@ -13,34 +13,29 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <math.h>
 #include <string.h>
 #include <glib.h>
 #include <libnr/nr-rect.h>
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-blit.h>
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_bpath.h>
-#include <libart_lgpl/art_vpath.h>
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_vpath_bpath.h>
-#include <libart_lgpl/art_rect_svp.h>
-#include <libart_lgpl/art_svp_vpath.h>
-#include <libart_lgpl/art_svp_vpath_stroke.h>
-#include <libart_lgpl/art_svp_wind.h>
-#include <libart_lgpl/art_svp_point.h>
-#include <libart_lgpl/art_gray_svp.h>
+#include <libnr/nr-path.h>
 #include "../style.h"
 #include "nr-arena.h"
 #include "nr-arena-glyphs.h"
 
 #ifdef test_glyph_liv
-#include "../helper/canvas-bpath.h"
+#include "../display/canvas-bpath.h"
 #include "../livarot/LivarotDefs.h"
 #include "../livarot/Path.h"
 #include "../livarot/Shape.h"
-#include "../livarot/Ligne.h"
 #include <libnr/nr-matrix-ops.h>
+#include <libnrtype/font-instance.h>
+#include <libnrtype/raster-glyph.h>
+#include <libnrtype/RasterFont.h>
 
 // defined in nr-arena-shape.cpp
 void nr_pixblock_render_shape_mask_or (NRPixBlock &m,Shape* theS);
@@ -52,7 +47,7 @@ static void nr_arena_glyphs_finalize (NRObject *object);
 
 static guint nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
 static guint nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
-static NRArenaItem *nr_arena_glyphs_pick (NRArenaItem *item, double x, double y, double delta, unsigned int sticky);
+static NRArenaItem *nr_arena_glyphs_pick (NRArenaItem *item, NR::Point p, double delta, unsigned int sticky);
 
 static NRArenaItemClass *glyphs_parent_class;
 
@@ -83,6 +78,7 @@ nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass)
 	glyphs_parent_class = (NRArenaItemClass *) ((NRObjectClass *) klass)->parent;
 
 	object_class->finalize = nr_arena_glyphs_finalize;
+	object_class->cpp_ctor = NRObject::invoke_ctor<NRArenaGlyphs>;
 
 	item_class->update = nr_arena_glyphs_update;
 	item_class->clip = nr_arena_glyphs_clip;
@@ -92,48 +88,49 @@ nr_arena_glyphs_class_init (NRArenaGlyphsClass *klass)
 static void
 nr_arena_glyphs_init (NRArenaGlyphs *glyphs)
 {
-	glyphs->curve = NULL;
 	glyphs->style = NULL;
+	nr_matrix_set_identity(&glyphs->g_transform);
+	glyphs->font = NULL;
+	glyphs->glyph = 0;
 
-#ifdef test_glyph_liv
-  glyphs->stroke_shp=NULL;
-  glyphs->cached_shp=NULL;
-  glyphs->cached_shp_dirty=false;
-  glyphs->cached_style_dirty=false;
-#else
-	glyphs->stroke_svp = NULL;
-#endif
+	glyphs->rfont = NULL;
+	glyphs->sfont = NULL;
+	glyphs->x = glyphs->y = 0.0;
+
+//	nr_matrix_set_identity(&glyphs->cached_tr);
+//	glyphs->cached_shp=NULL;
+//	glyphs->cached_shp_dirty=false;
+//	glyphs->cached_style_dirty=false;
+
+//	glyphs->stroke_shp=NULL;
 }
 
 static void
 nr_arena_glyphs_finalize (NRObject *object)
 {
-	NRArenaGlyphs *glyphs;
+	NRArenaGlyphs *glyphs=static_cast<NRArenaGlyphs *>(object);
 
-	glyphs = NR_ARENA_GLYPHS (object);
-
-#ifdef test_glyph_liv
-	if (glyphs->cached_shp) {
-		delete glyphs->cached_shp;
-		glyphs->cached_shp = NULL;
-	}
-	if (glyphs->stroke_shp) {
-		delete glyphs->stroke_shp;
-		glyphs->stroke_shp = NULL;
-	}
-#else
-	if (glyphs->stroke_svp) {
-		art_svp_free (glyphs->stroke_svp);
-		glyphs->stroke_svp = NULL;
-	}
-#endif
+//	if (glyphs->cached_shp) {
+//		delete glyphs->cached_shp;
+//		glyphs->cached_shp = NULL;
+//	}
+//	if (glyphs->stroke_shp) {
+//		delete glyphs->stroke_shp;
+//		glyphs->stroke_shp = NULL;
+//	}
   
 	if (glyphs->rfont) {
-		glyphs->rfont = nr_rasterfont_unref (glyphs->rfont);
+		glyphs->rfont->Unref();
+		glyphs->rfont=NULL;
+	}
+	if (glyphs->sfont) {
+		glyphs->sfont->Unref();
+		glyphs->sfont=NULL;
 	}
 
 	if (glyphs->font) {
-		glyphs->font = nr_font_unref (glyphs->font);
+		glyphs->font->Unref();
+		glyphs->font=NULL;
 	}
 
 	if (glyphs->style) {
@@ -141,10 +138,6 @@ nr_arena_glyphs_finalize (NRObject *object)
 		glyphs->style = NULL;
 	}
 
-	if (glyphs->curve) {
-		glyphs->curve = sp_curve_unref (glyphs->curve);
-	}
-
 	((NRObjectClass *) glyphs_parent_class)->finalize (object);
 }
 
@@ -152,13 +145,9 @@ static guint
 nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
 	NRArenaGlyphs *glyphs;
-	NRRasterFont *rfont;
+	raster_font *rfont;
 	NRMatrix t;
-#ifndef test_glyph_liv
-	ArtBpath *abp;
-	ArtVpath *vp, *pvp;
-#endif
-	ArtDRect bbox;
+	NRRect bbox;
 
 	glyphs = NR_ARENA_GLYPHS (item);
 
@@ -170,172 +159,75 @@ nr_arena_glyphs_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 	}
 
 	/* Release state data */
-#ifdef test_glyph_liv
-	if (glyphs->stroke_shp) {
-    delete glyphs->stroke_shp;
-		glyphs->stroke_shp = NULL;
-	}
-#else
-	if (glyphs->stroke_svp) {
-		art_svp_free (glyphs->stroke_svp);
-		glyphs->stroke_svp = NULL;
-	}
-#endif
+//	if (glyphs->stroke_shp) {
+//    delete glyphs->stroke_shp;
+//		glyphs->stroke_shp = NULL;
+//	}
 
-	if (!glyphs->font || !glyphs->curve || !glyphs->style) return NR_ARENA_ITEM_STATE_ALL;
+	if (!glyphs->font || !glyphs->style) return NR_ARENA_ITEM_STATE_ALL;
 	if ((glyphs->style->fill.type == SP_PAINT_TYPE_NONE) && (glyphs->style->stroke.type == SP_PAINT_TYPE_NONE)) return NR_ARENA_ITEM_STATE_ALL;
 
 	bbox.x0 = bbox.y0 = bbox.x1 = bbox.y1 = 0.0;
 
 	if (glyphs->style->fill.type != SP_PAINT_TYPE_NONE) {
-		NRRect area;
-		nr_matrix_multiply (&t, &glyphs->transform, &gc->transform);
-		rfont = nr_rasterfont_new (glyphs->font, &t);
-		if (glyphs->rfont) glyphs->rfont = nr_rasterfont_unref (glyphs->rfont);
-		glyphs->rfont = rfont;
+		NRRect narea;
+		nr_matrix_multiply (&t, &glyphs->g_transform, &gc->transform);
 		glyphs->x = t.c[4];
 		glyphs->y = t.c[5];
-		nr_rasterfont_glyph_area_get (rfont, glyphs->glyph, &area);
-		bbox.x0 = area.x0 + glyphs->x;
-		bbox.y0 = area.y0 + glyphs->y;
-		bbox.x1 = area.x1 + glyphs->x;
-		bbox.y1 = area.y1 + glyphs->y;
-	}
+    t.c[4]=0;
+    t.c[5]=0;
+		rfont = glyphs->font->RasterFont(t,0);
+		if (glyphs->rfont) glyphs->rfont->Unref();
+		glyphs->rfont = rfont;
+		if ( glyphs->rfont ) glyphs->rfont->BBox(glyphs->glyph, &narea);
+		bbox.x0 = narea.x0 + glyphs->x;
+		bbox.y0 = narea.y0 + glyphs->y;
+		bbox.x1 = narea.x1 + glyphs->x;
+		bbox.y1 = narea.y1 + glyphs->y;
+	}                               
 
 	if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
 		/* Build state data */
-#ifdef test_glyph_liv
-    bool   recache=false;
-    bool   antiIsometry=false;
-    if ( glyphs->cached_shp == NULL || glyphs->cached_shp_dirty || glyphs->cached_style_dirty ) {
-      recache=true;
-    } else {
-      NR::Matrix   oMat(glyphs->cached_tr);
-      NR::Matrix   nMat(gc->transform);
-      oMat=oMat.inverse();
-      NR::Matrix   p=oMat*nMat;
-      NR::Matrix   tp;
-      // trasnposition
-      tp[0]=p[0];
-      tp[1]=p[2];
-      tp[2]=p[1];
-      tp[3]=p[3];
-      NR::Matrix   isom=tp*p;
-      if ( fabs(isom[0]-1.0) < 0.01 && fabs(isom[3]-1.0) < 0.01 &&
-           fabs(isom[1]) < 0.01 && fabs(isom[2]) < 0.01 ) {
-        // the changed wrt the cached version is an isometry-> no need to recompute
-        // the uncrossed polygon
-        antiIsometry=(p.det()<0)?true:false;
-        recache=false;
-      } else {
-        recache=true;
-      }
+		NRRect narea;
+		nr_matrix_multiply (&t, &glyphs->g_transform, &gc->transform);
+		glyphs->x = t.c[4];
+		glyphs->y = t.c[5];
+    t.c[4]=0;
+    t.c[5]=0;
+	  const float scale = NR_MATRIX_DF_EXPANSION (&gc->transform);
+	  if ( fabs(glyphs->style->stroke_width.computed * scale) > 0.01 ) { // sinon c'est 0=oon veut pas de bord
+      font_style nstyl;
+      nstyl.transform=t;
+      nstyl.stroke_width=MAX (0.125, glyphs->style->stroke_width.computed * scale);
+      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_BUTT ) nstyl.stroke_cap=butt_straight;
+      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_ROUND ) nstyl.stroke_cap=butt_round;
+      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_SQUARE ) nstyl.stroke_cap=butt_square;
+      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_MITER ) nstyl.stroke_join=join_pointy;
+      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_ROUND ) nstyl.stroke_join=join_round;
+      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_BEVEL ) nstyl.stroke_join=join_straight;
+      nstyl.nbDash=0;
+      nstyl.dashes=NULL;
+		  if ( glyphs->style->stroke_dash.n_dash > 0 ) {
+        nstyl.nbDash=glyphs->style->stroke_dash.n_dash;
+        nstyl.dashes=(double*)malloc(nstyl.nbDash*sizeof(double));
+		    for (int i = 0; i < nstyl.nbDash; i++) nstyl.dashes[i]= glyphs->style->stroke_dash.dash[i] * scale;
+  		}
+		  rfont = glyphs->font->RasterFont( nstyl);
+      if ( nstyl.dashes ) free(nstyl.dashes);
+		  if (glyphs->sfont) glyphs->sfont->Unref();
+		  glyphs->sfont = rfont;
+		  if ( glyphs->sfont ) glyphs->sfont->BBox(glyphs->glyph, &narea);
+      narea.x0-=nstyl.stroke_width;
+      narea.y0-=nstyl.stroke_width;
+      narea.x1+=nstyl.stroke_width;
+      narea.y1+=nstyl.stroke_width;
+		  bbox.x0 = narea.x0 + glyphs->x;
+		  bbox.y0 = narea.y0 + glyphs->y;
+		  bbox.x1 = narea.x1 + glyphs->x;
+		  bbox.y1 = narea.y1 + glyphs->y;
     }
-    if ( recache ) {
-//      printf("r: %i %i\n",(glyphs->cached_shp_dirty)?1:0,(glyphs->cached_style_dirty)?1:0);
-      glyphs->cached_shp_dirty=glyphs->cached_style_dirty=false;
-      Path*  thePath=new Path;
-      Shape* theShape=new Shape;    
-      if ( glyphs->cached_shp == NULL ) glyphs->cached_shp=new Shape;
-      {
-        NR::Matrix   tempMat(gc->transform);
-        thePath->LoadArtBPath(glyphs->curve->bpath,tempMat,true);
-      }
-      thePath->Convert(0.25);
-      
-      gdouble width;
-      width = glyphs->style->stroke_width.computed * NR_MATRIX_DF_EXPANSION (&gc->transform);
-      glyphs->cached_tr=gc->transform;
-      width = MAX (0.125, width);
-      
-      JoinType join=join_straight;
-      ButtType butt=butt_straight;
-      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_BUTT ) butt=butt_straight;
-      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_ROUND ) butt=butt_round;
-      if ( glyphs->style->stroke_linecap.value == SP_STROKE_LINECAP_SQUARE ) butt=butt_square;
-      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_MITER ) join=join_pointy;
-      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_ROUND ) join=join_round;
-      if ( glyphs->style->stroke_linejoin.value == SP_STROKE_LINEJOIN_BEVEL ) join=join_straight;
-      thePath->Stroke(theShape,false,0.5*width, join,butt,width*glyphs->style->stroke_miterlimit.value);
-      
-      glyphs->cached_shp->ConvertToShape(theShape,fill_nonZero);
-      delete thePath;
-      delete theShape;
-    }
-    if ( glyphs->cached_shp == NULL ) {
-      if ( glyphs->stroke_shp == NULL ) glyphs->stroke_shp=new Shape;
-      glyphs->stroke_shp->Reset();
-    } else {
-      if ( glyphs->stroke_shp == NULL ) glyphs->stroke_shp=new Shape;
-      NR::Matrix   oMat(glyphs->cached_tr);
-      NR::Matrix   nMat(gc->transform);
-      oMat=oMat.inverse();
-      NR::Matrix   p=oMat*nMat; // attention a pas le mettre dans le sens inverse
-      bool antiIsometry=(p.det()<0)?true:false;
-      glyphs->stroke_shp->Reset(glyphs->cached_shp->nbPt,glyphs->cached_shp->nbAr);
-      for (int i=0;i<glyphs->cached_shp->nbPt;i++) {
-        glyphs->stroke_shp->AddPoint(glyphs->cached_shp->pts[i].x*p); // matrix * point multiplication is in reverse order?
-      }
-      if ( antiIsometry ) {
-        for (int i=0;i<glyphs->cached_shp->nbAr;i++) {
-          if ( glyphs->cached_shp->aretes[i].st < 0 || glyphs->cached_shp->aretes[i].en < 0 ) {
-            printf("inv\n");
-          }
-          glyphs->stroke_shp->AddEdge(glyphs->cached_shp->aretes[i].en,glyphs->cached_shp->aretes[i].st);
-        }
-      } else {
-        for (int i=0;i<glyphs->cached_shp->nbAr;i++) {
-          if ( glyphs->cached_shp->aretes[i].st < 0 || glyphs->cached_shp->aretes[i].en < 0 ) {
-            printf("inv\n");
-          }
-          glyphs->stroke_shp->AddEdge(glyphs->cached_shp->aretes[i].st,glyphs->cached_shp->aretes[i].en);
-        }
-      }
-      glyphs->stroke_shp->ForceToPolygon();
-      glyphs->stroke_shp->flags|=need_points_sorting;
-      glyphs->stroke_shp->flags|=need_edges_sorting;
-    }
-#else
-		abp = art_bpath_affine_transform (glyphs->curve->bpath, NR_MATRIX_D_TO_DOUBLE (&gc->transform));
-		vp = art_bez_path_to_vec (abp, 0.25);
-		art_free (abp);
-		pvp = art_vpath_perturb (vp);
-		art_free (vp);
-
-		if (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE) {
-			gdouble width;
-			width = glyphs->style->stroke_width.computed * NR_MATRIX_DF_EXPANSION (&gc->transform);
-			width = MAX (0.125, width);
-			glyphs->stroke_svp = art_svp_vpath_stroke (pvp,
-								   (ArtPathStrokeJoinType)glyphs->style->stroke_linejoin.value,
-								   (ArtPathStrokeCapType)glyphs->style->stroke_linecap.value,
-								   width,
-								   glyphs->style->stroke_miterlimit.value, 0.25);
-		}
-
-		art_free (pvp);
-#endif
 	}
-
-#ifdef test_glyph_liv
-  if ( glyphs->stroke_shp ) {
-    glyphs->stroke_shp->CalcBBox();
-    if ( bbox.x0 >= bbox.x1 || bbox.y0 >= bbox.y1 ) {
-      bbox.x0=glyphs->stroke_shp->leftX;
-      bbox.x1=glyphs->stroke_shp->rightX;
-      bbox.y0=glyphs->stroke_shp->topY;
-      bbox.y1=glyphs->stroke_shp->bottomY;
-    } else {
-      if ( glyphs->stroke_shp->leftX < bbox.x0 ) bbox.x0=glyphs->stroke_shp->leftX;
-      if ( glyphs->stroke_shp->rightX > bbox.x1 ) bbox.x1=glyphs->stroke_shp->rightX;
-      if ( glyphs->stroke_shp->topY < bbox.y0 ) bbox.y0=glyphs->stroke_shp->topY;
-      if ( glyphs->stroke_shp->bottomY > bbox.y1 ) bbox.y1=glyphs->stroke_shp->bottomY;
-    }
-  }
-#else 
-	if (glyphs->stroke_svp) art_drect_svp_union (&bbox, glyphs->stroke_svp);
-#endif
-	if (art_drect_empty (&bbox)) return NR_ARENA_ITEM_STATE_ALL;
+	if (nr_rect_d_test_empty(&bbox)) return NR_ARENA_ITEM_STATE_ALL;
 
 	item->bbox.x0 = (gint32)(bbox.x0 - 1.0);
 	item->bbox.y0 = (gint32)(bbox.y0 - 1.0);
@@ -353,7 +245,7 @@ nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 
 	glyphs = NR_ARENA_GLYPHS (item);
 
-	if (!glyphs->font || !glyphs->curve) return item->state;
+	if (!glyphs->font ) return item->state;
 
 	/* TODO : render to greyscale pixblock provided for clipping */
 
@@ -361,20 +253,21 @@ nr_arena_glyphs_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 }
 
 static NRArenaItem *
-nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky)
+nr_arena_glyphs_pick (NRArenaItem *item, NR::Point p, gdouble delta, unsigned int sticky)
 {
 	NRArenaGlyphs *glyphs;
 
 	glyphs = NR_ARENA_GLYPHS (item);
 
-	if (!glyphs->font || !glyphs->curve) return NULL;
+	if (!glyphs->font ) return NULL;
 	if (!glyphs->style) return NULL;
+	
+	const double x = p[NR::X];
+	const double y = p[NR::Y];
+	/* fixme: pt in rect*/
+	if ((x >= item->bbox.x0) && (y >= item->bbox.y0) && (x <= item->bbox.x1) && (y <= item->bbox.y1)) return item;
 
-	/* fixme: */
-	if ((x >= item->bbox.x0) && (y >= item->bbox.y0) && (x < item->bbox.x1) && (y < item->bbox.y1)) return item;
-
-#ifdef test_glyph_liv
-  NR::Point const thePt(x, y);
+/*	NR::Point const thePt = p;
 		if (glyphs->stroke_shp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
 			if (glyphs->stroke_shp->PtWinding(thePt) > 0 ) return item;
 		}
@@ -382,59 +275,31 @@ nr_arena_glyphs_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, un
 			if (glyphs->stroke_shp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
 				if ( glyphs->stroke_shp->DistanceLE(thePt, delta)) return item;
 			}
-		}
-#else
-	if (glyphs->stroke_svp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
-		if (art_svp_point_wind (glyphs->stroke_svp, x, y)) return item;
-	}
-	if (delta > 1e-3) {
-		if (glyphs->stroke_svp && (glyphs->style->stroke.type != SP_PAINT_TYPE_NONE)) {
-			if (art_svp_point_dist (glyphs->stroke_svp, x, y) <= delta) return item;
-		}
-	}
-#endif
+		}*/
   
 	return NULL;
 }
 
 void
-nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, unsigned int lieutenant, NRFont *font, gint glyph, const NRMatrix *transform)
+nr_arena_glyphs_set_path (NRArenaGlyphs *glyphs, SPCurve *curve, unsigned int lieutenant, font_instance *font, gint glyph, const NRMatrix *transform)
 {
 	nr_return_if_fail (glyphs != NULL);
 	nr_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
 
 	nr_arena_item_request_render (NR_ARENA_ITEM (glyphs));
   
-  glyphs->cached_shp_dirty=true;
+ // glyphs->cached_shp_dirty=true;
 
-	if (glyphs->curve) {
-		sp_curve_unref (glyphs->curve);
-		glyphs->curve = NULL;
+	if (transform) {
+		glyphs->g_transform = *transform;
+	} else {
+		nr_matrix_set_identity (&glyphs->g_transform);
 	}
-
-	if (curve) {
-		if (transform) {
-			ArtBpath *abp;
-			double a[6];
-			a[0] = transform->c[0];
-			a[1] = transform->c[1];
-			a[2] = transform->c[2];
-			a[3] = transform->c[3];
-			a[4] = transform->c[4];
-			a[5] = transform->c[5];
-			abp = art_bpath_affine_transform (curve->bpath, a);
-			curve = sp_curve_new_from_bpath (abp);
-			glyphs->curve = curve;
-			glyphs->transform = *transform;
-		} else {
-			glyphs->curve = curve;
-			sp_curve_ref (curve);
-			nr_matrix_set_identity (&glyphs->transform);
-		}
-	}
-
-	if (glyphs->font) glyphs->font = nr_font_unref (glyphs->font);
-	if (font) glyphs->font = nr_font_ref (font);
+		
+	//printf("glyph_setpath ");
+	if ( font ) font->Ref();
+	if ( glyphs->font ) glyphs->font->Unref();
+	glyphs->font=font;
 	glyphs->glyph = glyph;
 
 	nr_arena_item_request_update (NR_ARENA_ITEM (glyphs), NR_ARENA_ITEM_STATE_ALL, FALSE);
@@ -446,7 +311,7 @@ nr_arena_glyphs_set_style (NRArenaGlyphs *glyphs, SPStyle *style)
 	nr_return_if_fail (glyphs != NULL);
 	nr_return_if_fail (NR_IS_ARENA_GLYPHS (glyphs));
 
-  glyphs->cached_style_dirty=true;
+//  glyphs->cached_style_dirty=true;
   
 	if (style) sp_style_ref (style);
 	if (glyphs->style) sp_style_unref (glyphs->style);
@@ -465,7 +330,8 @@ nr_arena_glyphs_fill_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 	item = NR_ARENA_ITEM (glyphs);
 
 	if (glyphs->rfont && nr_rect_l_test_intersect (area, &item->bbox)) {
-		nr_rasterfont_glyph_mask_render (glyphs->rfont, glyphs->glyph, m, glyphs->x, glyphs->y);
+		raster_glyph* g=glyphs->rfont->GetGlyph(glyphs->glyph);
+		if ( g ) g->Blit(NR::Point(glyphs->x, glyphs->y),*m);
 	}
 
 	return item->state;
@@ -477,21 +343,17 @@ nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m
 	NRArenaItem *item;
 
 	item = NR_ARENA_ITEM (glyphs);
-#ifdef test_glyph_liv
-	if (glyphs->stroke_shp && nr_rect_l_test_intersect (area, &item->bbox)) {
-#else
-	if (glyphs->stroke_svp && nr_rect_l_test_intersect (area, &item->bbox)) {
-#endif
+	if (glyphs->sfont && nr_rect_l_test_intersect (area, &item->bbox)) {
+		raster_glyph* g=glyphs->sfont->GetGlyph(glyphs->glyph);
+		if ( g ) g->Blit(NR::Point(glyphs->x, glyphs->y),*m);
+	}
+/*	if (glyphs->stroke_shp && nr_rect_l_test_intersect (area, &item->bbox)) {
 		NRPixBlock gb;
 		gint x, y;
 		nr_pixblock_setup_fast (&gb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
-#ifdef test_glyph_liv
     // art_gray_svp_aa is just fillung apparently
     // dunno why it's used here instead of its libnr counterpart
     nr_pixblock_render_shape_mask_or (gb,glyphs->stroke_shp);    
-#else
-		art_gray_svp_aa (glyphs->stroke_svp, area->x0, area->y0, area->x1, area->y1, NR_PIXBLOCK_PX (&gb), gb.rs);
-#endif
 		for (y = area->y0; y < area->y1; y++) {
 			guchar *d, *s;
 			d = NR_PIXBLOCK_PX (m) + (y - area->y0) * m->rs;
@@ -504,11 +366,7 @@ nr_arena_glyphs_stroke_mask (NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m
 		}
 		nr_pixblock_release (&gb);
 		m->empty = FALSE;
-#ifdef test_glyph_liv
-  }
-#else
-	}
-#endif
+	}*/
   
 	return item->state;
 }
@@ -520,7 +378,7 @@ static void nr_arena_glyphs_group_finalize (NRObject *object);
 static guint nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
 static unsigned int nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags);
 static unsigned int nr_arena_glyphs_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
-static NRArenaItem *nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky);
+static NRArenaItem *nr_arena_glyphs_group_pick (NRArenaItem *item, NR::Point p, gdouble delta, unsigned int sticky);
 
 static NRArenaGroupClass *group_parent_class;
 
@@ -551,6 +409,7 @@ nr_arena_glyphs_group_class_init (NRArenaGlyphsGroupClass *klass)
 	group_parent_class = (NRArenaGroupClass *) ((NRObjectClass *) klass)->parent;
 
 	object_class->finalize = nr_arena_glyphs_group_finalize;
+	object_class->cpp_ctor = NRObject::invoke_ctor<NRArenaGlyphsGroup>;
 
 	item_class->update = nr_arena_glyphs_group_update;
 	item_class->render = nr_arena_glyphs_group_render;
@@ -572,9 +431,7 @@ nr_arena_glyphs_group_init (NRArenaGlyphsGroup *group)
 static void
 nr_arena_glyphs_group_finalize (NRObject *object)
 {
-	NRArenaGlyphsGroup *group;
-
-	group = NR_ARENA_GLYPHS_GROUP (object);
+	NRArenaGlyphsGroup *group=static_cast<NRArenaGlyphsGroup *>(object);
 
 	if (group->fill_painter) {
 		sp_painter_free (group->fill_painter);
@@ -591,15 +448,13 @@ nr_arena_glyphs_group_finalize (NRObject *object)
 		group->style = NULL;
 	}
 
-		((NRObjectClass *) group_parent_class)->finalize (object);
+	((NRObjectClass *) group_parent_class)->finalize (object);
 }
 
 static guint
 nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
 {
-	NRArenaGlyphsGroup *group;
-
-	group = NR_ARENA_GLYPHS_GROUP (item);
+	NRArenaGlyphsGroup *group = NR_ARENA_GLYPHS_GROUP (item);
 
 	if (group->fill_painter) {
 		sp_painter_free (group->fill_painter);
@@ -614,14 +469,20 @@ nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint
 	item->render_opacity = TRUE;
 	if (group->style->fill.type == SP_PAINT_TYPE_PAINTSERVER) {
 		group->fill_painter = sp_paint_server_painter_new (SP_STYLE_FILL_SERVER (group->style),
-								   NR_MATRIX_D_TO_DOUBLE (&gc->transform), &group->paintbox);
-	item->render_opacity = FALSE;
+							 NR::Matrix (&gc->transform), NR::Matrix (&gc->parent->transform),
+							 &group->paintbox);
+		item->render_opacity = FALSE;
 	}
 
 	if (group->style->stroke.type == SP_PAINT_TYPE_PAINTSERVER) {
 		group->stroke_painter = sp_paint_server_painter_new (SP_STYLE_STROKE_SERVER (group->style),
-								     NR_MATRIX_D_TO_DOUBLE (&gc->transform), &group->paintbox);
-	item->render_opacity = FALSE;
+							 NR::Matrix (&gc->transform), NR::Matrix (&gc->parent->transform),
+							 &group->paintbox);
+		item->render_opacity = FALSE;
+	}
+
+	if ( item->render_opacity == TRUE && group->style->stroke.type != SP_PAINT_TYPE_NONE && group->style->fill.type != SP_PAINT_TYPE_NONE ) {
+		item->render_opacity=FALSE;
 	}
 
 	if (((NRArenaItemClass *) group_parent_class)->update)
@@ -635,57 +496,49 @@ nr_arena_glyphs_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint
 static unsigned int
 nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags)
 {
-	NRArenaGroup *group;
-	NRArenaGlyphsGroup *ggroup;
 	NRArenaItem *child;
-	SPStyle *style;
-	guint ret;
 
-	group = NR_ARENA_GROUP (item);
-	ggroup = NR_ARENA_GLYPHS_GROUP (item);
-	style = ggroup->style;
+	NRArenaGroup *group = NR_ARENA_GROUP (item);
+	NRArenaGlyphsGroup *ggroup = NR_ARENA_GLYPHS_GROUP (item);
+	SPStyle const *style = ggroup->style;
 
-	ret = item->state;
+	guint ret = item->state;
 
 	/* Fill */
 	if (style->fill.type != SP_PAINT_TYPE_NONE) {
-		NRPixBlock mb;
+		NRPixBlock m;
 		guint32 rgba;
-		nr_pixblock_setup_fast (&mb, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
+		nr_pixblock_setup_fast (&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
 		/* Render children fill mask */
 		for (child = group->children; child != NULL; child = child->next) {
-			ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, &mb);
+			ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, &m);
 			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) {
-				nr_pixblock_release (&mb);
+				nr_pixblock_release (&m);
 				return ret;
 			}
 		}
 		/* Composite into buffer */
 		switch (style->fill.type) {
 		case SP_PAINT_TYPE_COLOR:
-			rgba = sp_color_get_rgba32_falpha (&style->fill.value.color,
-							   SP_SCALE24_TO_FLOAT (style->fill_opacity.value) *
-							   SP_SCALE24_TO_FLOAT (style->opacity.value));
-			nr_blit_pixblock_mask_rgba32 (pb, &mb, rgba);
+			if ( item->render_opacity ) {
+				rgba = sp_color_get_rgba32_falpha (&style->fill.value.color,
+                                           SP_SCALE24_TO_FLOAT (style->fill_opacity.value) *
+                                           SP_SCALE24_TO_FLOAT (style->opacity.value));
+			} else {
+				rgba = sp_color_get_rgba32_falpha (&style->fill.value.color, SP_SCALE24_TO_FLOAT (style->fill_opacity.value));
+			}
+			nr_blit_pixblock_mask_rgba32 (pb, &m, rgba);
 			pb->empty = FALSE;
 			break;
 		case SP_PAINT_TYPE_PAINTSERVER:
 			if (ggroup->fill_painter) {
-				NRPixBlock cb;
-				/* Need separate gradient buffer */
-				nr_pixblock_setup_fast (&cb, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
-				ggroup->fill_painter->fill (ggroup->fill_painter, &cb);
-				cb.empty = FALSE;
-				/* Composite */
-				nr_blit_pixblock_pixblock_mask (pb, &cb, &mb);
-				pb->empty = FALSE;
-				nr_pixblock_release (&cb);
+				nr_arena_render_paintserver_fill (pb, area, ggroup->fill_painter, SP_SCALE24_TO_FLOAT (style->fill_opacity.value), &m);
 			}
 			break;
 		default:
 			break;
 		}
-		nr_pixblock_release (&mb);
+		nr_pixblock_release (&m);
 	}
 
 	/* Stroke */
@@ -704,23 +557,20 @@ nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb,
 		/* Composite into buffer */
 		switch (style->stroke.type) {
 		case SP_PAINT_TYPE_COLOR:
-			rgba = sp_color_get_rgba32_falpha (&style->stroke.value.color,
-							   SP_SCALE24_TO_FLOAT (style->stroke_opacity.value) *
-							   SP_SCALE24_TO_FLOAT (style->opacity.value));
+			if ( item->render_opacity ) {
+				rgba = sp_color_get_rgba32_falpha (&style->stroke.value.color,
+                                           SP_SCALE24_TO_FLOAT (style->stroke_opacity.value) *
+                                           SP_SCALE24_TO_FLOAT (style->opacity.value));
+			} else {
+				rgba = sp_color_get_rgba32_falpha (&style->stroke.value.color,
+							 SP_SCALE24_TO_FLOAT (style->stroke_opacity.value));
+			}
 			nr_blit_pixblock_mask_rgba32 (pb, &m, rgba);
 			pb->empty = FALSE;
 			break;
 		case SP_PAINT_TYPE_PAINTSERVER:
 			if (ggroup->stroke_painter) {
-				NRPixBlock cb;
-				/* Need separate gradient buffer */
-				nr_pixblock_setup_fast (&cb, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
-				ggroup->stroke_painter->fill (ggroup->stroke_painter, &cb);
-				cb.empty = FALSE;
-				/* Composite */
-				nr_blit_pixblock_pixblock_mask (pb, &cb, &m);
-				pb->empty = FALSE;
-				nr_pixblock_release (&cb);
+				nr_arena_render_paintserver_fill (pb, area, ggroup->stroke_painter, SP_SCALE24_TO_FLOAT (style->stroke_opacity.value), &m);
 			}
 			break;
 		default:
@@ -735,18 +585,12 @@ nr_arena_glyphs_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb,
 static unsigned int
 nr_arena_glyphs_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 {
-	NRArenaGroup *group;
-	NRArenaGlyphsGroup *ggroup;
-	NRArenaItem *child;
-	guint ret;
+	NRArenaGroup *group = NR_ARENA_GROUP (item);
 
-	group = NR_ARENA_GROUP (item);
-	ggroup = NR_ARENA_GLYPHS_GROUP (item);
-
-	ret = item->state;
+	guint ret = item->state;
 
 	/* Render children fill mask */
-	for (child = group->children; child != NULL; child = child->next) {
+	for (NRArenaItem *child = group->children; child != NULL; child = child->next) {
 		ret = nr_arena_glyphs_fill_mask (NR_ARENA_GLYPHS (child), area, pb);
 		if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) return ret;
 	}
@@ -755,17 +599,12 @@ nr_arena_glyphs_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 }
 
 static NRArenaItem *
-nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, unsigned int sticky)
+nr_arena_glyphs_group_pick (NRArenaItem *item, NR::Point p, gdouble delta, unsigned int sticky)
 {
-	NRArenaGroup *group;
-	NRArenaItem *picked;
-
-	group = NR_ARENA_GROUP (item);
-
-	picked = NULL;
+	NRArenaItem *picked = NULL;
 
 	if (((NRArenaItemClass *) group_parent_class)->pick)
-		picked = ((NRArenaItemClass *) group_parent_class)->pick (item, x, y, delta, sticky);
+		picked = ((NRArenaItemClass *) group_parent_class)->pick (item, p, delta, sticky);
 
 	if (picked) picked = item;
 
@@ -775,9 +614,7 @@ nr_arena_glyphs_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble del
 void
 nr_arena_glyphs_group_clear (NRArenaGlyphsGroup *sg)
 {
-	NRArenaGroup *group;
-
-	group = NR_ARENA_GROUP (sg);
+	NRArenaGroup *group = NR_ARENA_GROUP (sg);
 
 	nr_arena_item_request_render (NR_ARENA_ITEM (group));
 
@@ -787,28 +624,24 @@ nr_arena_glyphs_group_clear (NRArenaGlyphsGroup *sg)
 }
 
 void
-nr_arena_glyphs_group_add_component (NRArenaGlyphsGroup *sg, NRFont *font, int glyph, const NRMatrix *transform)
+nr_arena_glyphs_group_add_component (NRArenaGlyphsGroup *sg, font_instance *font, int glyph, const NRMatrix *transform)
 {
 	NRArenaGroup *group;
 	NRBPath bpath;
 
 	group = NR_ARENA_GROUP (sg);
 
-	if (nr_font_glyph_outline_get (font, glyph, &bpath, FALSE) && bpath.path) {
-		SPCurve *curve;
+	if ( font ) bpath.path=(NArtBpath*)font->ArtBPath(glyph); else bpath.path=NULL;
+	if ( bpath.path ) {
 
 		nr_arena_item_request_render (NR_ARENA_ITEM (group));
 
-		curve = sp_curve_new_from_foreign_bpath (bpath.path);
-		if (curve) {
 			NRArenaItem *new_arena;
-			new_arena = nr_arena_item_new (NR_ARENA_ITEM (group)->arena, NR_TYPE_ARENA_GLYPHS);
+			new_arena = NRArenaGlyphs::create(group->arena);
 			nr_arena_item_append_child (NR_ARENA_ITEM (group), new_arena);
 			nr_arena_item_unref (new_arena);
-			nr_arena_glyphs_set_path (NR_ARENA_GLYPHS (new_arena), curve, FALSE, font, glyph, transform);
+			nr_arena_glyphs_set_path (NR_ARENA_GLYPHS (new_arena), NULL, FALSE, font, glyph, transform);
 			nr_arena_glyphs_set_style (NR_ARENA_GLYPHS (new_arena), sg->style);
-			sp_curve_unref (curve);
-		}
 	}
 
 }
@@ -837,7 +670,7 @@ nr_arena_glyphs_group_set_style (NRArenaGlyphsGroup *sg, SPStyle *style)
 }
 
 void
-nr_arena_glyphs_group_set_paintbox (NRArenaGlyphsGroup *gg, const ArtDRect *pbox)
+nr_arena_glyphs_group_set_paintbox (NRArenaGlyphsGroup *gg, const NRRect *pbox)
 {
 	nr_return_if_fail (gg != NULL);
 	nr_return_if_fail (NR_IS_ARENA_GLYPHS_GROUP (gg));
