@@ -13,21 +13,29 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include <map>
 #include <sigc++/sigc++.h>
-#include "xml/repr-action.h"
+#include "xml/sp-repr-action-fns.h"
 #include "sp-defs.h"
 #include "sp-root.h"
 #include "document.h"
 
 #define SP_DOCUMENT_DEFS(d) ((SPObject *) SP_ROOT (SP_DOCUMENT_ROOT (d))->defs)
 
-struct _SPDocumentPrivate {
+class SPReprAction;
+
+struct SPDocumentPrivate {
+	typedef std::map<GQuark, SPDocument::IDChangedSignal> IDChangedSignalMap;
+	typedef std::map<GQuark, SPDocument::ResourcesChangedSignal> ResourcesChangedSignalMap;
+
 	GHashTable *iddef;	/**< Dictionary of id -> SPObject mappings */
-	GHashTable *idsignals; /**< Dictionary of signals for id changes */
+	/** Dictionary of signals for id changes */
+	IDChangedSignalMap id_changed_signals;
 
 	/* Resources */
 	/* It is GHashTable of GSLists */
 	GHashTable *resources;
+	ResourcesChangedSignalMap resources_changed_signals;
 
 	/* Undo/Redo state */
 	guint sensitive: 1; /* If we save actions to undo stack */
