@@ -24,7 +24,7 @@
 #endif
 
 #include "xml/repr-private.h"
-#include "xml/repr-action.h"
+#include "xml/sp-repr-action-fns.h"
 #include "sp-object.h"
 #include "sp-item.h"
 #include "document-private.h"
@@ -33,6 +33,13 @@
 
 /*
  * Undo & redo
+ *
+ *   Since undo sensitivity needs to be nested, setting undo sensitivity
+ *   should be done like this:
+ *      gboolean saved = sp_document_get_undo_sensitive(document);
+ *      sp_document_set_undo_sensitive(document, FALSE);
+ *      ... do stuff ...
+ *      sp_document_set_undo_sensitive(document, saved);
  */
 
 void
@@ -41,7 +48,6 @@ sp_document_set_undo_sensitive (SPDocument *doc, gboolean sensitive)
 	g_assert (doc != NULL);
 	g_assert (SP_IS_DOCUMENT (doc));
 	g_assert (doc->priv != NULL);
-	g_return_if_fail ( !(sensitive) != !(doc->priv->sensitive) );
 
 	if ( !(sensitive) == !(doc->priv->sensitive) )
 		return;
@@ -55,7 +61,15 @@ sp_document_set_undo_sensitive (SPDocument *doc, gboolean sensitive)
 		);
 	}
 
-	doc->priv->sensitive = sensitive;
+	doc->priv->sensitive = !!sensitive;
+}
+
+gboolean sp_document_get_undo_sensitive(SPDocument const *document) {
+	g_assert(document != NULL);
+	g_assert(SP_IS_DOCUMENT(document));
+	g_assert(document->priv != NULL);
+
+	return document->priv->sensitive;
 }
 
 void
@@ -74,18 +88,18 @@ sp_document_reset_key (Inkscape::Application *inkscape, SPDesktop *desktop, GtkO
 void
 sp_document_maybe_done (SPDocument *doc, const gchar *key)
 {
-	SPReprAction *log;
-
 	g_assert (doc != NULL);
 	g_assert (SP_IS_DOCUMENT (doc));
 	g_assert (doc->priv != NULL);
 	g_assert (doc->priv->sensitive);
 
+	doc->collectOrphans();
+
 	sp_document_ensure_up_to_date (doc);
 
 	sp_document_clear_redo (doc);
 
-	log = sp_repr_coalesce_log (doc->priv->partial, sp_repr_commit_undoable (doc->rdoc));
+	SPReprAction *log = sp_repr_coalesce_log (doc->priv->partial, sp_repr_commit_undoable (doc->rdoc));
 	doc->priv->partial = NULL;
 
 	if (!log) {
@@ -102,6 +116,7 @@ sp_document_maybe_done (SPDocument *doc, const gchar *key)
 
 	doc->actionkey = key;
 
+	doc->virgin = FALSE;
 	if (!sp_repr_attr (doc->rroot, "sodipodi:modified")) {
 		sp_repr_set_attr (doc->rroot, "sodipodi:modified", "true");
 	}
@@ -139,6 +154,8 @@ sp_document_undo (SPDocument *doc)
 	g_assert (doc->priv != NULL);
 	g_assert (doc->priv->sensitive);
 
+	doc->priv->sensitive = FALSE;
+
 	doc->actionkey = NULL;
 	log = sp_repr_commit_undoable (doc->rdoc);
 
@@ -151,6 +168,9 @@ sp_document_undo (SPDocument *doc)
 		doc->priv->undo = g_slist_remove (doc->priv->undo, log);
 		sp_repr_undo_log (log);
 		doc->priv->redo = g_slist_prepend (doc->priv->redo, log);
+
+		sp_repr_set_attr (doc->rroot, "sodipodi:modified", "true");
+
 		ret = TRUE;
 	} else {
 		ret = FALSE;
@@ -158,6 +178,8 @@ sp_document_undo (SPDocument *doc)
 
 	sp_repr_begin_transaction (doc->rdoc);
 
+	doc->priv->sensitive = TRUE;
+
 	return ret;
 }
 
@@ -172,6 +194,8 @@ sp_document_redo (SPDocument *doc)
 	g_assert (doc->priv != NULL);
 	g_assert (doc->priv->sensitive);
 
+	doc->priv->sensitive = FALSE;
+
 	doc->actionkey = NULL;
 	log = sp_repr_commit_undoable (doc->rdoc);
 
@@ -184,6 +208,9 @@ sp_document_redo (SPDocument *doc)
 		doc->priv->redo = g_slist_remove (doc->priv->redo, log);
 		sp_repr_replay_log (log);
 		doc->priv->undo = g_slist_prepend (doc->priv->undo, log);
+
+		sp_repr_set_attr (doc->rroot, "sodipodi:modified", "true");
+
 		ret = TRUE;
 	} else {
 		ret = FALSE;
@@ -191,6 +218,8 @@ sp_document_redo (SPDocument *doc)
 
 	sp_repr_begin_transaction (doc->rdoc);
 
+	doc->priv->sensitive = TRUE;
+
 	return ret;
 }
 
