@@ -20,11 +20,13 @@
 #include <string.h>
 #include <gtk/gtkmain.h>
 #include "xml/repr.h"
-#include "xml/repr-action.h"
+#include "xml/sp-repr-action-fns.h"
 #include "helper/sp-marshal.h"
 #include "helper/sp-intl.h"
 #include "inkscape-private.h"
+#include "inkscape_version.h"
 #include "sp-object-repr.h"
+#include "sp-item-group.h"
 #include "sp-root.h"
 #include "sp-namedview.h"
 #include "document-private.h"
@@ -37,6 +39,8 @@
 #include "display/canvas-arena.h"
 #include "desktop-handles.h"
 
+#include "dialogs/rdf.h"
+
 #define A4_WIDTH_STR "210mm"
 #define A4_HEIGHT_STR "297mm"
 
@@ -118,16 +122,6 @@ sp_document_class_init(SPDocumentClass *klass)
 	object_class->dispose = sp_document_dispose;
 }
 
-static void
-free_id_signal(gpointer p)
-{
-	SigC::Signal1<void, SPObject *> *signal = reinterpret_cast<SigC::Signal1<void, SPObject *> *>(p);
-	if (!signal->empty()) {
-		g_warning("Lingering document id observers");
-	}
-	delete signal;
-}
-
 static void
 sp_document_init (SPDocument *doc)
 {
@@ -135,6 +129,7 @@ sp_document_init (SPDocument *doc)
 
 	doc->advertize = FALSE;
 	doc->keepalive = FALSE;
+	doc->virgin    = TRUE;
 
 	doc->modified_id = 0;
 
@@ -146,10 +141,11 @@ sp_document_init (SPDocument *doc)
 	doc->base = NULL;
 	doc->name = NULL;
 
-	p = g_new (SPDocumentPrivate, 1);
+	doc->_collection_queue = NULL;
+
+	p = new SPDocumentPrivate();
 
 	p->iddef = g_hash_table_new (g_direct_hash, g_direct_equal);
-	p->idsignals = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, free_id_signal);
 
 	p->resources = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -162,6 +158,27 @@ sp_document_init (SPDocument *doc)
 	doc->priv = p;
 }
 
+void SPDocument::queueForOrphanCollection(SPObject *object) {
+	g_return_if_fail(object != NULL);
+	g_return_if_fail(SP_OBJECT_DOCUMENT(object) == this);
+
+	sp_object_ref(object, NULL);
+	_collection_queue = g_slist_prepend(_collection_queue, object);
+}
+
+void SPDocument::collectOrphans() {
+	while (_collection_queue) {
+		GSList *objects=_collection_queue;
+		_collection_queue = NULL;
+		for ( GSList *iter=objects ; iter ; iter = iter->next ) {
+			SPObject *object=reinterpret_cast<SPObject *>(iter->data);
+			object->collectOrphan();
+			sp_object_unref(object, NULL);
+		}
+		g_slist_free(objects);
+	}
+}
+
 /**
  *  This routine removes the given document object from the application,
  *  clearing the undo/redo information, releasing its root object, releasing
@@ -173,6 +190,8 @@ sp_document_dispose (GObject *object)
 	SPDocument *doc = (SPDocument *) object;
 	SPDocumentPrivate *priv = doc->priv;
 
+	doc->collectOrphans();
+
 	if (priv) {
 		inkscape_remove_document (doc);
 
@@ -191,7 +210,6 @@ sp_document_dispose (GObject *object)
 		}
 
 		if (priv->iddef) g_hash_table_destroy (priv->iddef);
-		if (priv->idsignals) g_hash_table_destroy (priv->idsignals);
 
 		if (doc->rdoc) sp_repr_document_unref (doc->rdoc);
 
@@ -199,7 +217,7 @@ sp_document_dispose (GObject *object)
 		g_hash_table_foreach_remove (priv->resources, sp_document_resource_list_free, doc);
 		g_hash_table_destroy (priv->resources);
 
-		g_free (priv);
+		delete priv;
 		doc->priv = NULL;
 	}
 
@@ -253,23 +271,19 @@ sp_document_create (SPReprDoc *rdoc,
 	document->rroot = rroot;
 
 #ifndef WIN32
-	if (uri) { // compute absolute path
-		 char *full_path = (char *) g_malloc (1000);
-		inkscape_rel2abs (uri, g_get_current_dir(), full_path, 1000);
-		document->uri = g_strdup (full_path);
-		g_free (full_path);
-	} else {
-		document->uri = NULL;
-	}
+	prepend_current_dir_if_relative (&(document->uri), uri);
 #else
-	//TODO:WIN32: program the windows equivalent of the above code for finding out 
-	// the normalized absolute path of the file, including the drive letter
-	document->uri = g_strdup (uri);
+	// FIXME: it may be that prepend_current_dir_if_relative works OK on windows too, test!
+	document->uri = uri? g_strdup (uri) : NULL;
 #endif
 
 	// base is simply the part of the path before filename; e.g. when running "inkscape ../file.svg" the base is "../"
 	// which is why we use g_get_current_dir() in calculating the abs path above
-	document->base = g_strdup (base);
+        //This is NULL for a new document
+	if (base)
+            document->base = g_strdup (base);
+        else
+            document->base = NULL;
 	document->name = g_strdup (name);
 
 	document->root = sp_object_repr_build_tree (document, rroot);
@@ -278,7 +292,7 @@ sp_document_create (SPReprDoc *rdoc,
 
 	/* fixme: Not sure about this, but lets assume ::build updates */
 	sp_repr_set_attr (rroot, "sodipodi:version", SODIPODI_VERSION);
-	sp_repr_set_attr (rroot, "inkscape:version", VERSION);
+	sp_repr_set_attr (rroot, "inkscape:version", INKSCAPE_VERSION);
 	/* fixme: Again, I moved these here to allow version determining in ::build (Lauris) */
 
 	/* A quick hack to get namespaces into doc */
@@ -286,6 +300,9 @@ sp_document_create (SPReprDoc *rdoc,
 	sp_repr_set_attr (rroot, "xmlns:sodipodi", SP_SODIPODI_NS_URI);
 	sp_repr_set_attr (rroot, "xmlns:inkscape", SP_INKSCAPE_NS_URI);
 	sp_repr_set_attr (rroot, "xmlns:xlink", SP_XLINK_NS_URI);
+	sp_repr_set_attr (rroot, "xmlns:rdf", SP_RDF_NS_URI);
+	sp_repr_set_attr (rroot, "xmlns:cc", SP_CC_NS_URI);
+	sp_repr_set_attr (rroot, "xmlns:dc", SP_DC_NS_URI);
 	/* End of quick hack */
 
 	/* Quick hack 2 - get default image size into document */
@@ -297,7 +314,8 @@ sp_document_create (SPReprDoc *rdoc,
 	if (uri) {
 		/* fixme: Think, what this means for images (Lauris) */
 		sp_repr_set_attr (rroot, "sodipodi:docname", uri);
-		sp_repr_set_attr (rroot, "sodipodi:docbase", document->base);
+		if (document->base)
+                    sp_repr_set_attr (rroot, "sodipodi:docbase", document->base);
 	}
 	/* End of quick hack 3 */
 	/* between 0 and 0.25 */
@@ -330,6 +348,16 @@ sp_document_create (SPReprDoc *rdoc,
 		sp_repr_unref (rnew);
 	}
 
+	// Create metadata if it doesn't already exist
+	if (!sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "metadata")) {
+		// create repr
+		SPRepr * rnew = sp_repr_new ("metadata");
+		// insert into the document
+		sp_repr_add_child (rroot, rnew, NULL);
+		// clean up
+		sp_repr_unref (rnew);
+	}
+
 	/* Defs */
 	if (!SP_ROOT (document->root)->defs) {
 		SPRepr *r;
@@ -339,6 +367,9 @@ sp_document_create (SPReprDoc *rdoc,
 		g_assert (SP_ROOT (document->root)->defs);
 	}
 
+	/* Default RDF */
+	rdf_set_defaults ( document );
+
 	if (keepalive) {
 		inkscape_ref ();
 	}
@@ -358,11 +389,12 @@ sp_document_create (SPReprDoc *rdoc,
 }
 
 SPDocument *
-sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive)
+sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive, bool make_new)
 {
 	SPDocument *doc;
 	SPReprDoc *rdoc;
-	gchar *base, *name;
+	gchar *base = NULL;
+        gchar *name = NULL;
 
 	if (uri) {
 		SPRepr *rroot;
@@ -388,10 +420,17 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 		g_free (s);
 	} else {
 		rdoc = sp_repr_document_new ("svg");
+	}
+
+	if (make_new) {
 		base = NULL;
+		uri = NULL;
 		name = g_strdup_printf (_("New document %d"), ++doc_count);
 	}
 
+        //# These should be set by now
+        g_assert (name);
+
 	doc = sp_document_create (rdoc, uri, base, name, advertize, keepalive);
 
 	g_free (base);
@@ -488,15 +527,9 @@ void sp_document_set_uri(SPDocument *document, gchar const *uri)
 	if (uri) {
 
 #ifndef WIN32
-		// compute absolute path
-		char *full_path;
-		full_path = (char *) g_malloc (1000);
-		inkscape_rel2abs (uri, g_get_current_dir(), full_path, 1000);
-		document->uri = g_strdup (full_path);
-		g_free (full_path);
+		prepend_current_dir_if_relative (&(document->uri), uri);
 #else
-	//TODO:WIN32: program the windows equivalent of the above code for finding out 
-	// the normalized absolute path of the file, including the drive letter
+		// FIXME: it may be that prepend_current_dir_if_relative works OK on windows too, test!
 		document->uri = g_strdup (uri);
 #endif
 	
@@ -513,10 +546,13 @@ void sp_document_set_uri(SPDocument *document, gchar const *uri)
 	// Update saveable repr attributes.
 	SPRepr *repr = sp_document_repr_root(document);
 	// changing uri in the document repr must not be not undoable
+	gboolean saved = sp_document_get_undo_sensitive(document);
 	sp_document_set_undo_sensitive (document, FALSE);
-	sp_repr_set_attr (repr, "sodipodi:docbase", document->base);
+	if (document->base)
+            sp_repr_set_attr (repr, "sodipodi:docbase", document->base);
+
 	sp_repr_set_attr (repr, "sodipodi:docname", document->name);
-	sp_document_set_undo_sensitive (document, TRUE);
+	sp_document_set_undo_sensitive (document, saved);
 
 	g_signal_emit (G_OBJECT (document), signals [URI_SET], 0, document->uri);
 }
@@ -535,9 +571,6 @@ sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height)
 void sp_document_def_id(SPDocument *document, gchar const *id, SPObject *object)
 {
 	GQuark idq = g_quark_from_string(id);
-	SigC::Signal1<void, SPObject *> *signal
-	  = reinterpret_cast<SigC::Signal1<void, SPObject *> *>(g_hash_table_lookup(document->priv->idsignals,
-										    GINT_TO_POINTER(idq)));
 
 	if (object) {
 		g_assert(g_hash_table_lookup(document->priv->iddef, GINT_TO_POINTER(idq)) == NULL);
@@ -547,46 +580,37 @@ void sp_document_def_id(SPDocument *document, gchar const *id, SPObject *object)
 		g_hash_table_remove(document->priv->iddef, GINT_TO_POINTER(idq));
 	}
 
-	if (signal) {
-		if (!signal->empty()) {
-			signal->emit(object);
-		} else {
-			/* dispose of unused signal */
-			g_hash_table_remove(document->priv->idsignals, GINT_TO_POINTER(idq));
-			signal = NULL;
+	SPDocumentPrivate::IDChangedSignalMap &id_changed_signals = document->priv->id_changed_signals;
+	SPDocumentPrivate::IDChangedSignalMap::iterator pos;
+
+	pos = id_changed_signals.find(idq);
+	if ( pos != id_changed_signals.end() ) {
+		if (!(*pos).second.empty()) {
+			(*pos).second.emit(object);
+		} else { // discard unused signal
+			id_changed_signals.erase(pos);
 		}
 	}
 }
 
 SigC::Connection
 sp_document_id_changed_connect(SPDocument *doc, const gchar *id,
-                               SigC::Slot1<void, SPObject *> slot)
+                               SPDocument::IDChangedSignal::slot_type slot)
 {
-	SigC::Signal1<void, SPObject *> *signal;
-	GQuark idq;
-
-	idq = g_quark_from_string(id);
-	signal = reinterpret_cast<SigC::Signal1<void, SPObject *> *>(g_hash_table_lookup(doc->priv->idsignals, GINT_TO_POINTER(idq)));
-	if (!signal) {
-		signal = new SigC::Signal1<void, SPObject *>();
-		g_hash_table_insert(doc->priv->idsignals, GINT_TO_POINTER(idq), reinterpret_cast<gpointer>(signal));
-	}
-
-	return signal->connect(slot);
+	return doc->priv->id_changed_signals[g_quark_from_string(id)].connect(slot);
 }
 
-SPObject *
-sp_document_lookup_id (SPDocument *doc, const gchar *id)
+SPObject *SPDocument::getObjectById(const gchar *id)
 {
-	GQuark idq;
-
-	g_return_val_if_fail (doc != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (id != NULL, NULL);
 
-	idq = g_quark_from_string(id);
+	GQuark idq = g_quark_from_string(id);
+	return (SPObject*)g_hash_table_lookup (this->priv->iddef, GINT_TO_POINTER(idq));
+}
 
-	return (SPObject*)g_hash_table_lookup (doc->priv->iddef, GINT_TO_POINTER(idq));
+SPObject *SPDocument::getObjectByRepr(SPRepr *repr)
+{
+	return getObjectById(sp_repr_attr(repr, "id"));
 }
 
 /* Object modification root handler */
@@ -626,10 +650,10 @@ sp_document_ensure_up_to_date (SPDocument *doc)
 			ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
 			ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
 			nr_matrix_set_identity (&ctx.i2vp);
-			sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
+			doc->root->updateDisplay((SPCtx *)&ctx, 0);
 		}
 		/* Emit "modified" signal on objects */
-		sp_object_invoke_modified (doc->root, 0);
+		doc->root->emitModified(0);
 		/* Emit our own "modified" signal */
 		g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
 			       SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
@@ -665,12 +689,18 @@ sp_document_idle_handler (gpointer data)
 		ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
 		ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
 		nr_matrix_set_identity (&ctx.i2vp);
-		sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
+
+		gboolean saved = sp_document_get_undo_sensitive(doc);
+		sp_document_set_undo_sensitive (doc, FALSE);
+
+		doc->root->updateDisplay((SPCtx *)&ctx, 0);
+
+		sp_document_set_undo_sensitive (doc, saved);
 		/* if (doc->root->uflags & SP_OBJECT_MODIFIED_FLAG) return TRUE; */
 	}
 
 	/* Emit "modified" signal on objects */
-	sp_object_invoke_modified (doc->root, 0);
+	doc->root->emitModified(0);
 
 #ifdef SP_DOCUMENT_DEBUG_IDLE
 	g_print ("\n->");
@@ -689,44 +719,30 @@ sp_document_idle_handler (gpointer data)
 	return repeat;
 }
 
-SPObject *
-sp_document_add_repr (SPDocument *document, SPRepr *repr)
+static int
+is_within (const NRRect *area, const NRRect *box)
 {
-	GType type;
+	if (box->x0 > box->x1 || box->y0 > box->y1) // invalid (=empty) bbox, may happen e.g. with a whitespace-only text object
+		return false;
 
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (repr != NULL, NULL);
-
-	type = sp_repr_type_lookup (repr);
-
-	if (g_type_is_a (type, SP_TYPE_ITEM)) {
-		sp_repr_append_child (sp_document_repr_root(document), repr);
-	} else if (g_type_is_a (type, SP_TYPE_OBJECT)) {
-		sp_repr_append_child (SP_OBJECT_REPR (SP_DOCUMENT_DEFS(document)), repr);
-	}
-
-	return sp_document_lookup_id (document, sp_repr_attr (repr, "id"));
+	return (box->x0 > area->x0) && (box->x1 < area->x1)
+	    && (box->y0 > area->y0) && (box->y1 < area->y1);
 }
 
 static int
-is_within (const NRRect *what, const NRRect *box)
+overlaps (const NRRect *area, const NRRect *box)
 {
-	return (box->x0 > what->x0) && (box->x1 < what->x1)
-	    && (box->y0 > what->y0) && (box->y1 < what->y1);
-}
+	if (box->x0 > box->x1 || box->y0 > box->y1) // invalid (=empty) bbox, may happen e.g. with a whitespace-only text object
+		return false;
 
-static int
-overlaps (const NRRect *what, const NRRect *box)
-{
-	return (((what->x0 > box->x0) && (what->x0 < box->x1)) ||
-	        ((what->x1 > box->x0) && (what->x1 < box->x1))) &&
-	       (((what->y0 > box->y0) && (what->y0 < box->y1)) ||
-	        ((what->y1 > box->y0) && (what->y1 < box->y1)));
+	return (((area->x0 > box->x0) && (area->x0 < box->x1)) ||
+	        ((area->x1 > box->x0) && (area->x1 < box->x1))) &&
+	       (((area->y0 > box->y0) && (area->y0 < box->y1)) ||
+	        ((area->y1 > box->y0) && (area->y1 < box->y1)));
 }
 
 static GSList *
-find_items_in_area (GSList *s, SPGroup *group, NRRect const *area,
+find_items_in_area (GSList *s, SPGroup *group, unsigned int dkey, NRRect const *area,
                     int (*test)(const NRRect *, const NRRect *), bool take_insensitive = false)
 {
 	g_return_val_if_fail (SP_IS_GROUP (group), s);
@@ -734,14 +750,14 @@ find_items_in_area (GSList *s, SPGroup *group, NRRect const *area,
 	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL ; o = SP_OBJECT_NEXT(o) ) {
 		if (!SP_IS_ITEM (o)) continue;
 		if (SP_IS_GROUP (o) &&
-		    SP_GROUP (o)->mode == SP_GROUP_MODE_LAYER)
+		    SP_GROUP (o)->effectiveLayerMode(dkey) == SPGroup::LAYER )
 		{
-			s = find_items_in_area (s, SP_GROUP (o), area, test);
+			s = find_items_in_area(s, SP_GROUP(o), dkey, area, test);
 		} else {
 			SPItem *child = SP_ITEM(o);
 			NRRect box;
 			sp_item_bbox_desktop (child, &box);
-			if (test (area, &box) && (take_insensitive || child->sensitive)) {
+			if (test (area, &box) && (take_insensitive || child->isVisibleAndUnlocked(dkey))) {
 				s = g_slist_append (s, child);
 			}
 		}
@@ -752,27 +768,89 @@ find_items_in_area (GSList *s, SPGroup *group, NRRect const *area,
 
 extern gdouble nr_arena_global_delta;
 
+/**
+Returns true if an item is among the descendants of group (recursively).
+ */
+bool item_is_in_group (SPItem *item, SPGroup *group)
+{
+	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL ; o = SP_OBJECT_NEXT(o) ) {
+		if (!SP_IS_ITEM (o)) continue;
+		if (SP_ITEM (o) == item)
+			return true;
+		if (SP_IS_GROUP (o))
+			if (item_is_in_group (item, SP_GROUP(o)))
+				return true;
+	}
+	return false;
+}
+
+/**
+Returns the bottommost item from the list which is at the point, or NULL if none. 
+*/
 SPItem*
-find_item_at_point (gint dkey, SPGroup *group, NR::Point const p, gboolean into_groups, bool take_insensitive = false)
+sp_document_item_from_list_at_point_bottom (unsigned int dkey, SPGroup *group, const GSList *list, NR::Point const p, bool take_insensitive)
+{
+	g_return_val_if_fail (group, NULL);
+
+	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL ; o = SP_OBJECT_NEXT(o) ) {
+
+		if (!SP_IS_ITEM (o)) continue;
+
+		SPItem *item = SP_ITEM (o);
+		NRArenaItem *arenaitem = sp_item_get_arenaitem(item, dkey);
+		if (nr_arena_item_invoke_pick (arenaitem, p, nr_arena_global_delta, 1) != NULL 
+                      && (take_insensitive || item->isVisibleAndUnlocked(dkey))) {
+			if (g_slist_find((GSList *) list, item) != NULL)
+				return item;
+		}
+
+		if (SP_IS_GROUP (o)) {
+			SPItem *found = sp_document_item_from_list_at_point_bottom (dkey, SP_GROUP(o), list, p, take_insensitive);
+			if (found)
+				return found;
+		}
+
+	}
+	return NULL;
+}
+
+/**
+Returns the topmost (in z-order) item from the descendants of group (recursively) which
+is at the point p, or NULL if none. Honors into_groups on whether to recurse into
+non-layer groups or not. Honors take_insensitive on whether to return insensitive
+items. If upto != NULL, then if item upto is encountered (at any level), stops searching
+upwards in z-order and returns what it has found so far (i.e. the found item is
+guaranteed to be lower than upto).
+ */
+SPItem*
+find_item_at_point (unsigned int dkey, SPGroup *group, NR::Point const p, gboolean into_groups, bool take_insensitive = false, SPItem *upto = NULL)
 {
 	SPItem *seen = NULL, *newseen = NULL;
 
 	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL ; o = SP_OBJECT_NEXT(o) ) {
 		if (!SP_IS_ITEM (o)) continue;
-		if (SP_IS_GROUP (o) && (SP_GROUP (o)->mode == SP_GROUP_MODE_LAYER || into_groups))	{
+
+		if (upto && SP_ITEM (o) == upto)
+			break;
+
+		if (SP_IS_GROUP (o) && (SP_GROUP (o)->effectiveLayerMode(dkey) == SPGroup::LAYER || into_groups))	{
 			// if nothing found yet, recurse into the group
-			newseen = find_item_at_point (dkey, SP_GROUP (o), p, into_groups);
+			newseen = find_item_at_point (dkey, SP_GROUP (o), p, into_groups, take_insensitive, upto);
 			if (newseen) {
 				seen = newseen;
 				newseen = NULL;
 			}
+
+			if (item_is_in_group (upto, SP_GROUP (o)))
+				break;
+
 		} else {
 			SPItem *child = SP_ITEM(o);
 			NRArenaItem *arenaitem = sp_item_get_arenaitem(child, dkey);
 
 			// seen remembers the last (topmost) of items pickable at this point
-			if (nr_arena_item_invoke_pick (arenaitem, p[NR::X], p[NR::Y], nr_arena_global_delta, 1) != NULL 
-                           && (take_insensitive || child->sensitive)) {
+			if (nr_arena_item_invoke_pick (arenaitem, p, nr_arena_global_delta, 1) != NULL 
+                           && (take_insensitive || child->isVisibleAndUnlocked(dkey))) {
 				seen = child;
 			}
 		}
@@ -780,19 +858,23 @@ find_item_at_point (gint dkey, SPGroup *group, NR::Point const p, gboolean into_
 	return seen;
 }
 
+/**
+Returns the bottommost non-layer group from the descendants of group which is at point
+p, or NULL if none.
+ */
 SPItem*
-find_group_at_point (gint dkey, SPGroup *group, NR::Point const p)
+find_group_at_point (unsigned int dkey, SPGroup *group, NR::Point const p)
 {
 	SPItem *seen = NULL;
 
 	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL ; o = SP_OBJECT_NEXT(o) ) {
 		if (!SP_IS_ITEM (o)) continue;
-		if (SP_IS_GROUP (o) && SP_GROUP (o)->mode != SP_GROUP_MODE_LAYER) {
+		if (SP_IS_GROUP (o) && SP_GROUP (o)->effectiveLayerMode(dkey) != SPGroup::LAYER ) {
 			SPItem *child = SP_ITEM(o);
 			NRArenaItem *arenaitem = sp_item_get_arenaitem(child, dkey);
 
 			// seen remembers the last (topmost) of groups pickable at this point
-			if (nr_arena_item_invoke_pick (arenaitem, p[NR::X], p[NR::Y], nr_arena_global_delta, 1) != NULL) {
+			if (nr_arena_item_invoke_pick (arenaitem, p, nr_arena_global_delta, 1) != NULL) {
 				seen = child;
 			}
 		}
@@ -808,7 +890,7 @@ find_group_at_point (gint dkey, SPGroup *group, NR::Point const p)
  */
 
 GSList *
-sp_document_items_in_box (SPDocument *document, NRRect const *box)
+sp_document_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box)
 {
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
@@ -816,7 +898,7 @@ sp_document_items_in_box (SPDocument *document, NRRect const *box)
 	g_return_val_if_fail (box != NULL, NULL);
 
 	return find_items_in_area (NULL, SP_GROUP (document->root),
-	                           box, is_within);
+	                           dkey, box, is_within);
 }
 
 /*
@@ -827,7 +909,7 @@ sp_document_items_in_box (SPDocument *document, NRRect const *box)
  */
 
 GSList *
-sp_document_partial_items_in_box (SPDocument *document, NRRect const *box)
+sp_document_partial_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box)
 {
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
@@ -835,17 +917,17 @@ sp_document_partial_items_in_box (SPDocument *document, NRRect const *box)
 	g_return_val_if_fail (box != NULL, NULL);
 
 	return find_items_in_area (NULL, SP_GROUP (document->root),
-	                           box, overlaps);
+	                           dkey, box, overlaps);
 }
 
 SPItem*
-sp_document_item_at_point (SPDocument *document, unsigned int key, NR::Point p, gboolean into_groups)
+sp_document_item_at_point (SPDocument *document, unsigned int key, NR::Point p, gboolean into_groups, SPItem *upto)
 {
 	g_return_val_if_fail (document != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (document->priv != NULL, NULL);
 
- 	return find_item_at_point (key, SP_GROUP (document->root), p, into_groups);
+ 	return find_item_at_point (key, SP_GROUP (document->root), p, into_groups, false, upto);
 }
 
 SPItem*
@@ -865,6 +947,7 @@ gboolean
 sp_document_add_resource (SPDocument *document, const gchar *key, SPObject *object)
 {
 	GSList *rlist;
+	GQuark q=g_quark_from_string(key);
 
 	g_return_val_if_fail (document != NULL, FALSE);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), FALSE);
@@ -878,6 +961,8 @@ sp_document_add_resource (SPDocument *document, const gchar *key, SPObject *obje
 	rlist = g_slist_prepend (rlist, object);
 	g_hash_table_insert (document->priv->resources, (gpointer) key, rlist);
 
+	document->priv->resources_changed_signals[q].emit();
+
 	return TRUE;
 }
 
@@ -885,6 +970,7 @@ gboolean
 sp_document_remove_resource (SPDocument *document, const gchar *key, SPObject *object)
 {
 	GSList *rlist;
+	GQuark q=g_quark_from_string(key);
 
 	g_return_val_if_fail (document != NULL, FALSE);
 	g_return_val_if_fail (SP_IS_DOCUMENT (document), FALSE);
@@ -899,6 +985,8 @@ sp_document_remove_resource (SPDocument *document, const gchar *key, SPObject *o
 	rlist = g_slist_remove (rlist, object);
 	g_hash_table_insert (document->priv->resources, (gpointer) key, rlist);
 
+	document->priv->resources_changed_signals[q].emit();
+
 	return TRUE;
 }
 
@@ -913,6 +1001,15 @@ sp_document_get_resource_list (SPDocument *document, const gchar *key)
 	return (GSList*)g_hash_table_lookup (document->priv->resources, key);
 }
 
+sigc::connection sp_document_resources_changed_connect(
+	SPDocument *document,
+	const gchar *key,
+	SPDocument::ResourcesChangedSignal::slot_type slot
+) {
+	GQuark q=g_quark_from_string(key);
+	return document->priv->resources_changed_signals[q].connect(slot);
+}
+
 /* Helpers */
 
 gboolean
@@ -921,3 +1018,5 @@ sp_document_resource_list_free (gpointer key, gpointer value, gpointer data)
 	g_slist_free ((GSList *) value);
 	return TRUE;
 }
+
+
