@@ -17,17 +17,23 @@
 #include <glib.h>
 #include <gtk/gtksignal.h>
 #include <sigc++/sigc++.h>
-#include "xml/repr.h"
+#include <sigc++/class_slot.h>
+
 #include "forward.h"
 #include "libnr/nr-forward.h"
+#include "xml/xml-forward.h"
 
-typedef struct _SPDocumentPrivate SPDocumentPrivate;
+class SPDocumentPrivate;
+
+struct SPDocument {
+	typedef sigc::signal<void, SPObject *> IDChangedSignal;
+	typedef sigc::signal<void> ResourcesChangedSignal;
 
-struct _SPDocument {
 	GObject object;
 
 	unsigned int advertize : 1;
 	unsigned int keepalive : 1;
+	unsigned int virgin    : 1; /* Has the document never been touched? */
 
 	SPReprDoc *rdoc; /* Our SPReprDoc */
 	SPRepr *rroot; /* Root element of SPReprDoc */
@@ -37,16 +43,23 @@ struct _SPDocument {
 	gchar *base;
 	gchar *name;
 
-	/* fixme: remove this */
 	SPDocumentPrivate *priv;
 
 	/* Last action key */
 	const gchar *actionkey;
 	/* Handler ID */
 	guint modified_id;
+
+	SPObject *getObjectById(gchar const *id);
+	SPObject *getObjectByRepr(SPRepr *repr);
+
+	void queueForOrphanCollection(SPObject *object);
+	void collectOrphans();
+
+	GSList *_collection_queue;
 };
 
-struct _SPDocumentClass {
+struct SPDocumentClass {
 	GObjectClass parent_class;
 
 	void (* modified) (SPDocument *document, guint flags);
@@ -58,7 +71,7 @@ struct _SPDocumentClass {
  * Public document appear in document list
  */
 
-SPDocument *sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive);
+SPDocument *sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive, bool make_new = false);
 SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive);
 
 SPDocument *sp_document_ref (SPDocument *doc);
@@ -84,14 +97,17 @@ gdouble sp_document_height (SPDocument * document);
  */
 
 void sp_document_def_id(SPDocument *document, const gchar *id, SPObject *object);
-SigC::Connection sp_document_id_changed_connect(SPDocument *document, const gchar *id, SigC::Slot1<void, SPObject *>);
-SPObject *sp_document_lookup_id(SPDocument *document, const gchar *id);
+sigc::connection sp_document_id_changed_connect(SPDocument *document, const gchar *id, SPDocument::IDChangedSignal::slot_type slot);
+
+
+
 
 /*
  * Undo & redo
  */
 
 void sp_document_set_undo_sensitive (SPDocument * document, gboolean sensitive);
+gboolean sp_document_get_undo_sensitive (SPDocument const * document);
 
 void sp_document_clear_undo (SPDocument * document);
 void sp_document_clear_redo (SPDocument * document);
@@ -118,22 +134,12 @@ void sp_document_cancel (SPDocument *document);
 gboolean sp_document_undo (SPDocument * document);
 gboolean sp_document_redo (SPDocument * document);
 
-/* Adds repr to document, returning created object (if any) */
-/* Items will be added to root (fixme: should be namedview root) */
-/* Non-item objects will go to root-level defs group */
-SPObject *sp_document_add_repr (SPDocument *document, SPRepr *repr);
-
-#if 0
-/* Deletes repr from document */
-/* fixme: This is not needed anymore - remove it (Lauris) */
-/* Instead simply unparent repr */
-void sp_document_del_repr (SPDocument *document, SPRepr *repr);
-#endif
-
 /* Resource management */
 gboolean sp_document_add_resource (SPDocument *document, const gchar *key, SPObject *object);
 gboolean sp_document_remove_resource (SPDocument *document, const gchar *key, SPObject *object);
 const GSList *sp_document_get_resource_list (SPDocument *document, const gchar *key);
+sigc::connection sp_document_resources_changed_connect(SPDocument *document, const gchar *key, SPDocument::ResourcesChangedSignal::slot_type slot);
+
 
 /*
  * Ideas: How to overcome style invalidation nightmare
@@ -153,9 +159,10 @@ const GSList *sp_document_get_resource_list (SPDocument *document, const gchar *
  * Misc
  */
 
-GSList * sp_document_items_in_box (SPDocument *document, NRRect const *box);
-GSList * sp_document_partial_items_in_box (SPDocument *document, NRRect const *box);
-SPItem * sp_document_item_at_point (SPDocument *document, unsigned int key, NR::Point const p, gboolean into_groups);
+GSList * sp_document_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box);
+GSList * sp_document_partial_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box);
+SPItem* sp_document_item_from_list_at_point_bottom (unsigned int dkey, SPGroup *group, const GSList *list, NR::Point const p, bool take_insensitive = false);
+SPItem * sp_document_item_at_point (SPDocument *document, unsigned int key, NR::Point const p, gboolean into_groups, SPItem *upto = NULL);
 SPItem * sp_document_group_at_point (SPDocument *document, unsigned int key,  NR::Point const p);
 
 void sp_document_set_uri (SPDocument *document, const gchar *uri);
