@@ -20,32 +20,26 @@
 /*
  * TODO: Tue Oct  2 22:57:15 2001
  *  - Decide control point behavior when use_calligraphic==1.
- *  - Option dialog box support if it is availabe.
  *  - Decide to use NORMALIZED_COORDINATE or not.
- *  - Remove hard coded style attributes and move to customizable style.
  *  - Bug fix.
  */
 
 #define noDYNA_DRAW_VERBOSE
 
-#include <config.h>
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "svg/svg.h"
 #include "helper/sp-intl.h"
-#include "helper/curve.h"
-#include "helper/canvas-bpath.h"
-#include "helper/sodipodi-ctrl.h"
-#include "helper/bezier-utils.h"
+#include "display/curve.h"
+#include "display/canvas-bpath.h"
+#include "display/sodipodi-ctrl.h"
+#include "display/bezier-utils.h"
 
 #include "macros.h"
 #include "enums.h"
@@ -56,10 +50,14 @@
 #include "desktop-events.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
-#include "desktop-snap.h"
+#include "desktop-style.h"
+#include "snap.h"
+#include "sp-desktop-widget.h"
 #include "dyna-draw-context.h"
-
-#include <libnr/nr-point-fns.h>
+#include "widgets/spw-utilities.h"
+#include "libnr/n-art-bpath.h"
+#include "libnr/nr-point-fns.h"
+#include "xml/repr.h"
 
 #define DDC_RED_RGBA 0xff0000ff
 #define DDC_GREEN_RGBA 0x000000ff
@@ -72,7 +70,7 @@
 #define DYNA_MIN_WIDTH 1.0e-6
 
 #define DRAG_MIN 0.0
-#define DRAG_DEFAULT 0.5
+#define DRAG_DEFAULT 1.0
 #define DRAG_MAX 1.0
 
 static void sp_dyna_draw_context_class_init(SPDynaDrawContextClass *klass);
@@ -83,8 +81,6 @@ static void sp_dyna_draw_context_setup(SPEventContext *ec);
 static void sp_dyna_draw_context_set(SPEventContext *ec, gchar const *key, gchar const *val);
 static gint sp_dyna_draw_context_root_handler(SPEventContext *ec, GdkEvent *event);
 
-static GtkWidget *sp_dyna_draw_context_config_widget(SPEventContext *ec);
-
 static void clear_current(SPDynaDrawContext *dc);
 static void set_to_accumulated(SPDynaDrawContext *dc);
 static void concat_current_line(SPDynaDrawContext *dc);
@@ -136,7 +132,6 @@ sp_dyna_draw_context_class_init(SPDynaDrawContextClass *klass)
     event_context_class->setup = sp_dyna_draw_context_setup;
     event_context_class->set = sp_dyna_draw_context_set;
     event_context_class->root_handler = sp_dyna_draw_context_root_handler;
-    event_context_class->config_widget = sp_dyna_draw_context_config_widget;
 }
 
 static void
@@ -160,7 +155,6 @@ sp_dyna_draw_context_init(SPDynaDrawContext *ddc)
     ddc->del = NR::Point(0,0);
 
     /* attributes */
-    ddc->fixed_angle = FALSE;
     ddc->use_timeout = FALSE;
     ddc->use_calligraphic = TRUE;
     ddc->timer_id = 0;
@@ -171,6 +165,9 @@ sp_dyna_draw_context_init(SPDynaDrawContext *ddc)
     ddc->drag = DRAG_DEFAULT;
     ddc->angle = 30.0;
     ddc->width = 0.2;
+
+    ddc->vel_thin = 0.1;
+    ddc->flatness = 0.9;
 }
 
 static void
@@ -196,6 +193,10 @@ sp_dyna_draw_context_dispose(GObject *object)
         ddc->currentshape = NULL;
     }
 
+    if (ddc->_message_context) {
+        delete ddc->_message_context;
+    }
+
     G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
@@ -224,6 +225,12 @@ sp_dyna_draw_context_setup(SPEventContext *ec)
     sp_event_context_read(ec, "drag");
     sp_event_context_read(ec, "angle");
     sp_event_context_read(ec, "width");
+    sp_event_context_read(ec, "thinning");
+    sp_event_context_read(ec, "flatness");
+
+    ddc->is_drawing = false;
+
+    ddc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
 static void
@@ -232,19 +239,23 @@ sp_dyna_draw_context_set(SPEventContext *ec, gchar const *key, gchar const *val)
     SPDynaDrawContext *ddc = SP_DYNA_DRAW_CONTEXT(ec);
 
     if (!strcmp(key, "mass")) {
-        double const dval = ( val ? atof(val) : 0.2 );
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : 0.2 );
         ddc->mass = CLAMP(dval, -1000.0, 1000.0);
     } else if (!strcmp(key, "drag")) {
-        double const dval = ( val ? atof(val) : DRAG_DEFAULT );
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : DRAG_DEFAULT );
         ddc->drag = CLAMP(dval, DRAG_MIN, DRAG_MAX);
     } else if (!strcmp(key, "angle")) {
-        double const dval = ( val
-                              ? mod360(atof(val))
-                              : 0.0 );
-        ddc->angle = dval;
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : 0.0);
+        ddc->angle = CLAMP (dval, -90, 90);
     } else if (!strcmp(key, "width")) {
-        double const dval = ( val ? atof(val) : 0.1 );
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : 0.1 );
         ddc->width = CLAMP(dval, -1000.0, 1000.0);
+    } else if (!strcmp(key, "thinning")) {
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : 0.1 );
+        ddc->vel_thin = CLAMP(dval, -1.0, 1.0);
+    } else if (!strcmp(key, "flatness")) {
+        double const dval = ( val ? g_ascii_strtod (val, NULL) : 1.0 );
+        ddc->flatness = CLAMP(dval, 0, 1.0);
     }
 
     //g_print("DDC: %g %g %g %g\n", ddc->mass, ddc->drag, ddc->angle, ddc->width);
@@ -262,8 +273,8 @@ sp_dyna_draw_get_npoint(SPDynaDrawContext const *dc, NR::Point v)
 {
     NRRect drect;
     sp_desktop_get_display_area(SP_EVENT_CONTEXT(dc)->desktop, &drect);
-    return NR::Point(( v[NR::X] - drect.x0 ) / ( drect.x1 - drect.x0 ),
-                     ( v[NR::Y] - drect.y0 ) / ( drect.y1 - drect.y0 ));
+    double max = MAX (( drect.x1 - drect.x0 ), ( drect.y1 - drect.y0 ));
+    return NR::Point(( v[NR::X] - drect.x0 ) / max,  ( v[NR::Y] - drect.y0 ) / max);
 }
 
 /* Get view point */
@@ -272,8 +283,8 @@ sp_dyna_draw_get_vpoint(SPDynaDrawContext const *dc, NR::Point n)
 {
     NRRect drect;
     sp_desktop_get_display_area(SP_EVENT_CONTEXT(dc)->desktop, &drect);
-    return NR::Point(n[NR::X] * ( drect.x1 - drect.x0 ) + drect.x0,
-                     n[NR::Y] * ( drect.y1 - drect.y0 ) + drect.y0);
+    double max = MAX (( drect.x1 - drect.x0 ), ( drect.y1 - drect.y0 ));
+    return NR::Point(n[NR::X] * max + drect.x0, n[NR::Y] * max + drect.y0);
 }
 
 /* Get current view point */
@@ -281,8 +292,8 @@ static NR::Point sp_dyna_draw_get_curr_vpoint(SPDynaDrawContext const *dc)
 {
     NRRect drect;
     sp_desktop_get_display_area(SP_EVENT_CONTEXT(dc)->desktop, &drect);
-    return NR::Point(dc->cur[NR::X] * ( drect.x1 - drect.x0 ) + drect.x0,
-                     dc->cur[NR::Y] * ( drect.y1 - drect.y0 ) + drect.y0);
+    double max = MAX (( drect.x1 - drect.x0 ), ( drect.y1 - drect.y0 ));
+    return NR::Point(dc->cur[NR::X] * max + drect.x0, dc->cur[NR::Y] * max + drect.y0);
 }
 
 static void
@@ -316,18 +327,38 @@ sp_dyna_draw_apply(SPDynaDrawContext *dc, NR::Point p)
     dc->vel += dc->acc;
 
     /* Calculate angle of drawing tool */
-    if (dc->fixed_angle) {
-        double const radians = ( dc->angle / 180.0 ) * M_PI;
-        // should this be -sin, cos?
-        dc->ang = NR::Point(cos(radians),
-                            -sin(radians));
-    } else {
-        gdouble const mag_vel = NR::L2(dc->vel);
-        if ( mag_vel < DYNA_EPSILON ) {
-            return FALSE;
-        }
-        dc->ang = NR::rot90(dc->vel) / mag_vel;
+
+    // 1. fixed dc->angle (absolutely flat nib):
+    double const radians = ( (dc->angle - 90) / 180.0 ) * M_PI;
+    NR::Point ang1 = NR::Point(-sin(radians),  cos(radians));
+
+    // 2. perpendicular to dc->vel (absolutely non-flat nib):
+    gdouble const mag_vel = NR::L2(dc->vel);
+    if ( mag_vel < DYNA_EPSILON ) {
+        return FALSE;
     }
+    NR::Point ang2 = NR::rot90(dc->vel) / mag_vel;
+
+    // 3. Average them using flatness parameter:
+    // calculate angles
+    double a1 = atan2(ang1);
+    double a2 = atan2(ang2);
+    // flip a2 to force it to be in the same half-circle as a1
+    bool flipped = false;
+    if (fabs (a2-a1) > 0.5*M_PI) {
+        a2 += M_PI;
+        flipped = true;
+    }
+    // normalize a2
+    if (a2 > M_PI)
+        a2 -= 2*M_PI;
+    if (a2 < -M_PI)
+        a2 += 2*M_PI;
+    // find the flatness-weighted bisector angle, unflip if a2 was flipped
+    // FIXME: when dc->vel is oscillating around the fixed angle, the new_ang flips back and forth. How to avoid this?
+    double new_ang = a1 + (1 - dc->flatness) * (a2 - a1) - (flipped? M_PI : 0);
+    // convert to point
+    dc->ang = NR::Point (cos (new_ang), sin (new_ang));
 
     /* Apply drag */
     dc->vel *= 1.0 - drag;
@@ -347,12 +378,15 @@ sp_dyna_draw_brush(SPDynaDrawContext *dc)
     if (dc->use_calligraphic) {
         /* calligraphics */
 
-        /* fixme: */
-        double width = ( 0.05 - NR::L2(dc->vel) ) * dc->width;
-        if ( width < DYNA_MIN_WIDTH ) {
-            width = DYNA_MIN_WIDTH;
+        // How much velocity thins strokestyle
+        double vel_thin = flerp (0, 160, dc->vel_thin);
+
+        double width = ( 1 - vel_thin * NR::L2(dc->vel) ) * dc->width;
+        if ( width < 0.02 * dc->width ) {
+            width = 0.02 * dc->width;
         }
-        NR::Point del = width * dc->ang;
+
+        NR::Point del = 0.05 * width * dc->ang;
 
         dc->point1[dc->npoints] = sp_dyna_draw_get_vpoint(dc, dc->cur + del);
         dc->point2[dc->npoints] = sp_dyna_draw_get_vpoint(dc, dc->cur - del);
@@ -383,7 +417,7 @@ sp_dyna_draw_timeout_handler(gpointer data)
         return TRUE;
     }
     p = sp_dyna_draw_get_curr_vpoint(dc);
-    sp_desktop_free_snap(desktop, p);
+    namedview_free_snap(desktop->namedview, Snapper::SNAP_POINT, p);
     // something's not right here
     if ( dc->cur != dc->last ) {
         sp_dyna_draw_brush(dc);
@@ -394,6 +428,16 @@ sp_dyna_draw_timeout_handler(gpointer data)
     return TRUE;
 }
 
+void
+sp_ddc_update_toolbox (SPDesktop *desktop, const gchar *id, double value)
+{
+    gpointer hb = sp_search_by_data_recursive (desktop->owner->aux_toolbox, (gpointer) id);
+    if (hb && GTK_IS_WIDGET(hb) && GTK_IS_SPIN_BUTTON(hb)) {
+        GtkAdjustment *a = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON(hb));
+        gtk_adjustment_set_value (a, value);
+    }
+ }
+
 gint
 sp_dyna_draw_context_root_handler(SPEventContext *event_context,
                                   GdkEvent *event)
@@ -406,13 +450,25 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
     switch (event->type) {
     case GDK_BUTTON_PRESS:
         if ( event->button.button == 1 ) {
+
+            SPDesktop *desktop = SP_EVENT_CONTEXT_DESKTOP(dc);
+            SPItem *layer=SP_ITEM(desktop->currentLayer());
+            if ( !layer || desktop->itemIsHidden(layer)) {
+                dc->_message_context->flash(Inkscape::WARNING_MESSAGE, _("<b>Current layer is hidden</b>. Unhide it to be able to draw on it."));
+                return TRUE;
+            }
+            if ( !layer || layer->isLocked()) {
+                dc->_message_context->flash(Inkscape::WARNING_MESSAGE, _("<b>Current layer is locked</b>. Unlock it to be able to draw on it."));
+                return TRUE;
+            }
+
             NR::Point const button_w(event->button.x,
                                      event->button.y);
             NR::Point const button_dt(sp_desktop_w2d_xy_point(desktop, button_w));
             sp_dyna_draw_reset(dc, button_dt);
             sp_dyna_draw_apply(dc, button_dt);
             NR::Point p = sp_dyna_draw_get_curr_vpoint(dc);
-            sp_desktop_free_snap(desktop, p);
+            namedview_free_snap(desktop->namedview, Snapper::SNAP_POINT, p);
             /* TODO: p isn't subsequently used; we should probably get rid of the last
                1-2 statements (or use p).  Same for MOTION_NOTIFY below. */
             sp_curve_reset(dc->accumulated);
@@ -425,9 +481,11 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
 
             sp_canvas_item_grab(SP_CANVAS_ITEM(desktop->acetate),
                                 ( dc->use_timeout
-                                  ? ( GDK_BUTTON_RELEASE_MASK |
+                                  ? ( GDK_KEY_PRESS_MASK |
+                                      GDK_BUTTON_RELEASE_MASK |
                                       GDK_BUTTON_PRESS_MASK    )
-                                  : ( GDK_BUTTON_RELEASE_MASK |
+                                  : ( GDK_KEY_PRESS_MASK | 
+                                      GDK_BUTTON_RELEASE_MASK |
                                       GDK_POINTER_MOTION_MASK |
                                       GDK_BUTTON_PRESS_MASK    ) ),
                                 NULL,
@@ -437,10 +495,12 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
                 dc->timer_id = gtk_timeout_add(SAMPLE_TIMEOUT, sp_dyna_draw_timeout_handler, dc);
             }
             ret = TRUE;
+
+            dc->is_drawing = true;
         }
         break;
     case GDK_MOTION_NOTIFY:
-        if ( !dc->use_timeout && ( event->motion.state & GDK_BUTTON1_MASK ) ) {
+        if ( dc->is_drawing && !dc->use_timeout && ( event->motion.state & GDK_BUTTON1_MASK ) ) {
             dc->dragging = TRUE;
             dc->dynahand = TRUE;
 
@@ -453,7 +513,7 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
                 break;
             }
             NR::Point p = sp_dyna_draw_get_curr_vpoint(dc);
-            sp_desktop_free_snap(desktop, p);
+            namedview_free_snap(desktop->namedview, Snapper::SNAP_POINT, p);
             /* p unused; see comments above in BUTTON_PRESS. */
 
             if ( dc->cur != dc->last ) {
@@ -466,6 +526,8 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
         break;
 
     case GDK_BUTTON_RELEASE:
+        sp_canvas_item_ungrab(SP_CANVAS_ITEM(desktop->acetate), event->button.time);
+        dc->is_drawing = false;
         if ( event->button.button == 1
              && dc->use_timeout
              && dc->timer_id != 0 )
@@ -501,22 +563,62 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
                     }
                 }
             }
-
-            sp_canvas_item_ungrab(SP_CANVAS_ITEM(desktop->acetate), event->button.time);
             ret = TRUE;
         }
         break;
     case GDK_KEY_PRESS:
         switch (event->key.keyval) {
         case GDK_Up:
-        case GDK_Down:
         case GDK_KP_Up:
-        case GDK_KP_Down:
-            // prevent the zoom field from activation
             if (!MOD__CTRL_ONLY) {
+                dc->angle += 5.0;
+                if (dc->angle > 90.0) 
+                    dc->angle = 90.0;
+                sp_ddc_update_toolbox (desktop, "calligraphy-angle", dc->angle);
                 ret = TRUE;
             }
             break;
+        case GDK_Down:
+        case GDK_KP_Down:
+            if (!MOD__CTRL_ONLY) {
+                dc->angle -= 5.0;
+                if (dc->angle < -90.0) 
+                    dc->angle = -90.0;
+                sp_ddc_update_toolbox (desktop, "calligraphy-angle", dc->angle);
+                ret = TRUE;
+            }
+            break;
+        case GDK_Right:
+        case GDK_KP_Right:
+            if (!MOD__CTRL_ONLY) {
+                dc->width += 0.01;
+                if (dc->width > 1.0) 
+                    dc->width = 1.0;
+                sp_ddc_update_toolbox (desktop, "altx-calligraphy", dc->width); // the same spinbutton is for alt+x
+                ret = TRUE;
+            }
+            break;
+        case GDK_Left:
+        case GDK_KP_Left:
+            if (!MOD__CTRL_ONLY) {
+                dc->width -= 0.01;
+                if (dc->width < 0.0) 
+                    dc->width = 0.0;
+                sp_ddc_update_toolbox (desktop, "altx-calligraphy", dc->width); 
+                ret = TRUE;
+            }
+            break;
+        case GDK_x:
+        case GDK_X:
+            if (MOD__ALT_ONLY) {
+                gpointer hb = sp_search_by_data_recursive (desktop->owner->aux_toolbox, (gpointer) "altx-calligraphy");
+                if (hb && GTK_IS_WIDGET(hb)) {
+                    gtk_widget_grab_focus (GTK_WIDGET (hb));
+                }
+                ret = TRUE;
+            }
+            break;
+
         default:
             break;
         }
@@ -533,6 +635,7 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
     return ret;
 }
 
+
 static void
 clear_current(SPDynaDrawContext *dc)
 {
@@ -552,34 +655,28 @@ set_to_accumulated(SPDynaDrawContext *dc)
     SPDesktop *desktop = SP_EVENT_CONTEXT(dc)->desktop;
 
     if (!sp_curve_empty(dc->accumulated)) {
-        ArtBpath *abp;
-        gdouble d2doc[6];
+        NArtBpath *abp;
         gchar *str;
 
         if (!dc->repr) {
             /* Create object */
             SPRepr *repr = sp_repr_new("path");
+
             /* Set style */
-            SPRepr *style = inkscape_get_repr(INKSCAPE,
-                                              ( dc->use_calligraphic
-                                                ? "tools.calligraphic"
-                                                : "tools.freehand" ));
-            if (style) {
-                SPCSSAttr *css = sp_repr_css_attr_inherited(style, "style");
-                sp_repr_css_set(repr, css, "style");
-                sp_repr_css_attr_unref(css);
-            }
+            sp_desktop_apply_style_tool (desktop, repr, "tools.calligraphic", false);
+
             dc->repr = repr;
 
-            sp_document_add_repr(SP_DT_DOCUMENT(desktop), dc->repr);
+            SPItem *item=SP_ITEM(desktop->currentLayer()->appendChildRepr(dc->repr));
             sp_repr_unref(dc->repr);
-            sp_selection_set_repr(SP_DT_SELECTION(desktop), dc->repr);
+            item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
+            item->updateRepr();
+            SP_DT_SELECTION(desktop)->setRepr(dc->repr);
         }
-        sp_desktop_dt2root_affine(desktop, (NRMatrix *) d2doc);
-        abp = art_bpath_affine_transform(sp_curve_first_bpath(dc->accumulated), d2doc);
+        abp = nr_artpath_affine(sp_curve_first_bpath(dc->accumulated), sp_desktop_dt2root_affine(desktop));
         str = sp_svg_write_path(abp);
         g_assert( str != NULL );
-        art_free(abp);
+        nr_free(abp);
         sp_repr_set_attr(dc->repr, "d", str);
         g_free(str);
     } else {
@@ -596,19 +693,19 @@ static void
 concat_current_line(SPDynaDrawContext *dc)
 {
     if (!sp_curve_empty(dc->currentcurve)) {
-        ArtBpath *bpath;
+        NArtBpath *bpath;
         if (sp_curve_empty(dc->accumulated)) {
             bpath = sp_curve_first_bpath(dc->currentcurve);
-            g_assert( bpath->code == ART_MOVETO_OPEN );
+            g_assert( bpath->code == NR_MOVETO_OPEN );
             sp_curve_moveto(dc->accumulated, bpath->x3, bpath->y3);
         }
         bpath = sp_curve_last_bpath(dc->currentcurve);
-        if ( bpath->code == ART_CURVETO ) {
+        if ( bpath->code == NR_CURVETO ) {
             sp_curve_curveto(dc->accumulated,
                              bpath->x1, bpath->y1,
                              bpath->x2, bpath->y2,
                              bpath->x3, bpath->y3);
-        } else if ( bpath->code == ART_LINETO ) {
+        } else if ( bpath->code == NR_LINETO ) {
             sp_curve_lineto(dc->accumulated, bpath->x3, bpath->y3);
         } else {
             g_assert_not_reached();
@@ -703,12 +800,13 @@ fit_and_split_calligraphics(SPDynaDrawContext *dc, gboolean release)
     g_print("[F&S:R=%c]", release?'T':'F');
 #endif
 
-    g_assert( dc->npoints > 0 && dc->npoints < SAMPLING_SIZE );
+    if (!( dc->npoints > 0 && dc->npoints < SAMPLING_SIZE ))
+        return; // just clicked
 
     if ( dc->npoints == SAMPLING_SIZE - 1 || release ) {
 #define BEZIER_SIZE       4
-#define BEZIER_MAX_DEPTH  4
-#define BEZIER_MAX_LENGTH ( BEZIER_SIZE << ( BEZIER_MAX_DEPTH - 1 ) )
+#define BEZIER_MAX_BEZIERS  8
+#define BEZIER_MAX_LENGTH ( BEZIER_SIZE * BEZIER_MAX_BEZIERS )
 
 #ifdef DYNA_DRAW_VERBOSE
         g_print("[F&S:#] dc->npoints:%d, release:%s\n",
@@ -728,12 +826,12 @@ fit_and_split_calligraphics(SPDynaDrawContext *dc, gboolean release)
 
         NR::Point b1[BEZIER_MAX_LENGTH];
         gint const nb1 = sp_bezier_fit_cubic_r(b1, dc->point1, dc->npoints,
-                                               tolerance_sq, BEZIER_MAX_DEPTH);
+                                               tolerance_sq, BEZIER_MAX_BEZIERS);
         g_assert( nb1 * BEZIER_SIZE <= gint(G_N_ELEMENTS(b1)) );
 
         NR::Point b2[BEZIER_MAX_LENGTH];
         gint const nb2 = sp_bezier_fit_cubic_r(b2, dc->point2, dc->npoints,
-                                               tolerance_sq, BEZIER_MAX_DEPTH);
+                                               tolerance_sq, BEZIER_MAX_BEZIERS);
         g_assert( nb2 * BEZIER_SIZE <= gint(G_N_ELEMENTS(b2)) );
 
         if ( nb1 != -1 && nb2 != -1 ) {
@@ -824,154 +922,6 @@ draw_temporary_box(SPDynaDrawContext *dc)
     sp_canvas_bpath_set_bpath(SP_CANVAS_BPATH(dc->currentshape), dc->currentcurve);
 }
 
-/* Gtk stuff */
-
-static void
-sp_ddc_mass_value_changed(GtkAdjustment *adj, SPDynaDrawContext *ddc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(ddc), "mass", adj->value);
-}
-
-static void
-sp_ddc_drag_value_changed(GtkAdjustment *adj, SPDynaDrawContext *ddc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(ddc), "drag", adj->value);
-}
-
-static void
-sp_ddc_angle_value_changed(GtkAdjustment *adj, SPDynaDrawContext *ddc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(ddc), "angle", adj->value);
-}
-
-static void
-sp_ddc_width_value_changed(GtkAdjustment *adj, SPDynaDrawContext *ddc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(ddc), "width", adj->value);
-}
-
-static void sp_ddc_defaults(GtkWidget *, GtkObject *obj)
-{
-    struct KeyValue {
-        char const *key;
-        double value;
-    } const key_values[] = {
-        {"mass", 0.3},
-        {"drag", DRAG_DEFAULT},
-        {"angle", 30.0},
-        {"width", 0.2}
-    };
-
-    for (unsigned i = 0; i < G_N_ELEMENTS(key_values); ++i) {
-        KeyValue const &kv = key_values[i];
-        GtkAdjustment &adj = *static_cast<GtkAdjustment *>(gtk_object_get_data(obj, kv.key));
-        gtk_adjustment_set_value(&adj, kv.value);
-    }
-}
-
-static GtkWidget *
-sp_dyna_draw_context_config_widget(SPEventContext *ec)
-{
-    SPDynaDrawContext *ddc = SP_DYNA_DRAW_CONTEXT(ec);
-
-    GtkWidget *tbl = gtk_table_new(5, 2, FALSE);
-    gtk_container_set_border_width(GTK_CONTAINER(tbl), 4);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 4);
-
-    /* Mass */
-    {
-        GtkWidget *l = gtk_label_new(_("Mass:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 0, 1,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(ddc->mass, 0.0, 1.0, 0.01, 0.1, 0.1);
-        gtk_object_set_data(GTK_OBJECT(tbl), "mass", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 0, 1,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_ddc_mass_value_changed), ddc);
-    }
-
-    /* Drag */
-    {
-        GtkWidget *l = gtk_label_new(_("Drag:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 1, 2,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(ddc->drag, DRAG_MIN, DRAG_MAX, 0.01, 0.1, 0.1);
-        gtk_object_set_data(GTK_OBJECT(tbl), "drag", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 1, 2,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_ddc_drag_value_changed), ddc);
-    }
-
-    /* Angle */
-    {
-        GtkWidget *l = gtk_label_new(_("Angle:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 2, 3,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(ddc->angle, 0.0, 360.0, 1.0, 10.0, 10.0);
-        gtk_object_set_data(GTK_OBJECT(tbl), "angle", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 2, 3,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_ddc_angle_value_changed), ddc);
-    }
-
-    /* Width */
-    {
-        GtkWidget *l = gtk_label_new(_("Width:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 3, 4,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(ddc->width, 0.01, 1.0, 0.01, 0.1, 0.1);
-        gtk_object_set_data(GTK_OBJECT(tbl), "width", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 3, 4,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_ddc_width_value_changed), ddc);
-    }
-
-    /* Reset */
-    {
-        GtkWidget *b = gtk_button_new_with_label(_("Defaults"));
-        gtk_widget_show(b);
-        gtk_table_attach(GTK_TABLE(tbl), b, 0, 2, 4, 5,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(b), "clicked", GTK_SIGNAL_FUNC(sp_ddc_defaults), tbl);
-    }
-
-    return tbl;
-}
-
 /*
   Local Variables:
   mode:c++
