@@ -16,20 +16,19 @@
 #include <config.h>
 
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 
-#include "helper/sp-canvas.h"
-#include "xml/repr-private.h"
+#include "display/sp-canvas.h"
+#include "xml/sp-repr-event-vector.h"
+#include "xml/repr.h"
 #include "sp-cursor.h"
-
 #include "shortcuts.h"
-
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
 #include "selection.h"
-#include "event-context.h"
 #include "sp-item.h"
 #include "zoom-context.h"
 #include "select-context.h"
@@ -41,6 +40,10 @@
 #include "macros.h"
 #include "tools-switch.h"
 #include "prefs-utils.h"
+#include "message-context.h"
+#include "widgets/spw-utilities.h"
+
+#include "event-context.h"
 
 static void sp_event_context_class_init (SPEventContextClass *klass);
 static void sp_event_context_init (SPEventContext *event_context);
@@ -108,6 +111,8 @@ sp_event_context_init (SPEventContext *event_context)
 {
 	event_context->desktop = NULL;
 	event_context->cursor = NULL;
+	event_context->_message_context = NULL;
+	event_context->_selcue = NULL;
 }
 
 static void
@@ -117,6 +122,10 @@ sp_event_context_dispose (GObject *object)
 
 	ec = SP_EVENT_CONTEXT (object);
 
+	if (ec->_message_context) {
+		delete ec->_message_context;
+	}
+
 	if (ec->cursor != NULL) {
 		gdk_cursor_unref (ec->cursor);
 		ec->cursor = NULL;
@@ -135,18 +144,15 @@ sp_event_context_dispose (GObject *object)
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static void
-sp_event_context_private_setup (SPEventContext *ec)
+void
+sp_event_context_update_cursor (SPEventContext *ec)
 {
-	GtkWidget *w;
-	GdkBitmap *bitmap, * mask;
-
-	w = GTK_WIDGET (SP_DT_CANVAS (ec->desktop));
+	GtkWidget *w = GTK_WIDGET (SP_DT_CANVAS (ec->desktop));
 	if (w->window) {
 		/* fixme: */
 		if (ec->cursor_shape) {
-			bitmap = NULL;
-			mask = NULL;
+			GdkBitmap *bitmap = NULL;
+			GdkBitmap *mask = NULL;
 			sp_cursor_bitmap_and_mask_from_xpm (&bitmap, &mask, ec->cursor_shape);
 			if ((bitmap != NULL) && (mask != NULL)) {
 				ec->cursor = gdk_cursor_new_from_pixmap (bitmap, mask,
@@ -158,6 +164,12 @@ sp_event_context_private_setup (SPEventContext *ec)
 	}
 }
 
+static void
+sp_event_context_private_setup (SPEventContext *ec)
+{
+	sp_event_context_update_cursor (ec);
+}
+
 static void 
 sp_toggle_selector (SPDesktop *dt)
 {
@@ -360,8 +372,7 @@ static gint sp_event_context_private_root_handler(SPEventContext *event_context,
 
 			if (within_tolerance) {
 				dontgrab ++;
-				NR::Point const event_w(event->button.x,
-							event->button.y);
+				NR::Point const event_w(event->button.x, event->button.y);
 				NR::Point const event_dt(sp_desktop_w2d_xy_point(desktop, event_w));
 				double const zoom_power = ( (event->button.state & GDK_SHIFT_MASK)
 							    ? -dontgrab
@@ -464,10 +475,22 @@ static gint sp_event_context_private_root_handler(SPEventContext *event_context,
 			sp_toggle_selector (desktop);
 			ret= TRUE;
 			break;
+		case GDK_z:
+		case GDK_Z:
+                   if (MOD__ALT_ONLY) {
+                        gpointer hb = gtk_object_get_data (GTK_OBJECT (desktop->owner), "altz");
+                        if (hb && GTK_IS_WIDGET(hb)) {
+                            gtk_widget_grab_focus(GTK_WIDGET(hb));
+                        }
+                        ret = TRUE;
+			}
+			break;
+		default:
+			break;
 		}
 		break;
 	case GDK_SCROLL:
-		/* ctrl + wheel, pan left--right */
+		/* shift + wheel, pan left--right */
 		if (event->scroll.state & GDK_SHIFT_MASK) {
 			switch (event->scroll.direction) {
 			case GDK_SCROLL_UP:
@@ -479,21 +502,27 @@ static gint sp_event_context_private_root_handler(SPEventContext *event_context,
 			default:
 				break;
 			}
-			/* shift + wheel, zoom in--out */
+
+		/* ctrl + wheel, zoom in--out */
 		} else if (event->scroll.state & GDK_CONTROL_MASK) {
 			double rel_zoom;
 			switch (event->scroll.direction) {
-			case GDK_SCROLL_UP:   rel_zoom = zoom_inc;     break;
-			case GDK_SCROLL_DOWN: rel_zoom = 1 / zoom_inc; break;
-			default:              rel_zoom = 0.0;          break;
+			case GDK_SCROLL_UP:   
+				rel_zoom = zoom_inc;
+				break;
+			case GDK_SCROLL_DOWN: 
+				rel_zoom = 1 / zoom_inc;
+				break;
+			default:              
+				rel_zoom = 0.0;
+				break;
 			}
 			if (rel_zoom != 0.0) {
-				NR::Point const scroll_w(event->scroll.x,
-							 event->scroll.y);
-				NR::Point const scroll_dt(sp_desktop_w2d_xy_point(desktop, scroll_w));
+				NR::Point const scroll_dt = sp_desktop_point(desktop);
 				sp_desktop_zoom_relative_keep_point(desktop, scroll_dt, rel_zoom);
 			}
-			/* no modifier, pan up--down (left--right on multiwheel mice?) */
+
+		/* no modifier, pan up--down (left--right on multiwheel mice?) */
 		} else {
 			switch (event->scroll.direction) {
 			case GDK_SCROLL_UP:
@@ -541,11 +570,6 @@ sp_event_context_private_item_handler (SPEventContext *ec, SPItem *item, GdkEven
 	return ret;
 }
 
-static void sp_ec_repr_destroy(SPRepr *, gpointer)
-{
-	g_warning ("Oops! Repr destroyed while event context still present");
-}
-
 static unsigned int
 sp_ec_repr_change_attr (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, gpointer data)
 {
@@ -570,7 +594,6 @@ sp_ec_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval, co
 }
 
 SPReprEventVector sp_ec_event_vector = {
-	sp_ec_repr_destroy,
 	NULL, /* Add child */
 	NULL, /* Child added */
 	NULL, /* Remove child */
@@ -595,6 +618,7 @@ sp_event_context_new (GType type, SPDesktop *desktop, SPRepr *repr, unsigned int
 	ec = (SPEventContext*)g_object_new (type, NULL);
 
 	ec->desktop = desktop;
+	ec->_message_context = new Inkscape::MessageContext(desktop->messageStack());
 	ec->key = key;
 	ec->repr = repr;
 	if (ec->repr) {
@@ -614,6 +638,8 @@ sp_event_context_finish (SPEventContext *ec)
 	g_return_if_fail (ec != NULL);
 	g_return_if_fail (SP_IS_EVENT_CONTEXT (ec));
 
+	ec->enableSelectionCue(false);
+
 	if (ec->next) {
 		g_warning ("Finishing event context with active link\n");
 	}
@@ -622,6 +648,19 @@ sp_event_context_finish (SPEventContext *ec)
 		((SPEventContextClass *) G_OBJECT_GET_CLASS(ec))->finish (ec);
 }
 
+void SPEventContext::enableSelectionCue(bool enable) {
+	if (enable) {
+		if (!_selcue) {
+			_selcue = new SPSelCue(desktop);
+		}
+	} else {
+		if (_selcue) {
+			delete _selcue;
+			_selcue = NULL;
+		}
+	}
+}
+
 void
 sp_event_context_read (SPEventContext *ec, const gchar *key)
 {
@@ -685,18 +724,6 @@ sp_event_context_item_handler (SPEventContext * event_context, SPItem * item, Gd
 	return ret;
 }
 
-GtkWidget *
-sp_event_context_config_widget (SPEventContext *ec)
-{
-	g_return_val_if_fail (ec != NULL, NULL);
-	g_return_val_if_fail (SP_IS_EVENT_CONTEXT (ec), NULL);
-
-	if (((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->config_widget)
-		return ((SPEventContextClass *) G_OBJECT_GET_CLASS (ec))->config_widget (ec);
-
-	return NULL;
-}
-
 static void set_event_location(SPDesktop *desktop, GdkEvent *event)
 {
 	if (event->type != GDK_MOTION_NOTIFY) {
@@ -716,7 +743,7 @@ sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 
 	/* fixme: This is not what I want but works for now (Lauris) */
 	if (event->type == GDK_KEY_PRESS) {
-		item = sp_selection_item (SP_DT_SELECTION (desktop));
+		item = SP_DT_SELECTION(desktop)->singleItem();
 	}
 	menu = sp_ui_context_menu (SP_VIEW (desktop), item);
 	gtk_widget_show (menu);
@@ -733,3 +760,26 @@ sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 	}
 }
 
+void
+sp_event_show_modifier_tip (Inkscape::MessageContext *message_context, GdkEvent *event, const gchar *ctrl_tip, const gchar *shift_tip, const gchar *alt_tip)
+{
+	bool ctrl = ctrl_tip && (MOD__CTRL || (event->key.keyval == GDK_Control_L) || (event->key.keyval == GDK_Control_R));
+	bool shift = shift_tip && (MOD__SHIFT || (event->key.keyval == GDK_Shift_L) || (event->key.keyval == GDK_Shift_R));
+	bool alt = alt_tip && (MOD__ALT || (event->key.keyval == GDK_Alt_L) || (event->key.keyval == GDK_Alt_R) 
+		|| (event->key.keyval == GDK_Meta_L) || (event->key.keyval == GDK_Meta_R));
+
+	gchar *tip = g_strdup_printf ("%s%s%s%s%s", 
+						ctrl? ctrl_tip : "",
+						ctrl && (shift || alt)? "; " : "",
+						shift? shift_tip : "",
+						(ctrl || shift) && alt? "; " : "",
+						alt? alt_tip : ""
+						);
+
+
+	if (strlen (tip) > 0) {
+		message_context->flash(Inkscape::INFORMATION_MESSAGE, tip);
+	}
+
+	g_free (tip);
+}
