@@ -19,11 +19,16 @@
 
 #include <glib-object.h>
 #include <gtk/gtkwidget.h>
-#include "xml/repr.h"
+#include "xml/xml-forward.h"
+#include "selcue.h"
+
 #include "forward.h"
 
-struct _SPEventContext {
-	GObject object;
+namespace Inkscape { class MessageContext; }
+
+struct SPEventContext : public GObject {
+	void enableSelectionCue(bool enable=true);
+
 	/* Desktop eventcontext stack */
 	SPEventContext *next;
 	unsigned int key;
@@ -38,10 +43,17 @@ struct _SPEventContext {
 	bool within_tolerance; // are we still within tolerance of origin
 
 	SPItem *item_to_select; // the item where mouse_press occurred, to be selected if this is a click not drag
+
+	Inkscape::MessageContext *defaultMessageContext() {
+		return _message_context;
+	}
+
+	Inkscape::MessageContext *_message_context;
+
+	SPSelCue *_selcue;
 };
 
-struct _SPEventContextClass {
-	GObjectClass parent_class;
+struct SPEventContextClass : public GObjectClass {
 	void (* setup) (SPEventContext *ec);
 	void (* finish) (SPEventContext *ec);
 	void (* set) (SPEventContext *ec, const gchar *key, const gchar *val);
@@ -49,9 +61,6 @@ struct _SPEventContextClass {
 	void (* deactivate) (SPEventContext *ec);
 	gint (* root_handler) (SPEventContext *ec, GdkEvent *event);
 	gint (* item_handler) (SPEventContext *ec, SPItem *item, GdkEvent *event);
-	/* fixme: I do not like Gtk+ stuff here (Lauris) */
-	GtkWidget * (* config_widget) (SPEventContext *ec);
-
 };
 
 #define SP_EVENT_CONTEXT_DESKTOP(e) (SP_EVENT_CONTEXT (e)->desktop)
@@ -68,11 +77,13 @@ void sp_event_context_deactivate (SPEventContext *ec);
 gint sp_event_context_root_handler (SPEventContext *ec, GdkEvent *event);
 gint sp_event_context_item_handler (SPEventContext *ec, SPItem *item, GdkEvent *event);
 
-GtkWidget *sp_event_context_config_widget (SPEventContext *ec);
-
 void sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event);
 
 gint gobble_key_events (guint keyval, gint mask);
 gint gobble_motion_events (gint mask);
 
+void sp_event_context_update_cursor (SPEventContext *ec);
+
+void sp_event_show_modifier_tip (Inkscape::MessageContext *message_context, GdkEvent *event, const gchar *ctrl_tip, const gchar *shift_tip, const gchar *alt_tip);
+
 #endif
