@@ -27,11 +27,13 @@
 #include "inkscape.h"
 #include "sp-object.h"
 #include "document.h"
-#include "file.h"
-#include "interface.h"
+#include "prefs-utils.h"
 #include "extension.h"
+#include <extension/implementation/implementation.h>
+#include <libnr/nr-point.h>
 
 #include "db.h"
+#include "dependency.h"
 
 namespace Inkscape {
 namespace Extension {
@@ -56,7 +58,7 @@ Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
 
     id = NULL;
     name = NULL;
-    state = STATE_UNLOADED;
+    _state = STATE_UNLOADED;
     parameters = NULL;
 
     if (in_imp == NULL) {
@@ -67,16 +69,11 @@ Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
 
 //  printf("Extension Constructor: ");
     if (repr != NULL) {
-        const gchar *val;
-        gchar c[256];
-        SPRepr * child_repr;
-
-        child_repr = sp_repr_children(repr);
+        SPRepr *child_repr = sp_repr_children(repr);
         /* TODO: Handle what happens if we don't have these two */
         while (child_repr != NULL) {
             if (!strcmp(sp_repr_name(child_repr), "id")) {
-                val = sp_repr_content (sp_repr_children(child_repr));
-                val = db.get_unique_id (c, 256, val);
+                gchar const *val = sp_repr_content(sp_repr_children(child_repr));
                 id = g_strdup (val);
             } /* id */
             if (!strcmp(sp_repr_name(child_repr), "name")) {
@@ -85,6 +82,9 @@ Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
             if (!strcmp(sp_repr_name(child_repr), "param")) {
                 make_param(child_repr);
             } /* param */
+            if (!strcmp(sp_repr_name(child_repr), "dependency")) {
+                _deps.push_back(new Dependency(child_repr));
+            } /* param */
             child_repr = sp_repr_next(child_repr);
         }
 
@@ -106,11 +106,19 @@ Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
 */
 Extension::~Extension (void)
 {
+//	printf("Extension Destructor: %s\n", name);
+	set_state(STATE_UNLOADED);
 	db.unregister_ext(this);
     sp_repr_unref(repr);
     g_free(id);
     g_free(name);
-    /* TODO: Need to do parameters here */
+    /** \todo Need to do parameters here */
+
+	for (unsigned int i = 0 ; i < _deps.size(); i++) {
+		delete _deps[i];
+	}
+	_deps.clear();
+
     return;
 }
 
@@ -128,16 +136,26 @@ Extension::~Extension (void)
 void
 Extension::set_state (state_t in_state)
 {
-    if (in_state != state) {
-        /* TODO: Need some error checking here! */
-        if (in_state == STATE_LOADED) {
-            if (imp->load(this))
-                state = STATE_LOADED;
-        } else {
-            imp->unload(this);
-            state = STATE_UNLOADED;
-        }
+	if (_state == STATE_DEACTIVATED) return;
+    if (in_state != _state) {
+        /** \todo Need some more error checking here! */
+		switch (in_state) {
+			case STATE_LOADED:
+				if (imp->load(this))
+					_state = STATE_LOADED;
+				break;
+			case STATE_UNLOADED:
+				imp->unload(this);
+				_state = STATE_UNLOADED;
+				break;
+			case STATE_DEACTIVATED:
+				_state = STATE_DEACTIVATED;
+				break;
+			default:
+				break;
+		}
     }
+
     return;
 }
 
@@ -148,7 +166,7 @@ Extension::set_state (state_t in_state)
 Extension::state_t
 Extension::get_state (void)
 {
-    return state;
+    return _state;
 }
 
 /**
@@ -158,7 +176,40 @@ Extension::get_state (void)
 bool
 Extension::loaded (void)
 {
-    return state == STATE_LOADED;
+    return get_state() == STATE_LOADED;
+}
+
+/**
+    \return  A boolean saying whether the extension passed the checks
+	\brief   A function to check the validity of the extension
+
+	This function chekcs to make sure that there is an id, a name, a
+	repr and an implemenation for this extension.  Then it checks all
+	of the dependencies to see if they pass.  Finally, it asks the
+	implmentation to do a check of itself.
+*/
+bool
+Extension::check (void)
+{
+	// static int i = 0;
+	// std::cout << "Checking module[" << i++ << "]: " << name << std::endl;
+	if (id == NULL)
+		return FALSE;
+	if (name == NULL)
+		return FALSE;
+	if (repr == NULL)
+		return FALSE;
+	if (imp == NULL)
+		return FALSE;
+
+	for (unsigned int i = 0 ; i < _deps.size(); i++) {
+		if (_deps[i]->check() == FALSE) {
+			// std::cout << "Failed: " << *(_deps[i]) << std::endl;
+			return FALSE;
+		}
+	}
+
+	return imp->check(this);
 }
 
 /**
@@ -171,7 +222,7 @@ Extension::get_repr (void)
     return repr;
 }
 
-/*
+/**
     \return  The textual id of this extension
     \brief   Get the ID of this extension - not a copy don't delete!
 */
@@ -181,7 +232,7 @@ Extension::get_id (void)
     return id;
 }
 
-/*
+/**
     \return  The textual name of this extension
     \brief   Get the name of this extension - not a copy don't delete!
 */
@@ -191,6 +242,44 @@ Extension::get_name (void)
     return name;
 }
 
+/**
+    \return  None
+	\brief   This function diactivates the extension (which makes it
+	         unusable, but not deleted)
+	
+    This function is used to removed an extension from functioning, but
+	not delete it completely.  It sets the state to \c STATE_DEACTIVATED to
+	mark to the world that it has been deactivated.  It also removes
+	the current implementation and replaces it with a standard one.  This
+	makes it so that we don't have to continually check if there is an
+	implementation, but we are gauranteed to have a benign one.
+
+	\warning It is important to note that there is no 'activate' function.
+	Running this function is irreversable.
+*/
+void
+Extension::deactivate (void)
+{
+	set_state(STATE_DEACTIVATED);
+
+	/* Removing the old implementation, and making this use the default. */
+	/* This should save some memory */
+	delete imp;
+	imp = new Implementation::Implementation();
+
+	return;
+}
+
+/**
+    \return  Whether the extension has been deactivated
+	\brief   Find out the status of the extension
+*/
+bool
+Extension::deactivated (void)
+{
+	return get_state() == STATE_DEACTIVATED;
+}
+
 /**
     \return None
     \brief  This function creates a parameter that can be used later.  This
@@ -200,7 +289,7 @@ Extension::get_name (void)
     \param  paramrepr  The XML describing the parameter
 
     This function first grabs all of the data out of the Repr and puts
-    it into local variables.  Actually these are just pointers, and the
+    it into local variables.  Actually, these are just pointers, and the
     data is not duplicated so we need to be careful with it.  If there
     isn't a name or a type in the XML, then no parameter is created as
     the function just returns.
@@ -217,7 +306,7 @@ Extension::get_name (void)
     Finally the allocated parameter is put into the GSList that is called
     parameteres.
 
-    TODO: This function should pull up parameters that are stored
+    \todo This function should pull up parameters that are stored
     in the preferences somewhere.  This needs to be figured out.
 */
 void
@@ -226,6 +315,7 @@ Extension::make_param (SPRepr * paramrepr)
     const char * name;
     const char * type;
     const char * defaultval;
+	gchar * param_name;
     Extension::param_t * param;
 
     name = sp_repr_attr(paramrepr, "name");
@@ -240,6 +330,7 @@ Extension::make_param (SPRepr * paramrepr)
 
     param = new param_t();
     param->name = g_strdup(name);
+	param_name = g_strdup_printf("%s.%s", id, name);
     if (!strcmp(type, "boolean")) {
         param->type = Extension::PARAM_BOOL;
         if (defaultval != NULL && !strcmp(defaultval, "TRUE")) {
@@ -247,6 +338,7 @@ Extension::make_param (SPRepr * paramrepr)
         } else {
             param->val.t_bool = FALSE;
         }
+		param->val.t_bool = prefs_get_int_attribute("extensions", param_name, (gint)param->val.t_bool);
     } else if (!strcmp(type, "int")) { 
         param->type = Extension::PARAM_INT;
         if (defaultval != NULL) {
@@ -254,12 +346,21 @@ Extension::make_param (SPRepr * paramrepr)
         } else {
             param->val.t_int = 0;
         }
+		param->val.t_int = prefs_get_int_attribute("extensions", param_name, (gint)param->val.t_int);
     } else if (!strcmp(type, "string")) { 
+		const gchar * temp_str;
         param->type = Extension::PARAM_STRING;
-        param->val.t_string = g_strdup(defaultval);
+
+		temp_str = prefs_get_string_attribute("extensions", param_name);
+		if (temp_str == NULL)
+			temp_str = defaultval;
+
+        param->val.t_string = g_strdup(temp_str);
     } else {
+		g_free(param_name);
         return;
     }
+	g_free(param_name);
 
     parameters = g_slist_append(parameters, param);
     return;
@@ -292,7 +393,7 @@ Extension::make_param (SPRepr * paramrepr)
     called again with g_slist_next as a parameter.
 */
 inline Extension::param_t *
-Extension::param_shared (gchar * name, GSList * list)
+Extension::param_shared (const gchar * name, GSList * list)
 {
     Extension::param_t * output;
     
@@ -312,17 +413,11 @@ Extension::param_shared (gchar * name, GSList * list)
 }
 
 /**
-    \return   None
+    \return   A constant pointer to the string held by the parameters.
     \brief    Gets a parameter identified by name with the string placed
               in value.  It isn't duplicated into the value string.
     \param    name    The name of the parameter to get
-    \param    value   Place to put a pointer to the string
-
-    This function first checks to make sure that the value parameter
-    is not passed in as NULL.  If so, the function returns.  This
-    doesn't emit a warning - this will probably cause some sort of
-    error in the calling function (as it wouldn't be calling this function
-    if it didn't want the value) so it can be debugged there.
+	\param    doc    The document to look in for document specific parameters
 
     To get the parameter to be used the function param_shared is called.
     This function is inline so it shouldn't cause the stack to build
@@ -337,39 +432,26 @@ Extension::param_shared (gchar * name, GSList * list)
     Finally, if everything is okay, the string value that is stored in
     the parameter is placed in value.
 */
-void
-Extension::get_param (gchar * name, gchar ** value)
+const gchar *
+Extension::get_param_string (const gchar * name, const SPReprDoc * doc)
 {
     Extension::param_t * param;
     
-    if (value == NULL) {
-        /* This probably isn't a good error, but the calling function
-           will find out soon enough it doesn't have data there ;) */
-        return;
-    }
-
     param = Extension::param_shared(name, parameters);
 
     if (param->type != Extension::PARAM_STRING) {
         throw Extension::param_wrong_type();
     }
 
-    *value = param->val.t_string;
-    return;
+    return param->val.t_string;
 }
 
 /**
-    \return   None
+    \return   The value of the parameter identified by the name
     \brief    Gets a parameter identified by name with the bool placed
               in value.
     \param    name    The name of the parameter to get
-    \param    value   Place to put the bool value
-
-    This function first checks to make sure that the value parameter
-    is not passed in as NULL.  If so, the function returns.  This
-    doesn't emit a warning - this will probably cause some sort of
-    error in the calling function (as it wouldn't be calling this function
-    if it didn't want the value) so it can be debugged there.
+	\param    doc    The document to look in for document specific parameters
 
     To get the parameter to be used the function param_shared is called.
     This function is inline so it shouldn't cause the stack to build
@@ -384,39 +466,26 @@ Extension::get_param (gchar * name, gchar ** value)
     Finally, if everything is okay, the boolean value that is stored in
     the parameter is placed in value.
 */
-void
-Extension::get_param (gchar * name, bool * value)
+bool
+Extension::get_param_bool (const gchar * name, const SPReprDoc * doc)
 {
     Extension::param_t * param;
     
-    if (value == NULL) {
-        /* This probably isn't a good error, but the calling function
-           will find out soon enough it doesn't have data there ;) */
-        return;
-    }
-
     param = Extension::param_shared(name, parameters);
 
     if (param->type != Extension::PARAM_BOOL) {
         throw Extension::param_wrong_type();
     }
 
-    *value = param->val.t_bool;
-    return;
+    return param->val.t_bool;
 }
 
 /**
-    \return   None
+    \return   The integer value for the parameter specified
     \brief    Gets a parameter identified by name with the integer placed
               in value.
     \param    name    The name of the parameter to get
-    \param    value   Place to put the integer value
-
-    This function first checks to make sure that the value parameter
-    is not passed in as NULL.  If so, the function returns.  This
-    doesn't emit a warning - this will probably cause some sort of
-    error in the calling function (as it wouldn't be calling this function
-    if it didn't want the value) so it can be debugged there.
+	\param    doc    The document to look in for document specific parameters
 
     To get the parameter to be used the function param_shared is called.
     This function is inline so it shouldn't cause the stack to build
@@ -431,25 +500,18 @@ Extension::get_param (gchar * name, bool * value)
     Finally, if everything is okay, the integer value that is stored in
     the parameter is placed in value.
 */
-void
-Extension::get_param (gchar * name, int * value)
+int
+Extension::get_param_int (const gchar * name, const SPReprDoc * doc)
 {
     Extension::param_t * param;
     
-    if (value == NULL) {
-        /* This probably isn't a good error, but the calling function
-           will find out soon enough it doesn't have data there ;) */
-        return;
-    }
-
     param = Extension::param_shared(name, parameters);
 
     if (param->type != Extension::PARAM_INT) {
         throw Extension::param_wrong_type();
     }
 
-    *value = param->val.t_int;
-    return;
+    return param->val.t_int;
 }
 
 /**
@@ -458,6 +520,7 @@ Extension::get_param (gchar * name, int * value)
               in the parameter value.
     \param    name    The name of the parameter to set
     \param    value   The value to set the parameter to
+	\param    doc    The document to look in for document specific parameters
 
     To get the parameter to be used the function param_shared is called.
     This function is inline so it shouldn't cause the stack to build
@@ -473,9 +536,10 @@ Extension::get_param (gchar * name, int * value)
 	in is placed in the param.
 */
 bool
-Extension::set_param (gchar * name, bool value)
+Extension::set_param_bool (const gchar * name, bool value, SPReprDoc * doc)
 {
     Extension::param_t * param;
+	gchar * param_name;
     
     param = Extension::param_shared(name, parameters);
 
@@ -484,6 +548,11 @@ Extension::set_param (gchar * name, bool value)
     }
 
     param->val.t_bool = value;
+
+	param_name = g_strdup_printf("%s.%s", id, name);
+	prefs_set_int_attribute("extensions", param_name, value == TRUE ? 1 : 0);
+	g_free(param_name);
+
     return value;
 }
 
@@ -493,6 +562,7 @@ Extension::set_param (gchar * name, bool value)
               in the parameter value.
     \param    name    The name of the parameter to set
     \param    value   The value to set the parameter to
+	\param    doc    The document to look in for document specific parameters
 
     To get the parameter to be used the function param_shared is called.
     This function is inline so it shouldn't cause the stack to build
@@ -508,9 +578,10 @@ Extension::set_param (gchar * name, bool value)
 	in is placed in the param.
 */
 int
-Extension::set_param (gchar * name, int value)
+Extension::set_param_int (const gchar * name, int value, SPReprDoc * doc)
 {
     Extension::param_t * param;
+	gchar * param_name;
     
     param = Extension::param_shared(name, parameters);
 
@@ -519,6 +590,11 @@ Extension::set_param (gchar * name, int value)
     }
 
     param->val.t_int = value;
+
+	param_name = g_strdup_printf("%s.%s", id, name);
+	prefs_set_int_attribute("extensions", param_name, value);
+	g_free(param_name);
+
     return value;
 }
 
@@ -528,6 +604,7 @@ Extension::set_param (gchar * name, int value)
               in the parameter value.
     \param    name    The name of the parameter to set
     \param    value   The value to set the parameter to
+	\param    doc    The document to look in for document specific parameters
 
 	First this function makes sure that the incoming string is not
 	NULL.  The value can't be set to NULL.
@@ -545,10 +622,11 @@ Extension::set_param (gchar * name, int value)
     Finally, if everything is okay, the previous value is free'd and
 	the incoming value is duplicated and placed in the parameter.
 */
-gchar *
-Extension::set_param (gchar * name, gchar * value)
+const gchar *
+Extension::set_param_string (const gchar * name, const gchar * value, SPReprDoc * doc)
 {
     Extension::param_t * param;
+	gchar * param_name;
     
     if (value == NULL) {
         /* This probably isn't a good error, but the calling function
@@ -564,370 +642,14 @@ Extension::set_param (gchar * name, gchar * value)
 
     g_free(param->val.t_string);
     param->val.t_string = g_strdup(value);
+
+	param_name = g_strdup_printf("%s.%s", id, name);
+	prefs_set_string_attribute("extensions", param_name, value);
+	g_free(param_name);
+
     return value;
 }
 
-/* Inkscape::Extension::Input */
-
-/**
-    \return   None
-    \brief    Builds a SPModuleInput object from a XML description
-    \param    module  The module to be initialized
-    \param    repr    The XML description in a SPRepr tree
-
-    Okay, so you want to build a SPModuleInput object.
-
-    This function first takes and does the build of the parent class,
-    which is SPModule.  Then, it looks for the <input> section of the
-    XML description.  Under there should be several fields which
-    describe the input module to excruciating detail.  Those are parsed,
-    copied, and put into the structure that is passed in as module.
-    Overall, there are many levels of indentation, just to handle the
-    levels of indentation in the XML file.
-*/
-Input::Input (SPRepr * in_repr, Implementation::Implementation * in_imp) : Extension(in_repr, in_imp)
-{
-    mimetype = NULL;
-    extension = NULL;
-    filetypename = NULL;
-    filetypetooltip = NULL;
-	output_extension = NULL;
-
-    if (repr != NULL) {
-        SPRepr * child_repr;
-
-        child_repr = sp_repr_children(repr);
-
-        while (child_repr != NULL) {
-            if (!strcmp(sp_repr_name(child_repr), "input")) {
-                child_repr = sp_repr_children(child_repr);
-                while (child_repr != NULL) {
-                    if (!strcmp(sp_repr_name(child_repr), "extension")) {
-                        g_free (extension);
-                        extension = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "mimetype")) {
-                        g_free (mimetype);
-                        mimetype = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "filetypename")) {
-                        g_free (filetypename);
-                        filetypename = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "filetypetooltip")) {
-                        g_free (filetypetooltip);
-                        filetypetooltip = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "output_extension")) {
-                        g_free (output_extension);
-                        output_extension = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-
-                    child_repr = sp_repr_next(child_repr);
-                }
-
-                break;
-            }
-
-            child_repr = sp_repr_next(child_repr);
-        }
-
-    }
-
-    return;
-}
-
-Input::~Input (void)
-{
-    g_free(mimetype);
-    g_free(extension);
-    g_free(filetypename);
-    g_free(filetypetooltip);
-	g_free(output_extension);
-    return;
-}
-
-SPDocument *
-Input::open (const gchar *uri)
-{
-	SPDocument * doc;
-	SPRepr * repr;
-
-	gsize bytesRead = 0;
-	gsize bytesWritten = 0;
-	GError* error = NULL;
-	gchar* local_uri = g_filename_from_utf8 ( uri,
-                                 -1,  &bytesRead,  &bytesWritten, &error);
-
-	if (!g_file_test(local_uri, G_FILE_TEST_EXISTS)) {
-		g_free(local_uri);
-		return NULL;
-	}
-	g_free(local_uri);
-
-	doc = imp->open(this, uri);
-
-	if (doc != NULL) {
-		repr = sp_document_repr_root(doc);
-		sp_document_set_undo_sensitive (doc, FALSE);
-		sp_repr_set_attr(repr, "inkscape:output_extension", output_extension);
-		sp_document_set_undo_sensitive (doc, TRUE);
-	}
-
-	return doc;
-}
-
-gchar *
-Input::get_extension(void)
-{
-    return extension;
-}
-
-gchar *
-Input::get_filetypename(void)
-{
-    return filetypename;
-}
-
-gchar *
-Input::get_filetypetooltip(void)
-{
-    return filetypetooltip;
-}
-
-GtkDialog *
-Input::prefs (const gchar *uri)
-{
-    return imp->prefs(this, uri);
-}
-
-
-/* Inkscape::Extension::Output */
-
-/**
-    \return   None
-    \brief    Builds a SPModuleOutput object from a XML description
-    \param    module  The module to be initialized
-    \param    repr    The XML description in a SPRepr tree
-
-    Okay, so you want to build a SPModuleOutput object.
-
-    This function first takes and does the build of the parent class,
-    which is SPModule.  Then, it looks for the <output> section of the
-    XML description.  Under there should be several fields which
-    describe the output module to excruciating detail.  Those are parsed,
-    copied, and put into the structure that is passed in as module.
-    Overall, there are many levels of indentation, just to handle the
-    levels of indentation in the XML file.
-*/
-Output::Output (SPRepr * in_repr, Implementation::Implementation * in_imp) : Extension(in_repr, in_imp)
-{
-    mimetype = NULL;
-    extension = NULL;
-    filetypename = NULL;
-    filetypetooltip = NULL;
-	dataloss = TRUE;
-
-    if (repr != NULL) {
-        SPRepr * child_repr;
-
-        child_repr = sp_repr_children(repr);
-
-        while (child_repr != NULL) {
-            if (!strcmp(sp_repr_name(child_repr), "output")) {
-                child_repr = sp_repr_children(child_repr);
-                while (child_repr != NULL) {
-                    if (!strcmp(sp_repr_name(child_repr), "extension")) {
-                        g_free (extension);
-                        extension = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "mimetype")) {
-                        g_free (mimetype);
-                        mimetype = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "filetypename")) {
-                        g_free (filetypename);
-                        filetypename = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "filetypetooltip")) {
-                        g_free (filetypetooltip);
-                        filetypetooltip = g_strdup(sp_repr_content(sp_repr_children(child_repr)));
-                    }
-                    if (!strcmp(sp_repr_name(child_repr), "dataloss")) {
-                        if (!strcmp(sp_repr_content(sp_repr_children(child_repr)), "FALSE")) {
-							dataloss = FALSE;
-						}
-					}
-
-                    child_repr = sp_repr_next(child_repr);
-                }
-
-                break;
-            }
-
-            child_repr = sp_repr_next(child_repr);
-        }
-
-    }
-}
-
-Output::~Output (void)
-{
-    g_free(mimetype);
-    g_free(extension);
-    g_free(filetypename);
-    g_free(filetypetooltip);
-    return;
-}
-
-gchar *
-Output::get_extension(void)
-{
-    return extension;
-}
-
-gchar *
-Output::get_filetypename(void)
-{
-    return filetypename;
-}
-
-gchar *
-Output::get_filetypetooltip(void)
-{
-    return filetypetooltip;
-}
-
-GtkDialog *
-Output::prefs (void)
-{
-    return imp->prefs(this);
-}
-
-void
-Output::save (SPDocument * doc, const gchar * uri)
-{
-	SPRepr * repr;
-
-	repr = sp_document_repr_root(doc);
-
-	sp_document_set_undo_sensitive (doc, FALSE);
-	sp_repr_set_attr(repr, "inkscape:output_extension", NULL);
-	sp_repr_set_attr(repr, "inkscape:dataloss", NULL);
-	sp_repr_set_attr(repr, "sodipodi:modified", NULL);
-	sp_document_set_undo_sensitive (doc, TRUE);
-
-    imp->save(this, doc, uri);
-
-	sp_document_set_undo_sensitive (doc, FALSE);
-	sp_repr_set_attr(repr, "inkscape:output_extension", get_id());
-	if (dataloss) {
-		sp_repr_set_attr(repr, "inkscape:dataloss", "TRUE");
-	}
-	sp_document_set_undo_sensitive (doc, TRUE);
-
-	return;
-}
-
-/* Inkscape::Extension::Effect */
-
-Effect::Effect (SPRepr * in_repr, Implementation::Implementation * in_imp) : Extension(in_repr, in_imp)
-{
-    return;
-}
-
-Effect::~Effect (void)
-{
-    return;
-}
-
-GtkDialog *
-Effect::prefs (void)
-{
-    return imp->prefs(this);
-}
-
-void
-Effect::effect (SPDocument * doc)
-{
-    return imp->effect(this, doc);
-}
-
-/* Inkscape::Extension::Print */
-
-Print::Print (SPRepr * in_repr, Implementation::Implementation * in_imp) : Extension(in_repr, in_imp)
-{
-    base = NULL;
-    arena = NULL;
-    root = NULL;
-    dkey = 0;
-
-    return;
-}
-
-Print::~Print (void)
-{
-    return;
-}
-
-unsigned int
-Print::setup (void)
-{
-    return imp->setup(this);
-}
-
-unsigned int
-Print::set_preview (void)
-{
-    return imp->set_preview(this);
-}
-
-unsigned int
-Print::begin (SPDocument *doc)
-{
-    return imp->begin(this, doc);
-}
-
-unsigned int
-Print::finish (void)
-{
-    return imp->finish(this);
-}
-
-unsigned int
-Print::bind (const NRMatrix *transform, float opacity)
-{
-    return imp->bind (this, transform, opacity);
-}
-
-unsigned int
-Print::release (void)
-{
-    return imp->release(this);
-}
-
-unsigned int
-Print::fill (const NRBPath *bpath, const NRMatrix *ctm, const SPStyle *style,
-                   const NRRect *pbox, const NRRect *dbox, const NRRect *bbox)
-{
-    return imp->fill (this, bpath, ctm, style, pbox, dbox, bbox);
-}
-
-unsigned int
-Print::stroke (const NRBPath *bpath, const NRMatrix *transform, const SPStyle *style,
-                 const NRRect *pbox, const NRRect *dbox, const NRRect *bbox)
-{
-    return imp->stroke (this, bpath, transform, style, pbox, dbox, bbox);
-}
-
-unsigned int
-Print::image (unsigned char *px, unsigned int w, unsigned int h, unsigned int rs,
-                const NRMatrix *transform, const SPStyle *style)
-{
-    return imp->image (this, px, w, h, rs, transform, style);
-}
-
-
 }; /* namespace Extension */
 }; /* namespace Inkscape */
 
