@@ -10,8 +10,12 @@
     contains the base class for all of that.
 */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include "../extension.h"
 #include "implementation.h"
+#include <libnr/nr-point.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -19,7 +23,7 @@ namespace Implementation {
 
 /**
 	\return   Was the load sucessful?
-	\breif    This function is the stub load.  It just returns sucess.
+	\brief    This function is the stub load.  It just returns sucess.
 	\param    module   The Extension that should be loaded.
 */
 bool 
@@ -35,10 +39,16 @@ Implementation::unload (Inkscape::Extension::Extension * module) {
 	return;
 } /* Implementation::unload */
 
+bool
+Implementation::check (Inkscape::Extension::Extension * module) {
+	/* If there are no checks, they all pass */
+	return TRUE;
+} /* Implemenation::check */
+
 GtkDialog *
-Implementation::prefs (Inkscape::Extension::Input * module, const gchar * filename) {
+Implementation::prefs_input (Inkscape::Extension::Input * module, const gchar * filename) {
 	return NULL;
-} /* Implementation::prefs */
+} /* Implementation::prefs_input */
 
 SPDocument *
 Implementation::open (Inkscape::Extension::Input * module, const gchar * filename) {
@@ -47,9 +57,9 @@ Implementation::open (Inkscape::Extension::Input * module, const gchar * filenam
 } /* Implementation::open */
 
 GtkDialog *
-Implementation::prefs (Inkscape::Extension::Output * module) {
+Implementation::prefs_output (Inkscape::Extension::Output * module) {
 	return NULL;
-} /* Implementation::prefs */
+} /* Implementation::prefs_output */
 
 void
 Implementation::save (Inkscape::Extension::Output * module, SPDocument * doc, const gchar * filename) {
@@ -58,10 +68,10 @@ Implementation::save (Inkscape::Extension::Output * module, SPDocument * doc, co
 } /* Implementation::save */
 
 GtkDialog *
-Implementation::prefs (Inkscape::Extension::Effect * module) {
+Implementation::prefs_effect (Inkscape::Extension::Effect * module) {
 
 	return NULL;
-} /* Implementation::prefs */
+} /* Implementation::prefs_effect */
 
 void
 Implementation::effect (Inkscape::Extension::Effect * module, SPDocument * document) {
@@ -129,6 +139,28 @@ Implementation::image (Inkscape::Extension::Print * module, unsigned char *px, u
 	return 0;
 }
 
+unsigned int
+Implementation::text (Inkscape::Extension::Print * module, const char *text,
+                      NR::Point p, const SPStyle* style)
+{
+	return 0;
+}
+
+/**
+   \brief  Tell the printing engine whether text should be text or path
+   \retval TRUE  Render the text as a path
+   \retval FALSE Render text using the text function (above)
+ 
+    Default value is FALSE because most printing engines will support
+    paths more than they'll support text.  (at least they do today)
+*/
+bool
+Implementation::textToPath(Inkscape::Extension::Print * ext)
+{
+    return FALSE;
+}
+
+
 }; /* namespace Implementation */
 }; /* namespace Extension */
 }; /* namespace Inkscape */
@@ -142,4 +174,4 @@ Implementation::image (Inkscape::Extension::Print * module, unsigned char *px, u
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
