@@ -12,11 +12,15 @@
 
 #define __INKSCAPE_EXTENSION_IMPLEMENTATION_SCRIPT_C__
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include "path-prefix.h"
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#include <libart_lgpl/art_affine.h>
 #include <svg/svg.h>
 #include <xml/repr-private.h>
 #include <interface.h>
@@ -47,7 +51,7 @@ namespace Implementation {
 /* Real functions */
 /**
 	\return    A script object
-	\breif     This function creates a script object and sets up the
+	\brief     This function creates a script object and sets up the
 	           variables.
 
    This function just sets the command to NULL.  It should get built
@@ -57,6 +61,7 @@ namespace Implementation {
 Script::Script (void) : Implementation::Implementation()
 {
 	command = NULL;
+	helper_extension = NULL;
 	return;
 }
 
@@ -70,7 +75,7 @@ Script::Script (void) : Implementation::Implementation()
 
 	Basically this function looks at an attribute of the Repr, and makes
 	a decision based on that.  Currently, it is only working with the
-	'exetensions' relative directory, but there will be more of them.
+	'extensions' relative directory, but there will be more of them.
 	One thing to notice is that this function always returns an allocated
 	string.  This means that the caller of this function can always
 	free what they are given (and should do it too!).
@@ -86,7 +91,7 @@ Script::solve_reldir (SPRepr * reprin) {
 	}
 
 	if (!strcmp(reldir, "extensions")) {
-		return g_strdup_printf("%s/%s", INKSCAPE_EXTENSIONDIR, sp_repr_content(sp_repr_children(reprin)));
+		return g_build_filename(INKSCAPE_EXTENSIONDIR, sp_repr_content(sp_repr_children(reprin)), NULL);
 	} else {
 		return g_strdup(sp_repr_content(sp_repr_children(reprin)));
 	}
@@ -94,6 +99,88 @@ Script::solve_reldir (SPRepr * reprin) {
 	return NULL;
 }
 
+/**
+	\return   Whether the command given exists, including in the path
+	\brief    This function is used to find out if something exists for
+	          the check command.  It can look in the path if required.
+	\param    command   The command or file that should be looked for
+
+	The first thing that this function does is check to see if the
+	incoming file name has a directory delimiter in it.  This would
+	mean that it wants to control the directories, and should be
+	used directly.
+
+	If not, the path is used.  Each entry in the path is stepped through,
+	attached to the string, and then tested.  If the file is found
+	then a TRUE is returned.  If we get all the way through the path
+	then a FALSE is returned, the command could not be found.
+*/
+bool
+Script::check_existance (const gchar * command)
+{
+	gchar * path;
+	gchar * orig_path;
+
+	if (*command == '\0') {
+		/* We check the simple case first. */
+		return FALSE;
+	}
+
+	if (g_utf8_strchr (command, -1, G_DIR_SEPARATOR) != NULL) {
+		/* Don't search when it contains a slash.  */
+		if (g_file_test(command, G_FILE_TEST_EXISTS))
+			return TRUE;
+		else
+			return FALSE;
+    }
+
+
+	path = g_strdup(g_getenv("PATH"));
+
+	if (path == NULL) {
+		/* There is no `PATH' in the environment.
+		   The default search path is the current directory */
+		path = g_strdup(G_SEARCHPATH_SEPARATOR_S);
+	}
+	orig_path = path;
+
+	for (; path != NULL;) {
+		gchar * local_path;
+		gchar * final_name;
+
+		local_path = path;
+		path = g_utf8_strchr(path, -1, G_SEARCHPATH_SEPARATOR);
+		if (path == NULL) {
+			break;
+		}
+		/* Not sure whether this is UTF8 happy, but it would seem
+		   like it considering that I'm searching (and finding)
+		   the ':' character */
+		if (path != local_path && path != NULL) {
+			path[0] = '\0';
+			path++;
+		} else {
+			path = NULL;
+		}
+
+		if (local_path == '\0') {
+			final_name = g_strdup(command);
+		} else {
+			final_name = g_build_filename(local_path, command, NULL);
+		}
+
+		if (g_file_test(final_name, G_FILE_TEST_EXISTS)) {
+			g_free(final_name);
+			g_free(orig_path);
+			return TRUE;
+		}
+
+		g_free(final_name);
+	}
+
+	return FALSE;
+}
+
 /**
 	\return   none
 	\brief    This function 'loads' an extention, basically it determines
@@ -127,7 +214,7 @@ Script::load (Inkscape::Extension::Extension * module)
 	/* This should probably check to find the executable... */
 	child_repr = sp_repr_children(module->get_repr());
 	while (child_repr != NULL) {
-		if (!strcmp(sp_repr_name(child_repr), "extension")) {
+		if (!strcmp(sp_repr_name(child_repr), "script")) {
 			child_repr = sp_repr_children(child_repr);
 			while (child_repr != NULL) {
 				if (!strcmp(sp_repr_name(child_repr), "command")) {
@@ -149,11 +236,6 @@ Script::load (Inkscape::Extension::Extension * module)
 	g_free(command);
 	command = command_text;
 
-	if (helper_extension != NULL) {
-		g_free(helper_extension);
-		helper_extension = NULL;
-	}
-
 	return TRUE;
 }
 
@@ -173,16 +255,83 @@ Script::unload (Inkscape::Extension::Extension * module)
 	return;
 }
 
+/**
+	\return   Whether the check passed or not
+	\brief    Check every dependency that was given to make sure we should keep this extension
+	\param    module  The Extension in question
+
+*/
+bool
+Script::check (Inkscape::Extension::Extension * module)
+{
+	SPRepr * child_repr;
+	gchar * command_text;
+
+	child_repr = sp_repr_children(module->get_repr());
+	while (child_repr != NULL) {
+		if (!strcmp(sp_repr_name(child_repr), "script")) {
+			child_repr = sp_repr_children(child_repr);
+			while (child_repr != NULL) {
+				if (!strcmp(sp_repr_name(child_repr), "check")) {
+					command_text = solve_reldir(child_repr);
+					
+					if (command_text != NULL) {
+					/* I've got the command */
+						bool existance;
+
+						existance = check_existance(command_text);
+                                                // printf("%s exists %s\n", command_text, existance ? "TRUE" : "FALSE");
+						g_free(command_text);
+						if (!existance)
+							return FALSE;
+					}
+				}
+
+				if (!strcmp(sp_repr_name(child_repr), "helper_extension")) {
+					const gchar * helper;
+
+					helper = sp_repr_content(sp_repr_children(child_repr));
+					if (Inkscape::Extension::db.get(helper) == NULL) {
+						//printf("Couldn't find helper\n");
+						return FALSE;
+					}
+				}
+
+				child_repr = sp_repr_next(child_repr);
+			}
+
+			break;
+		}
+		child_repr = sp_repr_next(child_repr);
+	}
+
+	return TRUE;
+}
+
 /**
 	\return   A dialog for preferences
 	\brief    A stub funtion right now
 	\param    module    Module who's preferences need getting
-	\param    filename  Hey, the file your getting might be important
+	\param    filename  Hey, the file you're getting might be important
 
 	This function should really do something, right now it doesn't.
 */
 GtkDialog *
-Script::prefs (Inkscape::Extension::Input * module, const gchar * filename)
+Script::prefs_input (Inkscape::Extension::Input * module, const gchar * filename)
+{
+	/* Sad, this should really do something... */
+	return NULL;
+}
+
+/**
+	\return   A dialog for preferences
+	\brief    A stub funtion right now
+	\param    module    Module whose preferences need getting
+
+	This function should really do something, right now it doesn't.
+*/
+GtkDialog *
+Script::prefs_output (Inkscape::Extension::Output * module)
 {
 	/* Sad, this should really do something... */
 	return NULL;
@@ -196,21 +345,7 @@ Script::prefs (Inkscape::Extension::Input * module, const gchar * filename)
 	This function should really do something, right now it doesn't.
 */
 GtkDialog *
-Script::prefs (Inkscape::Extension::Output * module)
-{
-	/* Sad, this should really do something... */
-	return NULL;
-}
-
-/**
-	\return   A dialog for preferences
-	\brief    A stub funtion right now
-	\param    module    Module who's preferences need getting
-
-	This function should really do something, right now it doesn't.
-*/
-GtkDialog *
-Script::prefs (Inkscape::Extension::Effect * module)
+Script::prefs_effect (Inkscape::Extension::Effect * module)
 {
 	/* Sad, this should really do something... */
 	return NULL;
@@ -273,9 +408,9 @@ Script::open (Inkscape::Extension::Input * module, const gchar * filename)
 	g_free(local_filename);
 
 	if (helper_extension == NULL) {
-		mydoc = sp_module_system_open(Inkscape::Extension::db.get(SP_MODULE_KEY_INPUT_SVG), tempfilename_out);
+		mydoc = Inkscape::Extension::open(Inkscape::Extension::db.get(SP_MODULE_KEY_INPUT_SVG), tempfilename_out);
 	} else {
-		mydoc = sp_module_system_open(Inkscape::Extension::db.get(helper_extension), tempfilename_out);
+		mydoc = Inkscape::Extension::open(Inkscape::Extension::db.get(helper_extension), tempfilename_out);
 	}
 
 	sp_document_set_uri(mydoc, (const gchar *)filename);
@@ -334,9 +469,9 @@ Script::save (Inkscape::Extension::Output * module, SPDocument * doc, const gcha
 	}
 
 	if (helper_extension == NULL) {
-		sp_module_system_save(Inkscape::Extension::db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE), doc, tempfilename_in, FALSE, FALSE, FALSE);
+		Inkscape::Extension::save(Inkscape::Extension::db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE), doc, tempfilename_in, FALSE, FALSE, FALSE);
 	} else {
-		sp_module_system_save(Inkscape::Extension::db.get(helper_extension), doc, tempfilename_in, FALSE, FALSE, FALSE);
+		Inkscape::Extension::save(Inkscape::Extension::db.get(helper_extension), doc, tempfilename_in, FALSE, FALSE, FALSE);
 	}
 
 	gsize bytesRead = 0;
@@ -429,7 +564,7 @@ Script::effect (Inkscape::Extension::Effect * module, SPDocument * doc)
 		}
 	}
 
-	sp_module_system_save(Inkscape::Extension::db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE), doc, tempfilename_in, FALSE, FALSE, FALSE);
+	Inkscape::Extension::save(Inkscape::Extension::db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE), doc, tempfilename_in, FALSE, FALSE, FALSE);
 
 	/* TODO: I don't think this is the best way to do this, plus,
 	         it needs to handle all of the cases where there is more
@@ -448,7 +583,7 @@ Script::effect (Inkscape::Extension::Effect * module, SPDocument * doc)
 	execute(command, tempfilename_in, tempfilename_out);
 	g_free(command);
 
-	mydoc = sp_module_system_open(Inkscape::Extension::db.get(SP_MODULE_KEY_INPUT_SVG), tempfilename_out);
+	mydoc = Inkscape::Extension::open(Inkscape::Extension::db.get(SP_MODULE_KEY_INPUT_SVG), tempfilename_out);
 
 	unlink(tempfilename_in);
 	unlink(tempfilename_out);
@@ -489,14 +624,14 @@ Script::effect (Inkscape::Extension::Effect * module, SPDocument * doc)
 	opens the output file with the output file handle.  Both of these
 	operations are checked extensively for errors.
 
-	After both are opened, then the data is coppied from the output
+	After both are opened, then the data is copied from the output
 	of the pipe into the file out using fread and fwrite.  These two
 	functions are used because of their primitive nature they make
 	no assumptions about the data.  A buffer is used in the transfer,
 	but the output of fread is stored so the exact number of bytes
 	is handled gracefully.
 
-	At the very end (after the data has been coppied) both of the files
+	At the very end (after the data has been copied) both of the files
 	are closed, and we return to what we were doing.
 */
 void
@@ -528,7 +663,8 @@ Script::execute (const gchar * in_command, const gchar * filein, const gchar * f
 	  return;
 	}
 
-	pfile = fopen(fileout, "w");
+        Inkscape::IO::dump_fopen_call(fileout, "J");
+	pfile = Inkscape::IO::fopen_utf8name(fileout, "w");
 
 	if (pfile == NULL) {
 	  /* Error - could not open file */
