@@ -10,14 +10,27 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include "ps-out.h"
 #include <print.h>
 #include <extension/system.h>
+#include <extension/db.h>
 
 namespace Inkscape {
 namespace Extension {
 namespace Internal {
 
+bool
+PsOutput::check (Inkscape::Extension::Extension * module)
+{
+	if (NULL == Inkscape::Extension::db.get(SP_MODULE_KEY_PRINT_PS))
+		return FALSE;
+
+	return TRUE;
+}
+
 /**
     \brief  This function calls the print system with the filename
 	\param  mod   unused
@@ -48,17 +61,17 @@ PsOutput::save (Inkscape::Extension::Output *mod, SPDocument *doc, const gchar *
 void
 PsOutput::init (void)
 {
-    sp_module_system_build_from_mem(
-		"<spmodule>\n"
+	Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>Postscript Output</name>\n"
-			"<id>module.output.ps</id>\n"
+			"<id>org.inkscape.output.ps</id>\n"
 			"<output>\n"
 				"<extension>.ps</extension>\n"
 				"<mimetype>image/x-postscript</mimetype>\n"
 				"<filetypename>Postscript (*.ps)</filetypename>\n"
 				"<filetypetooltip>Postscript File</filetypetooltip>\n"
 			"</output>\n"
-		"</spmodule>", new PsOutput());
+		"</inkscape-extension>", new PsOutput());
 
 	return;
 }
