@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <math.h>
 
+#include <libnr/n-art-bpath.h>
 #include <libnr/nr-macros.h>
 #include <libnr/nr-matrix.h>
 
@@ -47,25 +48,34 @@
 
 #include "helper/sp-intl.h"
 #include "display/nr-arena-item.h"
-#include "helper/canvas-bpath.h"
+#include "display/canvas-bpath.h"
 #include "enums.h"
 #include "document.h"
 #include "inkscape.h"
+#include "sp-item.h"
 #include "style.h"
+#include "inkscape_version.h"
+
+#include <libnrtype/FontFactory.h>
+#include <libnrtype/font-instance.h>
+#include <libnrtype/font-style-to-pos.h>
 
 #include "ps.h"
 #include <extension/system.h>
+#include <extension/extension.h>
+#include <extension/print.h>
 
+#include "file.h"
 
 namespace Inkscape {
 namespace Extension {
 namespace Internal {
 
-PrintPS::PrintPS (void)
+PrintPS::PrintPS() :
+	_stream(NULL),
+ 	_dpi(72),
+	_bitmap(false)
 {
-	_dpi = 72;
-
-	return;
 }
 
 PrintPS::~PrintPS (void)
@@ -127,13 +137,12 @@ PrintPS::setup (Inkscape::Extension::Print * mod)
 	gtk_container_add (GTK_CONTAINER (f), vb);
 	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
 	/* Print type */
-	p2bm = FALSE;
-	mod->get_param("bitmap", &p2bm);
+	p2bm = mod->get_param_bool("bitmap");
 	rb = gtk_radio_button_new_with_label (NULL, _("Print using PostScript operators"));
 	gtk_tooltips_set_tip ((GtkTooltips *) tt, rb,
 						  _("Use PostScript vector operators. The resulting image is usually smaller "
 						    "in file size and can be arbitrarily scaled, but alpha transparency, "
-							"gradients, markers, and patterns will be lost."), NULL);
+							"gradients and patterns will be lost."), NULL);
 	if (!p2bm) gtk_toggle_button_set_active ((GtkToggleButton *) rb, TRUE);
 	gtk_box_pack_start (GTK_BOX (vb), rb, FALSE, FALSE, 0);
 	rb = gtk_radio_button_new_with_label (gtk_radio_button_get_group ((GtkRadioButton *) rb), _("Print as bitmap"));
@@ -162,8 +171,8 @@ PrintPS::setup (Inkscape::Extension::Print * mod)
 	gtk_combo_set_popdown_strings (GTK_COMBO (combo), sl);
 	g_list_free (sl);
 	if (1) {
-		gchar * val = NULL;
-		mod->get_param("resolution", &val);
+		const gchar * val = NULL;
+		val = mod->get_param_string("resolution");
 		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), val);
 	}
 	gtk_box_pack_end (GTK_BOX (hb), combo, FALSE, FALSE, 0);
@@ -183,8 +192,8 @@ PrintPS::setup (Inkscape::Extension::Print * mod)
 
 	e = gtk_entry_new ();
 	if (1) {
-		gchar *val;
-		mod->get_param("destination", &val);
+		const gchar *val;
+		val = mod->get_param_string("destination");
 		gtk_entry_set_text (GTK_ENTRY (e), val);
 	}
 	gtk_box_pack_start (GTK_BOX (vb), e, FALSE, FALSE, 0);
@@ -209,9 +218,9 @@ PrintPS::setup (Inkscape::Extension::Print * mod)
 		fn = gtk_entry_get_text (GTK_ENTRY (e));
 		/* g_print ("Printing to %s\n", fn); */
 
-		mod->set_param("bitmap", (_bitmap) ? (bool)TRUE : (bool)FALSE);
-		mod->set_param("resolution", (gchar *)sstr);
-		mod->set_param("destination", (gchar *)fn);
+		mod->set_param_bool("bitmap", _bitmap);
+		mod->set_param_string("resolution", (gchar *)sstr);
+		mod->set_param_string("destination", (gchar *)fn);
 		ret = TRUE;
 	}
 
@@ -223,11 +232,12 @@ PrintPS::setup (Inkscape::Extension::Print * mod)
 unsigned int
 PrintPS::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 {
+	Inkscape::SVGOStringStream os;
 	int res;
 	FILE *osf, *osp;
-	gchar * fn;
+	const gchar * fn;
 
-	mod->get_param("destination", (gchar **)&fn);
+	fn = mod->get_param_string("destination");
 
 	osf = NULL;
 	osp = NULL;
@@ -239,6 +249,11 @@ PrintPS::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
                                  -1,  &bytesRead,  &bytesWritten, &error);
 	fn = local_fn;
 
+	/* TODO: Replace the below fprintf's with something that does the right thing whether in
+	 * gui or batch mode (e.g. --print=blah).  Consider throwing an exception: currently one of
+	 * the callers (sp_print_document_to_file, "ret = mod->begin(doc)") wrongly ignores the
+	 * return code.
+	 */
 	if (fn != NULL) {
 		if (*fn == '|') {
 			fn += 1;
@@ -248,21 +263,38 @@ PrintPS::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 #else
 			osp = _popen (fn, "w");
 #endif
+			if (!osp) {
+				fprintf(stderr, "inkscape: popen(%s): %s\n",
+					fn, strerror(errno));
+				return 0;
+			}
 			_stream = osp;
 		} else if (*fn == '>') {
 			fn += 1;
 			while (isspace (*fn)) fn += 1;
-			osf = fopen (fn, "w+");
+			Inkscape::IO::dump_fopen_call(fn, "K");
+			osf = Inkscape::IO::fopen_utf8name(fn, "w+");
+			if (!osf) {
+				fprintf(stderr, "inkscape: fopen(%s): %s\n",
+					fn, strerror(errno));
+				return 0;
+			}
 			_stream = osf;
 		} else {
 			gchar *qn;
 			/* put cwd stuff in here */
+			/* FIXME: quote fn */
 			qn = g_strdup_printf ("lpr -P %s", fn);
 #ifndef WIN32
 			osp = popen (qn, "w");
 #else
 			osp = _popen (qn, "w");
 #endif
+			if (!osp) {
+				fprintf(stderr, "inkscape: popen(%s): %s\n",
+					qn, strerror(errno));
+				return 0;
+			}
 			g_free (qn);
 			_stream = osp;
 		}
@@ -275,7 +307,7 @@ PrintPS::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 #if !defined(_WIN32) && !defined(__WIN32__)
 		(void) signal(SIGPIPE, SIG_IGN);
 #endif
-	}
+	}	
 
 	res = fprintf (_stream, "%%!PS-Adobe-2.0\n");
 	/* flush this to test output stream as early as possible */
@@ -295,21 +327,88 @@ PrintPS::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 	_width = sp_document_width (doc);
 	_height = sp_document_height (doc);
 
-	if (_bitmap) return 0;
+	NRRect d;
+	bool   pageBoundingBox;
+	bool   pageLandscape;
+	pageBoundingBox = mod->get_param_bool("pageBoundingBox");
+	// printf("Page Bounding Box: %s\n", pageBoundingBox ? "TRUE" : "FALSE");
+	if (pageBoundingBox)
+	{
+		d.x0 = d.y0 = 0;
+		d.x1 = ceil (sp_document_width (doc));
+		d.y1 = ceil (sp_document_height (doc));
+	}
+	else
+	{
+		SPItem* doc_item = SP_ITEM (sp_document_root (doc));
+		sp_item_invoke_bbox(doc_item, &d, sp_item_i2r_affine(doc_item), TRUE);
+	}
 
-	if (res >= 0) res = fprintf (_stream, "%%%%BoundingBox: %i %i %i %i\n",
-															 0, 0,
-															 (int) ceil (sp_document_width (doc)),
-															 (int) ceil (sp_document_height (doc)));
-	if (res >= 0) res = fprintf (_stream, "%%%%HiResBoundingBox: %g %g %g %g\n",
-															 0.0, 0.0,
-															 sp_document_width (doc),
-															 sp_document_height (doc));
+	if (res >= 0) {
 
-	if (res >= 0) res = fprintf (_stream, "%g %g translate\n", 0.0, sp_document_height (doc));
-	if (res >= 0) res = fprintf (_stream, "0.8 -0.8 scale\n");
+		os << "%%Creator: " << PACKAGE_STRING << "\n";
+		// This will become problematic if inkscape gains the
+		// ability to handle multi paged documents. If this is
+		// the case the %%Orientation: comments should be
+		// renamed to %%PageOrientation: and moved to the
+		// respective pages.
+		os << "%%Pages: 1\n";
 
-	return res;
+		pageLandscape = (d.x1 - d.x0 > d.y1 - d.y0) ? true : false;
+
+		if (pageLandscape) {
+			os << "%%Orientation: Landscape\n";
+			os << "%%BoundingBox: " << (int) (_height - d.y1) << " "
+						<< (int) d.x0 << " "
+						<< (int) ceil (_height - d.y0) << " "
+						<< (int) ceil (d.x1) << "\n";
+			// According to Mike Sweet (the author of CUPS)
+			// HiResBoundingBox is only appropriate
+			// for EPS files. This means that we should
+			// distinguish if we export to ps or eps here.
+			// FIXME: I couldn't find HiResBoundingBox in the PS
+			// reference manual, so I guess we should skip
+			// it.
+			os << "%%HiResBoundingBox: " << (_height - d.y1) << " "
+						<< d.x0 << " "
+						<< (_height - d.y0) << " "
+						<< d.x1 << "\n";
+		}
+		else
+		{
+			os << "%%Orientation: Portrait\n";
+			os << "%%BoundingBox: " << (int) d.x0 << " "
+						<< (int) d.y0 << " "
+						<< (int) ceil (d.x1) << " "
+						<< (int) ceil (d.y1) << "\n";
+			os << "%%HiResBoundingBox: " << d.x0 << " "
+						<< d.y0 << " "
+						<< d.x1 << " "
+						<< d.y1 << "\n";
+		}
+		
+		os << "%%EndComments\n";
+		// This will become problematic if we print
+		// multi paged documents:
+		os << "%%Page: 1 1\n";
+		
+		if (pageLandscape) {
+			os << 90 << " rotate\n";
+			if (_bitmap) {
+				os << "0 " << (int) -ceil (_height) << " translate\n";
+			}
+		} else 
+			if (!_bitmap) {
+				os << "0 " << (int) ceil (_height) << " translate\n";
+			}
+		
+		if (!_bitmap) {
+			os << "0.8 -0.8 scale\n";
+		}
+
+	}
+
+	return fprintf (_stream, "%s", os.str().c_str());
 }
 
 unsigned int
@@ -350,7 +449,7 @@ PrintPS::finish (Inkscape::Extension::Print *mod)
 
 		for (y = 0; y < height; y += 64) {
 			NRRectL bbox;
-			NRGC gc;
+			NRGC gc(NULL);
 			NRMatrix imgt;
 			NRPixBlock pb;
 			/* Set area of interest */
@@ -400,10 +499,15 @@ PrintPS::bind (Inkscape::Extension::Print *mod, const NRMatrix *transform, float
 	if (!_stream) return 0;  // XXX: fixme, returning -1 as unsigned.
 	if (_bitmap) return 0;
 
-	return fprintf (_stream, "gsave [%g %g %g %g %g %g] concat\n",
-			transform->c[0], transform->c[1],
-			transform->c[2], transform->c[3],
-			transform->c[4], transform->c[5]);
+        Inkscape::SVGOStringStream os;
+	os << "gsave [" << transform->c[0] << " "
+			<< transform->c[1] << " "
+			<< transform->c[2] << " "
+			<< transform->c[3] << " "
+			<< transform->c[4] << " "
+			<< transform->c[5] << "] concat\n";
+
+	return fprintf (_stream, "%s", os.str().c_str());
 }
 
 unsigned int
@@ -415,6 +519,45 @@ PrintPS::release (Inkscape::Extension::Print *mod)
 	return fprintf (_stream, "grestore\n");
 }
 
+void
+PrintPS::print_fill_style (SVGOStringStream &os, const SPStyle *style)
+{
+		float rgb[3];
+		sp_color_get_rgb_floatv (&style->fill.value.color, rgb);
+
+		os << rgb[0] << " " << rgb[1] << " " << rgb[2] << " setrgbcolor\n";
+}
+
+void
+PrintPS::print_stroke_style (SVGOStringStream &os, const SPStyle *style)
+{
+	float rgb[3];
+	sp_color_get_rgb_floatv (&style->stroke.value.color, rgb);
+
+	os << rgb[0] << " " << rgb[1] << " " << rgb[2] << " setrgbcolor\n";
+
+	if (style->stroke_dasharray_set) {
+		if (style->stroke_dash.n_dash && style->stroke_dash.dash) {
+			int i;
+			os << "[";
+			for (i = 0; i < style->stroke_dash.n_dash; i++) {
+				if ((i)) {
+					os << " ";
+				}
+				os << style->stroke_dash.dash[i];
+			}
+			os << "] " << style->stroke_dash.offset << " setdash\n";
+		}
+	} else {
+		os << "[] 0 setdash\n";
+	}
+
+	os << style->stroke_width.computed << " setlinewidth\n";
+	os << style->stroke_linejoin.computed << " setlinejoin\n";
+	os << style->stroke_linecap.computed << " setlinecap\n";
+}
+
+
 unsigned int
 PrintPS::fill (Inkscape::Extension::Print *mod, const NRBPath *bpath, const NRMatrix *ctm, const SPStyle *style,
 			    const NRRect *pbox, const NRRect *dbox, const NRRect *bbox)
@@ -423,24 +566,24 @@ PrintPS::fill (Inkscape::Extension::Print *mod, const NRBPath *bpath, const NRMa
 	if (_bitmap) return 0;
 
 	if (style->fill.type == SP_PAINT_TYPE_COLOR) {
-		float rgb[3];
+        	Inkscape::SVGOStringStream os;
 
-		sp_color_get_rgb_floatv (&style->fill.value.color, rgb);
+		print_fill_style (os, style);
 
-		fprintf (_stream, "%g %g %g setrgbcolor\n", rgb[0], rgb[1], rgb[2]);
-
-		print_bpath (_stream, bpath->path);
+		print_bpath (os, bpath->path);
 
 		if (style->fill_rule.value == SP_WIND_RULE_EVENODD) {
-			fprintf (_stream, "eofill\n");
+			os << "eofill\n";
 		} else {
-			fprintf (_stream, "fill\n");
+			os << "fill\n";
 		}
+		fprintf (_stream, "%s", os.str().c_str());
 	}
 
 	return 0;
 }
 
+
 unsigned int
 PrintPS::stroke (Inkscape::Extension::Print *mod, const NRBPath *bpath, const NRMatrix *ctm, const SPStyle *style,
 			      const NRRect *pbox, const NRRect *dbox, const NRRect *bbox)
@@ -449,30 +592,15 @@ PrintPS::stroke (Inkscape::Extension::Print *mod, const NRBPath *bpath, const NR
 	if (_bitmap) return 0;
 
 	if (style->stroke.type == SP_PAINT_TYPE_COLOR) {
-		float rgb[3];
+        	Inkscape::SVGOStringStream os;
 
-		sp_color_get_rgb_floatv (&style->stroke.value.color, rgb);
+		print_stroke_style (os, style);
 
-		fprintf (_stream, "%g %g %g setrgbcolor\n", rgb[0], rgb[1], rgb[2]);
+		print_bpath (os, bpath->path);
 
-		print_bpath (_stream, bpath->path);
+		os << "stroke\n";
 
-		if (style->stroke_dash.n_dash > 0) {
-			int i;
-			fprintf (_stream, "[");
-			for (i = 0; i < style->stroke_dash.n_dash; i++) {
-				fprintf (_stream, (i) ? " %g" : "%g", style->stroke_dash.dash[i]);
-			}
-			fprintf (_stream, "] %g setdash\n", style->stroke_dash.offset);
-		} else {
-			fprintf (_stream, "[] 0 setdash\n");
-		}
-
-		fprintf (_stream, "%g setlinewidth\n", style->stroke_width.computed);
-		fprintf (_stream, "%d setlinejoin\n", style->stroke_linejoin.computed);
-		fprintf (_stream, "%d setlinecap\n", style->stroke_linecap.computed);
-
-		fprintf (_stream, "stroke\n");
+		fprintf (_stream, "%s", os.str().c_str());
 	}
 
 	return 0;
@@ -500,9 +628,9 @@ PrintPS::image (Inkscape::Extension::Print *mod, guchar *px, unsigned int w, uns
 	fprintf (_stream, "{currentfile rowdata readhexstring pop}\n");
 	fprintf (_stream, "false 3 colorimage\n");
 
-	for (r = 0; r < h; r++) {
+	for (unsigned int r = 0; r < h; r++) {
 		guchar *s;
-		int c0, c1, c;
+		unsigned int c0, c1, c;
 		s = px + r * rs;
 		for (c0 = 0; c0 < w; c0 += 24) {
 			c1 = MIN (w, c0 + 24);
@@ -526,36 +654,119 @@ PrintPS::image (Inkscape::Extension::Print *mod, guchar *px, unsigned int w, uns
 #endif
 }
 
+const char *
+PrintPS::PSFontName (const SPStyle *style)
+{
+	font_instance *tf = (font_factory::Default())->Face(style->text->font_family.value, font_style_to_pos(*style));
+
+	char const *n;
+	char name_buf[256];
+
+	if (tf) {
+		tf->PSName(name_buf, sizeof(name_buf));
+		n = name_buf;
+		tf->Unref();
+	} else { 
+		// this system does not have this font, so just use the name from SVG in the hope that PS interpreter will make sense of it
+		bool i = (style->font_style.value == SP_CSS_FONT_STYLE_ITALIC);
+		bool o = (style->font_style.value == SP_CSS_FONT_STYLE_OBLIQUE);
+		bool b = (style->font_weight.value == SP_CSS_FONT_WEIGHT_BOLD) || 
+			(style->font_weight.value >= SP_CSS_FONT_WEIGHT_500 && style->font_weight.value <= SP_CSS_FONT_WEIGHT_900);
+
+		n = g_strdup_printf ("%s%s%s%s", 
+								 g_strdelimit (style->text->font_family.value, " ", '-'),
+								 (b || i || o) ? "-" : "", 
+								 (b) ? "Bold" : "", 
+								 (i) ? "Italic" : ((o) ? "Oblique" : "") ); 
+	}
+
+	return g_strdup (n);
+}
+
+
+unsigned int
+PrintPS::text (Inkscape::Extension::Print *mod, const char *text, NR::Point p,
+	       const SPStyle* style)
+{
+  if (!_stream) return 0; // XXX: fixme, returning -1 as unsigned.
+  if (_bitmap) return 0;
+
+  Inkscape::SVGOStringStream os;
+
+  // Escape chars
+  // FlowResOut feeds us text char by char
+  if (!strcmp (text, "(")) {
+	text = "\\(";
+  } else if (!strcmp (text, ")")) {
+	text = "\\)";
+  } else if (!strcmp (text, "\\")) {
+	text = "\\\\";
+  }
+
+  // set font
+  const char *fn = PSFontName (style);
+  os << "/" << fn << " findfont\n";
+  os << style->font_size.computed << " scalefont\n";
+  os << "setfont\n";
+  g_free ((void *) fn);
+
+  if (style->fill.type == SP_PAINT_TYPE_COLOR) {
+	// set fill style
+	print_fill_style (os, style);
+	// paint fill
+	os << "newpath\n";
+	os << p[NR::X] << " " << p[NR::Y] << " moveto\n";
+	os << "(" << text << ") show\n";
+  }
+
+  if (style->stroke.type == SP_PAINT_TYPE_COLOR) {
+ 	// set stroke style
+	print_stroke_style (os, style);
+	// paint stroke
+	os << "newpath\n";
+	os << p[NR::X] << " " << p[NR::Y] << " moveto\n";
+	os << "(" << text << ") false charpath stroke\n";
+  }
+
+  fprintf (_stream, "%s", os.str().c_str());
+
+  return 0;
+}
+	
+
+
 /* PostScript helpers */
 
 void
-PrintPS::print_bpath (FILE *stream, const ArtBpath *bp)
+PrintPS::print_bpath (SVGOStringStream &os, const NArtBpath *bp)
 {
 	unsigned int closed;
 
-	fprintf (stream, "newpath\n");
+	os << "newpath\n";
 	closed = FALSE;
-	while (bp->code != ART_END) {
+	while (bp->code != NR_END) {
 		switch (bp->code) {
-		case ART_MOVETO:
+		case NR_MOVETO:
 			if (closed) {
-				fprintf (stream, "closepath\n");
+				os << "closepath\n";
 			}
 			closed = TRUE;
-			fprintf (stream, "%g %g moveto\n", bp->x3, bp->y3);
+			os << bp->x3 << " " << bp->y3 << " moveto\n";
 			break;
-		case ART_MOVETO_OPEN:
+		case NR_MOVETO_OPEN:
 			if (closed) {
-				fprintf (stream, "closepath\n");
+				os << "closepath\n";
 			}
 			closed = FALSE;
-			fprintf (stream, "%g %g moveto\n", bp->x3, bp->y3);
+			os << bp->x3 << " " << bp->y3 << " moveto\n";
 			break;
-		case ART_LINETO:
-			fprintf (stream, "%g %g lineto\n", bp->x3, bp->y3);
+		case NR_LINETO:
+			os << bp->x3 << " " << bp->y3 << " lineto\n";
 			break;
-		case ART_CURVETO:
-			fprintf (stream, "%g %g %g %g %g %g curveto\n", bp->x1, bp->y1, bp->x2, bp->y2, bp->x3, bp->y3);
+		case NR_CURVETO:
+			os << bp->x1 << " " << bp->y1 << " "
+			   << bp->x2 << " " << bp->y2 << " "
+			   << bp->x3 << " " << bp->y3 << " curveto\n";
 			break;
 		default:
 			break;
@@ -563,23 +774,33 @@ PrintPS::print_bpath (FILE *stream, const ArtBpath *bp)
 		bp += 1;
 	}
 	if (closed) {
-		fprintf (stream, "closepath\n");
+		os << "closepath\n";
 	}
 }
 
-/* The following code is licensed under GNU GPL */
+/* The following code is licensed under GNU GPL.
+** The packbits, ascii85 and imaging printing code
+** is from the gimp's postscript.c.
+*/
 
+/**
+* \param nin Number of bytes of source data.
+* \param src Source data.
+* \param nout Number of output bytes.
+* \param dst Buffer for output.
+*/
 void
 PrintPS::compress_packbits (int nin,
                             guchar *src,
                             int *nout,
                             guchar *dst)
 
-{register gchar c;
- int nrepeat, nliteral;
- guchar *run_start;
- guchar *start_dst = dst;
- guchar *last_literal = NULL;
+{
+  register guchar c;
+  int nrepeat, nliteral;
+  guchar *run_start;
+  guchar *start_dst = dst;
+  guchar *last_literal = NULL;
 
  for (;;)
  {
@@ -659,7 +880,7 @@ PrintPS::ascii85_init (void)
 }
 
 void
-PrintPS::ascii85_flush (FILE *ofp)
+PrintPS::ascii85_flush (SVGOStringStream &os)
 {
   char c[5];
   int i;
@@ -677,10 +898,10 @@ PrintPS::ascii85_flush (FILE *ofp)
     {
       if (ascii85_linewidth >= max_linewidth)
       {
-        putc ('\n', ofp);
+	os << '\n';
         ascii85_linewidth = 0;
       }
-      putc ('z', ofp);
+      os << 'z';
       ascii85_linewidth++;
     }
   else
@@ -689,10 +910,10 @@ PrintPS::ascii85_flush (FILE *ofp)
       {
         if ((ascii85_linewidth >= max_linewidth) && (c[i] != '%'))
         {
-          putc ('\n', ofp);
+	  os << '\n';
           ascii85_linewidth = 0;
         }
-	putc (c[i], ofp);
+	os << c[i];
         ascii85_linewidth++;
       }
     }
@@ -702,10 +923,10 @@ PrintPS::ascii85_flush (FILE *ofp)
 }
 
 inline void
-PrintPS::ascii85_out (gchar byte, FILE *ofp)
+PrintPS::ascii85_out (guchar byte, SVGOStringStream &os)
 {
   if (ascii85_len == 4)
-    ascii85_flush (ofp);
+    ascii85_flush (os);
 
   ascii85_buf <<= 8;
   ascii85_buf |= byte;
@@ -713,28 +934,26 @@ PrintPS::ascii85_out (gchar byte, FILE *ofp)
 }
 
 void
-PrintPS::ascii85_nout (int n, gchar *uptr, FILE *ofp)
+PrintPS::ascii85_nout (int n, guchar *uptr, SVGOStringStream &os)
 {
  while (n-- > 0)
  {
-   ascii85_out (*uptr, ofp);
+   ascii85_out (*uptr, os);
    uptr++;
  }
 }
 
 void
-PrintPS::ascii85_done (FILE *ofp)
+PrintPS::ascii85_done (SVGOStringStream &os)
 {
   if (ascii85_len)
     {
       /* zero any unfilled buffer portion, then flush */
       ascii85_buf <<= (8 * (4-ascii85_len));
-      ascii85_flush (ofp);
+      ascii85_flush (os);
     }
 
-  putc ('~', ofp);
-  putc ('>', ofp);
-  putc ('\n', ofp);
+  os << "~>\n";
 }
 
 unsigned int
@@ -744,33 +963,35 @@ PrintPS::print_image (FILE *ofp, guchar *px, unsigned int width, unsigned int he
 	unsigned int i, j;
 	/* gchar *data, *src; */
 	guchar *packb = NULL, *plane = NULL;
+        Inkscape::SVGOStringStream os;
+
+	os << "gsave\n";
+	os << "[" << transform->c[0] << " "
+		<< transform->c[1] << " "
+		<< transform->c[2] << " "
+		<< transform->c[3] << " "
+		<< transform->c[4] << " "
+		<< transform->c[5] << "] concat\n";
+	os << width << " " << height << " 8 ["
+		<< width << " 0 0 -" << height << " 0 " << height << "]\n";
 
-	fprintf (ofp, "gsave\n");
-	fprintf (ofp, "[%g %g %g %g %g %g] concat\n",
-		 transform->c[0],
-		 transform->c[1],
-		 transform->c[2],
-		 transform->c[3],
-		 transform->c[4],
-		 transform->c[5]);
-	fprintf (ofp, "%d %d 8 [%d 0 0 -%d 0 %d]\n", width, height, width, height, height);
 
 	/* Write read image procedure */
-	fprintf (ofp, "%% Strings to hold RGB-samples per scanline\n");
-	fprintf (ofp, "/rstr %d string def\n", width);
-	fprintf (ofp, "/gstr %d string def\n", width);
-	fprintf (ofp, "/bstr %d string def\n", width);
-	fprintf (ofp, "{currentfile /ASCII85Decode filter /RunLengthDecode filter rstr readstring pop}\n");
-	fprintf (ofp, "{currentfile /ASCII85Decode filter /RunLengthDecode filter gstr readstring pop}\n");
-	fprintf (ofp, "{currentfile /ASCII85Decode filter /RunLengthDecode filter bstr readstring pop}\n");
-	fprintf (ofp, "true 3\n");
+	os << "% Strings to hold RGB-samples per scanline\n";
+	os << "/rstr " << width << " string def\n";
+	os << "/gstr " << width << " string def\n";
+	os << "/bstr " << width << " string def\n";
+	os << "{currentfile /ASCII85Decode filter /RunLengthDecode filter rstr readstring pop}\n";
+	os << "{currentfile /ASCII85Decode filter /RunLengthDecode filter gstr readstring pop}\n";
+	os << "{currentfile /ASCII85Decode filter /RunLengthDecode filter bstr readstring pop}\n";
+	os << "true 3\n";
 
 	/* Allocate buffer for packbits data. Worst case: Less than 1% increase */
 	packb = (guchar *)g_malloc ((width * 105)/100+2);
 	plane = (guchar *)g_malloc (width);
 
 	/* ps_begin_data (ofp); */
-	fprintf (ofp, "colorimage\n");
+	os << "colorimage\n";
 
 #define GET_RGB_TILE(begin) \
   {int scan_lines; \
@@ -795,10 +1016,11 @@ PrintPS::print_image (FILE *ofp, guchar *px, unsigned int width, unsigned int he
 				src_ptr += 4;
 			}
 			compress_packbits (width, plane, &nout, packb);
+
 			ascii85_init ();
-			ascii85_nout (nout, (gchar*)packb, ofp);
-			ascii85_out (128, ofp); /* Write EOD of RunLengthDecode filter */
-			ascii85_done (ofp);
+			ascii85_nout (nout, packb, os);
+			ascii85_out (128, os); /* Write EOD of RunLengthDecode filter */
+			ascii85_done (os);
 		}
 	}
 	/* ps_end_data (ofp); */
@@ -819,27 +1041,37 @@ PrintPS::print_image (FILE *ofp, guchar *px, unsigned int width, unsigned int he
 	}
 #endif
 
-	fprintf (ofp, "grestore\n");
+	os << "grestore\n";
+
+	fprintf (ofp, "%s", os.str().c_str());
 
 	return 0;
 #undef GET_RGB_TILE
 }
 
+bool
+PrintPS::textToPath (Inkscape::Extension::Print * ext)
+{
+	return ext->get_param_bool("textToPath");
+}
+
 void
 PrintPS::init (void)
 {
 	Inkscape::Extension::Extension * ext;
 	
 	/* SVG in */
-    ext = sp_module_system_build_from_mem(
-		"<spmodule>\n"
+    ext = Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>Postscript Print</name>\n"
 			"<id>" SP_MODULE_KEY_PRINT_PS "</id>\n"
 			"<param name=\"bitmap\" type=\"boolean\">FALSE</param>\n"
 			"<param name=\"resolution\" type=\"string\">72</param>\n"
 			"<param name=\"destination\" type=\"string\">lp</param>\n"
+			"<param name=\"pageBoundingBox\" type=\"boolean\">TRUE</param>\n"
+			"<param name=\"textToPath\" type=\"boolean\">TRUE</param>\n"
 			"<print/>\n"
-		"</spmodule>", new PrintPS());
+		"</inkscape-extension>", new PrintPS());
 
 	return;
 }
