@@ -12,6 +12,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <string.h>
 #include <helper/sp-intl.h>
 #include <xml/repr.h>
@@ -21,6 +24,7 @@
 #include "../implementation/implementation.h"
 #include "svg.h"
 #include <extension/system.h>
+#include <extension/output.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -31,7 +35,7 @@ namespace Internal {
 	\brief    What would an SVG editor be without loading/saving SVG
 	          files.  This function sets that up.
 
-	For each module there is a call to sp_module_system_build_from_mem
+	For each module there is a call to Inkscape::Extension::build_from_mem
 	with a rather large XML file passed in.  This is a constant string
 	that describes the module.  At the end of this call a module is
 	returned that is basically filled out.  The one thing that it doesn't
@@ -44,22 +48,22 @@ Svg::init(void)
 	Inkscape::Extension::Extension * ext;
 	
 	/* SVG in */
-    ext = sp_module_system_build_from_mem(
-		"<spmodule>\n"
+    ext = Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>SVG Input</name>\n"
 			"<id>" SP_MODULE_KEY_INPUT_SVG "</id>\n"
 			"<input>\n"
 				"<extension>.svg</extension>\n"
 				"<mimetype>image/x-svg</mimetype>\n"
-				"<filetypename>Scalable Vector Graphic (SVG)</filetypename>\n"
+				"<filetypename>Scalable Vector Graphic (*.svg)</filetypename>\n"
 				"<filetypetooltip>Inkscape native file format and W3C standard</filetypetooltip>\n"
 				"<output_extension>" SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE "</output_extension>\n"
 			"</input>\n"
-		"</spmodule>", new Svg());
+		"</inkscape-extension>", new Svg());
 
 	/* SVG out Inkscape */
-    ext = sp_module_system_build_from_mem(
-		"<spmodule>\n"
+    ext = Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>SVG Output Inkscape</name>\n"
 			"<id>" SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE "</id>\n"
 			"<output>\n"
@@ -69,11 +73,11 @@ Svg::init(void)
 				"<filetypetooltip>SVG format with Inkscape extensions</filetypetooltip>\n"
 				"<dataloss>FALSE</dataloss>\n"
 			"</output>\n"
-		"</spmodule>", new Svg());
+		"</inkscape-extension>", new Svg());
 
 	/* SVG out */
-    ext = sp_module_system_build_from_mem(
-		"<spmodule>\n"
+    ext = Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>SVG Output</name>\n"
 			"<id>" SP_MODULE_KEY_OUTPUT_SVG "</id>\n"
 			"<output>\n"
@@ -82,7 +86,7 @@ Svg::init(void)
 				"<filetypename>Plain SVG (*.svg)</filetypename>\n"
 				"<filetypetooltip>Scalable Vector Graphics format as defined by the W3C</filetypetooltip>\n"
 			"</output>\n"
-		"</spmodule>", new Svg());
+		"</inkscape-extension>", new Svg());
 
 	return;
 }
@@ -129,45 +133,57 @@ Svg::open (Inkscape::Extension::Input *mod, const gchar *uri)
 void
 Svg::save (Inkscape::Extension::Output *mod, SPDocument *doc, const gchar *uri)
 {
-	SPRepr *repr;
-	gboolean spns;
-	const GSList *images, *l;
-	SPReprDoc *rdoc;
-	const gchar *save_path;
-
 	g_return_if_fail(doc != NULL);
 	g_return_if_fail(uri != NULL);
 
-	save_path = g_dirname (uri);
+	gchar const *save_path = g_dirname (uri);
 
-	spns = (!mod->get_id() || !strcmp (mod->get_id(), SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE));
+	gboolean const spns =
+	  (!mod->get_id() || !strcmp (mod->get_id(), SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE));
+
+	SPReprDoc *rdoc = NULL;
+	SPRepr *repr = NULL;
 	if (spns) {
-		rdoc = NULL;
 		repr = sp_document_repr_root (doc);
 	} else {
 		rdoc = sp_repr_document_new ("svg");
 		repr = sp_repr_document_root (rdoc);
-		repr = sp_object_invoke_write (sp_document_root (doc), repr, SP_OBJECT_WRITE_BUILD);
+		repr = sp_document_root (doc)->updateRepr(repr, SP_OBJECT_WRITE_BUILD);
 	}
 
-	images = sp_document_get_resource_list (doc, "image");
-	for (l = images; l != NULL; l = l->next) {
-		SPRepr *ir;
-		const gchar *href, *relname;
-		ir = SP_OBJECT_REPR (l->data);
-		href = sp_repr_attr (ir, "xlink:href");
+	GSList const *images = sp_document_get_resource_list (doc, "image");
+	for (GSList const *l = images; l != NULL; l = l->next) {
+		SPRepr *ir = SP_OBJECT_REPR (l->data);
+
+		// First try to figure out an absolute path to the asset
+		const gchar *href = sp_repr_attr (ir, "xlink:href");
 		if (spns && !g_path_is_absolute (href)) {
-			href = sp_repr_attr (ir, "sodipodi:absref");
+			const gchar *absref = sp_repr_attr (ir, "sodipodi:absref");
+
+			if ( absref && g_file_test(absref, G_FILE_TEST_EXISTS) )
+			{
+				// only switch over if the absref is still valid
+				href = absref;
+			}
 		}
+
+		// Once we have an absolute path, convert it relative to the new location
 		if (href && g_path_is_absolute (href)) {
-			relname = sp_relative_path_from_path (href, save_path);
+			const gchar *relname = sp_relative_path_from_path (href, save_path);
 			sp_repr_set_attr (ir, "xlink:href", relname);
 		}
+// TODO next refinement is to make the first choice keeping the relative path as-is if
+//      based on the new save location it gives us a valid file.
 	}
 
-	sp_repr_save_file (sp_repr_document (repr), uri);
+	gboolean const s = sp_repr_save_file (sp_repr_document (repr), uri);
+	if (s == FALSE) {
+		throw Inkscape::Extension::Output::save_failed();
+	}
 
-	if (!spns) sp_repr_document_unref (rdoc);
+	if (!spns) {
+		sp_repr_document_unref (rdoc);
+	}
 
 	return;
 }
