@@ -20,6 +20,7 @@
 
 #include "win32.h"
 #include "system.h"
+#include "extension/print.h"
 
 /* Initialization */
 
@@ -263,7 +264,7 @@ PrintWin32::finish (Inkscape::Extension::Print *mod)
 			0,       // biClrUsed
 			0        // biClrImportant
 		},
-		{0, 0, 0, 0} // bmiColors
+		{ { 0, 0, 0, 0 } } // bmiColors
 	};
 	//RECT wrect;
 	int res;
@@ -315,7 +316,7 @@ PrintWin32::finish (Inkscape::Extension::Print *mod)
 	for (row = 0; row < height; row += 64) {
 		NRPixBlock pb;
 		NRRectL bbox;
-		NRGC gc;
+		NRGC gc(NULL);
 		int num_rows;
 		int i;
 
@@ -479,12 +480,13 @@ PrintWin32::init (void)
 	Inkscape::Extension::Extension * ext;
 	
 	/* SVG in */
-    ext = sp_module_system_build_from_mem(
-		"<spmodule>\n"
+    ext = Inkscape::Extension::build_from_mem(
+		"<inkscape-extension>\n"
 			"<name>Windows 32-bit Print</name>\n"
 			"<id>" SP_MODULE_KEY_PRINT_WIN32 "</id>\n"
+			"<param name=\"textToPath\" type=\"boolean\">TRUE</param>\n"
 			"<print/>\n"
-		"</spmodule>", new PrintWin32());
+		"</inkscape-extension>", new PrintWin32());
 
 	return;
 }
