@@ -14,8 +14,11 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include <stdio.h>
+
 #include "forward.h"
-#include <extension/extension.h>
+#include <extension/extension-forward.h>
+#include <gtk/gtkwidget.h>
 
 /*######################
 ## N E W
@@ -24,7 +27,8 @@
 /**
  * Creates a new Inkscape document and window
  */
-void sp_file_new (void);
+void sp_file_new (const gchar *templ);
+void sp_file_new_default (void);
 
 /*######################
 ## D E L E T E
@@ -42,7 +46,7 @@ void sp_file_exit (void);
 /**
  * Opens a new file and window from the given URI
  */
-void sp_file_open (const gchar *uri, Inkscape::Extension::Extension *key);
+bool sp_file_open (const gchar *uri, Inkscape::Extension::Extension *key, bool add_to_recent = true);
 
 /**
  * Displays a file open dialog. Calls sp_file_open on
@@ -50,6 +54,11 @@ void sp_file_open (const gchar *uri, Inkscape::Extension::Extension *key);
  */
 void sp_file_open_dialog (gpointer object, gpointer data);
 
+/**
+ * Reverts file to disk-copy on "YES"
+ */
+void sp_file_revert_dialog ();
+
 /*######################
 ## S A V E
 ######################*/
@@ -85,6 +94,10 @@ gboolean sp_file_save_dialog (SPDocument *doc);
  */
 void sp_file_import (GtkWidget * widget);
 
+/**
+ * Imports a resource
+ */
+void file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension *key);
 
 /*######################
 ## E X P O R T
@@ -99,13 +112,14 @@ void sp_file_export_dialog (void *widget);
 
 /**
  * Export the given document as a Portable Network Graphics (PNG)
- * file.
+ * file.  Returns FALSE if an error was encountered while writing
+ * the file, TRUE otherwise.
  */
-void sp_export_png_file (SPDocument *doc, const gchar *filename,
+int sp_export_png_file (SPDocument *doc, const gchar *filename,
 			 double x0, double y0, double x1, double y1,
 			 unsigned int width, unsigned int height,
 			 unsigned long bgcolor,
-			 unsigned int (*status) (float, void *), void *data);
+			 unsigned int (*status) (float, void *), void *data, bool force_overwrite = false, GSList *items_only = NULL);
 
 
 /*######################
@@ -131,5 +145,26 @@ void sp_file_print_direct (void);
  */
 void sp_file_print_preview (gpointer object, gpointer data);
 
+/*#####################
+## U T I L I T Y
+#####################*/
+
+/**
+ * clean unused defs out of file
+ */
+void sp_file_vacuum ();
+
+
+namespace Inkscape {
+namespace IO {
+
+void dump_fopen_call( char const *utf8name, char const *id );
+FILE *fopen_utf8name( char const *utf8name, char const *mode );
+
+void fixupHrefs( SPDocument *doc, const gchar *uri, gboolean spns );
+
+}
+}
+
 
 #endif
