@@ -21,9 +21,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#include "path-prefix.h"
+
 #include "gtk/gtkiconfactory.h"
 
-#include "config.h"
 #include "inkscape-stock.h"
 
 static struct StockIcon
@@ -39,6 +43,15 @@ static struct StockIcon
     {INKSCAPE_STOCK_CAP_BUTT,                      "cap_butt.xpm"            },
     {INKSCAPE_STOCK_CAP_ROUND,                     "cap_round.xpm"           },
     {INKSCAPE_STOCK_CAP_SQUARE,                    "cap_square.xpm"          },
+    {INKSCAPE_STOCK_START_MARKER INKSCAPE_STOCK_MARKER_NONE, "marker_none_start.xpm" },
+    {INKSCAPE_STOCK_START_MARKER INKSCAPE_STOCK_MARKER_FILLED_ARROW, "marker_triangle_start.xpm" },
+    {INKSCAPE_STOCK_START_MARKER INKSCAPE_STOCK_MARKER_HOLLOW_ARROW, "marker_arrow_start.xpm" },
+    {INKSCAPE_STOCK_MID_MARKER INKSCAPE_STOCK_MARKER_NONE, "marker_none_start.xpm" },
+    {INKSCAPE_STOCK_MID_MARKER INKSCAPE_STOCK_MARKER_FILLED_ARROW, "marker_triangle_end.xpm" },
+    {INKSCAPE_STOCK_MID_MARKER INKSCAPE_STOCK_MARKER_HOLLOW_ARROW, "marker_arrow_end.xpm" },
+    {INKSCAPE_STOCK_END_MARKER INKSCAPE_STOCK_MARKER_NONE, "marker_none_end.xpm" },
+    {INKSCAPE_STOCK_END_MARKER INKSCAPE_STOCK_MARKER_FILLED_ARROW, "marker_triangle_end.xpm" },
+    {INKSCAPE_STOCK_END_MARKER INKSCAPE_STOCK_MARKER_HOLLOW_ARROW, "marker_arrow_end.xpm" },
 
     /*object properties */
     {INKSCAPE_STOCK_ROTATE_LEFT,                   "rotate_left.xpm"         },
@@ -152,7 +165,7 @@ inkscape_gtk_stock_init (void) {
 
         if (!g_file_test (localFilename, G_FILE_TEST_EXISTS)) {
             // testing
-            g_critical ("Unable to load stock pixmap %s\n", filename);
+            g_critical ("Unable to load stock pixmap RELOC %s\n", filename);
             // g_critical ("Unable to load stock pixmap %s\n", 
             //             INKSCAPE_PIXMAPDIR);
             // g_critical ("Unable to load stock pixmap %s\n", 
