@@ -12,7 +12,10 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#include "path-prefix.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -45,6 +48,7 @@
 #include "helper/sp-intl.h"
 #include "helper/sp-marshal.h"
 #include "xml/repr-private.h"
+#include "dialogs/debugdialog.h"
 
 #include "shortcuts.h"
 
@@ -58,19 +62,13 @@
 #include "inkscape-private.h"
 #include "prefs-utils.h"
 
+#include "extension/init.h"
+
 #ifdef WIN32
 #define mkdir(d,m) _mkdir((d))
 #endif
 
-/**
- * this kludge was removed from main.cpp, 
- * so that it will work in both inkscape
- * and inkview.
- * This should be removed entirely, eventually.
- */
-gboolean sp_bitmap_icons = FALSE;
-
-static Inkscape::Application *inkscape=NULL;
+static Inkscape::Application *inkscape = NULL;
 
 /* Backbones of configuration xml data */
 #include "preferences-skeleton.h"
@@ -86,7 +84,6 @@ enum {
     DEACTIVATE_DESKTOP,
     NEW_DOCUMENT,
     DESTROY_DOCUMENT,
-    COLOR_SET,
     SHUTDOWN_SIGNAL,
     DIALOGS_HIDE,
     DIALOGS_UNHIDE,
@@ -129,6 +126,8 @@ struct Inkscape::Application {
     GSList *desktops;
     gchar *argv0;
     gboolean dialogs_toggle;
+    gboolean use_gui;         // may want to consider a virtual function
+                              // for overriding things like the warning dlg's
 };
 
 struct Inkscape::ApplicationClass {
@@ -282,14 +281,6 @@ inkscape_class_init (Inkscape::ApplicationClass * klass)
                                sp_marshal_NONE__POINTER,
                                G_TYPE_NONE, 1,
                                G_TYPE_POINTER);
-    inkscape_signals[COLOR_SET] =        g_signal_new ("color_set",
-                               G_TYPE_FROM_CLASS (klass),
-                               G_SIGNAL_RUN_FIRST,
-                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, color_set),
-                               NULL, NULL,
-                               sp_marshal_NONE__POINTER_DOUBLE,
-                               G_TYPE_NONE, 2,
-                               G_TYPE_POINTER, G_TYPE_DOUBLE);
     inkscape_signals[SHUTDOWN_SIGNAL] =        g_signal_new ("shut_down",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_FIRST,
@@ -338,9 +329,7 @@ inkscape_init (SPObject * object)
 static void
 inkscape_dispose (GObject *object)
 {
-    Inkscape::Application *inkscape;
-
-    inkscape = (Inkscape::Application *) object;
+    Inkscape::Application *inkscape = (Inkscape::Application *) object;
 
     while (inkscape->documents) {
         g_object_unref (G_OBJECT (inkscape->documents->data));
@@ -399,29 +388,28 @@ static void
 inkscape_segv_handler (int signum)
 {
     static gint recursion = FALSE;
-    GSList *savednames, *failednames, *l;
-    const gchar *home;
-    gint count, nllen, len, pos;
-    time_t sptime;
-    struct tm *sptm;
-    gchar sptstr[256];
-    GtkWidget *msgbox;
+
+    /* let any SIGABRTs seen from within this handler dump core */
+    signal(SIGABRT, SIG_DFL);
 
     /* Kill loops */
-    if (recursion) abort ();
+    if (recursion) {
+        abort ();
+    }
     recursion = TRUE;
 
-    g_warning ("Emergency save activated");
+    fprintf(stderr, "\nEmergency save activated!\n");
 
-    home = g_get_home_dir ();
-    sptime = time (NULL);
-    sptm = localtime (&sptime);
+    const gchar *home = g_get_home_dir ();
+    time_t sptime = time (NULL);
+    struct tm *sptm = localtime (&sptime);
+    gchar sptstr[256];
     strftime (sptstr, 256, "%Y_%m_%d_%H_%M_%S", sptm);
 
-    count = 0;
-    savednames = NULL;
-    failednames = NULL;
-    for (l = inkscape->documents; l != NULL; l = l->next) {
+    gint count = 0;
+    GSList *savednames = NULL;
+    GSList *failednames = NULL;
+    for (GSList *l = inkscape->documents; l != NULL; l = l->next) {
         SPDocument *doc;
         SPRepr *repr;
         doc = (SPDocument *) l->data;
@@ -444,7 +432,7 @@ inkscape_segv_handler (int signum)
                         while (isdigit (*d) || (*d == '.') || (*d == '_')) d += 1;
                         if (*d) {
                             memcpy (n, docname, MIN (d0 - docname - 1, 64));
-                            n[64] = '\0';
+                            n[63] = '\0';
                             docname = n;
                         }
                     }
@@ -453,14 +441,17 @@ inkscape_segv_handler (int signum)
 
             if (!docname || !*docname) docname = "emergency";
             g_snprintf (c, 1024, "%s/.inkscape/%.256s.%s.%d", home, docname, sptstr, count);
-            file = fopen (c, "w");
+            Inkscape::IO::dump_fopen_call(c, "E");
+            file = Inkscape::IO::fopen_utf8name(c, "w");
             if (!file) {
                 g_snprintf (c, 1024, "%s/inkscape-%.256s.%s.%d", home, docname, sptstr, count);
-                file = fopen (c, "w");
+                Inkscape::IO::dump_fopen_call(c, "F");
+                file = Inkscape::IO::fopen_utf8name(c, "w");
             }
             if (!file) {
                 g_snprintf (c, 1024, "/tmp/inkscape-%.256s.%s.%d", docname, sptstr, count);
-                file = fopen (c, "w");
+                Inkscape::IO::dump_fopen_call(c, "G");
+                file = Inkscape::IO::fopen_utf8name(c, "w");
             }
             if (file) {
                 sp_repr_save_stream (sp_repr_document (repr), file);
@@ -477,14 +468,14 @@ inkscape_segv_handler (int signum)
     savednames = g_slist_reverse (savednames);
     failednames = g_slist_reverse (failednames);
     if (savednames) {
-        fprintf (stderr, "\nEmergency save locations:\n");
-        for (l = savednames; l != NULL; l = l->next) {
+        fprintf (stderr, "\nEmergency save document locations:\n");
+        for (GSList *l = savednames; l != NULL; l = l->next) {
             fprintf (stderr, "  %s\n", (gchar *) l->data);
         }
     }
     if (failednames) {
-        fprintf (stderr, "Failed to do emergency save for:\n");
-        for (l = failednames; l != NULL; l = l->next) {
+        fprintf (stderr, "\nFailed to do emergency save for documents:\n");
+        for (GSList *l = failednames; l != NULL; l = l->next) {
             fprintf (stderr, "  %s\n", (gchar *) l->data);
         }
     }
@@ -493,26 +484,26 @@ inkscape_segv_handler (int signum)
         inkscape_save_preferences (inkscape);
     }
 
-    g_print ("Emergency save completed. Inkscape will close now.\n");
-    g_print ("If you can reproduce this crash, please file a bug at www.inkscape.org\n");
-    g_print ("with a detailed description of the steps leading to the crash, so we can fix it.\n");
+    fprintf (stderr, "Emergency save completed. Inkscape will close now.\n");
+    fprintf (stderr, "If you can reproduce this crash, please file a bug at www.inkscape.org\n");
+    fprintf (stderr, "with a detailed description of the steps leading to the crash, so we can fix it.\n");
 
     /* Show nice dialog box */
 
     char const *istr = N_("Inkscape encountered an internal error and will close now.\n");
-    char const *sstr = N_("Automatic backups of unsaved documents were done to following locations:\n");
-    char const *fstr = N_("Automatic backup of following documents failed:\n");
-    nllen = strlen ("\n");
-    len = strlen (istr) + strlen (sstr) + strlen (fstr);
-    for (l = savednames; l != NULL; l = l->next) {
+    char const *sstr = N_("Automatic backups of unsaved documents were done to the following locations:\n");
+    char const *fstr = N_("Automatic backup of the following documents failed:\n");
+    gint nllen = strlen ("\n");
+    gint len = strlen (istr) + strlen (sstr) + strlen (fstr);
+    for (GSList *l = savednames; l != NULL; l = l->next) {
         len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
     }
-    for (l = failednames; l != NULL; l = l->next) {
+    for (GSList *l = failednames; l != NULL; l = l->next) {
         len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
     }
     len += 1;
     gchar *b = g_new (gchar, len);
-    pos = 0;
+    gint pos = 0;
     len = strlen (istr);
     memcpy (b + pos, istr, len);
     pos += len;
@@ -520,7 +511,7 @@ inkscape_segv_handler (int signum)
         len = strlen (sstr);
         memcpy (b + pos, sstr, len);
         pos += len;
-        for (l = savednames; l != NULL; l = l->next) {
+        for (GSList *l = savednames; l != NULL; l = l->next) {
             memset (b + pos, ' ', SP_INDENT);
             pos += SP_INDENT;
             len = strlen ((gchar *) l->data);
@@ -534,7 +525,7 @@ inkscape_segv_handler (int signum)
         len = strlen (fstr);
         memcpy (b + pos, fstr, len);
         pos += len;
-        for (l = failednames; l != NULL; l = l->next) {
+        for (GSList *l = failednames; l != NULL; l = l->next) {
             memset (b + pos, ' ', SP_INDENT);
             pos += SP_INDENT;
             len = strlen ((gchar *) l->data);
@@ -545,7 +536,7 @@ inkscape_segv_handler (int signum)
         }
     }
     *(b + pos) = '\0';
-    msgbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", b);
+    GtkWidget *msgbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", b);
     gtk_dialog_run (GTK_DIALOG (msgbox));
     gtk_widget_destroy (msgbox);
     g_free (b);
@@ -556,22 +547,42 @@ inkscape_segv_handler (int signum)
 
 
 void
-inkscape_application_init (const gchar *argv0)
+inkscape_application_init (const gchar *argv0, gboolean use_gui)
 {
     inkscape = (Inkscape::Application *)g_object_new (SP_TYPE_INKSCAPE, NULL);
     /* fixme: load application defaults */
-
-#ifndef WIN32
+  
     segv_handler = signal (SIGSEGV, inkscape_segv_handler);
-    signal (SIGFPE, inkscape_segv_handler);
-    signal (SIGILL, inkscape_segv_handler);
-#endif
+    signal (SIGFPE,  inkscape_segv_handler);
+    signal (SIGILL,  inkscape_segv_handler);
+#ifndef WIN32    
+    signal (SIGBUS,  inkscape_segv_handler);
+#endif    
+    signal (SIGABRT, inkscape_segv_handler);
 
+    inkscape->use_gui = use_gui;
     inkscape->argv0 = g_strdup(argv0);
 
     /* Attempt to load the preferences, and set the save_preferences flag to TRUE
        if we could, or FALSE if we couldn't */
     inkscape->save_preferences = inkscape_load_preferences(inkscape);
+
+    /* DebugDialog redirection.  On Linux, default to OFF, on Win32, default to ON */
+#ifdef WIN32
+#define DEFAULT_LOG_REDIRECT true
+#else
+#define DEFAULT_LOG_REDIRECT false
+#endif
+
+    if (prefs_get_int_attribute("dialogs.debug", "redirect", DEFAULT_LOG_REDIRECT))
+    {
+        Inkscape::UI::Dialogs::DebugDialog::getInstance()->captureLogMessages();
+    }
+
+    /* Initialize the extensions */
+    Inkscape::Extension::init();
+
+    return;
 }
 
 /**
@@ -594,12 +605,7 @@ inkscape_load_config (const gchar *filename, SPReprDoc *config, const gchar *ske
 		      unsigned int skel_size, const gchar *e_notreg, const gchar *e_notxml, 
 		      const gchar *e_notsp, const gchar *warn)
 {
-    gchar *fn;
-    GtkWidget * w;
-    SPReprDoc * doc;
-    SPRepr * root;
-
-    fn = profile_path(filename);
+    gchar *fn = profile_path(filename);
     if (!g_file_test(fn, G_FILE_TEST_EXISTS)) {
         /* No such file */
         inkscape_init_preferences (INKSCAPE);
@@ -609,26 +615,26 @@ inkscape_load_config (const gchar *filename, SPReprDoc *config, const gchar *ske
 
     if (!g_file_test(fn, G_FILE_TEST_IS_REGULAR)) {
         /* Not a regular file */
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notreg, fn, warn);
+        GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notreg, fn, warn);
         gtk_dialog_run (GTK_DIALOG (w));
         gtk_widget_destroy (w);
         g_free (fn);
         return FALSE;
     }
 
-    doc = sp_repr_read_file (fn, NULL);
+    SPReprDoc *doc = sp_repr_read_file (fn, NULL);
     if (doc == NULL) {
         /* Not an valid xml file */
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notxml, fn, warn);
+        GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notxml, fn, warn);
         gtk_dialog_run (GTK_DIALOG (w));
         gtk_widget_destroy (w);
         g_free (fn);
         return FALSE;
     }
 
-    root = sp_repr_document_root (doc);
+    SPRepr *root = sp_repr_document_root (doc);
     if (strcmp (sp_repr_name (root), "inkscape")) {
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notsp, fn, warn);
+        GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notsp, fn, warn);
         gtk_dialog_run (GTK_DIALOG (w));
         gtk_widget_destroy (w);
         sp_repr_document_unref (doc);
@@ -661,7 +667,7 @@ inkscape_load_preferences (Inkscape::Application *inkscape)
 				 _("%s is not a regular file.\n%s"),
 				 _("%s not a valid XML file, or\n"
 				   "you don't have read permissions on it.\n%s"),
-				 _("%s is not valid preferences file.\n%s"),
+				 _("%s is not a valid preferences file.\n%s"),
 				 _("Inkscape will run with default settings.\n"
                              "New settings will not be saved."));
 }
@@ -673,9 +679,7 @@ inkscape_load_preferences (Inkscape::Application *inkscape)
 gboolean
 inkscape_save_preferences (Inkscape::Application * inkscape)
 {
-    gchar * fn;
-
-    fn = profile_path(PREFERENCES_FILE);
+    gchar *fn = profile_path(PREFERENCES_FILE);
     gboolean retval = sp_repr_save_file (inkscape->preferences, fn);
 
     g_free (fn);
@@ -688,26 +692,26 @@ inkscape_save_preferences (Inkscape::Application * inkscape)
 SPRepr *
 inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key)
 {
-    SPRepr * repr;
-    gchar const *s;
+    if (key == NULL) {
+        return NULL;
+    }
 
-    if (key == NULL) return NULL;
-
-    repr = sp_repr_document_root (inkscape->preferences);
+    SPRepr *repr = sp_repr_document_root (inkscape->preferences);
     g_assert (!(strcmp (sp_repr_name (repr), "inkscape")));
 
-    s = key;
+    gchar const *s = key;
     while ((s) && (*s)) {
-        gchar const *e;
-        SPRepr * child;
-        guint len;
 
         /* Find next name */
-        if ((e = strchr (s, '.'))) {
+        gchar const *e = strchr (s, '.');
+        guint len;
+        if (e) {
             len = e++ - s;
         } else {
             len = strlen (s);
         }
+        
+        SPRepr* child;
         for (child = repr->children; child != NULL; child = child->next) {
             gchar const *id = sp_repr_attr (child, "id");
             if ((id) && (strlen (id) == len) && (!strncmp (id, s, len)))
@@ -715,7 +719,9 @@ inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key)
                 break;
             }
         }
-        if (child == NULL) return NULL;
+        if (child == NULL) {
+            return NULL;
+        }
 
         repr = child;
         s = e;
@@ -730,9 +736,8 @@ inkscape_selection_modified (SPSelection *selection, guint flags)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
-    g_return_if_fail (SP_IS_SELECTION (selection));
 
-    if (DESKTOP_IS_ACTIVE (selection->desktop)) {
+    if (DESKTOP_IS_ACTIVE (selection->desktop())) {
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[MODIFY_SELECTION], 0, selection, flags);
     }
 }
@@ -743,9 +748,8 @@ inkscape_selection_changed (SPSelection * selection)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
-    g_return_if_fail (SP_IS_SELECTION (selection));
 
-    if (DESKTOP_IS_ACTIVE (selection->desktop)) {
+    if (DESKTOP_IS_ACTIVE (selection->desktop())) {
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[CHANGE_SELECTION], 0, selection);
     }
 }
@@ -756,9 +760,8 @@ inkscape_selection_set (SPSelection * selection)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
-    g_return_if_fail (SP_IS_SELECTION (selection));
 
-    if (DESKTOP_IS_ACTIVE (selection->desktop)) {
+    if (DESKTOP_IS_ACTIVE (selection->desktop())) {
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[SET_SELECTION], 0, selection);
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[CHANGE_SELECTION], 0, selection);
     }
@@ -813,8 +816,7 @@ inkscape_remove_desktop (SPDesktop * desktop)
     if (DESKTOP_IS_ACTIVE (desktop)) {
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[DEACTIVATE_DESKTOP], 0, desktop);
         if (inkscape->desktops->next != NULL) {
-            SPDesktop * new_desktop;
-            new_desktop = (SPDesktop *) inkscape->desktops->next->data;
+            SPDesktop * new_desktop = (SPDesktop *) inkscape->desktops->next->data;
             inkscape->desktops = g_slist_remove (inkscape->desktops, new_desktop);
             inkscape->desktops = g_slist_prepend (inkscape->desktops, new_desktop);
             g_signal_emit (G_OBJECT (inkscape), inkscape_signals[ACTIVATE_DESKTOP], 0, new_desktop);
@@ -823,8 +825,7 @@ inkscape_remove_desktop (SPDesktop * desktop)
             g_signal_emit (G_OBJECT (inkscape), inkscape_signals[CHANGE_SELECTION], 0, SP_DT_SELECTION (new_desktop));
         } else {
             g_signal_emit (G_OBJECT (inkscape), inkscape_signals[SET_EVENTCONTEXT], 0, NULL);
-            g_signal_emit (G_OBJECT (inkscape), inkscape_signals[SET_SELECTION], 0, NULL);
-            g_signal_emit (G_OBJECT (inkscape), inkscape_signals[CHANGE_SELECTION], 0, NULL);
+	    desktop->selection->clear();
         }
     }
 
@@ -843,17 +844,17 @@ inkscape_remove_desktop (SPDesktop * desktop)
 void
 inkscape_activate_desktop (SPDesktop * desktop)
 {
-    SPDesktop * current;
-
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (desktop != NULL);
     g_return_if_fail (SP_IS_DESKTOP (desktop));
 
-    if (DESKTOP_IS_ACTIVE (desktop)) return;
+    if (DESKTOP_IS_ACTIVE (desktop)) {
+        return;
+    }
 
     g_assert (g_slist_find (inkscape->desktops, desktop));
 
-    current = (SPDesktop *) inkscape->desktops->data;
+    SPDesktop *current = (SPDesktop *) inkscape->desktops->data;
 
     g_signal_emit (G_OBJECT (inkscape), inkscape_signals[DEACTIVATE_DESKTOP], 0, current);
 
@@ -885,9 +886,7 @@ inkscape_reactivate_desktop (SPDesktop * desktop)
 SPDesktop *
 inkscape_find_desktop_by_dkey (unsigned int dkey)
 {
-    GSList *r;
-
-    for (r = inkscape->desktops; r; r = r->next) {
+    for (GSList *r = inkscape->desktops; r; r = r->next) {
         if (((SPDesktop *) r->data)->dkey == dkey)
             return ((SPDesktop *) r->data);
     }
@@ -900,16 +899,13 @@ inkscape_find_desktop_by_dkey (unsigned int dkey)
 unsigned int
 inkscape_maximum_dkey()
 {
-    GSList *r;
     unsigned int dkey = 0;
 
-    for (r = inkscape->desktops; r; r = r->next) {
+    for (GSList *r = inkscape->desktops; r; r = r->next) {
         if (((SPDesktop *) r->data)->dkey > dkey)
             dkey = ((SPDesktop *) r->data)->dkey;
     }
 
-    g_assert (dkey > 0);
-
     return dkey;
 }
 
@@ -920,19 +916,22 @@ inkscape_next_desktop ()
 {
     SPDesktop *d = NULL;
     unsigned int dkey_current = ((SPDesktop *) inkscape->desktops->data)->dkey;
-    unsigned int i;
 
     if (dkey_current < inkscape_maximum_dkey()) {
         // find next existing
-        for (i = dkey_current + 1; i <= inkscape_maximum_dkey(); i++) {
+        for (unsigned int i = dkey_current + 1; i <= inkscape_maximum_dkey(); i++) {
             d = inkscape_find_desktop_by_dkey (i);
-            if (d) break;
+            if (d) {
+                break;
+            }
         }
     } else {
         // find first existing
-        for (i = 1; i <= inkscape_maximum_dkey(); i++) {
+        for (unsigned int i = 0; i <= inkscape_maximum_dkey(); i++) {
             d = inkscape_find_desktop_by_dkey (i);
-            if (d) break;
+            if (d) {
+                break;
+            }
         }
     }
 
@@ -948,13 +947,14 @@ inkscape_prev_desktop ()
 {
     SPDesktop *d = NULL;
     unsigned int dkey_current = ((SPDesktop *) inkscape->desktops->data)->dkey;
-    unsigned int i;
 
-    if (dkey_current > 1) {
+    if (dkey_current > 0) {
         // find prev existing
-        for (i = dkey_current - 1; i >= 1; i--) {
+        for (signed int i = dkey_current - 1; i >= 0; i--) {
             d = inkscape_find_desktop_by_dkey (i);
-            if (d) break;
+            if (d) {
+                break;
+            }
         }
     }
     if (!d) {
@@ -972,9 +972,7 @@ inkscape_prev_desktop ()
 void
 inkscape_switch_desktops_next ()
 {
-    GtkWindow *w;
-
-    w = (GtkWindow *) g_object_get_data (G_OBJECT (inkscape_next_desktop ()), "window");
+    GtkWindow *w = (GtkWindow *) g_object_get_data (G_OBJECT (inkscape_next_desktop ()), "window");
     gtk_window_present (w);
 }
 
@@ -983,9 +981,7 @@ inkscape_switch_desktops_next ()
 void
 inkscape_switch_desktops_prev ()
 {
-    GtkWindow *w;
-
-    w = (GtkWindow *) g_object_get_data (G_OBJECT (inkscape_prev_desktop ()), "window");
+    GtkWindow *w = (GtkWindow *) g_object_get_data (G_OBJECT (inkscape_prev_desktop ()), "window");
     gtk_window_present (w);
 }
 
@@ -1016,10 +1012,11 @@ inkscape_dialogs_unhide ()
 void
 inkscape_dialogs_toggle ()
 {
-    if (inkscape->dialogs_toggle)
+    if (inkscape->dialogs_toggle) {
         inkscape_dialogs_hide ();
-    else
+    } else {
         inkscape_dialogs_unhide ();
+    }
 }
 
 
@@ -1048,7 +1045,6 @@ inkscape_remove_document (SPDocument *document)
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (document != NULL);
     g_return_if_fail (SP_IS_DOCUMENT (document));
-    gint i;
 
     g_assert (g_slist_find (inkscape->documents, document));
 
@@ -1056,57 +1052,25 @@ inkscape_remove_document (SPDocument *document)
 
     inkscape->documents = g_slist_remove (inkscape->documents, document);
 
-    int max_documents = prefs_get_int_attribute ("options.maxrecentdocuments", "value", 20);
-
-    if (document->advertize && SP_DOCUMENT_URI (document)) {
-        SPRepr *recent;
-        recent = inkscape_get_repr (INKSCAPE, "documents.recent");
-        if (recent) {
-            SPRepr *child;
-            child = sp_repr_lookup_child (recent, "uri", SP_DOCUMENT_URI (document));
-            if (child) {
-                sp_repr_change_order (recent, child, NULL);
-            } else {
-                if (sp_repr_n_children (recent) >= max_documents) {
-                    child = recent->children;
-                    // count to the last
-                    for (i = 0; i < max_documents - 2; i ++) child = child->next;
-                    // remove all after the last
-                    while (child->next) sp_repr_unparent (child->next);
-                }
-                child = sp_repr_new ("document");
-                sp_repr_set_attr (child, "uri", SP_DOCUMENT_URI (document));
-                sp_repr_add_child (recent, child, NULL);
-            }
-            sp_repr_set_attr (child, "name", SP_DOCUMENT_NAME (document));
-        }
-    }
+    return;
 }
 
-
-
-void
-inkscape_set_color (SPColor *color, float opacity)
-{
-    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[COLOR_SET], 0, color, (double) opacity);
-}
-
-
-
 SPDesktop *
 inkscape_active_desktop (void)
 {
-    if (inkscape->desktops == NULL) return NULL;
+    if (inkscape->desktops == NULL) {
+        return NULL;
+    }
 
     return (SPDesktop *) inkscape->desktops->data;
 }
 
-
-
 SPDocument *
 inkscape_active_document (void)
 {
-    if (SP_ACTIVE_DESKTOP) return SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP);
+    if (SP_ACTIVE_DESKTOP) {
+        return SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP);
+    }
 
     return NULL;
 }
@@ -1116,7 +1080,9 @@ inkscape_active_document (void)
 SPEventContext *
 inkscape_active_event_context (void)
 {
-    if (SP_ACTIVE_DESKTOP) return SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP);
+    if (SP_ACTIVE_DESKTOP) {
+        return SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP);
+    }
 
     return NULL;
 }
@@ -1136,53 +1102,75 @@ inkscape_init_config (SPReprDoc *doc, const gchar *config_name, const gchar *ske
 		      const gchar *e_cwf, 
 		      const gchar *warn)
 {
-    gchar * dn, *fn;
-    FILE *fh;
-    GtkWidget * w;
-
-    dn = profile_path(NULL);
+    gchar *dn = profile_path(NULL);
     if (!g_file_test(dn, G_FILE_TEST_EXISTS)) {
         if (mkdir (dn, S_IRWXU | S_IRGRP | S_IXGRP))
         {
-            /* Cannot create directory */
-            w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_mkdir, dn, warn);
+            if (inkscape->use_gui) {
+                /* Cannot create directory */
+                GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_mkdir, dn, warn);
+                gtk_dialog_run (GTK_DIALOG (w));
+                gtk_widget_destroy (w);
+                g_free (dn);
+                return;
+            } else {
+                g_warning(e_mkdir, dn, warn);
+                g_free (dn);
+                return;
+            }
+        }
+    } else if (!g_file_test(dn, G_FILE_TEST_IS_DIR)) {
+        if (inkscape->use_gui) {
+            /* Not a directory */
+            GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notdir, dn, warn);
             gtk_dialog_run (GTK_DIALOG (w));
             gtk_widget_destroy (w);
             g_free (dn);
             return;
+        } else {
+            g_warning(e_notdir, dn, warn);
+            g_free(dn);
+            return;
         }
-    } else if (!g_file_test(dn, G_FILE_TEST_IS_DIR)) {
-        /* Not a directory */
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notdir, dn, warn);
-        gtk_dialog_run (GTK_DIALOG (w));
-        gtk_widget_destroy (w);
-        g_free (dn);
-        return;
     }
     g_free (dn);
 
-    fn = profile_path(config_name);
+    gchar *fn = profile_path(config_name);
 
-    fh = fopen(fn, "w");
+    Inkscape::IO::dump_fopen_call(fn, "H");
+    FILE *fh = Inkscape::IO::fopen_utf8name(fn, "w");
     if (!fh) {
-        /* Cannot create file */
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_ccf, fn, warn);
-        gtk_dialog_run (GTK_DIALOG (w));
-        gtk_widget_destroy (w);
-        g_free (fn);
-        return;
+        if (inkscape->use_gui) {
+            /* Cannot create file */
+            GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_ccf, fn, warn);
+            gtk_dialog_run (GTK_DIALOG (w));
+            gtk_widget_destroy (w);
+            g_free (fn);
+            return;
+        } else {
+            g_warning(e_ccf, fn, warn);
+            g_free(fn);
+            return;
+        }
     }
     if ( fwrite(skeleton, 1, skel_size, fh) != skel_size ) {
-        /* Cannot create file */
-        w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_cwf, fn, warn);
-        gtk_dialog_run (GTK_DIALOG (w));
-        gtk_widget_destroy (w);
-        g_free (fn);
-        fclose(fh);
-        return;
+        if (inkscape->use_gui) {
+            /* Cannot create file */
+            GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_cwf, fn, warn);
+            gtk_dialog_run (GTK_DIALOG (w));
+            gtk_widget_destroy (w);
+            g_free (fn);
+            fclose(fh);
+            return;
+        } else {
+            g_warning(e_cwf, fn, warn);
+            g_free(fn);
+            fclose(fh);
+            return;
+        }
     }
 
-    g_free (fn);
+    g_free(fn);
     fclose(fh);
 }
 
@@ -1210,9 +1198,7 @@ inkscape_init_preferences (Inkscape::Application *inkscape)
 void
 inkscape_refresh_display (Inkscape::Application *inkscape)
 {
-    GSList *l;
-
-    for (l = inkscape->desktops; l != NULL; l = l->next) {
+    for (GSList *l = inkscape->desktops; l != NULL; l = l->next) {
         sp_view_request_redraw (SP_VIEW (l->data));
     }
 }
@@ -1242,7 +1228,7 @@ inkscape_exit (Inkscape::Application *inkscape)
 gchar *
 profile_path(const char *filename)
 {
-    static const gchar *homedir=NULL;
+    static const gchar *homedir = NULL;
     if (!homedir) {
         homedir = g_get_home_dir();
 #ifdef HAS_SHGetSpecialFolderPath
@@ -1260,4 +1246,14 @@ profile_path(const char *filename)
 }
 
 
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
