@@ -43,6 +43,8 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
 
+#include "gc-core.h"
+
 #include "helper/sp-intl.h"
 #include "document.h"
 #include "svg-view.h"
@@ -61,21 +63,22 @@ Inkscape::Application *inkscape;
 #define bind_textdomain_codeset(p,c)
 #endif
 
-struct _SPSlideShow {
+struct SPSlideShow {
     char **slides;
     int size;
     int length;
     int current;
     SPDocument *doc;
     GtkWidget *view;
+    GtkWindow *window;
     bool fullscreen;
 };
 
-static GtkWidget *sp_svgview_control_show (struct _SPSlideShow *ss);
-static void sp_svgview_show_next (struct _SPSlideShow *ss);
-static void sp_svgview_show_prev (struct _SPSlideShow *ss);
-static void sp_svgview_goto_first (struct _SPSlideShow *ss);
-static void sp_svgview_goto_last (struct _SPSlideShow *ss);
+static GtkWidget *sp_svgview_control_show (struct SPSlideShow *ss);
+static void sp_svgview_show_next (struct SPSlideShow *ss);
+static void sp_svgview_show_prev (struct SPSlideShow *ss);
+static void sp_svgview_goto_first (struct SPSlideShow *ss);
+static void sp_svgview_goto_last (struct SPSlideShow *ss);
 
 static int sp_svgview_show_next_cb (GtkWidget *widget, void *data);
 static int sp_svgview_show_prev_cb (GtkWidget *widget, void *data);
@@ -90,14 +93,14 @@ static GtkWidget *ctrlwin = NULL;
 
 
 static int
-sp_svgview_main_delete (GtkWidget *widget, GdkEvent *event, struct _SPSlideShow *ss)
+sp_svgview_main_delete (GtkWidget *widget, GdkEvent *event, struct SPSlideShow *ss)
 {
     gtk_main_quit ();
     return FALSE;
 }
 
 static int
-sp_svgview_main_key_press (GtkWidget *widget, GdkEventKey *event, struct _SPSlideShow *ss)
+sp_svgview_main_key_press (GtkWidget *widget, GdkEventKey *event, struct SPSlideShow *ss)
 {
     switch (event->keyval) {
     case GDK_Up:
@@ -122,18 +125,26 @@ sp_svgview_main_key_press (GtkWidget *widget, GdkEventKey *event, struct _SPSlid
     case GDK_Return:
 	sp_svgview_control_show (ss);
 	break;
+    case GDK_KP_Page_Down:
+    case GDK_Page_Down:
     case GDK_Right:
+    case GDK_space:
 	sp_svgview_show_next (ss);
 	break;
+    case GDK_KP_Page_Up:
+    case GDK_Page_Up:
     case GDK_Left:
 	sp_svgview_show_prev (ss);
 	break;
     case GDK_Escape:
+    case GDK_q:
+    case GDK_Q:
 	gtk_main_quit();
 	break;
     default:
 	break;
     }
+    gtk_window_set_title(GTK_WINDOW(widget), SP_DOCUMENT_NAME(ss->doc));
     return FALSE;
 }
 
@@ -144,7 +155,7 @@ main (int argc, const char **argv)
 	usage();
     }
     
-    struct _SPSlideShow ss; 
+    struct SPSlideShow ss; 
 	
     GtkWidget *w;
     int i;
@@ -154,8 +165,10 @@ main (int argc, const char **argv)
     textdomain (GETTEXT_PACKAGE);
 
     LIBXML_TEST_VERSION
+    
+    Inkscape::GC::init();
 
-	gtk_init (&argc, (char ***) &argv);
+    gtk_init (&argc, (char ***) &argv);
 
 #ifdef lalaWITH_MODULES
     g_warning ("Have to autoinit modules (lauris)");
@@ -236,7 +249,7 @@ main (int argc, const char **argv)
 		}
 		
 		ss.slides[ss.length++] = strdup (argv[i]);
-		ss.doc = sp_document_new (ss.slides[ss.current], TRUE, TRUE);
+		ss.doc = sp_document_new (ss.slides[ss.current], TRUE, TRUE, false);
 		
 		if (!ss.doc && ++ss.current >= ss.length) {
 		    /* No loadable documents */
@@ -283,7 +296,7 @@ sp_svgview_ctrlwin_delete (GtkWidget *widget, GdkEvent *event, void *data)
 }
 
 static GtkWidget *
-sp_svgview_control_show (struct _SPSlideShow *ss)
+sp_svgview_control_show (struct SPSlideShow *ss)
 {
     if (!ctrlwin) {
 	GtkWidget *t, *b;
@@ -326,40 +339,40 @@ sp_svgview_control_show (struct _SPSlideShow *ss)
 static int 
 sp_svgview_show_next_cb (GtkWidget *widget, void *data)
 {
-    sp_svgview_show_next(static_cast<_SPSlideShow *>(data));
+    sp_svgview_show_next(static_cast<struct SPSlideShow *>(data));
     return FALSE;
 }
 
 static int 
 sp_svgview_show_prev_cb (GtkWidget *widget, void *data)
 {
-    sp_svgview_show_prev(static_cast<struct _SPSlideShow *>(data));
+    sp_svgview_show_prev(static_cast<struct SPSlideShow *>(data));
     return FALSE;
 }
 
 static int 
 sp_svgview_goto_first_cb (GtkWidget *widget, void *data)
 {
-    sp_svgview_goto_first(static_cast<struct _SPSlideShow *>(data));
+    sp_svgview_goto_first(static_cast<struct SPSlideShow *>(data));
     return FALSE;
 }
 
 static int 
 sp_svgview_goto_last_cb (GtkWidget *widget, void *data)
 {
-    sp_svgview_goto_last(static_cast<struct _SPSlideShow *>(data));
+    sp_svgview_goto_last(static_cast<struct SPSlideShow *>(data));
     return FALSE;
 }
 
 static void
-sp_svgview_show_next (struct _SPSlideShow *ss)
+sp_svgview_show_next (struct SPSlideShow *ss)
 {
     SPDocument *doc;
     int current;
     doc = NULL;
     current = ss->current;
     while (!doc && (current < ss->length - 1)) {
-	doc = sp_document_new (ss->slides[++current], TRUE, TRUE);
+	doc = sp_document_new (ss->slides[++current], TRUE, TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -370,14 +383,14 @@ sp_svgview_show_next (struct _SPSlideShow *ss)
 }
 
 static void
-sp_svgview_show_prev (struct _SPSlideShow *ss)
+sp_svgview_show_prev (struct SPSlideShow *ss)
 {
     SPDocument *doc;
     int current;
     doc = NULL;
     current = ss->current;
     while (!doc && (current > 0)) {
-	doc = sp_document_new (ss->slides[--current], TRUE, TRUE);
+	doc = sp_document_new (ss->slides[--current], TRUE, TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -388,12 +401,12 @@ sp_svgview_show_prev (struct _SPSlideShow *ss)
 }
 
 static void
-sp_svgview_goto_first (struct _SPSlideShow *ss)
+sp_svgview_goto_first (struct SPSlideShow *ss)
 {
     SPDocument *doc = NULL;
     int current = 0;
     for ( ; !doc && (current < ss->length); current++) {
-	doc = sp_document_new (ss->slides[current], TRUE, TRUE);
+	doc = sp_document_new (ss->slides[current], TRUE, TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -404,12 +417,12 @@ sp_svgview_goto_first (struct _SPSlideShow *ss)
 }
 
 static void
-sp_svgview_goto_last (struct _SPSlideShow *ss)
+sp_svgview_goto_last (struct SPSlideShow *ss)
 {
     SPDocument *doc = NULL;
     int current = ss->length - 1;
     for ( ; !doc && (current >= 0); current--) {
-	doc = sp_document_new (ss->slides[current], TRUE, TRUE);
+	doc = sp_document_new (ss->slides[current], TRUE, TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -456,3 +469,15 @@ SPRepr *inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key) {r
 #include "widgets/menu.h"
 void sp_menu_append (SPMenu *menu, const gchar *name, const gchar *tip, const void *data) {}
 #endif
+
+
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
