@@ -12,14 +12,17 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <math.h>
 #include <glib-object.h>
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
 #include "helper/sp-intl.h"
 #include "helper/sp-marshal.h"
-#include "helper/sp-canvas-util.h"
-#include "helper/sodipodi-ctrl.h"
+#include "display/sp-canvas-util.h"
+#include "display/sodipodi-ctrl.h"
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "desktop-events.h"
@@ -29,6 +32,8 @@
 #include "document.h"
 #include "prefs-utils.h"
 #include <libnr/nr-point-fns.h>
+#include "message-stack.h"
+#include "event-context.h"
 
 #include "sp-guide.h"
 
@@ -56,6 +61,7 @@ enum {
 	PROP_IMAGE, PROP_IMAGE_MOUSEOVER, PROP_IMAGE_DRAGGING,
 	PROP_CURSOR, PROP_CURSOR_MOUSEOVER, PROP_CURSOR_DRAGGING,
 	PROP_PIXBUF,
+	PROP_TIP,
 
 	PROP_LAST
 };
@@ -222,6 +228,10 @@ sp_knot_class_init (SPKnotClass * klass)
 					 PROP_PIXBUF,
 					 g_param_spec_pointer ("pixbuf", "Pixbuf", "",
 							       (GParamFlags)G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_TIP,
+					 g_param_spec_pointer ("tip", "Tip", "",
+							       (GParamFlags)G_PARAM_READWRITE));
 
 	knot_signals[EVENT] = g_signal_new ("event",
 					    G_TYPE_FROM_CLASS(klass),
@@ -297,6 +307,7 @@ sp_knot_init (SPKnot * knot)
 	knot->anchor = GTK_ANCHOR_CENTER;
 	knot->shape = SP_KNOT_SHAPE_SQUARE;
 	knot->mode = SP_KNOT_MODE_XOR;
+	knot->tip = NULL;
 
 	knot->fill [SP_KNOT_STATE_NORMAL] = 0xffffff00;
 	knot->fill [SP_KNOT_STATE_MOUSEOVER] = 0xff0000ff;
@@ -338,6 +349,11 @@ sp_knot_dispose (GObject * object)
 		}
 	}
 
+	if (knot->tip) {
+		g_free (knot->tip);
+		knot->tip = NULL;
+	}
+
 	if (((GObjectClass *) (parent_class))->dispose)
 		(* ((GObjectClass *) (parent_class))->dispose) (object);
 }
@@ -424,6 +440,9 @@ sp_knot_set_property (GObject * object, guint prop_id, const GValue *value, GPar
 	case PROP_PIXBUF:
 	        knot->pixbuf = g_value_get_pointer (value);
 	        break;
+	case PROP_TIP:
+	        knot->tip = g_strdup ((const gchar *) g_value_get_pointer (value));
+	        break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -468,6 +487,7 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 
 			p = sp_desktop_w2d_xy_point (knot->desktop, NR::Point(event->button.x, event->button.y));
 			knot->grabbed_rel_pos = p - knot->pos;
+			knot->drag_origin = knot->pos;
 			if (!nograb) {
 				sp_canvas_item_grab (knot->item,
 						     KNOT_EVENT_MASK,
@@ -539,10 +559,28 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 		break;
 	case GDK_ENTER_NOTIFY:
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, TRUE);
+
+		sp_knot_set_flag (knot, SP_KNOT_GRABBED, FALSE);
+
+		if (knot->tip)
+			knot->desktop->event_context->defaultMessageContext()->set(Inkscape::NORMAL_MESSAGE, knot->tip);
+
+		grabbed = FALSE;
+		moved = FALSE;
+
 		consumed = TRUE;
 		break;
 	case GDK_LEAVE_NOTIFY:
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, FALSE);
+
+		sp_knot_set_flag (knot, SP_KNOT_GRABBED, FALSE);
+
+		if (knot->tip)
+			knot->desktop->event_context->defaultMessageContext()->clear();
+
+		grabbed = FALSE;
+		moved = FALSE;
+
 		consumed = TRUE;
 		break;
 	case GDK_KEY_PRESS: // keybindings for knot
@@ -560,7 +598,7 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 					       knot_signals[UNGRABBED], 0,
 					       event->button.state);
 				sp_document_undo (SP_DT_DOCUMENT (knot->desktop));
-				sp_view_set_statusf_flash (SP_VIEW(knot->desktop), _("Node or handle drag cancelled."));
+				knot->desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Node or handle drag canceled."));
 				transform_escaped = true;
 				consumed = TRUE;
 			} 
@@ -581,7 +619,7 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 }
 
 SPKnot *
-sp_knot_new (SPDesktop * desktop)
+sp_knot_new (SPDesktop * desktop, const gchar *tip)
 {
 	g_return_val_if_fail (desktop != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DESKTOP (desktop), NULL);
@@ -591,6 +629,8 @@ sp_knot_new (SPDesktop * desktop)
 
 	knot->desktop = desktop;
 	knot->flags = SP_KNOT_VISIBLE;
+	if (tip)
+	        knot->tip = g_strdup (tip);
 
 	knot->item = sp_canvas_item_new (SP_DT_CONTROLS (desktop),
 		SP_TYPE_CTRL,
@@ -674,7 +714,7 @@ sp_knot_set_position (SPKnot * knot, NR::Point *p, guint state)
 	knot->pos = *p;
 
 	if (knot->item)
-		sp_ctrl_moveto (SP_CTRL (knot->item), *p);
+		SP_CTRL (knot->item)->moveto (*p);
 
 	g_signal_emit (G_OBJECT (knot),
 		       knot_signals[MOVED], 0,
