@@ -15,14 +15,19 @@
 
 #include <glib.h>
 #include <gtk/gtksignal.h>
+
 #include "document.h"
 #include "desktop.h"
 #include "sp-item.h"
 #include "sp-shape.h"
+#include "knot.h"
 #include "knotholder.h"
+#include "knot-holder-entity.h"
+#include <libnr/nr-matrix-div.h>
 #include <libnr/nr-matrix-ops.h>
 
-static void knot_moved_handler (SPKnot *knot, NR::Point *p, guint state, gpointer data);
+static void knot_clicked_handler (SPKnot *knot, guint state, gpointer data);
+static void knot_moved_handler(SPKnot *knot, NR::Point const *p, guint state, gpointer data);
 static void knot_ungrabbed_handler (SPKnot *knot, unsigned int state, SPKnotHolder *kh);
 
 #ifdef KNOT_HOLDER_DEBUG
@@ -82,17 +87,19 @@ sp_knot_holder_destroy	(SPKnotHolder *kh)
 }
 
 void
-sp_knot_holder_add (SPKnotHolder *knot_holder, SPKnotHolderSetFunc knot_set, SPKnotHolderGetFunc knot_get)
+sp_knot_holder_add (SPKnotHolder *knot_holder, SPKnotHolderSetFunc knot_set, SPKnotHolderGetFunc knot_get, void (* knot_click) (SPItem *item, guint state), const gchar *tip)
 {
-	sp_knot_holder_add_full (knot_holder, knot_set, knot_get, SP_KNOT_SHAPE_DIAMOND, SP_KNOT_MODE_XOR);
+	sp_knot_holder_add_full (knot_holder, knot_set, knot_get, knot_click, SP_KNOT_SHAPE_DIAMOND, SP_KNOT_MODE_XOR, tip);
 }
 
 void
 sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 			 SPKnotHolderSetFunc knot_set,
 			 SPKnotHolderGetFunc knot_get,
+			 void (* knot_click) (SPItem *item, guint state),
 			 SPKnotShapeType     shape,
-			 SPKnotModeType      mode)
+			 SPKnotModeType      mode,
+			 const gchar *tip)
 {
 	g_return_if_fail (knot_holder != NULL);
 	g_return_if_fail (knot_set != NULL);
@@ -102,9 +109,14 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 
 	/* create new SPKnotHolderEntry */
 	SPKnotHolderEntity *e = g_new (SPKnotHolderEntity, 1);
-	e->knot = sp_knot_new (knot_holder->desktop);
+	e->knot = sp_knot_new (knot_holder->desktop, tip);
 	e->knot_set = knot_set;
 	e->knot_get = knot_get;
+	if (knot_click) {
+		e->knot_click = knot_click;
+	} else {
+		e->knot_click = NULL;
+	}
 
 	g_object_set (G_OBJECT (e->knot->item), "shape", shape, NULL);
 	g_object_set (G_OBJECT (e->knot->item), "mode", mode, NULL);
@@ -115,6 +127,7 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 	sp_knot_set_position (e->knot, &dp, SP_KNOT_STATE_NORMAL);
 
 	e->handler_id = g_signal_connect (G_OBJECT (e->knot), "moved", G_CALLBACK (knot_moved_handler), knot_holder);
+	g_signal_connect (G_OBJECT (e->knot), "clicked", G_CALLBACK (knot_clicked_handler), knot_holder);
 
 	g_signal_connect (G_OBJECT (e->knot), "ungrabbed", G_CALLBACK (knot_ungrabbed_handler), knot_holder);
 
@@ -124,25 +137,13 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 	sp_knot_show (e->knot);
 }
 
+/**
+ * \param p In desktop coordinates.
+ */
+
 static void
-knot_moved_handler (SPKnot *knot, NR::Point *p, guint state, gpointer data)
+knotholder_update_knots (SPKnotHolder *knot_holder, SPItem *item)
 {
-	SPKnotHolder *knot_holder = (SPKnotHolder *) data;
-	SPItem *item  = SP_ITEM (knot_holder->item);
-	// this was a local change and the knotholder does not need to be recreated:
-	knot_holder->local_change = TRUE; 
-
-	for (GSList *el = knot_holder->entity; el; el = el->next) {
-		SPKnotHolderEntity *e = (SPKnotHolderEntity *)el->data;
-		if (e->knot == knot) {
-			NR::Point const q = *p * sp_item_i2d_affine(item).inverse();
-			e->knot_set (item, q, state);
-			break;
-		}
-	}
-	
-	sp_shape_set_shape (SP_SHAPE (item));
-
 	NR::Matrix const i2d(sp_item_i2d_affine(item));
 
 	for (GSList *el = knot_holder->entity; el; el = el->next) {
@@ -156,6 +157,51 @@ knot_moved_handler (SPKnot *knot, NR::Point *p, guint state, gpointer data)
 	}
 }
 
+static void
+knot_clicked_handler(SPKnot *knot, guint state, gpointer data)
+{
+	SPKnotHolder *knot_holder = (SPKnotHolder *) data;
+	SPItem *item  = SP_ITEM (knot_holder->item);
+
+	for (GSList *el = knot_holder->entity; el; el = el->next) {
+		SPKnotHolderEntity *e = (SPKnotHolderEntity *) el->data;
+		if (e->knot == knot) {
+			if (e->knot_click)
+				e->knot_click (item, state);
+			break;
+		}
+	}
+
+	sp_shape_set_shape (SP_SHAPE (item));
+
+	knotholder_update_knots (knot_holder, item);
+
+	// for drag, this is done by ungrabbed_handler, but for click we must do it here
+	sp_document_done (SP_OBJECT_DOCUMENT (knot_holder->item)); 
+}
+
+static void
+knot_moved_handler(SPKnot *knot, NR::Point const *p, guint state, gpointer data)
+{
+	SPKnotHolder *knot_holder = (SPKnotHolder *) data;
+	SPItem *item  = SP_ITEM (knot_holder->item);
+	// this was a local change and the knotholder does not need to be recreated:
+	knot_holder->local_change = TRUE; 
+
+	for (GSList *el = knot_holder->entity; el; el = el->next) {
+		SPKnotHolderEntity *e = (SPKnotHolderEntity *)el->data;
+		if (e->knot == knot) {
+			NR::Point const q = *p / sp_item_i2d_affine(item);
+			e->knot_set (item, q, e->knot->drag_origin / sp_item_i2d_affine(item), state);
+			break;
+		}
+	}
+
+	sp_shape_set_shape (SP_SHAPE (item));
+
+	knotholder_update_knots (knot_holder, item);
+}
+
 static void
 knot_ungrabbed_handler (SPKnot *knot, unsigned int state, SPKnotHolder *kh)
 {
@@ -163,7 +209,7 @@ knot_ungrabbed_handler (SPKnot *knot, unsigned int state, SPKnotHolder *kh)
 		kh->released (kh->item);
 	} else {
 		SPObject *object = (SPObject *) kh->item;
-		sp_object_invoke_write (object, object->repr, SP_OBJECT_WRITE_EXT);
+		object->updateRepr(object->repr, SP_OBJECT_WRITE_EXT);
 		sp_document_done (SP_OBJECT_DOCUMENT (kh->item));
 	}
 }
