@@ -13,77 +13,44 @@
 #include "nr-rect-l.h"
 #include "nr-matrix.h"
 
+/**
+ *    \param r0 Rectangle.
+ *    \param r1 Another rectangle.
+ *    \param d Filled in with the intersection of r0 and r1.
+ *    \return d.
+ */
+
+NRRectL *nr_rect_l_intersect(NRRectL *d, const NRRectL *r0, const NRRectL *r1)
+{
+  NR::ICoord t;
+  t = std::max(r0->x0, r1->x0);
+  d->x1 = std::min(r0->x1, r1->x1);
+  d->x0 = t;
+  t = std::max(r0->y0, r1->y0);
+  d->y1 = std::min(r0->y1, r1->y1);
+  d->y0 = t;
+	
+  return d;
+}
+
 NRRect *
 nr_rect_d_intersect (NRRect *d, const NRRect *r0, const NRRect *r1)
 {
-	if (!r0) {
-		if (!r1) {
-			d->x0 = -NR_HUGE;
-			d->y0 = -NR_HUGE;
-			d->x1 = NR_HUGE;
-			d->y1 = NR_HUGE;
-		} else {
-			*d = *r1;
-		}
-	} else {
-		if (!r1) {
-			*d = *r0;
-		} else {
-			NR::Coord t;
-			t = MAX (r0->x0, r1->x0);
-			d->x1 = MIN (r0->x1, r1->x1);
-			d->x0 = t;
-			t = MAX (r0->y0, r1->y0);
-			d->y1 = MIN (r0->y1, r1->y1);
-			d->y0 = t;
-		}
-	}
-	return d;
-}
-
-NRRect *
-nr_rect_f_intersect (NRRect *d, const NRRect *r0, const NRRect *r1)
-{
-	return nr_rect_d_intersect(d, r0, r1);
-}
-
-NRRectL *
-nr_rect_l_intersect (NRRectL *d, const NRRectL *r0, const NRRectL *r1)
-{
-	if (!r0) {
-		if (!r1) {
-			d->x0 = -NR_HUGE_L - 1;
-			d->y0 = -NR_HUGE_L - 1;
-			d->x1 = NR_HUGE_L;
-			d->y1 = NR_HUGE_L;
-		} else {
-			*d = *r1;
-		}
-	} else {
-		if (!r1) {
-			*d = *r0;
-		} else {
-			NR::ICoord t;
-			t = MAX (r0->x0, r1->x0);
-			d->x1 = MIN (r0->x1, r1->x1);
-			d->x0 = t;
-			t = MAX (r0->y0, r1->y0);
-			d->y1 = MIN (r0->y1, r1->y1);
-			d->y0 = t;
-		}
-	}
+	NR::Coord t;
+	t = MAX (r0->x0, r1->x0);
+	d->x1 = MIN (r0->x1, r1->x1);
+	d->x0 = t;
+	t = MAX (r0->y0, r1->y0);
+	d->y1 = MIN (r0->y1, r1->y1);
+	d->y0 = t;
+	
 	return d;
 }
 
 NRRect *
 nr_rect_d_union (NRRect *d, const NRRect *r0, const NRRect *r1)
 {
-	if (!r0 || !r1) {
-		d->x0 = -NR_HUGE;
-		d->y0 = -NR_HUGE;
-		d->x1 = NR_HUGE;
-		d->y1 = NR_HUGE;
-	} else if (NR_RECT_DFLS_TEST_EMPTY (r0)) {
+	if (NR_RECT_DFLS_TEST_EMPTY (r0)) {
 		if (NR_RECT_DFLS_TEST_EMPTY (r1)) {
 			nr_rect_d_set_empty (d);
 		} else {
@@ -105,21 +72,10 @@ nr_rect_d_union (NRRect *d, const NRRect *r0, const NRRect *r1)
 	return d;
 }
 
-NRRect *
-nr_rect_f_union (NRRect *d, const NRRect *r0, const NRRect *r1)
-{
-	return nr_rect_d_union(d, r0, r1);
-}
-
 NRRectL *
 nr_rect_l_union (NRRectL *d, const NRRectL *r0, const NRRectL *r1)
 {
-	if (!r0 || !r1) {
-		d->x0 = -NR_HUGE_L - 1;
-		d->y0 = -NR_HUGE_L - 1;
-		d->x1 = NR_HUGE_L;
-		d->y1 = NR_HUGE_L;
-	} else if (NR_RECT_DFLS_TEST_EMPTY (r0)) {
+	if (NR_RECT_DFLS_TEST_EMPTY (r0)) {
 		if (NR_RECT_DFLS_TEST_EMPTY (r1)) {
 			nr_rect_l_set_empty (d);
 		} else {
@@ -141,48 +97,32 @@ nr_rect_l_union (NRRectL *d, const NRRectL *r0, const NRRectL *r1)
 	return d;
 }
 
+NRRect *
+nr_rect_union_pt(NRRect *dst, NR::Point const &p)
+{
+	using NR::X;
+	using NR::Y;
+
+	return nr_rect_d_union_xy(dst, p[X], p[Y]);
+}
+
 NRRect *
 nr_rect_d_union_xy (NRRect *d, NR::Coord x, NR::Coord y)
 {
-	if (d) {
-		if ((d->x0 <= d->x1) && (d->y0 <= d->y1)) {
-			d->x0 = MIN (d->x0, x);
-			d->y0 = MIN (d->y0, y);
-			d->x1 = MAX (d->x1, x);
-			d->y1 = MAX (d->y1, y);
-		} else {
-			d->x0 = d->x1 = x;
-			d->y0 = d->y1 = y;
-		}
+	if ((d->x0 <= d->x1) && (d->y0 <= d->y1)) {
+		d->x0 = MIN (d->x0, x);
+		d->y0 = MIN (d->y0, y);
+		d->x1 = MAX (d->x1, x);
+		d->y1 = MAX (d->y1, y);
+	} else {
+		d->x0 = d->x1 = x;
+		d->y0 = d->y1 = y;
 	}
 	return d;
 }
 
 NRRect *
-nr_rect_f_union_xy (NRRect *d, float x, float y)
-{
-	return nr_rect_d_union_xy(d, x, y);
-}
-
-NRRectL *
-nr_rect_l_union_xy (NRRectL *d, NR::ICoord x, NR::ICoord y)
-{
-	if (d) {
-		if ((d->x0 <= d->x1) && (d->y0 <= d->y1)) {
-			d->x0 = MIN (d->x0, x);
-			d->y0 = MIN (d->y0, y);
-			d->x1 = MAX (d->x1, x);
-			d->y1 = MAX (d->y1, y);
-		} else {
-			d->x0 = d->x1 = x;
-			d->y0 = d->y1 = y;
-		}
-	}
-	return d;
-}
-
-NRRect *
-nr_rect_d_matrix_d_transform (NRRect *d, NRRect *s, NRMatrix *m)
+nr_rect_d_matrix_transform (NRRect *d, NRRect *s, NRMatrix const *m)
 {
 	if (nr_rect_d_test_empty (s)) {
 		nr_rect_d_set_empty (d);
@@ -212,13 +152,6 @@ nr_rect_d_matrix_d_transform (NRRect *d, NRRect *s, NRMatrix *m)
 	return d;
 }
 
-NRRect *
-nr_rect_f_matrix_f_transform (NRRect *d, NRRect *s, NRMatrix *m)
-{
-	return nr_rect_d_matrix_d_transform(d, s, m);
-}
-
-
 namespace NR {
 
 Rect::Rect(const Point &p0, const Point &p1)
@@ -227,15 +160,15 @@ Rect::Rect(const Point &p0, const Point &p1)
 
 /** returns the four corners of the rectangle in the correct winding order */
 Point Rect::corner(unsigned i) const {
-	switch(i % 4) {
+	switch (i % 4) {
 	case 0:
-		return topleft();
+		return _min;
 	case 1:
-		return topright();
+		return Point(_max[X], _min[Y]);
 	case 2:
-		return bottomright();
+		return _max;
 	default: /* i.e. 3 */
-		return bottomleft();
+		return Point(_min[X], _max[Y]);
 	}
 }
 
@@ -288,3 +221,15 @@ Rect Rect::union_bounds(const Rect &a, const Rect &b) {
 }
 
 };
+
+
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
