@@ -7,6 +7,9 @@
 
 namespace NR {
 
+/** Notionally an NR::Matrix corresponding to rotation about the origin.
+    Behaves like NR::Matrix for multiplication.
+**/
 class rotate {
 public:
     Point vec;
@@ -17,6 +20,7 @@ private:
 public:
     explicit rotate(Coord theta);
     explicit rotate(Point const &p) : vec(p) {}
+    explicit rotate(Coord const x, Coord const y) : vec(x, y) {}
 
     bool operator==(rotate const &o) const {
         return vec == o.vec;
@@ -26,6 +30,9 @@ public:
         return vec != o.vec;
     }
 
+    inline rotate &operator*=(rotate const &b);
+    /* Defined in nr-rotate-ops.h. */
+
     rotate inverse() const {
         /* TODO: In the usual case that vec is a unit vector (within rounding error),
            dividing by len_sq is either a noop or numerically harmful.
