@@ -10,15 +10,18 @@
  * which is Copyright (C) Masatake Yamato 2002
  */
 
+#include <config.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <string.h>
 #include "macros.h"
 #include "xml/repr.h"
 #include "svg/svg.h"
-#include "helper/sp-canvas-util.h"
+#include "helper/sp-intl.h"
+#include "display/sp-canvas-util.h"
 #include "object-edit.h"
 #include "sp-path.h"
+#include "path-chemistry.h"
 #include "rubberband.h"
 #include "desktop.h"
 #include "desktop-affine.h"
@@ -33,7 +36,7 @@
 #include "document.h"
 #include "prefs-utils.h"
 #include "xml/repr.h"
-#include "xml/repr-private.h"
+#include "xml/sp-repr-event-vector.h"
 
 static void sp_node_context_class_init (SPNodeContextClass * klass);
 static void sp_node_context_init (SPNodeContext * node_context);
@@ -43,11 +46,8 @@ static void sp_node_context_setup (SPEventContext *ec);
 static gint sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 static gint sp_node_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event);
 
-static gboolean sp_node_context_stamp (SPNodeContext * node_context);
-
 static void nodepath_event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value, const gchar * new_value, bool is_interactive, gpointer data);
 static SPReprEventVector nodepath_repr_events = {
-	NULL, /* destroy */
 	NULL, /* add_child */
 	NULL, /* child_added */
 	NULL, /* remove_child */
@@ -126,16 +126,18 @@ sp_node_context_init (SPNodeContext * node_context)
 	node_context -> rightalt = FALSE;
 	node_context -> leftctrl = FALSE;
 	node_context -> rightctrl = FALSE;
+
+	new (&node_context->sel_changed_connection) sigc::connection();
 }
 
 static void
 sp_node_context_dispose (GObject *object)
 {
-	SPNodeContext * nc;
-	SPEventContext * ec;
+	SPNodeContext * nc = SP_NODE_CONTEXT (object);
+	SPEventContext * ec = SP_EVENT_CONTEXT (object);
 
-	nc = SP_NODE_CONTEXT (object);
-	ec = SP_EVENT_CONTEXT (object);
+	nc->sel_changed_connection.disconnect();
+	nc->sel_changed_connection.~connection();
 
 	if (nc->nodepath) {
 		sp_repr_remove_listener_by_data (nc->nodepath->repr, ec);
@@ -151,8 +153,8 @@ sp_node_context_dispose (GObject *object)
 		nc->knot_holder = NULL;
 	}
 
-	if (SP_EVENT_CONTEXT_DESKTOP (nc)) {
-		sp_signal_disconnect_by_data (SP_DT_SELECTION (SP_EVENT_CONTEXT_DESKTOP (nc)), nc);
+	if (nc->_node_message_context) {
+		delete nc->_node_message_context;
 	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -161,18 +163,16 @@ sp_node_context_dispose (GObject *object)
 static void
 sp_node_context_setup (SPEventContext *ec)
 {
-	SPNodeContext *nc;
-	SPItem *item;
-	nc = SP_NODE_CONTEXT (ec);
-	SPRepr *repr;
+	SPNodeContext *nc = SP_NODE_CONTEXT (ec);
 
 	if (((SPEventContextClass *) parent_class)->setup)
 		((SPEventContextClass *) parent_class)->setup (ec);
 
-	g_signal_connect (G_OBJECT (SP_DT_SELECTION (ec->desktop)), 
-		"changed", G_CALLBACK (sp_node_context_selection_changed), nc);
+	nc->sel_changed_connection.disconnect();
+	nc->sel_changed_connection = SP_DT_SELECTION(ec->desktop)->connectChanged(sigc::bind(sigc::ptr_fun(&sp_node_context_selection_changed), (gpointer)nc));
 
-	item = sp_selection_item (SP_DT_SELECTION (ec->desktop));
+	SPSelection *selection = SP_DT_SELECTION (ec->desktop);
+	SPItem *item = selection->singleItem();
 
 	nc->nodepath = NULL;
 	nc->knot_holder = NULL;
@@ -182,21 +182,26 @@ sp_node_context_setup (SPEventContext *ec)
 		if ( nc->nodepath) {
 			//point pack to parent in case nodepath is deleted
 			nc->nodepath->nodeContext = nc;
-		}
-		else {
+		} else {
 			nc->knot_holder = sp_item_knot_holder (item, ec->desktop);
 		}
 
-
-		// setting listener
-		repr = SP_OBJECT (item)->repr;
-		if (repr) {
-			sp_repr_ref (repr);
-			sp_repr_add_listener (repr, &nodepath_repr_events, ec);
-			sp_repr_synthesize_events (repr, &nodepath_repr_events, ec);
+		if (nc->nodepath || nc->knot_holder) {
+			// setting listener
+			SPRepr *repr = SP_OBJECT_REPR (item);
+			if (repr) {
+				sp_repr_ref (repr);
+				sp_repr_add_listener (repr, &nodepath_repr_events, ec);
+				sp_repr_synthesize_events (repr, &nodepath_repr_events, ec);
+			}
 		}
 	}
 
+	if (prefs_get_int_attribute("tools.nodes", "selcue", 0) != 0) {
+		ec->enableSelectionCue();
+	}
+
+	nc -> _node_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 	sp_nodepath_update_statusbar (nc->nodepath);
 }
 
@@ -207,14 +212,10 @@ destroys old and creates new nodepath and reassigns listeners to the new selecte
 void
 sp_node_context_selection_changed (SPSelection * selection, gpointer data)
 {
-	SPNodeContext * nc;
-	SPEventContext * ec;
-	SPDesktop *desktop;
-	SPItem * item;
-	SPRepr *old_repr = NULL, *repr;
+	SPNodeContext * nc = SP_NODE_CONTEXT (data);
+	SPEventContext * ec = SP_EVENT_CONTEXT (nc);
 
-	nc = SP_NODE_CONTEXT (data);
-	ec = SP_EVENT_CONTEXT (nc);
+	SPRepr *old_repr = NULL;
 
 	if (nc->nodepath) {
 		old_repr = nc->nodepath->repr;
@@ -230,22 +231,27 @@ sp_node_context_selection_changed (SPSelection * selection, gpointer data)
 		sp_repr_unref (old_repr);
 	}
 
-	item = sp_selection_item (selection);
+	SPItem * item = selection->singleItem();
 	
-	desktop = selection->desktop;
+	SPDesktop *desktop = selection->desktop();
 	nc->nodepath = NULL;
 	nc->knot_holder = NULL;
 	if (item) {
 		nc->nodepath = sp_nodepath_new (desktop, item);
-		if (! nc->nodepath) {
+		if (nc->nodepath) {
+			nc->nodepath->nodeContext = nc;
+		} else {
 			nc->knot_holder = sp_item_knot_holder (item, desktop);
 		}
-		// setting new listener
-		repr = SP_OBJECT (item)->repr;
-		if (repr) {
-			sp_repr_ref (repr);
-			sp_repr_add_listener (repr, &nodepath_repr_events, ec);
-			sp_repr_synthesize_events (repr, &nodepath_repr_events, ec);
+
+		if (nc->nodepath || nc->knot_holder) {
+			// setting new listener
+			SPRepr *repr = SP_OBJECT_REPR (item);
+			if (repr) {
+				sp_repr_ref (repr);
+				sp_repr_add_listener (repr, &nodepath_repr_events, ec);
+				sp_repr_synthesize_events (repr, &nodepath_repr_events, ec);
+			}
 		}
 	}
 	sp_nodepath_update_statusbar (nc->nodepath);
@@ -273,7 +279,8 @@ sp_nodepath_update_from_item (SPNodeContext *nc, SPItem *item)
 		sp_knot_holder_destroy (nc->knot_holder);
 	}
 
-	item = sp_selection_item (SP_DT_SELECTION (desktop));
+	SPSelection *selection = SP_DT_SELECTION (desktop);
+	item = selection->singleItem();
 
 	nc->nodepath = NULL;
 	nc->knot_holder = NULL;
@@ -334,6 +341,15 @@ nodepath_event_attr_changed (SPRepr * repr, const gchar * name, const gchar * ol
 	sp_nodepath_update_statusbar (nc->nodepath);
 }
 
+void
+sp_node_context_show_modifier_tip (SPEventContext *event_context, GdkEvent *event)
+{
+       sp_event_show_modifier_tip (event_context->defaultMessageContext(), event,
+                 _("<b>Ctrl</b>: toggle node type, snap handle angle, move hor/vert; <b>Ctrl+Alt</b>: move along handles"),
+                 _("<b>Shift</b>: toggle node selection, disable snapping, rotate both handles"),
+                 _("<b>Alt</b>: lock handle length; <b>Ctrl+Alt</b>: move along handles"));
+}
+
 static gint
 sp_node_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
 {
@@ -354,8 +370,8 @@ sp_node_context_item_handler (SPEventContext * event_context, SPItem * item, Gdk
 				if (!nc->drag) {
 					// find out clicked item, disregarding groups
 					item_ungrouped = sp_desktop_item_at_point (desktop, NR::Point(event->button.x, event->button.y), TRUE);
-					sp_selection_set_item (SP_DT_SELECTION (desktop), item_ungrouped);
-					ret = FALSE;
+					SP_DT_SELECTION (desktop)->setItem (item_ungrouped);
+					ret = TRUE;
 				}
 				break;
 			}
@@ -380,7 +396,7 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	double const nudge = prefs_get_double_attribute_limited("options.nudgedistance", "value", 2.8346457, 0, 1000); // default is 1 mm
 	tolerance = prefs_get_int_attribute_limited ("options.dragtolerance", "value", 0, 0, 100);
 	int const snaps = prefs_get_int_attribute("options.rotationsnapsperpi", "value", 12);
-	double const offset = prefs_get_double_attribute_limited("options.defaultoffsetwidth", "value", 2, 0, 1000);
+	double const offset = prefs_get_double_attribute_limited("options.defaultscale", "value", 2, 0, 1000);
 
 	gint ret = FALSE;
 
@@ -429,8 +445,12 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 					sp_nodepath_select_rect (nc->nodepath, &b, event->button.state & GDK_SHIFT_MASK);
 				}
 			} else {
-				if (!(nodeedit_rb_escaped)) // unless something was cancelled
-					sp_nodepath_deselect (nc->nodepath); 
+				if (!(nodeedit_rb_escaped)) { // unless something was cancelled
+					if (nc->nodepath && nc->nodepath->selected)
+						sp_nodepath_deselect (nc->nodepath);
+					else 
+						SP_DT_SELECTION(desktop)->clear();
+				}
 			}
 			ret = TRUE;
 			sp_rubberband_stop ();
@@ -498,30 +518,25 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 		case GDK_L:
 		case GDK_l:
 			if (MOD__SHIFT_ONLY) {
-				sp_node_selected_set_line_type (ART_LINETO);
+				sp_node_selected_set_line_type (NR_LINETO);
 				ret = TRUE;
 			}
 			break;
 		case GDK_K:
 		case GDK_k:
 			if (MOD__SHIFT_ONLY) {
-				sp_node_selected_set_line_type (ART_CURVETO);
+				sp_node_selected_set_line_type (NR_CURVETO);
 				ret = TRUE;
 			}
 			break;
-		case GDK_space:
-			// stamping when node-editing is a strange idea - the entire shape is duplicated 
-			// it may be useful however, so let it be 
-			if ((event->key.state & GDK_BUTTON1_MASK)
-					&& (nc->nodepath == NULL)) {
-				ret = sp_node_context_stamp(nc);
+		case GDK_R:
+		case GDK_r:
+			if (!MOD__CTRL && !MOD__ALT) {
+				// FIXME: add top panel button
+				sp_selected_path_reverse ();
+				ret = TRUE;
 			}
 			break;
-
-			// arrow keys:
-			// this was copied over from select-context.c, 
-			// with "sp_selection_" replaced by "sp_node_selected"; 
-			// maybe we could make this a shared function?
 		case GDK_Left: // move selection left
 		case GDK_KP_Left: 
 		case GDK_KP_4: 
@@ -601,7 +616,10 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 				sp_rubberband_stop ();
 				nodeedit_rb_escaped = 1;
 			} else {
-				sp_nodepath_deselect (nc->nodepath); // deselect
+				if (nc->nodepath && nc->nodepath->selected)
+					sp_nodepath_deselect (nc->nodepath);
+				else 
+					SP_DT_SELECTION(desktop)->clear();
 			}
 			ret = TRUE;
 			break;
@@ -692,15 +710,25 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 
 		case GDK_Alt_L:
 			nc->leftalt = TRUE;
+			sp_node_context_show_modifier_tip (event_context, event);
 			break;
 		case GDK_Alt_R:
 			nc->rightalt = TRUE;
+			sp_node_context_show_modifier_tip (event_context, event);
 			break;
 		case GDK_Control_L:
 			nc->leftctrl = TRUE;
+			sp_node_context_show_modifier_tip (event_context, event);
 			break;
 		case GDK_Control_R:
 			nc->rightctrl = TRUE;
+			sp_node_context_show_modifier_tip (event_context, event);
+			break;
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Meta_L:
+		case GDK_Meta_R:
+			sp_node_context_show_modifier_tip (event_context, event);
 			break;
 		default:
 			ret = node_key (event);
@@ -711,15 +739,25 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 		switch (event->key.keyval) {
 		case GDK_Alt_L:
 			nc->leftalt = FALSE;
+			event_context->defaultMessageContext()->clear();
 			break;
 		case GDK_Alt_R:
 			nc->rightalt = FALSE;
+			event_context->defaultMessageContext()->clear();
 			break;
 		case GDK_Control_L:
 			nc->leftctrl = FALSE;
+			event_context->defaultMessageContext()->clear();
 			break;
 		case GDK_Control_R:
 			nc->rightctrl = FALSE;
+			event_context->defaultMessageContext()->clear();
+			break;
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Meta_L:
+		case GDK_Meta_R:
+			event_context->defaultMessageContext()->clear();
 			break;
 		}
 		break;
@@ -735,30 +773,3 @@ sp_node_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 	return ret;
 }
 
-static gboolean
-sp_node_context_stamp (SPNodeContext * nc)
-{
-	SPItem * original_item, * copy_item;
-	SPRepr * original_repr, * copy_repr;
-	gchar tstr[80];
-	SPEventContext * ec;
-	
-	ec = SP_EVENT_CONTEXT(nc);
-	original_item = sp_selection_item (SP_DT_SELECTION (ec->desktop));
-	if (original_item) {
-		original_repr = (SPRepr *)(SP_OBJECT (original_item)->repr);
-		copy_repr = sp_repr_duplicate (original_repr);
-		copy_item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (ec->desktop), 
-							     copy_repr);
-		
-		if (sp_svg_transform_write (tstr, 80, &original_item->transform)) {
-			sp_repr_set_attr (copy_repr, "transform", tstr);
-		} else {
-			sp_repr_set_attr (copy_repr, "transform", NULL);
-		}
-		
-		sp_repr_unref (copy_repr);
-		sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-	}
-	return TRUE;
-}
