@@ -1,20 +1,24 @@
 #ifndef __SP_NODEPATH_H__
 #define __SP_NODEPATH_H__
 
-/**
+/** \file
  * Path handler in node edit mode
- *
+ */
+
+/*
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
  * This code is in public domain
  */
 
-#include "xml/repr.h"
 #include "knot.h"
 #include "sp-path.h"
 #include "desktop-handles.h"
+#include "libnr/nr-path-code.h"
+#include "xml/xml-forward.h"
 
+#include <list>
 
 /** Radial objects are represented by an angle and a distance from
  * 0,0.  0,0 is represented by a == big_num.
@@ -55,7 +59,7 @@ operator NR::Point() const
 
 
 /** defined in node-context.h */
-typedef struct _SPNodeContext SPNodeContext;
+class SPNodeContext;
 
 
 
@@ -115,6 +119,8 @@ class Path {
 	NR::Matrix i2d, d2i;
 /**  The DOM node which describes this NodePath */
 	SPRepr *repr;
+	//STL compliant method to get the selected nodes
+	void selection(std::list<Node *> &l);
 };
 
 
@@ -271,9 +277,15 @@ gboolean node_key (GdkEvent * event);
  */
 void sp_nodepath_update_statusbar (Path::Path *nodepath);
 
+/**
+ * Align selected nodes on the specified axis
+ */
+void sp_nodepath_selected_align(Path::Path *nodepath, NR::Dim2 axis);
 
-
-
+/**
+ * Distribute selected nodes on the specified axis
+ */
+void sp_nodepath_selected_distribute(Path::Path *nodepath, NR::Dim2 axis);
 
 /* possibly private functions */
 
@@ -321,7 +333,7 @@ void sp_node_selected_set_type (Path::NodeType type);
 /**
  *
  */
-void sp_node_selected_set_line_type (ArtPathcode code);
+void sp_node_selected_set_line_type (NRPathcode code);
 
 /**
  *
@@ -347,4 +359,6 @@ void sp_nodepath_selected_nodes_scale_screen (Path::Path * nodepath, gdouble gro
 
 
 
+
+
 #endif
