@@ -24,7 +24,7 @@
 #include "sp-item.h"
 #include "style.h"
 #include "sp-paint-server.h"
-#include "extension/extension.h"
+#include "extension/print.h"
 #include "extension/system.h"
 #include "print.h"
 
@@ -42,10 +42,6 @@
 
 /* Identity typedef */
 
-struct _SPPrintContext {
-	Inkscape::Extension::Print * module;
-};
-
 unsigned int sp_print_bind(SPPrintContext *ctx, NR::Matrix const &transform, float opacity)
 {
 	NRMatrix const ntransform(transform);
@@ -86,6 +82,11 @@ sp_print_image_R8G8B8A8_N (SPPrintContext *ctx,
 	return ctx->module->image(px, w, h, rs, transform, style);
 }
 
+unsigned int sp_print_text (SPPrintContext* ctx, const char* text, NR::Point p,
+			    const SPStyle* style)
+{
+  return ctx->module->text(text, p, style);
+}
 
 #include "display/nr-arena.h"
 #include "display/nr-arena-item.h"
@@ -100,7 +101,7 @@ sp_print_preview_document (SPDocument *doc)
 
 	sp_document_ensure_up_to_date (doc);
 
-	mod = sp_module_system_get_print(SP_MODULE_KEY_PRINT_DEFAULT);
+	mod = Inkscape::Extension::get_print(SP_MODULE_KEY_PRINT_DEFAULT);
 
 	ret = mod->set_preview ();
 
@@ -111,7 +112,7 @@ sp_print_preview_document (SPDocument *doc)
 		/* fixme: This has to go into module constructor somehow */
 		/* Create new arena */
 		mod->base = SP_ITEM (sp_document_root (doc));
-		mod->arena = (NRArena *) nr_object_new (NR_TYPE_ARENA);
+		mod->arena = NRArena::create();
 		mod->dkey = sp_item_display_key_new (1);
 		mod->root = sp_item_invoke_show (mod->base, mod->arena, mod->dkey, SP_ITEM_SHOW_PRINT);
 		/* Print document */
@@ -139,9 +140,9 @@ sp_print_document (SPDocument *doc, unsigned int direct)
 	sp_document_ensure_up_to_date (doc);
 
 	if (direct) {
-		mod = sp_module_system_get_print(SP_MODULE_KEY_PRINT_PS);
+		mod = Inkscape::Extension::get_print(SP_MODULE_KEY_PRINT_PS);
 	} else {
-		mod = sp_module_system_get_print(SP_MODULE_KEY_PRINT_DEFAULT);
+		mod = Inkscape::Extension::get_print(SP_MODULE_KEY_PRINT_DEFAULT);
 	}
 
 	ret = mod->setup();
@@ -153,7 +154,7 @@ sp_print_document (SPDocument *doc, unsigned int direct)
 		/* fixme: This has to go into module constructor somehow */
 		/* Create new arena */
 		mod->base = SP_ITEM (sp_document_root (doc));
-		mod->arena = (NRArena *) nr_object_new (NR_TYPE_ARENA);
+		mod->arena = NRArena::create();
 		mod->dkey = sp_item_display_key_new (1);
 		mod->root = sp_item_invoke_show (mod->base, mod->arena, mod->dkey, SP_ITEM_SHOW_PRINT);
 		/* Print document */
@@ -177,22 +178,23 @@ sp_print_document_to_file (SPDocument *doc, const gchar *filename)
 {
 	Inkscape::Extension::Print *mod;
 	SPPrintContext context;
+	const gchar * oldconst;
 	gchar * oldoutput;
 	unsigned int ret;
 
 	sp_document_ensure_up_to_date (doc);
 
-	mod = sp_module_system_get_print(SP_MODULE_KEY_PRINT_PS);
-	mod->get_param("destination", (gchar **)&oldoutput);
-	oldoutput = g_strdup(oldoutput);
-	mod->set_param("destination", (gchar *)filename);
+	mod = Inkscape::Extension::get_print(SP_MODULE_KEY_PRINT_PS);
+	oldconst = mod->get_param_string("destination");
+	oldoutput = g_strdup(oldconst);
+	mod->set_param_string("destination", (gchar *)filename);
 
 /* Start */
 	context.module = mod;
 	/* fixme: This has to go into module constructor somehow */
 	/* Create new arena */
 	mod->base = SP_ITEM (sp_document_root (doc));
-	mod->arena = (NRArena *) nr_object_new (NR_TYPE_ARENA);
+	mod->arena = NRArena::create();
 	mod->dkey = sp_item_display_key_new (1);
 	mod->root = sp_item_invoke_show (mod->base, mod->arena, mod->dkey, SP_ITEM_SHOW_PRINT);
 	/* Print document */
@@ -208,7 +210,7 @@ sp_print_document_to_file (SPDocument *doc, const gchar *filename)
 	mod->arena = NULL;
 /* end */
 
-	mod->set_param("destination", (gchar *)oldoutput);
+	mod->set_param_string("destination", oldoutput);
 	g_free(oldoutput);
 
 	return;
