@@ -13,7 +13,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -21,24 +22,28 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "macros.h"
-#include "helper/sp-canvas.h"
+#include "display/sp-canvas.h"
 #include "sp-rect.h"
 #include "inkscape.h"
 #include "document.h"
 #include "selection.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
-#include "desktop-snap.h"
+#include "snap.h"
+#include "desktop.h"
+#include "desktop-style.h"
 #include "pixmaps/cursor-rect.xpm"
 #include "rect-context.h"
+#include "sp-desktop-widget.h"
 #include "sp-metrics.h"
 #include "helper/sp-intl.h"
 #include "object-edit.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/repr-private.h"
+#include "xml/sp-repr-event-vector.h"
 #include "prefs-utils.h"
-
+#include "widgets/spw-utilities.h"
+#include "selcue.h"
 
 static void sp_rect_context_class_init(SPRectContextClass *klass);
 static void sp_rect_context_init(SPRectContext *rect_context);
@@ -49,7 +54,6 @@ static void sp_rect_context_set(SPEventContext *ec, gchar const *key, gchar cons
 
 static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent *event);
 static gint sp_rect_context_item_handler(SPEventContext *event_context, SPItem *item, GdkEvent *event);
-static GtkWidget *sp_rect_context_config_widget(SPEventContext *ec);
 
 static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state);
 static void sp_rect_finish(SPRectContext *rc);
@@ -85,13 +89,10 @@ static void sp_rect_context_class_init(SPRectContextClass *klass)
 
     object_class->dispose = sp_rect_context_dispose;
 
-#if 1
     event_context_class->setup = sp_rect_context_setup;
     event_context_class->set = sp_rect_context_set;
-#endif
     event_context_class->root_handler  = sp_rect_context_root_handler;
     event_context_class->item_handler  = sp_rect_context_item_handler;
-    event_context_class->config_widget = sp_rect_context_config_widget;
 }
 
 static void sp_rect_context_init(SPRectContext *rect_context)
@@ -108,9 +109,13 @@ static void sp_rect_context_init(SPRectContext *rect_context)
     event_context->item_to_select = NULL;
 
     rect_context->item = NULL;
+    rect_context->repr = NULL;
+    rect_context->knot_holder = NULL;
 
-    rect_context->rx_ratio = 0.0;
-    rect_context->ry_ratio = 0.0;
+    rect_context->rx = 0.0;
+    rect_context->ry = 0.0;
+
+    new (&rect_context->sel_changed_connection) sigc::connection();
 }
 
 static void sp_rect_context_dispose(GObject *object)
@@ -118,6 +123,9 @@ static void sp_rect_context_dispose(GObject *object)
     SPRectContext *rc = SP_RECT_CONTEXT(object);
     SPEventContext *ec = SP_EVENT_CONTEXT(object);
 
+    rc->sel_changed_connection.disconnect();
+    rc->sel_changed_connection.~connection();
+
     /* fixme: This is necessary because we do not grab */
     if (rc->item) {
         sp_rect_finish(rc);
@@ -134,8 +142,8 @@ static void sp_rect_context_dispose(GObject *object)
         rc->repr = 0;
     }
 
-    if (SP_EVENT_CONTEXT_DESKTOP(rc)) {
-        sp_signal_disconnect_by_data(SP_DT_SELECTION(SP_EVENT_CONTEXT_DESKTOP(rc)), rc);
+    if (rc->_message_context) {
+        delete rc->_message_context;
     }
 
     G_OBJECT_CLASS(parent_class)->dispose(object);
@@ -155,7 +163,7 @@ static void shape_event_attr_changed(SPRepr *repr,
 
     SPDesktop *desktop = ec->desktop;
 
-    SPItem *item = sp_selection_item(SP_DT_SELECTION(desktop));
+    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
 
     if (item) {
         rc->knot_holder = sp_item_knot_holder(item, desktop);
@@ -163,7 +171,6 @@ static void shape_event_attr_changed(SPRepr *repr,
 }
 
 static SPReprEventVector shape_repr_events = {
-    NULL, /* destroy */
     NULL, /* add_child */
     NULL, /* child_added */
     NULL, /* remove_child */
@@ -196,7 +203,7 @@ void sp_rect_context_selection_changed(SPSelection *selection, gpointer data)
         rc->repr = 0;
     }
 
-    SPItem *item = sp_selection_item(selection);
+    SPItem *item = selection->singleItem();
     if (item) {
         rc->knot_holder = sp_item_knot_holder(item, ec->desktop);
         SPRepr *repr = SP_OBJECT_REPR(item);
@@ -217,7 +224,7 @@ static void sp_rect_context_setup(SPEventContext *ec)
         ((SPEventContextClass *) parent_class)->setup(ec);
     }
 
-    SPItem *item = sp_selection_item(SP_DT_SELECTION(ec->desktop));
+    SPItem *item = SP_DT_SELECTION(ec->desktop)->singleItem();
     if (item) {
         rc->knot_holder = sp_item_knot_holder(item, ec->desktop);
         SPRepr *repr = SP_OBJECT_REPR(item);
@@ -229,11 +236,19 @@ static void sp_rect_context_setup(SPEventContext *ec)
         }
     }
 
-    g_signal_connect(G_OBJECT(SP_DT_SELECTION(ec->desktop)),
-                     "changed", G_CALLBACK(sp_rect_context_selection_changed), rc);
+    rc->sel_changed_connection.disconnect();
+    rc->sel_changed_connection = SP_DT_SELECTION(ec->desktop)->connectChanged(
+        sigc::bind(sigc::ptr_fun(&sp_rect_context_selection_changed), (gpointer)rc)
+    );
 
-    sp_event_context_read(ec, "rx_ratio");
-    sp_event_context_read(ec, "ry_ratio");
+    sp_event_context_read(ec, "rx");
+    sp_event_context_read(ec, "ry");
+
+    if (prefs_get_int_attribute("tools.shapes", "selcue", 0) != 0) {
+        ec->enableSelectionCue();
+    }
+
+    rc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
 static void sp_rect_context_set(SPEventContext *ec, gchar const *key, gchar const *val)
@@ -242,16 +257,14 @@ static void sp_rect_context_set(SPEventContext *ec, gchar const *key, gchar cons
 
     /* fixme: Proper error handling for non-numeric data.  Use a locale-independent function like
      * g_ascii_strtod (or a thin wrapper that does the right thing for invalid values inf/nan). */
-    if ( strcmp(key, "rx_ratio") == 0 ) {
-        rc->rx_ratio = ( val
-                         ? atof(val)
+    if ( strcmp(key, "rx") == 0 ) {
+        rc->rx = ( val
+                         ? g_ascii_strtod (val, NULL)
                          : 0.0 );
-        rc->rx_ratio = CLAMP(rc->rx_ratio, 0.0, 1.0);
-    } else if ( strcmp(key, "ry_ratio") == 0 ) {
-        rc->ry_ratio = ( val
-                         ? atof(val)
+    } else if ( strcmp(key, "ry") == 0 ) {
+        rc->ry = ( val
+                         ? g_ascii_strtod (val, NULL)
                          : 0.0 );
-        rc->ry_ratio = CLAMP(rc->ry_ratio, 0.0, 1.0);
     }
 }
 
@@ -317,7 +330,7 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
             NR::Point const button_dt(sp_desktop_w2d_xy_point(event_context->desktop, button_w));
             /* Snap center to nearest magnetic point */
             rc->center = button_dt;
-            sp_desktop_free_snap(event_context->desktop, rc->center);
+            namedview_free_snap_all_types(event_context->desktop->namedview, rc->center);
             sp_canvas_item_grab(SP_CANVAS_ITEM(desktop->acetate),
                                 ( GDK_BUTTON_RELEASE_MASK       |
                                   GDK_POINTER_MOTION_MASK       |
@@ -358,11 +371,10 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
                 sp_rect_finish(rc);
             } else if (event_context->item_to_select) {
                 // no dragging, select clicked item if any
-                sp_selection_set_item(SP_DT_SELECTION(desktop),
-                                      event_context->item_to_select);
+                SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
             } else {
                 // click in an empty space
-                sp_selection_empty(SP_DT_SELECTION(desktop));
+                SP_DT_SELECTION(desktop)->clear();
             }
 
             event_context->item_to_select = NULL;
@@ -373,6 +385,19 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
         break;
     case GDK_KEY_PRESS:
         switch (event->key.keyval) {
+        case GDK_Alt_L:
+        case GDK_Alt_R:
+        case GDK_Control_L:
+        case GDK_Control_R:
+        case GDK_Shift_L:
+        case GDK_Shift_R:
+        case GDK_Meta_L:  // Meta is when you press Shift+Alt (at least on my machine)
+        case GDK_Meta_R:
+            sp_event_show_modifier_tip (event_context->defaultMessageContext(), event,
+                                        _("<b>Ctrl</b>: make square or integer-ratio rect, lock a rounded corner circular"),
+                                        _("<b>Shift</b>: draw around the starting point"),
+                                        NULL);
+            break;
         case GDK_Up:
         case GDK_Down:
         case GDK_KP_Up:
@@ -381,12 +406,41 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
             if (!MOD__CTRL_ONLY)
                 ret = TRUE;
             break;
+
+        case GDK_x:
+        case GDK_X:
+            if (MOD__ALT_ONLY) {
+                gpointer hb = sp_search_by_data_recursive (desktop->owner->aux_toolbox, (gpointer) "altx-rect");
+                if (hb && GTK_IS_WIDGET(hb)) {
+                    gtk_widget_grab_focus (GTK_WIDGET (hb));
+                }
+                ret = TRUE;
+            }
+            break;
+
         case GDK_Escape:
-            sp_selection_empty(SP_DT_SELECTION(desktop)); // deselect
+            SP_DT_SELECTION(desktop)->clear();
             //TODO: make dragging escapable by Esc
         default:
             break;
         }
+        break;
+    case GDK_KEY_RELEASE:
+        switch (event->key.keyval) {
+        case GDK_Alt_L:
+        case GDK_Alt_R:
+        case GDK_Control_L:
+        case GDK_Control_R:
+        case GDK_Shift_L:
+        case GDK_Shift_R:
+        case GDK_Meta_L:  // Meta is when you press Shift+Alt
+        case GDK_Meta_R:
+            event_context->defaultMessageContext()->clear();
+            break;
+        default:
+            break;
+        }
+        break;
     default:
         break;
     }
@@ -405,20 +459,28 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
     SPDesktop *desktop = SP_EVENT_CONTEXT(&rc)->desktop;
 
     if (!rc.item) {
+
+        SPItem *layer=SP_ITEM(desktop->currentLayer());
+        if ( !layer || desktop->itemIsHidden(layer)) {
+            rc._message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is hidden</b>. Unhide it to be able to draw on it."));
+            return;
+        }
+        if ( !layer || layer->isLocked()) {
+            rc._message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is locked</b>. Unlock it to be able to draw on it."));
+            return;
+        }
+
         /* Create object */
         SPRepr *repr = sp_repr_new("rect");
-        /* Set style */
-        SPRepr *style = inkscape_get_repr(INKSCAPE, "tools.shapes.rect");
-        if (style) {
-            SPCSSAttr *css = sp_repr_css_attr_inherited(style, "style");
-            sp_repr_css_set(repr, css, "style");
-            sp_repr_css_attr_unref(css);
-        }
-        rc.item = (SPItem *) sp_document_add_repr(SP_DT_DOCUMENT(desktop), repr);
-        sp_repr_unref(repr);
-    }
 
-    /* This is bit ugly, but so we are */
+        /* Set style */
+        sp_desktop_apply_style_tool (desktop, repr, "tools.shapes.rect", false);
+
+        rc.item = (SPItem *) desktop->currentLayer()->appendChildRepr(repr);
+        sp_repr_unref(repr);
+        rc.item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
+        rc.item->updateRepr();
+    }
 
     NR::Point p0, p1;
     if ( state & GDK_CONTROL_MASK ) {
@@ -434,8 +496,8 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
         p1 = rc.center + delta;
         if ( state & GDK_SHIFT_MASK ) {
             p0 = rc.center - delta;
-            NR::Coord const l0 = sp_desktop_vector_snap(desktop, p0, p0 - p1);
-            NR::Coord const l1 = sp_desktop_vector_snap(desktop, p1, p1 - p0);
+            NR::Coord const l0 = namedview_vector_snap_all_types(desktop->namedview, p0, p0 - p1);
+            NR::Coord const l1 = namedview_vector_snap_all_types(desktop->namedview, p1, p1 - p0);
 
             if (l0 < l1) {
                 p1 = 2 * rc.center - p0;
@@ -444,8 +506,7 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
             }
         } else {
             p0 = rc.center;
-            sp_desktop_vector_snap(desktop, p1,
-                                   p1 - p0);
+            namedview_vector_snap_all_types(desktop->namedview, p1, p1 - p0);
         }
     } else if ( state & GDK_SHIFT_MASK ) {
         /* Corner point movements are bound */
@@ -453,8 +514,8 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
         p0 = 2 * rc.center - p1;
         for (unsigned d = 0 ; d < 2 ; ++d) {
             double snap_movement[2];
-            snap_movement[0] = sp_desktop_dim_snap(desktop, p0, d);
-            snap_movement[1] = sp_desktop_dim_snap(desktop, p1, d);
+            snap_movement[0] = namedview_dim_snap_all_types(desktop->namedview, p0, NR::Dim2(d));
+            snap_movement[1] = namedview_dim_snap_all_types(desktop->namedview, p1, NR::Dim2(d));
             if ( snap_movement[0] <
                  snap_movement[1] )
             {
@@ -468,7 +529,7 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
         /* Free movement for corner point */
         p0 = rc.center;
         p1 = pt;
-        sp_desktop_free_snap(desktop, p1);
+        namedview_free_snap_all_types(desktop->namedview, p1);
     }
 
     p0 = sp_desktop_dt2root_xy_point(desktop, p0);
@@ -477,135 +538,50 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
     // TODO: use NR::Rect
     using NR::X;
     using NR::Y;
-    NR::Coord const x0 = MIN(p0[X],
-                             p1[X]);
-    NR::Coord const y0 = MIN(p0[Y],
-                             p1[Y]);
-    NR::Coord const x1 = MAX(p0[X],
-                             p1[X]);
-    NR::Coord const y1 = MAX(p0[Y],
-                             p1[Y]);
+    NR::Coord const x0 = MIN(p0[X], p1[X]);
+    NR::Coord const y0 = MIN(p0[Y], p1[Y]);
+    NR::Coord const x1 = MAX(p0[X], p1[X]);
+    NR::Coord const y1 = MAX(p0[Y], p1[Y]);
     NR::Coord const w  = x1 - x0;
     NR::Coord const h  = y1 - y0;
 
     sp_rect_position_set(SP_RECT(rc.item), x0, y0, w, h);
-    if ( rc.rx_ratio != 0.0 ) {
-        sp_rect_set_rx(SP_RECT(rc.item), TRUE, 0.5 * rc.rx_ratio * w);
+    if ( rc.rx != 0.0 ) {
+        sp_rect_set_rx (SP_RECT(rc.item), TRUE, rc.rx);
     }
-    if ( rc.ry_ratio != 0.0 ) {
-        sp_rect_set_ry(SP_RECT(rc.item), TRUE, 0.5 * rc.ry_ratio * h);
+    if ( rc.ry != 0.0 ) {
+        if (rc.rx == 0.0)
+            sp_rect_set_ry (SP_RECT(rc.item), TRUE, CLAMP(rc.ry, 0, MIN(w, h)/2));
+        else 
+            sp_rect_set_ry (SP_RECT(rc.item), TRUE, CLAMP(rc.ry, 0, h));
     }
 
     // status text
-    gchar status[80];
     GString *xs = SP_PT_TO_METRIC_STRING(fabs( x1 - x0 ), SP_DEFAULT_METRIC);
     GString *ys = SP_PT_TO_METRIC_STRING(fabs( y1 - y0 ), SP_DEFAULT_METRIC);
-    g_snprintf(status, 80, _("Draw rectangle: %s x %s"), xs->str, ys->str);
-    sp_view_set_status(SP_VIEW(desktop), status, FALSE);
-    /* FIXME: I'd guess that arg2 should be TRUE below, that otherwise we'd have mem leak.
-     * Check with valgrind. */
+    rc._message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Rectangle</b>: %s x %s; with <b>Ctrl</b> to make square or integer-ratio rectangle; with <b>Shift</b> to draw around the starting point"), xs->str, ys->str);
     g_string_free(xs, FALSE);
     g_string_free(ys, FALSE);
 }
 
 static void sp_rect_finish(SPRectContext *rc)
 {
+    rc->_message_context->clear();
+
     if ( rc->item != NULL ) {
         SPDesktop * dt;
 
         dt = SP_EVENT_CONTEXT_DESKTOP(rc);
 
-        sp_object_invoke_write(SP_OBJECT(rc->item), SP_OBJECT_REPR(rc->item), SP_OBJECT_WRITE_EXT);
+        SP_OBJECT(rc->item)->updateRepr();
 
-        sp_selection_set_item(SP_DT_SELECTION(dt), rc->item);
+        SP_DT_SELECTION(dt)->setItem(rc->item);
         sp_document_done(SP_DT_DOCUMENT(dt));
 
         rc->item = NULL;
     }
 }
 
-static void sp_rc_rx_ratio_value_changed(GtkAdjustment *adj, SPRectContext *rc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(rc), "rx_ratio", adj->value);
-}
-
-static void sp_rc_ry_ratio_value_changed(GtkAdjustment *adj, SPRectContext *rc)
-{
-    sp_repr_set_double(SP_EVENT_CONTEXT_REPR(rc), "ry_ratio", adj->value);
-}
-
-static void sp_rc_defaults(GtkWidget *, GtkObject *obj)
-{
-    char const * const keys[] = {"rx_ratio",
-                                 "ry_ratio"};
-    for (unsigned i = 0; i < G_N_ELEMENTS(keys); ++i) {
-        GtkAdjustment &adj = *static_cast<GtkAdjustment *>(gtk_object_get_data(obj, keys[i]));
-        gtk_adjustment_set_value(&adj, 0.0);
-    }
-}
-
-static GtkWidget *sp_rect_context_config_widget(SPEventContext *ec)
-{
-    SPRectContext *rc = SP_RECT_CONTEXT(ec);
-
-    GtkWidget *tbl = gtk_table_new(3, 2, FALSE);
-    gtk_container_set_border_width(GTK_CONTAINER(tbl), 4);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 4);
-
-    /* rx_ratio */
-    {
-        GtkWidget *l = gtk_label_new(_("Roundness ratio for x:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 0, 1,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(rc->rx_ratio, 0.0, 1.0, 0.01, 0.1, 0.1);
-        gtk_object_set_data(GTK_OBJECT(tbl), "rx_ratio", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 0, 1,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_rc_rx_ratio_value_changed), rc);
-    }
-
-    /* ry_ratio */
-    {
-        GtkWidget *l = gtk_label_new(_("Roundness ratio for y:"));
-        gtk_widget_show(l);
-        gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-        gtk_table_attach(GTK_TABLE(tbl), l, 0, 1, 1, 2,
-                         (GtkAttachOptions) 0,
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        GtkObject *a = gtk_adjustment_new(rc->ry_ratio, 0.0, 1.0, 0.01, 0.1, 0.1);
-        gtk_object_set_data(GTK_OBJECT(tbl), "ry_ratio", a);
-        GtkWidget *sb = gtk_spin_button_new(GTK_ADJUSTMENT(a), 0.1, 2);
-        gtk_widget_show(sb);
-        gtk_table_attach(GTK_TABLE(tbl), sb, 1, 2, 1, 2,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) 0,
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(a), "value_changed", GTK_SIGNAL_FUNC(sp_rc_ry_ratio_value_changed), rc);
-    }
-
-    /* Reset */
-    {
-        GtkWidget *b = gtk_button_new_with_label(_("Defaults"));
-        gtk_widget_show(b);
-        gtk_table_attach(GTK_TABLE(tbl), b, 0, 2, 2, 3,
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         (GtkAttachOptions) ( GTK_EXPAND | GTK_FILL ),
-                         0, 0);
-        gtk_signal_connect(GTK_OBJECT(b), "clicked", GTK_SIGNAL_FUNC(sp_rc_defaults), tbl);
-    }
-
-    return tbl;
-}
-
 /*
   Local Variables:
   mode:c++
