@@ -16,14 +16,22 @@
 
 #include "forward.h"
 #include "libnr/nr-forward.h"
+#include "xml/xml-forward.h"
 
-void sp_edit_cleanup (gpointer object, gpointer data);
+void sp_selection_cleanup();
 
 void sp_selection_delete();
 void sp_selection_duplicate();
 void sp_edit_clear_all();
 void sp_edit_select_all();
 
+void sp_selection_clone();
+void sp_selection_unlink();
+void sp_select_clone_original ();
+
+void sp_selection_tile(bool apply = true);
+void sp_selection_untile();
+
 void sp_selection_group();
 void sp_selection_ungroup();
 
@@ -32,12 +40,14 @@ void sp_selection_raise_to_top();
 void sp_selection_lower();
 void sp_selection_lower_to_bottom();
 
+SPCSSAttr *take_style_from_item (SPItem *item);
+
 void sp_selection_cut();
 void sp_selection_copy();
 void sp_selection_paste(bool in_place);
 void sp_selection_paste_style();
 
-void sp_selection_apply_affine(SPSelection *selection, NR::Matrix const &affine);
+void sp_selection_apply_affine(SPSelection *selection, NR::Matrix const &affine, bool set_i2d = true);
 void sp_selection_remove_transform (void);
 void sp_selection_scale_absolute (SPSelection *selection, double x0, double x1, double y0, double y1);
 void sp_selection_scale_relative(SPSelection *selection, NR::Point const &align, NR::scale const &scale);
@@ -60,9 +70,13 @@ void sp_selection_move_screen (gdouble dx, gdouble dy);
 void sp_selection_item_next (void);
 void sp_selection_item_prev (void);
 
+void scroll_to_show_item(SPDesktop *desktop, SPItem *item);
+
 void sp_undo (SPDesktop *desktop, SPDocument *doc);
 void sp_redo (SPDesktop *desktop, SPDocument *doc);
 
+void sp_selection_create_bitmap_copy ();
+
 /* selection cycling */
 
 typedef enum
