@@ -18,6 +18,7 @@
 
 #include "display/nr-arena.h"
 #include "display/nr-arena-group.h"
+#include "xml/repr.h"
 
 #include "enums.h"
 #include "attributes.h"
@@ -26,7 +27,7 @@
 
 #include "sp-clippath.h"
 
-struct _SPClipPathView {
+struct SPClipPathView {
 	SPClipPathView *next;
 	unsigned int key;
 	NRArenaItem *arenaitem;
@@ -154,7 +155,7 @@ sp_clippath_set (SPObject *object, unsigned int key, const gchar *value)
 				cp->clipPathUnits_set = TRUE;
 			}
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) parent_class)->set)
@@ -175,7 +176,7 @@ sp_clippath_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	((SPObjectClass *) (parent_class))->child_added (object, child, ref);
 
 	/* Show new object */
-	ochild = sp_document_lookup_id (SP_OBJECT_DOCUMENT (object), sp_repr_attr (child, "id"));
+	ochild = SP_OBJECT_DOCUMENT (object)->getObjectByRepr(child);
 	if (SP_IS_ITEM (ochild)) {
 		SPClipPathView *v;
 		for (v = cp->display; v != NULL; v = v->next) {
@@ -214,7 +215,7 @@ sp_clippath_update (SPObject *object, SPCtx *ctx, guint flags)
 		child = SP_OBJECT (l->data);
 		l = g_slist_remove (l, child);
 		if (flags || (child->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
-			sp_object_invoke_update (child, ctx, flags);
+			child->updateDisplay(ctx, flags);
 		}
 		g_object_unref (G_OBJECT (child));
 	}
@@ -256,7 +257,7 @@ sp_clippath_modified (SPObject *object, guint flags)
 		child = SP_OBJECT (l->data);
 		l = g_slist_remove (l, child);
 		if (flags || (child->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
-			sp_object_invoke_modified (child, flags);
+			child->emitModified(flags);
 		}
 		g_object_unref (G_OBJECT (child));
 	}
@@ -290,7 +291,7 @@ sp_clippath_show (SPClipPath *cp, NRArena *arena, unsigned int key)
 	g_return_val_if_fail (arena != NULL, NULL);
 	g_return_val_if_fail (NR_IS_ARENA (arena), NULL);
 
-	ai = nr_arena_item_new (arena, NR_TYPE_ARENA_GROUP);
+	ai = NRArenaGroup::create(arena);
 	cp->display = sp_clippath_view_new_prepend (cp->display, key, ai);
 
 	for (child = sp_object_first_child(SP_OBJECT(cp)) ; child != NULL; child = SP_OBJECT_NEXT(child)) {
@@ -353,7 +354,7 @@ sp_clippath_set_bbox (SPClipPath *cp, unsigned int key, NRRect *bbox)
 			    !NR_DF_TEST_CLOSE (v->bbox.x1, bbox->x1, NR_EPSILON) ||
 			    !NR_DF_TEST_CLOSE (v->bbox.y1, bbox->y1, NR_EPSILON)) {
 				v->bbox = *bbox;
-				sp_object_request_update (SP_OBJECT (cp), SP_OBJECT_MODIFIED_FLAG);
+				SP_OBJECT(cp)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 			}
 			break;
 		}
@@ -371,7 +372,7 @@ sp_clippath_view_new_prepend (SPClipPathView *list, unsigned int key, NRArenaIte
 
 	new_path_view->next = list;
 	new_path_view->key = key;
-	new_path_view->arenaitem = nr_arena_item_ref (arenaitem);
+	new_path_view->arenaitem = nr_arena_item_ref(arenaitem);
 	new_path_view->bbox.x0 = new_path_view->bbox.x1 = 0.0;
 	new_path_view->bbox.y0 = new_path_view->bbox.y1 = 0.0;
 
