@@ -18,17 +18,22 @@
 #include <math.h>
 #include <string.h>
 
+#include "libnr/n-art-bpath.h"
 #include "libnr/nr-matrix.h"
 #include "libnr/nr-matrix-ops.h"
 #include "svg/svg.h"
 #include "svg/stringstream.h"
+#include "xml/repr.h"
 #include "attributes.h"
 #include "style.h"
 #include "version.h"
+#include "display/curve.h"
 #include "helper/sp-intl.h"
 
 #include "sp-ellipse.h"
 
+#include "prefs-utils.h"
+
 /* Common parent class */
 
 #define noELLIPSE_VERBOSE
@@ -67,7 +72,7 @@ static void sp_genericellipse_init (SPGenericEllipse *ellipse);
 
 static void sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags);
 
-static int sp_genericellipse_snappoints(SPItem *item, NR::Point p[], int size);
+static void sp_genericellipse_snappoints(SPItem const *item, SnapPointsIter p);
 
 static void sp_genericellipse_set_shape (SPShape *shape);
 static SPRepr *sp_genericellipse_write (SPObject *object, SPRepr *repr, guint flags);
@@ -131,12 +136,9 @@ static void
 sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags)
 {
 	if (flags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
-		SPGenericEllipse *ellipse;
-		SPStyle *style;
-		double d;
-		ellipse = (SPGenericEllipse *) object;
-		style = object->style;
-		d = 1.0 / NR_MATRIX_DF_EXPANSION (&((SPItemCtx *) ctx)->i2vp);
+		SPGenericEllipse *ellipse = (SPGenericEllipse *) object;
+		SPStyle *style = object->style;
+		double const d = 1.0 / NR_MATRIX_DF_EXPANSION (&((SPItemCtx *) ctx)->i2vp);
 		sp_svg_length_update (&ellipse->cx, style->font_size.computed, style->font_size.computed * 0.5, d);
 		sp_svg_length_update (&ellipse->cy, style->font_size.computed, style->font_size.computed * 0.5, d);
 		sp_svg_length_update (&ellipse->rx, style->font_size.computed, style->font_size.computed * 0.5, d);
@@ -152,15 +154,11 @@ sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags)
 
 /* fixme: Think (Lauris) */
 
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_bpath.h>
-
 static void sp_genericellipse_set_shape(SPShape *shape)
 {
 	double cx, cy, rx, ry, s, e;
 	double x0, y0, x1, y1, x2, y2, x3, y3;
 	double len;
-	double affine[6];
 	gint slice = FALSE;
 	gint i;
 
@@ -185,17 +183,17 @@ static void sp_genericellipse_set_shape(SPShape *shape)
 	} else {
 		slice = TRUE;
 	}
+	
+	NR::Matrix aff = NR::Matrix(NR::scale(rx, ry));
+	aff[4] = ellipse->cx.computed;
+	aff[5] = ellipse->cy.computed;
 
-	nr_matrix_set_scale (NR_MATRIX_D_FROM_DOUBLE (affine), rx, ry);
-	affine[4] = ellipse->cx.computed;
-	affine[5] = ellipse->cy.computed;
-
-	ArtBpath bpath[16];
+	NArtBpath bpath[16];
 	i = 0;
 	if (ellipse->closed) {
-		bpath[i].code = ART_MOVETO;
+		bpath[i].code = NR_MOVETO;
 	} else {
-		bpath[i].code = ART_MOVETO_OPEN;
+		bpath[i].code = NR_MOVETO_OPEN;
 	}
 	bpath[i].x3 = cos (ellipse->start);
 	bpath[i].y3 = sin (ellipse->start);
@@ -218,7 +216,7 @@ static void sp_genericellipse_set_shape(SPShape *shape)
 g_print ("step %d s %f e %f coords %f %f %f %f %f %f\n",
 	i, s, e, x1, y1, x2, y2, x3, y3);
 #endif
-		bpath[i].code = ART_CURVETO;
+		bpath[i].code = NR_CURVETO;
 		bpath[i].x1 = x1;
 		bpath[i].y1 = y1;
 		bpath[i].x2 = x2;
@@ -229,11 +227,11 @@ g_print ("step %d s %f e %f coords %f %f %f %f %f %f\n",
 	}
 
 	if (slice && ellipse->closed) {
-		bpath[i].code = ART_LINETO;
+		bpath[i].code = NR_LINETO;
 		bpath[i].x3 = 0.0;
 		bpath[i].y3 = 0.0;
 		i++;
-		bpath[i].code = ART_LINETO;
+		bpath[i].code = NR_LINETO;
 		bpath[i].x3 = bpath[0].x3;
 		bpath[i].y3 = bpath[0].y3;
 		i++;
@@ -242,30 +240,30 @@ g_print ("step %d s %f e %f coords %f %f %f %f %f %f\n",
 		bpath[i-1].y3 = bpath[0].y3;
 	}
 
-	bpath[i].code = ART_END;
-
-	ArtBpath *abp = art_bpath_affine_transform(bpath, affine);
-	SPCurve *c = sp_curve_new_from_bpath(abp);
+	bpath[i].code = NR_END;
+	SPCurve *c = sp_curve_new_from_bpath(nr_artpath_affine(bpath, aff));
+	g_assert(c != NULL);
+	
 	sp_shape_set_curve_insync ((SPShape *) ellipse, c, TRUE);
 	sp_curve_unref (c);
 }
 
-static int sp_genericellipse_snappoints(SPItem *item, NR::Point p[], int size)
+static void sp_genericellipse_snappoints(SPItem const *item, SnapPointsIter p)
 {
-	SPGenericEllipse *ge = SP_GENERICELLIPSE (item);
+	SPGenericEllipse const *ge = SP_GENERICELLIPSE (item);
 
-	/* We use corners of item and center of ellipse. */
-	int pos = 0;
-	if (((SPItemClass *) ge_parent_class)->snappoints) {
-		pos = ((SPItemClass *) ge_parent_class)->snappoints (item, p, size);
-	}
-	if (pos < size) {
-		p[pos++] = ( NR::Point(ge->cx.computed,
-				       ge->cy.computed)
-			     * sp_item_i2d_affine(item) );
-	}
+	NR::Matrix const i2d = sp_item_i2d_affine(item);
 
-	return pos;
+	/* Use the bounding box corners */
+	NRRect bbox;
+	sp_item_invoke_bbox(item, &bbox, i2d, TRUE);
+	*p = NR::Point(bbox.x0, bbox.y0);
+	*p = NR::Point(bbox.x1, bbox.y0);
+	*p = NR::Point(bbox.x1, bbox.y1);
+	*p = NR::Point(bbox.x0, bbox.y1);
+
+	/* And the centre */
+	*p = NR::Point(ge->cx.computed, ge->cy.computed) * i2d;
 }
 
 void
@@ -297,7 +295,8 @@ static SPRepr *sp_genericellipse_write (SPObject *object, SPRepr *repr, guint fl
 		sp_repr_set_double (repr, "sodipodi:rx", ellipse->rx.computed);
 		sp_repr_set_double (repr, "sodipodi:ry", ellipse->ry.computed);
 
-		sp_arc_set_elliptical_path_attribute (SP_ARC (object), SP_OBJECT_REPR (object));
+		if (SP_IS_ARC(ellipse))
+			sp_arc_set_elliptical_path_attribute (SP_ARC (object), SP_OBJECT_REPR (object));
 	}
 
 	if (((SPObjectClass *) ge_parent_class)->write)
@@ -388,8 +387,6 @@ sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags)
 	sp_repr_set_double (repr, "rx", ellipse->rx.computed);
 	sp_repr_set_double (repr, "ry", ellipse->ry.computed);
 
-	sp_arc_set_elliptical_path_attribute (SP_ARC (object), repr);
-	
 	if (((SPObjectClass *) ellipse_parent_class)->write)
 		(* ((SPObjectClass *) ellipse_parent_class)->write) (object, repr, flags);
 
@@ -408,25 +405,25 @@ sp_ellipse_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!sp_svg_length_read (value, &ellipse->cx)) {
 			sp_svg_length_unset (&ellipse->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_CY:
 		if (!sp_svg_length_read (value, &ellipse->cy)) {
 			sp_svg_length_unset (&ellipse->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_RX:
 		if (!sp_svg_length_read (value, &ellipse->rx) || (ellipse->rx.value <= 0.0)) {
 			sp_svg_length_unset (&ellipse->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_RY:
 		if (!sp_svg_length_read (value, &ellipse->ry) || (ellipse->ry.value <= 0.0)) {
 			sp_svg_length_unset (&ellipse->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) ellipse_parent_class)->set)
@@ -437,7 +434,7 @@ sp_ellipse_set (SPObject *object, unsigned int key, const gchar *value)
 
 static gchar *sp_ellipse_description(SPItem *item)
 {
-	return g_strdup(_("Ellipse"));
+	return g_strdup(_("<b>Ellipse</b>"));
 }
 
 
@@ -456,7 +453,7 @@ sp_ellipse_position_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, g
 	ge->rx.computed = rx;
 	ge->ry.computed = ry;
 
-	sp_object_request_update ((SPObject *) ge, SP_OBJECT_MODIFIED_FLAG);
+	((SPObject *)ge)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 }
 
 /* SVG <circle> element */
@@ -539,7 +536,7 @@ sp_circle_write (SPObject *object, SPRepr *repr, guint flags)
 	sp_repr_set_double (repr, "cx", ellipse->cx.computed);
 	sp_repr_set_double (repr, "cy", ellipse->cy.computed);
 	sp_repr_set_double (repr, "r", ellipse->rx.computed);
-	
+
 	if (((SPObjectClass *) circle_parent_class)->write)
 		((SPObjectClass *) circle_parent_class)->write (object, repr, flags);
 
@@ -558,20 +555,20 @@ sp_circle_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!sp_svg_length_read (value, &ge->cx)) {
 			sp_svg_length_unset (&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_CY:
 		if (!sp_svg_length_read (value, &ge->cy)) {
 			sp_svg_length_unset (&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_R:
 		if (!sp_svg_length_read (value, &ge->rx) || (ge->rx.value <= 0.0)) {
 			sp_svg_length_unset (&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		ge->ry = ge->rx;
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) circle_parent_class)->set)
@@ -582,7 +579,7 @@ sp_circle_set (SPObject *object, unsigned int key, const gchar *value)
 
 static gchar *sp_circle_description (SPItem *item)
 {
-	return g_strdup(_("Circle"));
+	return g_strdup(_("<b>Circle</b>"));
 }
 
 /* <path sodipodi:type="arc"> element */
@@ -704,9 +701,9 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 		NR::Point ph = sp_arc_get_xy (arc, (ge->start + ge->end) / 2.0);
 		os << "M " << p1[NR::X] << " " <<  p1[NR::Y]
 			<< " A " << ge->rx.computed << " " << ge->ry.computed
-			<< " 0 1 " << ((dt > 0)?1:0) << " " << ph[NR::X] << "," << ph[NR::Y]
+			<< " 0 1 1 " << " " << ph[NR::X] << "," << ph[NR::Y]
 			<< " A " << ge->rx.computed << " " << ge->ry.computed
-			<< " 0 1 " << ((dt > 0)?1:0) << " " << p2[NR::X] << " " << p2[NR::Y] << " z";
+			<< " 0 1 1 " << " " << p2[NR::X] << " " << p2[NR::Y] << " z";
 	} else {
 		fa = (fabs (dt) > M_PI) ? 1 : 0;
 		fs = (dt > 0) ? 1 : 0;
@@ -722,7 +719,7 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 			os << "M " << p1[NR::X] << "," << p1[NR::Y]
 				<< " A " << ge->rx.computed << "," << ge->ry.computed
 				<< " 0 " << fa << " " << fs << " " << p2[NR::X] << "," << p2[NR::Y];
-			
+
 		}
 	}
 	return sp_repr_set_attr (repr, "d", os.str().c_str());
@@ -772,7 +769,7 @@ static void
 sp_arc_set (SPObject *object, unsigned int key, const gchar *value)
 {
 	SPGenericEllipse *ge = SP_GENERICELLIPSE (object);
-	
+
 	SPVersion version = sp_object_get_sodipodi_version (object);
 
 	if (sp_version_inside_range (version, 0, 0, 0, 25)) {
@@ -797,25 +794,25 @@ sp_arc_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!sp_svg_length_read (value, &ge->cx)) {
 			sp_svg_length_unset (&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_CY:
 		if (!sp_svg_length_read (value, &ge->cy)) {
 			sp_svg_length_unset (&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_RX:
 		if (!sp_svg_length_read (value, &ge->rx) || (ge->rx.computed <= 0.0)) {
 			sp_svg_length_unset (&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_RY:
 		if (!sp_svg_length_read (value, &ge->ry) || (ge->ry.computed <= 0.0)) {
 			sp_svg_length_unset (&ge->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_START:
 		if (value) {
@@ -823,7 +820,7 @@ sp_arc_set (SPObject *object, unsigned int key, const gchar *value)
 		} else {
 			ge->start = 0;
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_END:
 		if (value) {
@@ -831,11 +828,11 @@ sp_arc_set (SPObject *object, unsigned int key, const gchar *value)
 		} else {
 			ge->end = 2 * M_PI;
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SODIPODI_OPEN:
 		ge->closed = (!value);
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) arc_parent_class)->set)
@@ -857,7 +854,7 @@ sp_arc_modified (SPObject *object, guint flags)
 
 static gchar *sp_arc_description(SPItem *item)
 {
-	return g_strdup(_("Arc"));
+	return g_strdup(_("<b>Arc</b>"));
 }
 
 void
@@ -872,8 +869,16 @@ sp_arc_position_set (SPArc *arc, gdouble x, gdouble y, gdouble rx, gdouble ry)
 	ge->cy.computed = y;
 	ge->rx.computed = rx;
 	ge->ry.computed = ry;
+	if (prefs_get_double_attribute ("tools.shapes.arc", "start", 0.0) != 0)
+		ge->start = prefs_get_double_attribute ("tools.shapes.arc", "start", 0.0);
+	if (prefs_get_double_attribute ("tools.shapes.arc", "end", 0.0) != 0) 
+		ge->end = prefs_get_double_attribute ("tools.shapes.arc", "end", 0.0);
+	if (!prefs_get_string_attribute ("tools.shapes.arc", "open")) 
+		ge->closed = 1; 
+	else 
+		ge->closed = 0;
 
-	sp_object_request_update ((SPObject *) arc, SP_OBJECT_MODIFIED_FLAG);
+	((SPObject *)arc)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 }
 
 NR::Point sp_arc_get_xy (SPArc *arc, gdouble arg)
