@@ -8,8 +8,10 @@
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
+ * Copyright (C) 2004 David Turner
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
+ *
  */
 
 #define noSP_GRADIENT_VERBOSE
@@ -19,10 +21,17 @@
 #include <stdlib.h>
 
 #include <libnr/nr-gradient.h>
+#include <libnr/nr-matrix.h>
+#include <libnr/nr-matrix-div.h>
+#include <libnr/nr-matrix-fns.h>
+#include <libnr/nr-matrix-ops.h>
+#include <libnr/nr-matrix-scale-ops.h>
+#include <libnr/nr-matrix-translate-ops.h>
+#include "libnr/nr-scale-translate-ops.h"
 
 #include <gtk/gtksignal.h>
 
-#include "helper/nr-gradient-gpl.h"
+#include "display/nr-gradient-gpl.h"
 #include "svg/svg.h"
 #include "svg/stringstream.h"
 #include "xml/repr-private.h"
@@ -30,6 +39,13 @@
 #include "document-private.h"
 #include "sp-object-repr.h"
 #include "sp-gradient.h"
+#include "sp-gradient-fns.h"
+#include "sp-gradient-reference.h"
+#include "sp-gradient-vector.h"
+#include "sp-linear-gradient.h"
+#include "sp-radial-gradient.h"
+#include "sp-stop.h"
+#include "uri.h"
 #include "uri-references.h"
 
 #define SP_MACROS_SILENT
@@ -99,10 +115,6 @@ static void sp_stop_build(SPObject *object, SPDocument *document, SPRepr *repr)
 static void
 sp_stop_set (SPObject *object, unsigned int key, const gchar *value)
 {
-	guint32 color;
-	gdouble opacity;
-	const gchar *p;
-
 	SPStop *stop = SP_STOP (object);
 
 	switch (key) {
@@ -110,31 +122,38 @@ sp_stop_set (SPObject *object, unsigned int key, const gchar *value)
 		/* fixme: We are reading simple values 3 times during
 		 *        build (Lauris) */
 		/* fixme: We need presentation attributes etc. */
-		p = sp_object_get_style_property (object, "stop-color", "black");
-		color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
+
+		{
+		const gchar *p = sp_object_get_style_property (object, "stop-color", "black");
+		guint32 color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
 		sp_color_set_rgb_rgba32 (&stop->color, color);
-		p = sp_object_get_style_property (object, "stop-opacity", "1");
-		opacity = sp_svg_read_percentage (p, stop->opacity);
+		}
+		{
+		const gchar *p = sp_object_get_style_property (object, "stop-opacity", "1");
+		gdouble opacity = sp_svg_read_percentage (p, stop->opacity);
 		stop->opacity = opacity;
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+		}
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 		break;
 	case SP_PROP_STOP_COLOR:
-		/* fixme: We need presentation attributes etc. */
-		p = sp_object_get_style_property (object, "stop-color", "black");
-		color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
+		{
+		const gchar *p = sp_object_get_style_property (object, "stop-color", "black");
+		guint32 color = sp_svg_read_color (p, sp_color_get_rgba32_ualpha (&stop->color, 0x00));
 		sp_color_set_rgb_rgba32 (&stop->color, color);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+		}
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 		break;
 	case SP_PROP_STOP_OPACITY:
-		/* fixme: We need presentation attributes etc. */
-		p = sp_object_get_style_property (object, "stop-opacity", "1");
-		opacity = sp_svg_read_percentage (p, stop->opacity);
+		{
+		const gchar *p = sp_object_get_style_property (object, "stop-opacity", "1");
+		gdouble opacity = sp_svg_read_percentage (p, stop->opacity);
 		stop->opacity = opacity;
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+		}
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_OFFSET:
 		stop->offset = sp_svg_read_percentage (value, 0.0);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) stop_parent_class)->set)
@@ -146,16 +165,16 @@ sp_stop_set (SPObject *object, unsigned int key, const gchar *value)
 static SPRepr *
 sp_stop_write (SPObject *object, SPRepr *repr, guint flags)
 {
-	gchar c[64];
-	Inkscape::SVGOStringStream os;	
-
 	SPStop *stop = SP_STOP (object);
 
 	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 		repr = sp_repr_new ("stop");
 	}
 
+	gchar c[64];
 	sp_svg_write_color (c, 64, sp_color_get_rgba32_ualpha (&stop->color, 255));
+
+	Inkscape::SVGOStringStream os;	
 	os << "stop-color:" << c << ";stop-opacity:" << stop->opacity << ";";
 	sp_repr_set_attr (repr, "style", os.str().c_str());
 	sp_repr_set_attr (repr, "stop-color", NULL);
@@ -232,14 +251,17 @@ static void
 sp_gradient_init (SPGradient *gr)
 {
 	gr->ref = new SPGradientReference(SP_OBJECT(gr));
-	gr->ref->changedSignal().connect(SigC::bind(SigC::slot(gradient_ref_changed), gr));
+	gr->ref->changedSignal().connect(sigc::bind(sigc::ptr_fun(gradient_ref_changed), gr));
 
 	/* Fixme: reprs being rearranged (e.g. via the XML editor)
 	 *        may require us to clear the state */
 	gr->state = SP_GRADIENT_STATE_UNKNOWN;
 
 	gr->units = SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX;
-	nr_matrix_set_identity (NR_MATRIX_D_FROM_DOUBLE (gr->transform));
+	gr->units_set = FALSE;
+
+	gr->gradientTransform = NR::identity();
+	gr->gradientTransform_set = FALSE;
 
 	gr->spread = SP_GRADIENT_SPREAD_PAD;
 	gr->spread_set = FALSE;
@@ -247,8 +269,8 @@ sp_gradient_init (SPGradient *gr)
 	gr->has_stops = FALSE;
 
 	gr->vector = NULL;
-	gr->color = NULL;
 
+	gr->color = NULL;
 	gr->len = 0.0;
 }
 
@@ -292,6 +314,9 @@ sp_gradient_release (SPObject *object)
 	}
 
 	if (gradient->ref) {
+		if (gradient->ref->getObject()) {
+			sp_signal_disconnect_by_data(gradient->ref->getObject(), gradient);
+		}
 		gradient->ref->detach();
 		delete gradient->ref;
 		gradient->ref = NULL;
@@ -316,7 +341,6 @@ sp_gradient_set (SPObject *object, unsigned int key, const gchar *value)
 {
 	SPGradient *gr = SP_GRADIENT (object);
 
-	/* fixme: We should unset properties, if val == NULL */
 	switch (key) {
 	case SP_ATTR_GRADIENTUNITS:
 		if (value) {
@@ -327,21 +351,21 @@ sp_gradient_set (SPObject *object, unsigned int key, const gchar *value)
 			}
 			gr->units_set = TRUE;
 		} else {
+			gr->units = SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX;
 			gr->units_set = FALSE;
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GRADIENTTRANSFORM: {
-		NRMatrix t;
+		NR::Matrix t;
 		if (value && sp_svg_transform_read (value, &t)) {
-			int i;
-			for (i = 0; i < 6; i++) gr->transform[i] = t.c[i];
-			gr->transform_set = TRUE;
+			gr->gradientTransform = t;
+			gr->gradientTransform_set = TRUE;
 		} else {
-			nr_matrix_set_identity (NR_MATRIX_D_FROM_DOUBLE (gr->transform));
-			gr->transform_set = FALSE;
+			gr->gradientTransform = NR::identity();
+			gr->gradientTransform_set = FALSE;
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	}
 	case SP_ATTR_SPREADMETHOD:
@@ -357,9 +381,9 @@ sp_gradient_set (SPObject *object, unsigned int key, const gchar *value)
 		} else {
 			gr->spread_set = FALSE;
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
-	case SP_ATTR_XLINK_HREF: {
+	case SP_ATTR_XLINK_HREF: 
 		if (value) {
 			try {
 				gr->ref->attach(Inkscape::URI(value));
@@ -371,7 +395,6 @@ sp_gradient_set (SPObject *object, unsigned int key, const gchar *value)
 			gr->ref->detach();
 		}
 		break;
-	}
 	default:
 		if (((SPObjectClass *) gradient_parent_class)->set)
 			((SPObjectClass *) gradient_parent_class)->set (object, key, value);
@@ -379,6 +402,9 @@ sp_gradient_set (SPObject *object, unsigned int key, const gchar *value)
 	}
 }
 
+/**
+Gets called when the gradient is (re)attached to another gradient
+*/
 static void
 gradient_ref_changed(SPObject *old_ref, SPObject *ref, SPGradient *gr)
 {
@@ -408,7 +434,7 @@ sp_gradient_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	}
 
 	/* Fixme: should we schedule "modified" here? */
-	sp_object_request_modified(object, SP_OBJECT_MODIFIED_FLAG);
+	object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 static void
@@ -421,7 +447,6 @@ sp_gradient_remove_child (SPObject *object, SPRepr *child)
 	if (((SPObjectClass *) gradient_parent_class)->remove_child)
 		(* ((SPObjectClass *) gradient_parent_class)->remove_child) (object, child);
 
-	/* Fixme: (Lauris) */
 	gr->has_stops = FALSE;
 	SPObject *ochild;
 	for ( ochild = sp_object_first_child(object) ; ochild != NULL ; ochild = SP_OBJECT_NEXT(ochild) ) {
@@ -432,7 +457,7 @@ sp_gradient_remove_child (SPObject *object, SPRepr *child)
 	}
 
 	/* Fixme: should we schedule "modified" here? */
-	sp_object_request_modified(object, SP_OBJECT_MODIFIED_FLAG);
+	object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 static void
@@ -447,6 +472,7 @@ sp_gradient_modified (SPObject *object, guint flags)
 	if (flags & SP_OBJECT_MODIFIED_FLAG) flags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
 	flags &= SP_OBJECT_MODIFIED_CASCADE;
 
+	// FIXME: climb up the ladder of hrefs
 	GSList *l = NULL;
 	for (SPObject *child = sp_object_first_child(object) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
 		g_object_ref (G_OBJECT (child));
@@ -457,7 +483,7 @@ sp_gradient_modified (SPObject *object, guint flags)
 		SPObject *child = SP_OBJECT (l->data);
 		l = g_slist_remove (l, child);
 		if (flags || (child->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
-			sp_object_invoke_modified (child, flags);
+			child->emitModified(flags);
 		}
 		g_object_unref (G_OBJECT (child));
 	}
@@ -476,7 +502,7 @@ sp_gradient_write (SPObject *object, SPRepr *repr, guint flags)
 		l = NULL;
 		for ( SPObject *child = sp_object_first_child(object) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
 			SPRepr *crepr;
-			crepr = sp_object_invoke_write (child, NULL, flags);
+			crepr = child->updateRepr(NULL, flags);
 			if (crepr) l = g_slist_prepend (l, crepr);
 		}
 		while (l) {
@@ -503,12 +529,9 @@ sp_gradient_write (SPObject *object, SPRepr *repr, guint flags)
 		}
 	}
 
-	if ((flags & SP_OBJECT_WRITE_ALL) || gr->transform_set) {
+	if ((flags & SP_OBJECT_WRITE_ALL) || gr->gradientTransform_set) {
 		gchar c[256];
-		NRMatrix t;
-		int i;
-		for (i = 0; i < 6; i++) t.c[i] = gr->transform[i];
-		if (sp_svg_transform_write (c, 256, &t)) {
+		if (sp_svg_transform_write(c, 256, gr->gradientTransform)) {
 			sp_repr_set_attr (repr, "gradientTransform", c);
 		} else {
 			sp_repr_set_attr (repr, "gradientTransform", NULL);
@@ -566,7 +589,7 @@ sp_gradient_set_vector (SPGradient *gradient, SPGradientVector *vector)
 	}
 	memcpy (gradient->vector, vector, sizeof (SPGradientVector) + (vector->nstops - 1) * sizeof (SPGradientStop));
 
-	sp_object_request_modified (SP_OBJECT (gradient), SP_OBJECT_MODIFIED_FLAG);
+	SP_OBJECT (gradient)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 void
@@ -575,7 +598,7 @@ sp_gradient_set_units (SPGradient *gr, SPGradientUnits units)
 	if (units != gr->units) {
 		gr->units = units;
 		gr->units_set = TRUE;
-		sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+		SP_OBJECT (gr)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 	}
 }
 
@@ -585,7 +608,7 @@ sp_gradient_set_spread (SPGradient *gr, SPGradientSpread spread)
 	if (spread != gr->spread) {
 		gr->spread = spread;
 		gr->spread_set = TRUE;
-		sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+		SP_OBJECT (gr)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 	}
 }
 
@@ -632,16 +655,6 @@ sp_gradient_repr_set_vector (SPGradient *gr, SPRepr *repr, SPGradientVector *vec
 	while (cl) {
 		SPRepr *child = static_cast<SPRepr *>(cl->data);
 		sp_repr_add_child(repr, child, NULL);
-		/* TODO: Previously, the below passed as first
-		   argument an expression always equal to NULL (which
-		   would generate a CRITICAL warning from
-		   sp_repr_unref if this code were executed).  I've
-		   guessed that the intention is to unref cl->data,
-		   but I can't work out how to get this code called,
-		   to check that it's doing the right thing.  Remove
-		   the below g_warning once you're sure that this new
-		   version is correct. */
-		g_warning("Congratulations, you've found a way to get this code to run.  Please run under valgrind (or other detector of leaks and premature free's) to check that the below unreferencing (from the above sp_repr_new call) is correct.");
 		sp_repr_unref(child);
 		cl = g_slist_remove(cl, child);
 	}
@@ -651,7 +664,7 @@ static void
 gradient_ref_modified (SPObject *href, guint flags, SPGradient *gradient)
 {
 	sp_gradient_invalidate_vector (gradient);
-	sp_object_request_modified (SP_OBJECT (gradient), SP_OBJECT_MODIFIED_FLAG);
+	SP_OBJECT (gradient)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 /* Creates normalized color vector */
@@ -683,13 +696,17 @@ sp_gradient_rebuild_vector (SPGradient *gr)
 		if (SP_IS_STOP (child)) {
 			SPStop *stop = SP_STOP (child);
 			if (!oset) {
+				/* first stop */
 				oset = TRUE;
 				offsets = offsete = stop->offset;
-				len += 1;
-			} else if (stop->offset > (offsete + 1e-9)) {
-				offsete = stop->offset;
-				len += 1;
+			} else {
+			        if (stop->offset < offsete) {
+				        stop->offset = offsete;
+				} else {
+				        offsete = stop->offset;
+				}
 			}
+			len += 1;
 			sp_color_copy (&color, &stop->color);
 			opacity = stop->opacity;
 		}
@@ -720,6 +737,7 @@ sp_gradient_rebuild_vector (SPGradient *gr)
 		gr->vector->nstops = vlen;
 	}
 
+	/*fixme: SVG spec says that if len is 1, treat as solid, if len is 0, treat as none*/
 	if (len < 2) {
 		gr->vector->start = 0.0;
 		gr->vector->end = 1.0;
@@ -738,17 +756,19 @@ sp_gradient_rebuild_vector (SPGradient *gr)
 
 	gint pos = 0;
 	gdouble offset = offsets;
+	gboolean first = TRUE;
 	gr->vector->stops[0].offset = 0.0;
 	for (SPObject *child = sp_object_first_child(SP_OBJECT(gr)) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
 		if (SP_IS_STOP (child)) {
 			SPStop *stop;
 			stop = SP_STOP (child);
-			if (stop->offset > (offset + 1e-9)) {
-				pos += 1;
-				g_assert( pos < gr->vector->nstops );
-				gr->vector->stops[pos].offset = (stop->offset - offsets) / (offsete - offsets);
-				offset = stop->offset;
+			if (!first) {
+			  pos += 1;
 			}
+			first = FALSE;
+			g_assert( pos < gr->vector->nstops );
+			gr->vector->stops[pos].offset = (stop->offset - offsets) / (offsete - offsets);
+			offset = stop->offset;
 			sp_color_copy (&gr->vector->stops[pos].color, &stop->color);
 			gr->vector->stops[pos].opacity = stop->opacity;
 		}
@@ -963,100 +983,71 @@ sp_gradient_render_vector_block_rgb (SPGradient *gradient, guchar *buf, gint wid
 }
 
 NRMatrix *
-sp_gradient_get_g2d_matrix_f (SPGradient *gr, NRMatrix *ctm, NRRect *bbox, NRMatrix *g2d)
+sp_gradient_get_g2d_matrix_f(SPGradient const *gr, NRMatrix const *ctm, NRRect const *bbox, NRMatrix *g2d)
 {
-	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		NRMatrix bb2u;
-		
-		bb2u.c[0] = bbox->x1 - bbox->x0;
-		bb2u.c[1] = 0.0;
-		bb2u.c[2] = 0.0;
-		bb2u.c[3] = bbox->y1 - bbox->y0;
-		bb2u.c[4] = bbox->x0;
-		bb2u.c[5] = bbox->y0;
-
-		nr_matrix_multiply (g2d, &bb2u, ctm);
-	} else {
-		*g2d = *ctm;
-	}
-
+	*g2d = sp_gradient_get_g2d_matrix(gr, *ctm, *bbox);
 	return g2d;
 }
 
-NRMatrix *
-sp_gradient_get_gs2d_matrix_f (SPGradient *gr, NRMatrix *ctm, NRRect *bbox, NRMatrix *gs2d)
+NR::Matrix
+sp_gradient_get_g2d_matrix(SPGradient const *gr, NR::Matrix const &ctm, NR::Rect const &bbox)
 {
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		NRMatrix bb2u, gs2u;
-
-		bb2u.c[0] = bbox->x1 - bbox->x0;
-		bb2u.c[1] = 0.0;
-		bb2u.c[2] = 0.0;
-		bb2u.c[3] = bbox->y1 - bbox->y0;
-		bb2u.c[4] = bbox->x0;
-		bb2u.c[5] = bbox->y0;
-
-		nr_matrix_multiply (&gs2u, (NRMatrix *) gr->transform, &bb2u);
-		nr_matrix_multiply (gs2d, &gs2u, ctm);
+		return ( NR::scale(bbox.dimensions())
+			 * NR::translate(bbox.min())
+			 * ctm );
 	} else {
-		nr_matrix_multiply (gs2d, (NRMatrix *) gr->transform, ctm);
+		return ctm;
 	}
+}
 
+NRMatrix *
+sp_gradient_get_gs2d_matrix_f(SPGradient const *gr, NRMatrix const *ctm, NRRect const *bbox, NRMatrix *gs2d)
+{
+	*gs2d = sp_gradient_get_gs2d_matrix(gr, *ctm, *bbox);
 	return gs2d;
 }
 
-void
-sp_gradient_set_gs2d_matrix_f (SPGradient *gr, NRMatrix *ctm, NRRect *bbox, NRMatrix *gs2d)
+NR::Matrix
+sp_gradient_get_gs2d_matrix(SPGradient const *gr, NR::Matrix const &ctm, NR::Rect const &bbox)
 {
-	NRMatrix g2d, d2g, gs2g;
-
-	SP_PRINT_MATRIX ("* GS2D:", gs2d);
-
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		NRMatrix bb2u;
-
-		bb2u.c[0] = bbox->x1 - bbox->x0;
-		bb2u.c[1] = 0.0;
-		bb2u.c[2] = 0.0;
-		bb2u.c[3] = bbox->y1 - bbox->y0;
-		bb2u.c[4] = bbox->x0;
-		bb2u.c[5] = bbox->y0;
-
-		SP_PRINT_MATRIX ("* BB2U:", &bb2u);
-
-		nr_matrix_multiply (&g2d, &bb2u, ctm);
+		return ( gr->gradientTransform
+			 * NR::scale(bbox.dimensions())
+			 * NR::translate(bbox.min())
+			 * ctm );
 	} else {
-		g2d = *ctm;
+		return gr->gradientTransform * ctm;
 	}
+}
 
-	SP_PRINT_MATRIX ("* G2D:", &g2d);
+void
+sp_gradient_set_gs2d_matrix_f(SPGradient *gr, NRMatrix const *ctm, NRRect const *bbox, NRMatrix const *gs2d)
+{
+	sp_gradient_set_gs2d_matrix(gr, *ctm, *bbox, *gs2d);
+}
 
-	nr_matrix_invert (&d2g, &g2d);
-	SP_PRINT_MATRIX ("* D2G:", &d2g);
-	SP_PRINT_MATRIX ("* G2D:", &g2d);
-	nr_matrix_invert (&g2d, &d2g);
-	SP_PRINT_MATRIX ("* D2G:", &d2g);
-	SP_PRINT_MATRIX ("* G2D:", &g2d);
+void
+sp_gradient_set_gs2d_matrix(SPGradient *gr, NR::Matrix const &ctm, NR::Rect const &bbox, NR::Matrix const &gs2d)
+{
+	gr->gradientTransform = gs2d / ctm;
+	if ( gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX ) {
+		gr->gradientTransform = ( gr->gradientTransform
+					  / NR::translate(bbox.min())
+					  / NR::scale(bbox.dimensions()) );
+	}
+	gr->gradientTransform_set = TRUE;
 
-
-	nr_matrix_multiply (&gs2g, gs2d, &d2g);
-	SP_PRINT_MATRIX ("* GS2G:", &gs2g);
-	
-	for(int i = 0; i < 6; i++)
-		gr->transform[i] = gs2g.c[i];
-
-	gr->transform_set = TRUE;
-
-	sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
+	SP_OBJECT(gr)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 /*
  * Linear Gradient
  */
 
-typedef struct _SPLGPainter SPLGPainter;
+class SPLGPainter;
 
-struct _SPLGPainter {
+struct SPLGPainter {
 	SPPainter painter;
 	SPLinearGradient *lg;
 
@@ -1070,7 +1061,7 @@ static void sp_lineargradient_build(SPObject *object, SPDocument *document, SPRe
 static void sp_lineargradient_set(SPObject *object, unsigned key, gchar const *value);
 static SPRepr *sp_lineargradient_write(SPObject *object, SPRepr *repr, guint flags);
 
-static SPPainter *sp_lineargradient_painter_new (SPPaintServer *ps, double const *affine, NRRect const *bbox);
+static SPPainter *sp_lineargradient_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &parent_transform, NRRect const *bbox);
 static void sp_lineargradient_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static void sp_lg_fill (SPPainter *painter, NRPixBlock *pb);
@@ -1141,25 +1132,25 @@ sp_lineargradient_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!sp_svg_length_read (value, &lg->x1)) {
 			sp_svg_length_unset (&lg->x1, SP_SVG_UNIT_PERCENT, 0.0, 0.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_Y1:
 		if (!sp_svg_length_read (value, &lg->y1)) {
 			sp_svg_length_unset (&lg->y1, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_X2:
 		if (!sp_svg_length_read (value, &lg->x2)) {
 			sp_svg_length_unset (&lg->x2, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_Y2:
 		if (!sp_svg_length_read (value, &lg->y2)) {
 			sp_svg_length_unset (&lg->y2, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) lg_parent_class)->set)
@@ -1201,15 +1192,15 @@ sp_lineargradient_write (SPObject *object, SPRepr *repr, guint flags)
  * 2) gradientTransform
  * 3) bbox2user
  * 4) ctm == userspace to pixel grid
+
+   See also (*) in sp-pattern about why we may need parent_transform.
  */
 
 static SPPainter *
-sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const NRRect *bbox)
+sp_lineargradient_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &parent_transform, const NRRect *bbox)
 {
 	SPGradient *gr;
 	SPLGPainter *lgp;
-	gdouble color2norm[6], color2px[6];
-	NRMatrix v2px;
 
 	SPLinearGradient *lg = SP_LINEARGRADIENT (ps);
 	gr = SP_GRADIENT (ps);
@@ -1231,67 +1222,34 @@ sp_lineargradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const NRRe
 	 *        garbage or something similar (Lauris) */
 	/* fixme: Originally I had 1023.9999 here - not sure whether
 	 *        we have really to cut out ceil int (Lauris) */
-	nr_matrix_set_identity (NR_MATRIX_D_FROM_DOUBLE (color2norm));
+	NR::Matrix color2norm (NR::identity());
 
+	NR::Matrix color2px;
+	
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		gdouble norm2pos[6], bbox2user[6];
-		gdouble color2pos[6], color2tpos[6], color2user[6];
-
-		nr_matrix_set_identity (NR_MATRIX_D_FROM_DOUBLE (norm2pos));
-
-		/* gradientTransform goes here (Lauris) */
-		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
+		NR::Matrix norm2pos (NR::identity());
 
 		/* BBox to user coordinate system */
-		bbox2user[0] = bbox->x1 - bbox->x0;
-		bbox2user[1] = 0.0;
-		bbox2user[2] = 0.0;
-		bbox2user[3] = bbox->y1 - bbox->y0;
-		bbox2user[4] = bbox->x0;
-		bbox2user[5] = bbox->y0;
-		SP_PRINT_TRANSFORM ("bbox2user", bbox2user);
+		NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0, 0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
-		/* CTM goes here */
-		SP_PRINT_TRANSFORM ("ctm", ctm);
+		NR::Matrix color2pos = color2norm * norm2pos;
+		NR::Matrix color2tpos = color2pos * gr->gradientTransform;
+		NR::Matrix color2user = color2tpos * bbox2user;
+		color2px = color2user * full_transform;
 
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2pos), NR_MATRIX_D_FROM_DOUBLE (color2norm), NR_MATRIX_D_FROM_DOUBLE (norm2pos));
-		SP_PRINT_TRANSFORM ("color2pos", color2pos);
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2tpos), NR_MATRIX_D_FROM_DOUBLE (color2pos), NR_MATRIX_D_FROM_DOUBLE (gr->transform));
-		SP_PRINT_TRANSFORM ("color2tpos", color2tpos);
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2user), NR_MATRIX_D_FROM_DOUBLE (color2tpos), NR_MATRIX_D_FROM_DOUBLE (bbox2user));
-		SP_PRINT_TRANSFORM ("color2user", color2user);
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2px), NR_MATRIX_D_FROM_DOUBLE (color2user), NR_MATRIX_D_FROM_DOUBLE (ctm));
-		SP_PRINT_TRANSFORM ("color2px", color2px);
 	} else {
-		gdouble norm2pos[6];
-		gdouble color2pos[6], color2tpos[6];
-		/* Problem: What to do, if we have mixed lengths and
-		 * percentages? */
-		/* Currently we do ignore percentages at all, but that
-		 * is not good (lauris) */
+		/* Problem: What to do, if we have mixed lengths and percentages? */
+		/* Currently we do ignore percentages at all, but that is not good (lauris) */
 
-		nr_matrix_set_identity (NR_MATRIX_D_FROM_DOUBLE (norm2pos));
+		NR::Matrix norm2pos (NR::identity());
+		NR::Matrix color2pos = color2norm * norm2pos;
+		NR::Matrix color2tpos = color2pos * gr->gradientTransform;
+		color2px = color2tpos * full_transform;
 
-		/* gradientTransform goes here (Lauris) */
-		SP_PRINT_TRANSFORM ("gradientTransform", gr->transform);
-
-		/* CTM goes here */
-		SP_PRINT_TRANSFORM ("ctm", ctm);
-
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2pos), NR_MATRIX_D_FROM_DOUBLE (color2norm), NR_MATRIX_D_FROM_DOUBLE (norm2pos));
-		SP_PRINT_TRANSFORM ("color2pos", color2pos);
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2tpos), NR_MATRIX_D_FROM_DOUBLE (color2pos), NR_MATRIX_D_FROM_DOUBLE (gr->transform));
-		SP_PRINT_TRANSFORM ("color2tpos", color2tpos);
-		nr_matrix_multiply (NR_MATRIX_D_FROM_DOUBLE (color2px), NR_MATRIX_D_FROM_DOUBLE (color2tpos), NR_MATRIX_D_FROM_DOUBLE (ctm));
-		SP_PRINT_TRANSFORM ("color2px", color2px);
 	}
 
-	v2px.c[0] = color2px[0];
-	v2px.c[1] = color2px[1];
-	v2px.c[2] = color2px[2];
-	v2px.c[3] = color2px[3];
-	v2px.c[4] = color2px[4];
-	v2px.c[5] = color2px[5];
+	NRMatrix v2px;
+	color2px.copyto (&v2px);
 
 	nr_lgradient_renderer_setup (&lgp->lgr, gr->color, gr->spread, &v2px,
 				     lg->x1.computed, lg->y1.computed, lg->x2.computed, lg->y2.computed);
@@ -1317,7 +1275,7 @@ sp_lineargradient_set_position (SPLinearGradient *lg, gdouble x1, gdouble y1, gd
 	sp_svg_length_set (&lg->x2, SP_SVG_UNIT_NONE, x2, x2);
 	sp_svg_length_set (&lg->y2, SP_SVG_UNIT_NONE, y2, y2);
 
-	sp_object_request_modified (SP_OBJECT (lg), SP_OBJECT_MODIFIED_FLAG);
+	SP_OBJECT (lg)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 /* Builds flattened repr tree of gradient - i.e. no href */
@@ -1330,7 +1288,7 @@ sp_lineargradient_build_repr (SPLinearGradient *lg, gboolean vector)
 
 	SPRepr *repr = sp_repr_new ("linearGradient");
 
-	sp_object_invoke_write ((SPObject *) lg, repr, SP_OBJECT_WRITE_EXT | SP_OBJECT_WRITE_ALL);
+	SP_OBJECT(lg)->updateRepr(repr, SP_OBJECT_WRITE_EXT | SP_OBJECT_WRITE_ALL);
 
 	if (vector) {
 		sp_gradient_ensure_vector ((SPGradient *) lg);
@@ -1352,9 +1310,9 @@ sp_lg_fill (SPPainter *painter, NRPixBlock *pb)
  * Radial Gradient
  */
 
-typedef struct _SPRGPainter SPRGPainter;
+class SPRGPainter;
 
-struct _SPRGPainter {
+struct SPRGPainter {
 	SPPainter painter;
 	SPRadialGradient *rg;
 	NRRGradientRenderer rgr;
@@ -1367,7 +1325,7 @@ static void sp_radialgradient_build (SPObject *object, SPDocument *document, SPR
 static void sp_radialgradient_set (SPObject *object, unsigned int key, const gchar *value);
 static SPRepr *sp_radialgradient_write (SPObject *object, SPRepr *repr, guint flags);
 
-static SPPainter *sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *affine, const NRRect *bbox);
+static SPPainter *sp_radialgradient_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &parent_transform, const NRRect *bbox);
 static void sp_radialgradient_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static void sp_rg_fill (SPPainter *painter, NRPixBlock *pb);
@@ -1446,7 +1404,7 @@ sp_radialgradient_set (SPObject *object, unsigned int key, const gchar *value)
 			rg->fx.value = rg->cx.value;
 			rg->fx.computed = rg->cx.computed;
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_CY:
 		if (!sp_svg_length_read (value, &rg->cy)) {
@@ -1456,25 +1414,25 @@ sp_radialgradient_set (SPObject *object, unsigned int key, const gchar *value)
 			rg->fy.value = rg->cy.value;
 			rg->fy.computed = rg->cy.computed;
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_R:
 		if (!sp_svg_length_read (value, &rg->r)) {
 			sp_svg_length_unset (&rg->r, SP_SVG_UNIT_PERCENT, 0.5, 0.5);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_FX:
 		if (!sp_svg_length_read (value, &rg->fx)) {
 			sp_svg_length_unset (&rg->fx, rg->cx.unit, rg->cx.value, rg->cx.computed);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_FY:
 		if (!sp_svg_length_read (value, &rg->fy)) {
 			sp_svg_length_unset (&rg->fy, rg->cy.unit, rg->cy.value, rg->cy.computed);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) rg_parent_class)->set)
@@ -1505,10 +1463,9 @@ sp_radialgradient_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 static SPPainter *
-sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const NRRect *bbox)
+sp_radialgradient_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &parent_transform, const NRRect *bbox)
 {
 	SPRGPainter *rgp;
-	NRMatrix gs2px;
 
 	SPRadialGradient *rg = SP_RADIALGRADIENT (ps);
 	SPGradient *gr = SP_GRADIENT (ps);
@@ -1522,45 +1479,29 @@ sp_radialgradient_painter_new (SPPaintServer *ps, const gdouble *ctm, const NRRe
 
 	rgp->rg = rg;
 
-	/* fixme: We may try to normalize here too, look at
-	 *        linearGradient (Lauris) */
+	NR::Matrix gs2px;
 
 	if (gr->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
-		NRMatrix bbox2user;
-		NRMatrix gs2user;
-
-		/* fixme: We may try to normalize here too, look at
-		 *        linearGradient (Lauris) */
-
-		/* gradientTransform goes here (Lauris) */
+		/* fixme: We may try to normalize here too, look at linearGradient (Lauris) */
 
 		/* BBox to user coordinate system */
-		// translate(bbox.topleft()) * scale(bbox.dimension())
-		bbox2user.c[0] = bbox->x1 - bbox->x0;
-		bbox2user.c[1] = 0.0;
-		bbox2user.c[2] = 0.0;
-		bbox2user.c[3] = bbox->y1 - bbox->y0;
-		bbox2user.c[4] = bbox->x0;
-		bbox2user.c[5] = bbox->y0;
+		NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0, 0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
-		/* fixme: (Lauris) */
-		nr_matrix_multiply (&gs2user, (NRMatrix *) gr->transform, &bbox2user);
-		nr_matrix_multiply (&gs2px, &gs2user, (NRMatrix *) ctm);
+		NR::Matrix gs2user = gr->gradientTransform * bbox2user;
+
+		gs2px = gs2user * full_transform;
 	} else {
-		/* Problem: What to do, if we have mixed lengths and
-		 * percentages? */
-		/* Currently we do ignore percentages at all, but that
-		 * is not good (lauris) */
+		/* Problem: What to do, if we have mixed lengths and percentages? */
+		/* Currently we do ignore percentages at all, but that is not good (lauris) */
 
-		/* fixme: We may try to normalize here too, look at
-		 *        linearGradient (Lauris) */
-
-		/* fixme: (Lauris) */
-		nr_matrix_multiply (&gs2px, (NRMatrix *) gr->transform, (NRMatrix *) ctm);
+		gs2px = gr->gradientTransform * full_transform;
 	}
 
+	NRMatrix gs2px_nr;
+	gs2px.copyto (&gs2px_nr);
+
 	nr_rgradient_renderer_setup (&rgp->rgr, gr->color, gr->spread,
-				     &gs2px,
+				     &gs2px_nr,
 				     rg->cx.computed, rg->cy.computed,
 				     rg->fx.computed, rg->fy.computed,
 				     rg->r.computed);
@@ -1587,7 +1528,7 @@ sp_radialgradient_set_position (SPRadialGradient *rg, gdouble cx, gdouble cy, gd
 	sp_svg_length_set (&rg->fy, SP_SVG_UNIT_NONE, fy, fy);
 	sp_svg_length_set (&rg->r, SP_SVG_UNIT_NONE, r, r);
 
-	sp_object_request_modified (SP_OBJECT (rg), SP_OBJECT_MODIFIED_FLAG);
+	SP_OBJECT (rg)->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
 
@@ -1601,7 +1542,7 @@ sp_radialgradient_build_repr (SPRadialGradient *rg, gboolean vector)
 
 	SPRepr *repr = sp_repr_new ("radialGradient");
 
-	sp_object_invoke_write ((SPObject *) rg, repr, SP_OBJECT_WRITE_EXT | SP_OBJECT_WRITE_ALL);
+	SP_OBJECT(rg)->updateRepr(repr, SP_OBJECT_WRITE_EXT | SP_OBJECT_WRITE_ALL);
 
 	if (vector) {
 		sp_gradient_ensure_vector ((SPGradient *) rg);
