@@ -10,10 +10,11 @@
  *
  */
 
-#include "helper/helper-forward.h"
-#include "sp-object.h"
-#include "libnr/nr-point.h"
 #include <vector>
+
+#include "display/display-forward.h"
+#include "libnr/nr-point.h"
+#include "sp-object.h"
 class SPGuideAttachment;
 
 #define SP_TYPE_GUIDE            (sp_guide_get_type ())
@@ -23,8 +24,7 @@ class SPGuideAttachment;
 #define SP_IS_GUIDE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_GUIDE))
 
 /* Represents the constraint on p that dot(g.direction, p) == g.position. */
-struct _SPGuide {
-	SPObject object;
+struct SPGuide : public SPObject {
 	NR::Point normal;
 	gdouble position;
 	guint32 color;
@@ -33,7 +33,7 @@ struct _SPGuide {
 	std::vector<SPGuideAttachment> attached_items;
 };
 
-struct _SPGuideClass {
+struct SPGuideClass {
 	SPObjectClass parent_class;
 };
 
