@@ -1,9 +1,11 @@
 #ifndef __SP_ITEM_H__
 #define __SP_ITEM_H__
 
+/** \file
+ * Some things pertinent to all visible shapes: SPItem, SPItemView, SPItemCtx, SPItemClass, SPEvent.
+ */
+
 /*
- * Abstract base class for all nodes
- *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
@@ -17,7 +19,6 @@
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-rect.h>
 
-#include "helper/units.h"
 #include "display/nr-arena-forward.h"
 #include "forward.h"
 #include "sp-object.h"
@@ -44,14 +45,14 @@ enum {
 	SP_EVENT_MOUSEOUT
 };
 
-struct _SPEvent {
+struct SPEvent {
 	unsigned int type;
 	gpointer data;
 };
 
-typedef struct _SPItemView SPItemView;
+class SPItemView;
 
-struct _SPItemView {
+struct SPItemView {
 	SPItemView *next;
 	unsigned int flags;
 	unsigned int key;
@@ -72,9 +73,9 @@ struct _SPItemView {
  */
 #define SP_ITEM_REFERENCE_FLAGS SP_ITEM_SHOW_PRINT
 
-typedef struct _SPItemCtx SPItemCtx;
+class SPItemCtx;
 
-struct _SPItemCtx {
+struct SPItemCtx {
 	SPCtx ctx;
 	/* Item to document transformation */
 	NRMatrix i2doc;
@@ -84,14 +85,13 @@ struct _SPItemCtx {
 	NRMatrix i2vp;
 };
 
-struct _SPItem {
-	SPObject object;
-
+/** Abstract base class for all visible shapes. */
+struct SPItem : public SPObject {
 	unsigned int sensitive : 1;
 	unsigned int printable : 1;
 	unsigned int stop_paint: 1;
 
-	NRMatrix transform;
+	NR::Matrix transform;
 
 	SPClipPathReference *clip_ref;
 	SPMaskReference *mask_ref;
@@ -99,13 +99,44 @@ struct _SPItem {
 	SPItemView *display;
 
 	std::vector<SPGuideConstraint> constraints;
+
+	sigc::signal<void, NR::Matrix const *, SPItem *> _transformed_signal;
+
+	bool isLocked() const;
+	void setLocked(bool lock);
+
+	bool isHidden() const;
+	void setHidden(bool hidden);
+
+	bool isHidden(unsigned display_key) const;
+
+	bool isExplicitlyHidden() const;
+
+	void setExplicitlyHidden(bool val);
+
+	bool isVisibleAndUnlocked() const;
+
+	bool isVisibleAndUnlocked(unsigned display_key) const;
+
+	NR::Matrix getRelativeTransform(SPObject const *obj) const;
+
+	void raiseOne();
+	void lowerOne();
+	void raiseToTop();
+	void lowerToBottom();
+
+	sigc::connection connectTransformed(sigc::slot<void, NR::Matrix const *, SPItem *> slot)  {
+                return _transformed_signal.connect(slot);
+        }
 };
 
-struct _SPItemClass {
+typedef std::back_insert_iterator<std::vector<NR::Point> > SnapPointsIter;
+
+struct SPItemClass {
 	SPObjectClass parent_class;
 
 	/* BBox union in given coordinate system */
-	void (* bbox) (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags);
+	void (* bbox) (SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
 
 	/* Printing method. Assumes ctm is set to item affine matrix */
 	/* fixme: Think about it, and maybe implement generic export method instead (Lauris) */
@@ -117,11 +148,13 @@ struct _SPItemClass {
 	NRArenaItem * (* show) (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 	void (* hide) (SPItem *item, unsigned int key);
 
-	/* Returns the number of points used. */
-	int (* snappoints) (SPItem *item, NR::Point points[], int size);
+	/* Write to an iterator the points that should be considered for snapping
+	** as the item's `nodes'.
+	*/
+        void (* snappoints) (SPItem const *item, SnapPointsIter p);
 
-	/* Write item transform to repr optimally */
-	void (* write_transform) (SPItem *item, SPRepr *repr, NRMatrix *transform);
+	/* Apply the transform optimally, and return any residual transformation */
+	NR::Matrix (* set_transform)(SPItem *item, NR::Matrix const &transform);
 
 	/* Emit event, if applicable */
 	gint (* event) (SPItem *item, SPEvent *event);
@@ -133,8 +166,8 @@ struct _SPItemClass {
 
 /* Methods */
 
-void sp_item_invoke_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int clear);
-void sp_item_invoke_bbox_full (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags, unsigned int clear);
+void sp_item_invoke_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const clear);
+void sp_item_invoke_bbox_full (SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags, unsigned const clear);
 
 unsigned sp_item_pos_in_parent(SPItem *item);
 
@@ -146,9 +179,10 @@ unsigned int sp_item_display_key_new (unsigned int numkeys);
 NRArenaItem *sp_item_invoke_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 void sp_item_invoke_hide (SPItem *item, unsigned int key);
 
-int sp_item_snappoints(SPItem *item, NR::Point points[], int size);
+void sp_item_snappoints(SPItem const *item, SnapPointsIter p);
 
-void sp_item_write_transform (SPItem *item, SPRepr *repr, NRMatrix *transform);
+void sp_item_write_transform(SPItem *item, SPRepr *repr, NRMatrix const *transform, NR::Matrix const *adv = NULL);
+void sp_item_write_transform(SPItem *item, SPRepr *repr, NR::Matrix const &transform, NR::Matrix const *adv = NULL);
 
 gint sp_item_event (SPItem *item, SPEvent *event);
 
@@ -159,6 +193,9 @@ NRArenaItem *sp_item_get_arenaitem (SPItem *item, unsigned int key);
 void sp_item_bbox_desktop (SPItem *item, NRRect *bbox);
 NR::Rect sp_item_bbox_desktop(SPItem *item);
 
+NR::Matrix i2anc_affine(SPObject const *item, SPObject const *ancestor);
+NR::Matrix i2i_affine(SPObject const *src, SPObject const *dest);
+
 NR::Matrix sp_item_i2doc_affine(SPItem const *item);
 NR::Matrix sp_item_i2root_affine(SPItem const *item);
 
@@ -166,9 +203,6 @@ NR::Matrix sp_item_i2root_affine(SPItem const *item);
 NRMatrix *sp_item_i2doc_affine (SPItem const *item, NRMatrix *transform);
 NRMatrix *sp_item_i2root_affine(SPItem const *item, NRMatrix *transform);
 
-/* Transformation to normalized (0,0-1,1) viewport */
-NRMatrix *sp_item_i2vp_affine (SPItem const *item, NRMatrix *transform);
-
 /* fixme: - these are evil, but OK */
 
 /* Fill *TRANSFORM with the item-to-desktop transform.  See doc/coordinates.txt
@@ -179,17 +213,14 @@ NRMatrix *sp_item_i2vp_affine (SPItem const *item, NRMatrix *transform);
  */
 NR::Matrix sp_item_i2d_affine(SPItem const *item);
 NRMatrix *sp_item_i2d_affine(SPItem const *item, NRMatrix *transform);
-NR::Matrix sp_item_i2d_affine(SPItem const *item);
+
+NR::Matrix sp_item_i2r_affine(SPItem const *item);
 
 void sp_item_set_i2d_affine(SPItem *item, NR::Matrix const &transform);
 
+NR::Matrix sp_item_dt2i_affine(SPItem const *item, SPDesktop *dt);
 NRMatrix *sp_item_dt2i_affine(SPItem const *item, SPDesktop *dt, NRMatrix *transform);
 
-/* Convert distances into SVG units */
-
-gdouble sp_item_distance_to_svg_viewport (SPItem *item, gdouble distance, const SPUnit *unit);
-gdouble sp_item_distance_to_svg_bbox (SPItem *item, gdouble distance, const SPUnit *unit);
-
-int sp_item_repr_compare_position (SPItem * first, SPItem * second);
+int sp_item_repr_compare_position(SPItem *first, SPItem *second);
 
 #endif
