@@ -11,13 +11,22 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <math.h>
 #include <string.h>
 #include "svg/svg.h"
 #include "attributes.h"
 #include "style.h"
 #include "sp-line.h"
+#include "display/curve.h"
 #include "helper/sp-intl.h"
+#include <libnr/nr-point.h>
+#include <libnr/nr-matrix.h>
+#include <libnr/nr-matrix-fns.h>
+#include <libnr/nr-matrix-ops.h>
+#include <xml/repr.h>
 
 static void sp_line_class_init (SPLineClass *klass);
 static void sp_line_init (SPLine *line);
@@ -27,9 +36,10 @@ static void sp_line_set (SPObject *object, unsigned int key, const gchar *value)
 static SPRepr *sp_line_write (SPObject *object, SPRepr *repr, guint flags);
 
 static gchar *sp_line_description (SPItem * item);
-static void sp_line_write_transform (SPItem *item, SPRepr *repr, NRMatrix *t);
+static NR::Matrix sp_line_set_transform(SPItem *item, NR::Matrix const &xform);
 
-static void sp_line_set_shape (SPLine * line);
+static void sp_line_update (SPObject *object, SPCtx *ctx, guint flags);
+static void sp_line_set_shape (SPShape *shape);
 
 static SPShapeClass *parent_class;
 
@@ -59,22 +69,21 @@ sp_line_get_type (void)
 static void
 sp_line_class_init (SPLineClass *klass)
 {
-	GObjectClass * gobject_class;
-	SPObjectClass * sp_object_class;
-	SPItemClass * item_class;
-
-	gobject_class = (GObjectClass *) klass;
-	sp_object_class = (SPObjectClass *) klass;
-	item_class = (SPItemClass *) klass;
-
-	parent_class = (SPShapeClass *)g_type_class_ref (SP_TYPE_SHAPE);
+	parent_class = (SPShapeClass *) g_type_class_ref (SP_TYPE_SHAPE);
 
+	SPObjectClass *sp_object_class = (SPObjectClass *) klass;
 	sp_object_class->build = sp_line_build;
 	sp_object_class->set = sp_line_set;
 	sp_object_class->write = sp_line_write;
 
+	SPItemClass *item_class = (SPItemClass *) klass;
 	item_class->description = sp_line_description;
-	item_class->write_transform = sp_line_write_transform;
+	item_class->set_transform = sp_line_set_transform;
+
+	sp_object_class->update = sp_line_update;
+
+	SPShapeClass *shape_class = (SPShapeClass *) klass;
+	shape_class->set_shape = sp_line_set_shape;
 }
 
 static void
@@ -90,9 +99,9 @@ sp_line_init (SPLine * line)
 static void
 sp_line_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
-
-	if (((SPObjectClass *) parent_class)->build)
+        if (((SPObjectClass *) parent_class)->build) {
 		((SPObjectClass *) parent_class)->build (object, document, repr);
+        }
 
 	sp_object_read_attr (object, "x1");
 	sp_object_read_attr (object, "y1");
@@ -103,9 +112,7 @@ sp_line_build (SPObject * object, SPDocument * document, SPRepr * repr)
 static void
 sp_line_set (SPObject *object, unsigned int key, const gchar *value)
 {
-	SPLine * line;
-
-	line = SP_LINE (object);
+	SPLine * line = SP_LINE (object);
 
 	/* fixme: we should really collect updates */
 
@@ -114,25 +121,25 @@ sp_line_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!sp_svg_length_read (value, &line->x1)) {
 			sp_svg_length_unset (&line->x1, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_line_set_shape (line);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_Y1:
 		if (!sp_svg_length_read (value, &line->y1)) {
 			sp_svg_length_unset (&line->y1, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_line_set_shape (line);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_X2:
 		if (!sp_svg_length_read (value, &line->x2)) {
 			sp_svg_length_unset (&line->x2, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_line_set_shape (line);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_Y2:
 		if (!sp_svg_length_read (value, &line->y2)) {
 			sp_svg_length_unset (&line->y2, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_line_set_shape (line);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) parent_class)->set)
@@ -141,12 +148,31 @@ sp_line_set (SPObject *object, unsigned int key, const gchar *value)
 	}
 }
 
+static void
+sp_line_update (SPObject *object, SPCtx *ctx, guint flags)
+{
+	if (flags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
+		SPLine *line = SP_LINE (object);
+
+ 		SPStyle *style = object->style;
+ 		double const d = 1.0 / NR_MATRIX_DF_EXPANSION (&((SPItemCtx *) ctx)->i2vp);
+ 		sp_svg_length_update (&line->x1, style->font_size.computed, style->font_size.computed * 0.5, d);
+ 		sp_svg_length_update (&line->x2, style->font_size.computed, style->font_size.computed * 0.5, d);
+ 		sp_svg_length_update (&line->y1, style->font_size.computed, style->font_size.computed * 0.5, d);
+ 		sp_svg_length_update (&line->y2, style->font_size.computed, style->font_size.computed * 0.5, d);
+
+		sp_shape_set_shape ((SPShape *) object);
+	}
+
+	if (((SPObjectClass *) parent_class)->update)
+		((SPObjectClass *) parent_class)->update (object, ctx, flags);
+}
+
+
 static SPRepr *
 sp_line_write (SPObject *object, SPRepr *repr, guint flags)
 {
-	SPLine *line;
-
-	line = SP_LINE (object);
+	SPLine *line  = SP_LINE (object);
 
 	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 		repr = sp_repr_new ("line");
@@ -156,6 +182,11 @@ sp_line_write (SPObject *object, SPRepr *repr, guint flags)
 		sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
 	}
 
+	sp_repr_set_double (repr, "x1", line->x1.computed);
+	sp_repr_set_double (repr, "y1", line->y1.computed);
+	sp_repr_set_double (repr, "x2", line->x2.computed);
+	sp_repr_set_double (repr, "y2", line->y2.computed);
+
 	if (((SPObjectClass *) (parent_class))->write)
 		((SPObjectClass *) (parent_class))->write (object, repr, flags);
 
@@ -165,53 +196,45 @@ sp_line_write (SPObject *object, SPRepr *repr, guint flags)
 static gchar *
 sp_line_description (SPItem * item)
 {
-	return g_strdup ("Line");
+	return g_strdup ("<b>Line</b>");
+}
+
+static NR::Matrix
+sp_line_set_transform (SPItem *item, NR::Matrix const &xform)
+{
+	SPLine *line = SP_LINE (item);
+	NR::Point points[2];
+
+	points[0] = NR::Point(line->x1.computed, line->y1.computed);
+	points[1] = NR::Point(line->x2.computed, line->y2.computed);
+
+	points[0] *= xform;
+	points[1] *= xform;
+
+	line->x1.computed = points[0][NR::X];
+	line->y1.computed = points[0][NR::Y];
+	line->x2.computed = points[1][NR::X];
+	line->y2.computed = points[1][NR::Y];
+
+	sp_shape_adjust_stroke(item, NR::expansion(xform));
+
+	SP_OBJECT (item)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+
+	return NR::identity();
 }
 
 static void
-sp_line_write_transform (SPItem *item, SPRepr *repr, NRMatrix *t)
+sp_line_set_shape (SPShape *shape)
 {
-	double sw, sh;
-	SPLine *line;
+	SPLine *line = SP_LINE (shape);
 
-	line = SP_LINE (item);
-
-	/* fixme: Would be nice to preserve units here */
-	sp_repr_set_double (repr, "x1", t->c[0] * line->x1.computed + t->c[2] * line->y1.computed + t->c[4]);
-	sp_repr_set_double (repr, "y1", t->c[1] * line->x1.computed + t->c[3] * line->y1.computed + t->c[5]);
-	sp_repr_set_double (repr, "x2", t->c[0] * line->x2.computed + t->c[2] * line->y2.computed + t->c[4]);
-	sp_repr_set_double (repr, "y2", t->c[1] * line->x2.computed + t->c[3] * line->y2.computed + t->c[5]);
-
-	/* Scalers */
-	sw = sqrt (t->c[0] * t->c[0] + t->c[1] * t->c[1]);
-	sh = sqrt (t->c[2] * t->c[2] + t->c[3] * t->c[3]);
-
-	/* And last but not least */
-	if ((fabs (sw - 1.0) > 1e-9) || (fabs (sh - 1.0) > 1e-9)) {
-		SPStyle *style;
-		gchar *str;
-		/* Scale changed, so we have to adjust stroke width */
-		style = SP_OBJECT_STYLE (item);
-		style->stroke_width.computed *= sqrt (fabs (sw * sh));
-		str = sp_style_write_difference (style, SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
-		sp_repr_set_attr (SP_OBJECT_REPR (item), "style", str);
-		g_free (str);
-	}
-}
-
-static void
-sp_line_set_shape (SPLine * line)
-{
-	SPCurve * c;
-	
-	if (hypot (line->x2.computed - line->x1.computed, line->y2.computed - line->y1.computed) < 1e-12) return;
-
-	c = sp_curve_new ();
+	SPCurve *c = sp_curve_new ();
 
 	sp_curve_moveto (c, line->x1.computed, line->y1.computed);
 	sp_curve_lineto (c, line->x2.computed, line->y2.computed);
 
-	sp_shape_set_curve (SP_SHAPE (line), c, TRUE);
+	sp_shape_set_curve_insync (shape, c, TRUE); // *_insync does not call update, avoiding infinite recursion when set_shape is called by update
+
 	sp_curve_unref (c);
 }
 
