@@ -16,8 +16,12 @@
 #include <math.h>
 #include <string.h>
 
+#include "libnr/nr-matrix-fns.h"
+#include "libnr/nr-scale-matrix-ops.h"
+#include "libnr/nr-rotate-fns.h"
 #include "svg/svg.h"
 #include "display/nr-arena-group.h"
+#include "xml/repr.h"
 #include "attributes.h"
 #include "print.h"
 #include "document.h"
@@ -26,7 +30,7 @@
 #include "sp-namedview.h"
 #include "sp-marker.h"
 
-struct _SPMarkerView {
+struct SPMarkerView {
 	SPMarkerView *next;
 	unsigned int key;
 	unsigned int size;
@@ -44,7 +48,7 @@ static SPRepr *sp_marker_write (SPObject *object, SPRepr *repr, guint flags);
 
 static NRArenaItem *sp_marker_private_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 static void sp_marker_private_hide (SPItem *item, unsigned int key);
-static void sp_marker_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags);
+static void sp_marker_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
 static void sp_marker_print (SPItem *item, SPPrintContext *ctx);
 
 static void sp_marker_view_remove (SPMarker *marker, SPMarkerView *view, unsigned int destroyitems);
@@ -165,31 +169,31 @@ sp_marker_set (SPObject *object, unsigned int key, const gchar *value)
 				marker->markerUnits_set = TRUE;
 			}
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_REFX:
 		if (!sp_svg_length_read (value, &marker->refX)) {
 			sp_svg_length_unset (&marker->refX, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_REFY:
 		if (!sp_svg_length_read (value, &marker->refY)) {
 			sp_svg_length_unset (&marker->refX, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_MARKERWIDTH:
 		if (!sp_svg_length_read (value, &marker->markerWidth)) {
 			sp_svg_length_unset (&marker->markerWidth, SP_SVG_UNIT_NONE, 3.0, 3.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_MARKERHEIGHT:
 		if (!sp_svg_length_read (value, &marker->markerHeight)) {
 			sp_svg_length_unset (&marker->markerHeight, SP_SVG_UNIT_NONE, 3.0, 3.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_ORIENT:
 		marker->orient_set = FALSE;
@@ -203,7 +207,7 @@ sp_marker_set (SPObject *object, unsigned int key, const gchar *value)
 				marker->orient_set = TRUE;
 			}
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_VIEWBOX:
 		marker->viewBox_set = FALSE;
@@ -213,13 +217,13 @@ sp_marker_set (SPObject *object, unsigned int key, const gchar *value)
 			/* fixme: We have to take original item affine into account */
 			/* fixme: Think (Lauris) */
 			eptr = (gchar *) value;
-			x = strtod (eptr, &eptr);
+			x = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			y = strtod (eptr, &eptr);
+			y = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			width = strtod (eptr, &eptr);
+			width = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			height = strtod (eptr, &eptr);
+			height = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
 			if ((width > 0) && (height > 0)) {
 				/* Set viewbox */
@@ -230,14 +234,14 @@ sp_marker_set (SPObject *object, unsigned int key, const gchar *value)
 				marker->viewBox_set = TRUE;
 			}
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_PRESERVEASPECTRATIO:
 		/* Do setup before, so we can use break to escape */
 		marker->aspect_set = FALSE;
 		marker->aspect_align = SP_ASPECT_NONE;
 		marker->aspect_clip = SP_ASPECT_MEET;
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		if (value) {
 			int len;
 			gchar c[256];
@@ -415,7 +419,6 @@ sp_marker_update (SPObject *object, SPCtx *ctx, guint flags)
 	nr_matrix_set_translate (&q, -marker->refX.computed, -marker->refY.computed);
 	nr_matrix_multiply (&marker->c2p, &q, &marker->c2p);
 
-
 	nr_matrix_multiply (&rctx.i2doc, &marker->c2p, &rctx.i2doc);
 
 	/* If viewBox is set reinitialize child viewport */
@@ -515,7 +518,7 @@ sp_marker_private_hide (SPItem *item, unsigned int key)
 }
 
 static void
-sp_marker_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags)
+sp_marker_bbox(SPItem const *, NRRect *, NR::Matrix const &, unsigned const)
 {
 	/* Break propagation */
 }
@@ -528,6 +531,13 @@ sp_marker_print (SPItem *item, SPPrintContext *ctx)
 
 /* fixme: Remove link if zero-sized (Lauris) */
 
+/**
+* First of all, removes any SPMarkerViews that a marker has with a specific key.   
+* Set up the NRArenaItem array's size in the specified SPMarker's SPMarkerView.
+* \param marker Marker to create views in.
+* \param key Key to give each SPMarkerView.
+* \param size Number of NRArenaItems to put in the SPMarkerView.
+*/
 void
 sp_marker_show_dimension (SPMarker *marker, unsigned int key, unsigned int size)
 {
@@ -557,11 +567,9 @@ sp_marker_show_dimension (SPMarker *marker, unsigned int key, unsigned int size)
 NRArenaItem *
 sp_marker_show_instance (SPMarker *marker, NRArenaItem *parent,
 			 unsigned int key, unsigned int pos,
-			 NRMatrix *base, float linewidth)
+			 NR::Matrix const &base, float linewidth)
 {
-	SPMarkerView *v;
-
-	for (v = marker->views; v != NULL; v = v->next) {
+	for (SPMarkerView *v = marker->views; v != NULL; v = v->next) {
 		if (v->key == key) {
 			if (pos >= v->size) {
 			  return NULL;
@@ -579,22 +587,19 @@ sp_marker_show_instance (SPMarker *marker, NRArenaItem *parent,
 				}
 			}
 			if (v->items[pos]) {
-				NRMatrix m;
+				NR::Matrix m;
 				if (marker->orient_auto) {
-					m = *base;
+					m = base;
 				} else {
 					/* fixme: Orient units (Lauris) */
-					nr_matrix_set_rotate (&m, marker->orient * M_PI / 180.0);
-					m.c[4] = base->c[4];
-					m.c[5] = base->c[5];
+					m = NR::Matrix(rotate_degrees(marker->orient));
+					m *= get_translation(base);
 				}
 				if (marker->markerUnits == SP_MARKER_UNITS_STROKEWIDTH) {
-					m.c[0] *= linewidth;
-					m.c[1] *= linewidth;
-					m.c[2] *= linewidth;
-					m.c[3] *= linewidth;
+					m = NR::scale(linewidth) * m;
 				}
-				nr_arena_item_set_transform (v->items[pos], &m);
+
+				nr_arena_item_set_transform(v->items[pos], m);
 			}
 			return v->items[pos];
 		}
@@ -605,6 +610,9 @@ sp_marker_show_instance (SPMarker *marker, NRArenaItem *parent,
 
 /* This replaces SPItem implementation because we have own views */
 
+/**
+* \param key SPMarkerView key to hide.
+*/
 void
 sp_marker_hide (SPMarker *marker, unsigned int key)
 {
