@@ -12,14 +12,20 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "config.h"
+
 #include <math.h>
+
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <gtk/gtk.h>
 
-#include "helper/canvas-grid.h"
+#include "display/canvas-grid.h"
+#include "helper/units.h"
 #include "svg/svg.h"
+#include "xml/repr.h"
 #include "attributes.h"
 #include "document.h"
 #include "desktop.h"
@@ -29,11 +35,16 @@
 #include "sp-item-group.h"
 #include "sp-namedview.h"
 #include "prefs-utils.h"
+#include "desktop.h"
+
+#include "isnan.h" //temp fox for isnan().  include last
 
 #define PTPERMM (72.0 / 25.4)
 
 #define DEFAULTTOLERANCE 5.0
 #define DEFAULTGRIDCOLOR 0x3f3fff2f
+#define DEFAULTGRIDEMPCOLOR 0x3f3fff52
+#define DEFAULTGRIDEMPSPACING 5
 #define DEFAULTGUIDECOLOR 0x0000ff7f
 #define DEFAULTGUIDEHICOLOR 0xff00007f
 #define DEFAULTBORDERCOLOR 0x000000ff
@@ -106,13 +117,16 @@ sp_namedview_init (SPNamedView * nv)
 {
 	nv->editable = TRUE;
 	nv->showgrid = FALSE;
-	nv->snaptogrid = FALSE;
 	nv->showguides = TRUE;
-	nv->snaptoguides = FALSE;
 	nv->showborder = TRUE;
 
 	nv->guides = NULL;
 	nv->viewcount = 0;
+
+	nv->default_layer_id = 0;
+
+	new (&nv->grid_snapper) GridSnapper(nv, 0);
+	new (&nv->guide_snapper) GuideSnapper(nv, 0);
 }
 
 static void
@@ -130,17 +144,18 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 
 	sp_object_read_attr (object, "viewonly");
 	sp_object_read_attr (object, "showgrid");
-	sp_object_read_attr (object, "snaptogrid");
 	sp_object_read_attr (object, "showguides");
-	sp_object_read_attr (object, "snaptoguides");
 	sp_object_read_attr (object, "gridtolerance");
 	sp_object_read_attr (object, "guidetolerance");
 	sp_object_read_attr (object, "gridoriginx");
 	sp_object_read_attr (object, "gridoriginy");
 	sp_object_read_attr (object, "gridspacingx");
 	sp_object_read_attr (object, "gridspacingy");
+	sp_object_read_attr (object, "gridempspacing");
 	sp_object_read_attr (object, "gridcolor");
+	sp_object_read_attr (object, "gridempcolor");
 	sp_object_read_attr (object, "gridopacity");
+	sp_object_read_attr (object, "gridempopacity");
 	sp_object_read_attr (object, "guidecolor");
 	sp_object_read_attr (object, "guideopacity");
 	sp_object_read_attr (object, "guidehicolor");
@@ -159,6 +174,11 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_object_read_attr (object, "inkscape:window-height");
 	sp_object_read_attr (object, "inkscape:window-x");
 	sp_object_read_attr (object, "inkscape:window-y");
+	sp_object_read_attr (object, "inkscape:grid-bbox");
+	sp_object_read_attr (object, "inkscape:guide-bbox");
+	sp_object_read_attr (object, "inkscape:grid-points");
+	sp_object_read_attr (object, "inkscape:guide-points");
+	sp_object_read_attr (object, "inkscape:current-layer");
 
 	/* Construct guideline list */
 
@@ -184,21 +204,14 @@ sp_namedview_release (SPObject * object)
 		namedview->guides = NULL;
 	}
 
-	/* 
-	  TODO:  This assert is getting hit when the XML editor is open and the user
-	  cancels an application quit (see Inkscape bug #864791).  But this assert
-	  doesn't seem to provide any value, since this routine doesn't refer to views
-	  at all.  So this should be reviewed in more depth to see if commenting out
-	  the assert introduces any new undesired behavior.
-
-	 */
-	g_assert (!namedview->views); 
-
 	while (namedview->gridviews) {
-		gtk_object_destroy (GTK_OBJECT (namedview->gridviews->data));
+		gtk_object_unref (GTK_OBJECT (namedview->gridviews->data));
 		namedview->gridviews = g_slist_remove (namedview->gridviews, namedview->gridviews->data);
 	}
 
+	namedview->grid_snapper.~GridSnapper();
+	namedview->guide_snapper.~GuideSnapper();
+
 	if (((SPObjectClass *) parent_class)->release)
 		((SPObjectClass *) parent_class)->release (object);
 }
@@ -206,31 +219,21 @@ sp_namedview_release (SPObject * object)
 static void
 sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 {
-	SPNamedView *nv;
-	const SPUnit *pt = NULL;
-	const SPUnit *px = NULL;
-	const SPUnit *mm = NULL;
-	GSList * l;
+	SPNamedView *nv = SP_NAMEDVIEW(object);
 
-	nv = SP_NAMEDVIEW (object);
-
-	if (!pt) pt = sp_unit_get_by_abbreviation ("pt");
-	if (!px) px = sp_unit_get_by_abbreviation ("px");
-	if (!mm) mm = sp_unit_get_by_abbreviation ("mm");
+	SPUnit const * const pt = &sp_unit_get_by_id(SP_UNIT_PT);
+	SPUnit const * const px = &sp_unit_get_by_id(SP_UNIT_PX);
+	SPUnit const * const mm = &sp_unit_get_by_id(SP_UNIT_MM);
 
 	switch (key) {
 	case SP_ATTR_VIEWONLY:
 		nv->editable = (!value);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SHOWGRID:
 		nv->showgrid = sp_str_to_bool (value);
 		sp_namedview_setup_grid (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		break;
-	case SP_ATTR_SNAPTOGRID:
-		nv->snaptogrid = sp_str_to_bool (value);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SHOWGUIDES:
 		if (!value) { // show guides if not specified, for backwards compatibility
@@ -239,11 +242,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 			nv->showguides = sp_str_to_bool (value);
 		}
 		sp_namedview_setup_guides (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
-		break;
-	case SP_ATTR_SNAPTOGUIDES:
-		nv->snaptoguides = sp_str_to_bool (value);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GRIDTOLERANCE:
 		nv->gridtoleranceunit = px;
@@ -251,7 +250,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		if (value) {
 			sp_nv_read_length (value, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->gridtolerance, &nv->gridtoleranceunit);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GUIDETOLERANCE:
 		nv->guidetoleranceunit = px;
@@ -259,7 +258,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		if (value) {
 			sp_nv_read_length (value, SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE, &nv->guidetolerance, &nv->guidetoleranceunit);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GRIDORIGINX:
 	case SP_ATTR_GRIDORIGINY:
@@ -272,7 +271,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		}
 		sp_convert_distance (&nv->gridorigin[d], nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	}
 	case SP_ATTR_GRIDSPACINGX:
@@ -286,7 +285,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		}
 		sp_convert_distance (&nv->gridspacing[d], nv->gridunit, pt);
 		sp_namedview_setup_grid (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	}
 	case SP_ATTR_GRIDCOLOR:
@@ -295,114 +294,155 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 			nv->gridcolor = (nv->gridcolor & 0xff) | sp_svg_read_color (value, nv->gridcolor);
 		}
 		sp_namedview_setup_grid (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_GRIDEMPCOLOR:
+		nv->gridempcolor = (nv->gridempcolor & 0xff) | (DEFAULTGRIDEMPCOLOR & 0xffffff00);
+		if (value) {
+			nv->gridempcolor = (nv->gridempcolor & 0xff) | sp_svg_read_color (value, nv->gridempcolor);
+		}
+		sp_namedview_setup_grid (nv);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GRIDOPACITY:
 		nv->gridcolor = (nv->gridcolor & 0xffffff00) | (DEFAULTGRIDCOLOR & 0xff);
 		sp_nv_read_opacity (value, &nv->gridcolor);
 		sp_namedview_setup_grid (nv);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_GRIDEMPOPACITY:
+		nv->gridempcolor = (nv->gridempcolor & 0xffffff00) | (DEFAULTGRIDEMPCOLOR & 0xff);
+		sp_nv_read_opacity (value, &nv->gridempcolor);
+		sp_namedview_setup_grid (nv);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_GRIDEMPSPACING:
+		nv->gridempspacing = DEFAULTGRIDEMPSPACING;
+		if (value != NULL)
+			nv->gridempspacing = atoi(value);
+		sp_namedview_setup_grid (nv);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GUIDECOLOR:
 		nv->guidecolor = (nv->guidecolor & 0xff) | (DEFAULTGUIDECOLOR & 0xffffff00);
 		if (value) {
 			nv->guidecolor = (nv->guidecolor & 0xff) | sp_svg_read_color (value, nv->guidecolor);
 		}
-		for (l = nv->guides; l != NULL; l = l->next) {
+		for (GSList *l = nv->guides; l != NULL; l = l->next) {
 			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GUIDEOPACITY:
 		nv->guidecolor = (nv->guidecolor & 0xffffff00) | (DEFAULTGUIDECOLOR & 0xff);
 		sp_nv_read_opacity (value, &nv->guidecolor);
-		for (l = nv->guides; l != NULL; l = l->next) {
+		for (GSList *l = nv->guides; l != NULL; l = l->next) {
 			g_object_set (G_OBJECT (l->data), "color", nv->guidecolor, NULL);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GUIDEHICOLOR:
 		nv->guidehicolor = (nv->guidehicolor & 0xff) | (DEFAULTGUIDEHICOLOR & 0xffffff00);
 		if (value) {
 			nv->guidehicolor = (nv->guidehicolor & 0xff) | sp_svg_read_color (value, nv->guidehicolor);
 		}
-		for (l = nv->guides; l != NULL; l = l->next) {
+		for (GSList *l = nv->guides; l != NULL; l = l->next) {
 			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_GUIDEHIOPACITY:
 		nv->guidehicolor = (nv->guidehicolor & 0xffffff00) | (DEFAULTGUIDEHICOLOR & 0xff);
 		sp_nv_read_opacity (value, &nv->guidehicolor);
-		for (l = nv->guides; l != NULL; l = l->next) {
+		for (GSList *l = nv->guides; l != NULL; l = l->next) {
 			g_object_set (G_OBJECT (l->data), "hicolor", nv->guidehicolor, NULL);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_SHOWBORDER:
 		nv->showborder = (value) ? sp_str_to_bool (value) : TRUE;
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_BORDERLAYER:
 		nv->borderlayer = SP_BORDER_LAYER_BOTTOM;
 		if (value && !strcasecmp (value, "top")) nv->borderlayer = SP_BORDER_LAYER_TOP;
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_BORDERCOLOR:
 		nv->bordercolor = (nv->bordercolor & 0xff) | (DEFAULTBORDERCOLOR & 0xffffff00);
 		if (value) {
 			nv->bordercolor = (nv->bordercolor & 0xff) | sp_svg_read_color (value, nv->bordercolor);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_BORDEROPACITY:
 		nv->bordercolor = (nv->bordercolor & 0xffffff00) | (DEFAULTBORDERCOLOR & 0xff);
 		sp_nv_read_opacity (value, &nv->bordercolor);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_PAGECOLOR:
 		nv->pagecolor = (nv->pagecolor & 0xff) | (DEFAULTPAGECOLOR & 0xffffff00);
 		if (value) {
 			nv->pagecolor = (nv->pagecolor & 0xff) | sp_svg_read_color (value, nv->pagecolor);
 		}
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_PAGEOPACITY:
 		nv->pagecolor = (nv->pagecolor & 0xffffff00) | (DEFAULTPAGECOLOR & 0xff);
 		sp_nv_read_opacity (value, &nv->pagecolor);
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_PAGESHADOW:
 		nv->pageshadow = value? atoi (value) : 2; // 2 is the default
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_ZOOM:
-		nv->zoom = value? atof (value) : 0; // zero means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		nv->zoom = value? g_ascii_strtod (value, NULL) : 0; // zero means not set
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_CX:
-		nv->cx = value? atof (value) : HUGE_VAL; // HUGE_VAL means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		nv->cx = value? g_ascii_strtod (value, NULL) : HUGE_VAL; // HUGE_VAL means not set
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_CY:
-		nv->cy = value? atof (value) : HUGE_VAL; // HUGE_VAL means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		nv->cy = value? g_ascii_strtod (value, NULL) : HUGE_VAL; // HUGE_VAL means not set
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_WINDOW_WIDTH:
 		nv->window_width = value? atoi (value) : -1; // -1 means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_WINDOW_HEIGHT:
 		nv->window_height = value? atoi (value) : -1; // -1 means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_WINDOW_X:
 		nv->window_x = value? atoi (value) : -1; // -1 means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_WINDOW_Y:
 		nv->window_y = value? atoi (value) : -1; // -1 means not set
-		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_INKSCAPE_GRID_BBOX:
+		nv->grid_snapper.setSnapTo(Snapper::BBOX_POINT, sp_str_to_bool(value));
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_INKSCAPE_GUIDE_BBOX:
+		nv->guide_snapper.setSnapTo(Snapper::BBOX_POINT, sp_str_to_bool(value));
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_INKSCAPE_GRID_POINTS:
+		nv->grid_snapper.setSnapTo(Snapper::SNAP_POINT, sp_str_to_bool(value));
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_INKSCAPE_GUIDE_POINTS:
+		nv->guide_snapper.setSnapTo(Snapper::SNAP_POINT, sp_str_to_bool(value));
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+		break;
+	case SP_ATTR_INKSCAPE_CURRENT_LAYER:
+		nv->default_layer_id = value ? g_quark_from_string(value) : 0;
+		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	default:
 		if (((SPObjectClass *) (parent_class))->set)
@@ -425,7 +465,7 @@ sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 		(* ((SPObjectClass *) (parent_class))->child_added) (object, child, ref);
 
 	id = sp_repr_attr (child, "id");
-	no = sp_document_lookup_id (object->document, id);
+	no = object->document->getObjectById(id);
 	g_assert (SP_IS_OBJECT (no));
 
 	if (SP_IS_GUIDE (no)) {
@@ -523,10 +563,15 @@ sp_namedview_show (SPNamedView * nv, gpointer desktop)
 	nv->views = g_slist_prepend (nv->views, desktop);
 
 	item = sp_canvas_item_new (SP_DT_GRID (dt), SP_TYPE_CGRID, NULL);
+	// since we're keeping a copy, we need to bump up the ref count
+	gtk_object_ref (GTK_OBJECT(item));
 	nv->gridviews = g_slist_prepend (nv->gridviews, item);
 	sp_namedview_setup_grid_item (nv, item);
 }
 
+/*
+ * Restores window geometry from the document settings
+ */
 void 
 sp_namedview_window_from_document (SPDesktop *desktop)
 {
@@ -534,22 +579,51 @@ sp_namedview_window_from_document (SPDesktop *desktop)
 	GtkWindow *win = GTK_WINDOW(gtk_object_get_data (GTK_OBJECT(desktop->owner), "window"));
 	gint save_geometry = prefs_get_int_attribute ("options.savewindowgeometry", "value", 0);
 
-	if (nv->zoom != 0 && nv->cx != HUGE_VAL && nv->cy != HUGE_VAL) {
-		sp_desktop_zoom_absolute (desktop, nv->cx, nv->cy, nv->zoom);
-	} 
+	// restore window size and position
 	if (save_geometry && win) {
 		if (nv->window_width != -1 && nv->window_height != -1) {
-			gtk_window_resize (win, nv->window_width, nv->window_height);
+			gtk_window_set_default_size (win, nv->window_width, nv->window_height);
+			gtk_window_reshow_with_initial_size (win);
 		}
 		if (nv->window_x != -1 && nv->window_y != -1) {
 			gtk_window_move (win, nv->window_x, nv->window_y);
 		}
 	}
+
+	// restore zoom and view
+	if (nv->zoom != 0 && nv->zoom != HUGE_VAL && !isNaN (nv->zoom)
+			&& nv->cx != HUGE_VAL && !isNaN (nv->cx) 
+			&& nv->cy != HUGE_VAL && !isNaN (nv->cy)) {
+		sp_desktop_zoom_absolute (desktop, nv->cx, nv->cy, nv->zoom);
+	} 
+
 	// cancel any history of zooms up to this point
 	if (desktop->zooms_past) {
 		g_list_free (desktop->zooms_past);
 		desktop->zooms_past = NULL;
 	}
+
+	SPObject *layer=NULL;
+	SPDocument *document=SP_VIEW_DOCUMENT(desktop);
+	if ( nv->default_layer_id != 0 ) {
+		layer = document->getObjectById(g_quark_to_string(nv->default_layer_id));
+	}
+	// don't use that object if it's not at least group
+	if ( !layer || !SP_IS_GROUP(layer) ) {
+		layer = NULL;
+	}
+	// if that didn't work out, look for the topmost layer
+	if (!layer) {
+		SPObject *iter=sp_object_first_child(SP_DOCUMENT_ROOT(document));
+		for ( ; iter ; iter = SP_OBJECT_NEXT(iter) ) {
+			if (desktop->isLayer(iter)) {
+				layer = iter;
+			}
+		}
+	}
+	if (layer) {
+		desktop->setCurrentLayer(layer);
+	}
 }
 
 void 
@@ -564,6 +638,7 @@ sp_namedview_document_from_window (SPDesktop *desktop)
 	sp_desktop_get_display_area (desktop, &r);
 
 	// saving window geometry is not undoable
+	gboolean saved = sp_document_get_undo_sensitive(SP_DT_DOCUMENT (desktop));
 	sp_document_set_undo_sensitive (SP_DT_DOCUMENT (desktop), FALSE);
 
 	sp_repr_set_double (view, "inkscape:zoom", SP_DESKTOP_ZOOM (desktop));
@@ -579,8 +654,10 @@ sp_namedview_document_from_window (SPDesktop *desktop)
 		sp_repr_set_int (view, "inkscape:window-y", y);
 	}
 
+	sp_repr_set_attr(view, "inkscape:current-layer", SP_OBJECT_ID(desktop->currentLayer()));
+
 	// restore undoability
-	sp_document_set_undo_sensitive (SP_DT_DOCUMENT (desktop), TRUE);
+	sp_document_set_undo_sensitive (SP_DT_DOCUMENT (desktop), saved);
 }
 
 void
@@ -609,7 +686,7 @@ sp_namedview_hide (SPNamedView * nv, gpointer desktop)
 
 	g_assert (l);
 
-	gtk_object_destroy (GTK_OBJECT (l->data));
+	gtk_object_unref (GTK_OBJECT (l->data));
 	nv->gridviews = g_slist_remove (nv->gridviews, l->data);
 }
 
@@ -651,7 +728,7 @@ sp_namedview_setup_guides (SPNamedView *nv)
 }
 
 void
-sp_namedview_toggle_guides (SPRepr *repr)
+sp_namedview_toggle_guides (SPDocument *doc, SPRepr *repr)
 {
 	unsigned int v;
 	unsigned int set = sp_repr_get_boolean (repr, "showguides", &v);
@@ -660,28 +737,32 @@ sp_namedview_toggle_guides (SPRepr *repr)
 	} else {
 		v = !v;
 	}
-	sp_repr_set_boolean (repr, "showguides", v);
 
-	if (v) {
-		sp_repr_set_boolean (repr, "snaptoguides", TRUE);
-	} else {
-		sp_repr_set_boolean (repr, "snaptoguides", FALSE);
-	}
+	gboolean saved = sp_document_get_undo_sensitive(doc);
+	sp_document_set_undo_sensitive(doc, FALSE);
+
+	sp_repr_set_boolean (repr, "showguides", v);
+	sp_repr_set_boolean (repr, "inkscape:guide-bbox", v);
+
+	sp_repr_set_attr (doc->rroot, "sodipodi:modified", "true");
+	sp_document_set_undo_sensitive(doc, saved);
 }
 
 void
-sp_namedview_toggle_grid (SPRepr *repr)
+sp_namedview_toggle_grid (SPDocument *doc, SPRepr *repr)
 {
 	unsigned int v;
 	sp_repr_get_boolean (repr, "showgrid", &v);
 	v = !v;
-	sp_repr_set_boolean (repr, "showgrid", v);
 
-	if (v) {
-		sp_repr_set_boolean (repr, "snaptogrid", TRUE);
-	} else {
-		sp_repr_set_boolean (repr, "snaptogrid", FALSE);
-	}
+	gboolean saved = sp_document_get_undo_sensitive(doc);
+	sp_document_set_undo_sensitive(doc, FALSE);
+
+	sp_repr_set_boolean (repr, "showgrid", v);
+	sp_repr_set_boolean (repr, "inkscape:grid-bbox", v);
+
+	sp_repr_set_attr (doc->rroot, "sodipodi:modified", "true");
+	sp_document_set_undo_sensitive(doc, saved);
 }
 
 static void
@@ -697,22 +778,20 @@ sp_namedview_setup_grid (SPNamedView * nv)
 static void
 sp_namedview_setup_grid_item (SPNamedView * nv, SPCanvasItem * item)
 {
-	const SPUnit *pt = NULL;
-
 	if (nv->showgrid) {
 		sp_canvas_item_show (item);
 	} else {
 		sp_canvas_item_hide (item);
 	}
 
-	if (!pt) pt = sp_unit_get_identity (SP_UNIT_ABSOLUTE);
-
 	sp_canvas_item_set ((GtkObject *) item,
 			       "color", nv->gridcolor,
 			       "originx", nv->gridorigin[NR::X],
 			       "originy", nv->gridorigin[NR::Y],
 			       "spacingx", nv->gridspacing[NR::X],
 			       "spacingy", nv->gridspacing[NR::Y],
+				   "empcolor", nv->gridempcolor,
+				   "empspacing", nv->gridempspacing,
 			       NULL);
 }
 
@@ -762,7 +841,7 @@ sp_nv_read_length (const gchar *str, guint base, gdouble *val, const SPUnit **un
 
 	if (!str) return FALSE;
 
-	v = strtod (str, &u);
+	v = g_ascii_strtod (str, &u);
 	if (!u) return FALSE;
 	while (isspace (*u)) u += 1;
 
@@ -774,37 +853,23 @@ sp_nv_read_length (const gchar *str, guint base, gdouble *val, const SPUnit **un
 
 	if (base & SP_UNIT_DEVICE) {
 		if (u[0] && u[1] && !isalnum (u[2]) && !strncmp (u, "px", 2)) {
-			static const SPUnit *device = NULL;
-			if (!device) device = sp_unit_get_identity (SP_UNIT_DEVICE);
-			*unit = device;
+			*unit = &sp_unit_get_by_id(SP_UNIT_PX);
 			*val = v;
 			return TRUE;
 		}
 	}
 
 	if (base & SP_UNIT_ABSOLUTE) {
-		static const SPUnit *pt = NULL;
-		static const SPUnit *mm = NULL;
-		static const SPUnit *cm = NULL;
-		static const SPUnit *m = NULL;
-		static const SPUnit *in = NULL;
-		if (!pt) {
-			pt = sp_unit_get_by_abbreviation ("pt");
-			mm = sp_unit_get_by_abbreviation ("mm");
-			cm = sp_unit_get_by_abbreviation ("cm");
-			m = sp_unit_get_by_abbreviation ("m");
-			in = sp_unit_get_by_abbreviation ("in");
-		}
 		if (!strncmp (u, "pt", 2)) {
-			*unit = pt;
+			*unit = &sp_unit_get_by_id(SP_UNIT_PT);
 		} else if (!strncmp (u, "mm", 2)) {
-			*unit = mm;
+			*unit = &sp_unit_get_by_id(SP_UNIT_MM);
 		} else if (!strncmp (u, "cm", 2)) {
-			*unit = cm;
+			*unit = &sp_unit_get_by_id(SP_UNIT_CM);
 		} else if (!strncmp (u, "m", 1)) {
-			*unit = m;
+			*unit = &sp_unit_get_by_id(SP_UNIT_M);
 		} else if (!strncmp (u, "in", 2)) {
-			*unit = in;
+			*unit = &sp_unit_get_by_id(SP_UNIT_IN);
 		} else {
 			return FALSE;
 		}
