@@ -18,9 +18,10 @@
 #define SP_IS_NAMEDVIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_NAMEDVIEW))
 #define SP_IS_NAMEDVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_NAMEDVIEW))
 
-#include "helper/units.h"
+#include "helper/helper-forward.h"
 #include "sp-object-group.h"
 #include "libnr/nr-point.h"
+#include "snap.h"
 
 
 
@@ -29,13 +30,10 @@ enum {
 	SP_BORDER_LAYER_TOP
 };
 
-struct _SPNamedView {
-	SPObjectGroup objectgroup;
+struct SPNamedView : public SPObjectGroup {
 	unsigned int editable : 1;
 	unsigned int showgrid : 1;
-	unsigned int snaptogrid : 1;
 	unsigned int showguides : 1;
-	unsigned int snaptoguides : 1;
 	unsigned int showborder : 1;
 	unsigned int borderlayer : 2;
 
@@ -47,10 +45,14 @@ struct _SPNamedView {
 	gint window_x;
 	gint window_y;
 
+	GridSnapper grid_snapper;
+	GuideSnapper guide_snapper;
+
 	const SPUnit *gridunit;
 	/* Grid data is in points regardless of unit */
 	NR::Point gridorigin;
 	gdouble gridspacing[2];
+	gint gridempspacing;
 
 	const SPUnit *gridtoleranceunit;
 	gdouble gridtolerance;
@@ -58,7 +60,10 @@ struct _SPNamedView {
 	const SPUnit *guidetoleranceunit;
 	gdouble guidetolerance;
 
+	GQuark default_layer_id;
+
 	guint32 gridcolor;
+	guint32 gridempcolor;
 	guint32 guidecolor;
 	guint32 guidehicolor;
 	guint32 bordercolor;
@@ -71,7 +76,7 @@ struct _SPNamedView {
 	gint viewcount;
 };
 
-struct _SPNamedViewClass {
+struct SPNamedViewClass {
 	SPObjectGroupClass parent_class;
 };
 
@@ -90,8 +95,8 @@ SPNamedView *sp_document_namedview (SPDocument * document, const gchar * name);
 void sp_namedview_window_from_document (SPDesktop *desktop);
 void sp_namedview_document_from_window (SPDesktop *desktop);
 
-void sp_namedview_toggle_guides (SPRepr *repr);
-void sp_namedview_toggle_grid (SPRepr *repr);
+void sp_namedview_toggle_guides (SPDocument *doc, SPRepr *repr);
+void sp_namedview_toggle_grid (SPDocument *doc, SPRepr *repr);
 
 #endif
 
