@@ -13,9 +13,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <list>
 
 #include "helper/sp-marshal.h"
 #include "xml/repr-private.h"
+#include "xml/sp-repr-event-vector.h"
 #include "attributes.h"
 #include "document.h"
 #include "style.h"
@@ -23,9 +25,21 @@
 #include "sp-root.h"
 
 #include "sp-object.h"
+#include "algorithms/longest-common-suffix.h"
+
+#define noSP_OBJECT_DEBUG_CASCADE
 
 #define noSP_OBJECT_DEBUG
-#define noSP_OBJECT_DEBUG_CASCADE
+
+#ifdef SP_OBJECT_DEBUG
+# define debug(f, a...) { g_print("%s(%d) %s:", \
+                                  __FILE__,__LINE__,__FUNCTION__); \
+                          g_print(f, ## a); \
+                          g_print("\n"); \
+                        }
+#else
+# define debug(f, a...) /**/
+#endif
 
 static void sp_object_class_init (SPObjectClass * klass);
 static void sp_object_init (SPObject * object);
@@ -55,10 +69,11 @@ static void sp_object_repr_order_changed (SPRepr *repr, SPRepr *child, SPRepr *o
 
 static gchar * sp_object_get_unique_id (SPObject * object, const gchar * defid);
 
+guint update_in_progress = 0; // guard against update-during-update
+
 enum {RELEASE, MODIFIED, LAST_SIGNAL};
 
 SPReprEventVector object_event_vector = {
-	NULL, /* Destroy */
 	NULL, /* Add child */
 	sp_object_repr_child_added,
 	NULL, /* Remove child */
@@ -135,29 +150,46 @@ sp_object_class_init (SPObjectClass * klass)
 static void
 sp_object_init (SPObject * object)
 {
-#ifdef SP_OBJECT_DEBUG
-	g_print("sp_object_init: id=%x, typename=%s\n", object, g_type_name_from_instance((GTypeInstance*)object));
-#endif
+	debug("id=%x, typename=%s",object, g_type_name_from_instance((GTypeInstance*)object));
 
 	object->hrefcount = 0;
+	object->_total_hrefcount = 0;
 	object->document = NULL;
 	object->children = NULL;
 	object->parent = object->next = NULL;
 	object->repr = NULL;
 	object->id = NULL;
 	object->style = NULL;
+
+	object->_collection_policy = SPObject::COLLECT_WITH_PARENT;
+
+	new (&object->_delete_signal) sigc::signal<void, SPObject *>();
+	object->_successor = NULL;
+
+	object->_label = NULL;
+	object->_default_label = NULL;
 }
 
 static void
 sp_object_finalize (GObject * object)
 {
-	SPObject *spobject;
+	SPObject *spobject = (SPObject *)object;
 
-	spobject = (SPObject *) object;
+	g_free(spobject->_label);
+	g_free(spobject->_default_label);
+	spobject->_label = NULL;
+	spobject->_default_label = NULL;
+
+	if (spobject->_successor) {
+		sp_object_unref(spobject->_successor, NULL);
+		spobject->_successor = NULL;
+	}
 
 	if (((GObjectClass *) (parent_class))->finalize) {
 		(* ((GObjectClass *) (parent_class))->finalize) (object);
 	}
+
+	spobject->_delete_signal.~signal();
 }
 
 /*
@@ -198,6 +230,7 @@ sp_object_href (SPObject *object, gpointer owner)
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 
 	object->hrefcount++;
+	object->_updateTotalHRefCount(1);
 
 	return object;
 }
@@ -210,10 +243,145 @@ sp_object_hunref (SPObject *object, gpointer owner)
 	g_return_val_if_fail (object->hrefcount > 0, NULL);
 
 	object->hrefcount--;
+	object->_updateTotalHRefCount(-1);
 
 	return NULL;
 }
 
+void SPObject::_updateTotalHRefCount(int increment) {
+	SPObject *topmost_collectable=NULL;
+	for ( SPObject *iter=this ; iter ; iter = SP_OBJECT_PARENT(iter) ) {
+		iter->_total_hrefcount += increment;
+		if ( iter->_total_hrefcount < iter->hrefcount ) {
+			g_critical("HRefs overcounted");
+		}
+		if ( iter->_total_hrefcount == 0 &&
+		     iter->_collection_policy != COLLECT_WITH_PARENT )
+		{
+			topmost_collectable = iter;
+		}
+	}
+	if (topmost_collectable) {
+		topmost_collectable->requestOrphanCollection();
+	}
+}
+
+bool SPObject::isAncestorOf(SPObject const *object) const {
+	g_return_val_if_fail(object != NULL, false);
+	object = SP_OBJECT_PARENT(object);
+	while (object) {
+		if ( object == this ) {
+			return true;
+		}
+		object = SP_OBJECT_PARENT(object);
+	}
+	return false;
+}
+
+namespace {
+
+bool same_objects(SPObject const &a, SPObject const &b) {
+	return &a == &b;
+}
+
+}
+
+SPObject const *SPObject::nearestCommonAncestor(SPObject const *object) const {
+	g_return_val_if_fail(object != NULL, NULL);
+
+	using Inkscape::Algorithms::longest_common_suffix;
+	return longest_common_suffix<SPObject::ConstParentIterator>(this, object, NULL, &same_objects);
+}
+
+SPObject *SPObject::appendChildRepr(SPRepr *repr) {
+	if (!SP_OBJECT_IS_CLONED(this)) {
+		sp_repr_append_child(SP_OBJECT_REPR(this), repr);
+		return SP_OBJECT_DOCUMENT(this)->getObjectByRepr(repr);
+	} else {
+		g_critical("Attempt to append repr as child of cloned object");
+		return NULL;
+	}
+}
+
+/** Gets the label property for the object or a default if no label 
+ *  is defined.
+ */
+gchar const *
+SPObject::label() const {
+    return _label;
+}
+
+/** Returns a default label property for the object. */
+gchar const *
+SPObject::defaultLabel() const {
+	if (_label) {
+		return _label;
+	} else {
+		if (!_default_label) {
+			gchar const *id=SP_OBJECT_ID(this);
+			if (id) {
+				_default_label = g_strdup_printf("#%s", id);
+			} else {
+				_default_label = g_strdup_printf("<%s>", sp_repr_name(SP_OBJECT_REPR(this)));
+			}
+		}
+		return _default_label;
+	}
+}
+
+/** Sets the label property for the object */
+void SPObject::setLabel(gchar const *label) {
+    sp_repr_set_attr(SP_OBJECT_REPR(this), "inkscape:label", label, false);
+}
+
+
+/** Queues the object for orphan collection */
+void SPObject::requestOrphanCollection() {
+	g_return_if_fail(document != NULL);
+	document->queueForOrphanCollection(this);
+
+	// This is a temporary hack added to make fill&stroke rebuild its gradient list when
+	// the defs are vacuumed.  gradient-vector.cpp listens to the modified signal on defs,
+	// and now we give it that signal.  Mental says that this should be made automatic by
+	// merging SPObjectGroup with SPObject; SPObjectGroup would issue this signal
+	// automatically. Or maybe just derive SPDefs from SPObjectGroup?
+	this->requestModified (SP_OBJECT_CHILD_MODIFIED_FLAG);
+}
+
+/** Sends the delete signal to all children of this object recursively */
+void SPObject::_sendDeleteSignalRecursive() {
+	for (SPObject *child = sp_object_first_child(this); child; child = SP_OBJECT_NEXT (child)) {
+		child->_delete_signal.emit(child);
+		child->_sendDeleteSignalRecursive();
+	}
+}
+
+/* Deletes the object reference, unparenting it from its parent.
+   If the \a propagate parameter is set to true, it emits a delete
+   signal.  If the \a propagate_descendants parameter is true, it
+   recursively sends the delete signal to children.
+*/
+void SPObject::deleteObject(bool propagate, bool propagate_descendants)
+{
+	sp_object_ref(this, NULL);
+	if (propagate) {
+		_delete_signal.emit(this);
+	}
+	if (propagate_descendants) {
+		this->_sendDeleteSignalRecursive();
+	}
+
+	SPRepr *repr=SP_OBJECT_REPR(this);
+	if (repr && sp_repr_parent(repr)) {
+		sp_repr_unparent(repr);
+	}
+
+	if (_successor) {
+		_successor->deleteObject(propagate, propagate_descendants);
+	}
+	sp_object_unref(this, NULL);
+}
+
 /*
  * Attaching/detaching
  */
@@ -242,11 +410,14 @@ sp_object_attach_reref (SPObject *parent, SPObject *object, SPObject *next)
 	sp_object_ref (object, parent);
 	g_object_unref (G_OBJECT (object));
 	object->parent = parent;
+	parent->_updateTotalHRefCount(object->_total_hrefcount);
+
 	object->next = next;
 	*ref = object;
 }
 
-void sp_object_reorder(SPObject *object, SPObject *next) {
+void sp_object_reorder(SPObject *object, SPObject *next)
+{
 	g_return_if_fail(object != NULL);
 	g_return_if_fail(SP_IS_OBJECT(object));
 	g_return_if_fail(object->parent != NULL);
@@ -254,9 +425,10 @@ void sp_object_reorder(SPObject *object, SPObject *next) {
 	g_return_if_fail(!next || SP_IS_OBJECT(next));
 	g_return_if_fail(!next || next->parent == object->parent);
 
-	SPObject *parent=object->parent;
-	SPObject **ref, **old_ref, **new_ref;
-	old_ref = new_ref = NULL;
+	SPObject *parent = object->parent;
+	SPObject **ref;
+	SPObject **old_ref = NULL;
+	SPObject **new_ref = NULL;
 	for ( ref = &parent->children ; *ref ; ref = &(*ref)->next ) {
 		if ( *ref == object ) {
 			old_ref = ref;
@@ -276,7 +448,7 @@ void sp_object_reorder(SPObject *object, SPObject *next) {
 	*new_ref = object;
 }
 
-static void detach_object(SPObject *parent, SPObject *object, bool unref) {
+void sp_object_detach (SPObject *parent, SPObject *object) {
 	g_return_if_fail (parent != NULL);
 	g_return_if_fail (SP_IS_OBJECT (parent));
 	g_return_if_fail (object != NULL);
@@ -296,31 +468,43 @@ static void detach_object(SPObject *parent, SPObject *object, bool unref) {
 	object->parent = NULL;
 
 	sp_object_invoke_release (object);
-
-	if (unref) {
-		sp_object_unref(object, parent);
-	}
+	parent->_updateTotalHRefCount(-object->_total_hrefcount);
 }
 
-void sp_object_detach (SPObject *parent, SPObject *object) {
-	detach_object(parent, object, false);
+void sp_object_detach_unref (SPObject *parent, SPObject *object)
+{
+	g_return_if_fail (parent != NULL);
+	g_return_if_fail (SP_IS_OBJECT (parent));
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (SP_IS_OBJECT (object));
+	g_return_if_fail (object->parent == parent);
+
+	sp_object_detach(parent, object);
+	sp_object_unref(object, parent);
 }
 
-void sp_object_detach_unref (SPObject *parent, SPObject *object) {
-	detach_object(parent, object, true);
-}
-
-SPObject *sp_object_first_child(SPObject *parent) {
+SPObject *sp_object_first_child(SPObject *parent)
+{
 	return parent->children;
 }
 
-SPObject *sp_object_get_child_by_repr(SPObject *object, SPRepr *repr) {
+SPObject *sp_object_last_child(SPObject *parent)
+{
+	if (!sp_object_first_child(parent))
+		return NULL;
+	SPObject *child;
+	for (child = sp_object_first_child(parent); SP_OBJECT_NEXT (child); child = SP_OBJECT_NEXT (child)) 
+		;
+	return child;
+}
+
+SPObject *sp_object_get_child_by_repr(SPObject *object, SPRepr *repr)
+{
 	g_return_val_if_fail(object != NULL, NULL);
 	g_return_val_if_fail(SP_IS_OBJECT(object), NULL);
 	g_return_val_if_fail(repr != NULL, NULL);
 
-	SPObject *child;
-	for ( child = object->children ; child ; child = child->next ) {
+	for ( SPObject *child = object->children ; child ; child = child->next ) {
 		if ( SP_OBJECT_REPR(child) == repr ) {
 			return child;
 		}
@@ -329,9 +513,9 @@ SPObject *sp_object_get_child_by_repr(SPObject *object, SPRepr *repr) {
 	return NULL;
 }
 
-static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * ref) {
-	GType type;
-	type = sp_repr_type_lookup(child);
+static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
+{
+	GType type = sp_repr_type_lookup(child);
 	if (!type) {
 		return;
 	}
@@ -349,13 +533,17 @@ static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * r
 	sp_object_invoke_build(ochild, object->document, child, SP_OBJECT_IS_CLONED(object));
 }
 
-static void sp_object_remove_child (SPObject * object, SPRepr * child) {
+static void sp_object_remove_child (SPObject * object, SPRepr * child)
+{
+	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
 	SPObject *ochild = sp_object_get_child_by_repr(object, child);
 	g_return_if_fail(ochild != NULL);
 	sp_object_detach_unref(object, ochild);
 }
 
-static void sp_object_order_changed (SPObject * object, SPRepr * child, SPRepr * old_ref, SPRepr * new_ref) {
+static void sp_object_order_changed (SPObject * object, SPRepr * child, SPRepr * old_ref,
+				     SPRepr * new_ref)
+{
 	SPObject *ochild = sp_object_get_child_by_repr(object, child);
 	g_return_if_fail(ochild != NULL);
 	SPObject *prev = new_ref ? sp_object_get_child_by_repr(object, new_ref) : NULL;
@@ -369,7 +557,9 @@ static void sp_object_order_changed (SPObject * object, SPRepr * child, SPRepr *
 	}
 }
 
-static void sp_object_release(SPObject *object) {
+static void sp_object_release(SPObject *object)
+{
+	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
 	while (object->children) {
 		sp_object_detach_unref(object, object->children);
 	}
@@ -383,21 +573,18 @@ static void
 sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 {
 	/* Nothing specific here */
-#ifdef SP_OBJECT_DEBUG
-	g_print("sp_object_build: id=%x, typename=%s\n", object, g_type_name_from_instance((GTypeInstance*)object));
-#endif
+	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
 
 	sp_object_read_attr (object, "xml:space");
+	sp_object_read_attr (object, "inkscape:label");
+	sp_object_read_attr (object, "inkscape:collect");
 
-	SPRepr * rchild;
-        for (rchild = repr->children; rchild != NULL; rchild = rchild->next) {
-		GType type;
-		SPObject * child;
-		type = sp_repr_type_lookup (rchild);
+        for (SPRepr *rchild = repr->children; rchild != NULL; rchild = rchild->next) {
+		GType type = sp_repr_type_lookup (rchild);
 		if (!type) {
 			continue;
 		}
-		child = SP_OBJECT(g_object_new (type, 0));
+		SPObject *child = SP_OBJECT(g_object_new (type, 0));
 		sp_object_attach_reref (object, child, NULL);
 		sp_object_invoke_build (child, document, rchild, SP_OBJECT_IS_CLONED (object));
 	}
@@ -406,9 +593,8 @@ sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 void
 sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, unsigned int cloned)
 {
-#ifdef SP_OBJECT_DEBUG
-	g_print("sp_object_invoke_build: id=%x, typename=%s\n", object, g_type_name_from_instance((GTypeInstance*)object));
-#endif
+	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
+
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (document != NULL);
@@ -428,11 +614,8 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 
 	/* If we are not cloned, force unique id */
 	if (!SP_OBJECT_IS_CLONED (object)) {
-		const gchar * id;
-		gchar * realid;
-
-		id = sp_repr_attr (object->repr, "id");
-		realid = sp_object_get_unique_id (object, id);
+		const gchar *id = sp_repr_attr (object->repr, "id");
+		gchar *realid = sp_object_get_unique_id (object, id);
 		g_assert (realid != NULL);
 
 		sp_document_def_id (object->document, realid, object);
@@ -440,10 +623,13 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 
 		/* Redefine ID, if required */
 		if ((id == NULL) || (strcmp (id, realid) != 0)) {
-			int ret;
-			ret = sp_repr_set_attr (object->repr, "id", realid);
+			gboolean undo_sensitive=sp_document_get_undo_sensitive(document);
+			sp_document_set_undo_sensitive(document, FALSE);
+			int ret = sp_repr_set_attr (object->repr, "id", realid);
+			sp_document_set_undo_sensitive(document, undo_sensitive);
+
 			if (!ret) {
-				g_error ("Cannot change id %s -> %s - probably there is stale ref", id, realid);
+				g_error ("Update id %s to unique id %s has been vetoed; cannot maintain SPObject <-> SPRepr binding", id, realid);
 			}
 		}
 	} else {
@@ -466,8 +652,8 @@ sp_object_invoke_release (SPObject *object)
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 
-	/* Parent refcount us, so there shouldn't be any */
-	g_assert (!object->parent);
+	// we need to remember our parent
+	// g_assert (!object->parent);
 	g_assert (!object->next);
 	g_assert (object->document);
 	g_assert (object->repr);
@@ -476,14 +662,16 @@ sp_object_invoke_release (SPObject *object)
 
 	g_signal_emit (G_OBJECT (object), object_signals[RELEASE], 0);
 
-	/* all hrefs should be released by the "release" handler */
+	/* all hrefs should be released by the "release" handlers */
 	g_assert (object->hrefcount == 0);
 
 	if (!SP_OBJECT_IS_CLONED (object)) {
 		g_assert (object->id);
 		sp_document_def_id (object->document, object->id, NULL);
 		g_free (object->id);
+		g_free (object->_default_label);
 		object->id = NULL;
+		object->_default_label = NULL;
 	} else {
 		g_assert (!object->id);
 	}
@@ -501,9 +689,7 @@ sp_object_invoke_release (SPObject *object)
 static void
 sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
 {
-	SPObject * object; 
-
-	object = SP_OBJECT (data);
+	SPObject *object = SP_OBJECT (data);
 
 	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->child_added)
 		(*((SPObjectClass *)G_OBJECT_GET_CLASS(object))->child_added) (object, child, ref);
@@ -512,9 +698,7 @@ sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer d
 static void
 sp_object_repr_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
 {
-	SPObject * object;
-
-	object = SP_OBJECT (data);
+	SPObject *object = SP_OBJECT (data);
 
 	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->remove_child) {
 		(* ((SPObjectClass *)G_OBJECT_GET_CLASS(object))->remove_child) (object, child);
@@ -526,9 +710,7 @@ sp_object_repr_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer
 static void
 sp_object_repr_order_changed (SPRepr * repr, SPRepr * child, SPRepr * old, SPRepr * newer, gpointer data)
 {
-	SPObject * object;
-
-	object = SP_OBJECT (data);
+	SPObject * object = SP_OBJECT (data);
 
 	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->order_changed) {
 		(* ((SPObjectClass *)G_OBJECT_GET_CLASS(object))->order_changed) (object, child, old, newer);
@@ -548,13 +730,35 @@ sp_object_private_set (SPObject *object, unsigned int key, const gchar *value)
 		if (!SP_OBJECT_IS_CLONED (object)) {
 			g_assert (value != NULL);
 			g_assert (strcmp ((const char*)value, object->id));
-			g_assert (!sp_document_lookup_id (object->document, (const char*)value));
+			g_assert (!object->document->getObjectById((const char*)value));
 			sp_document_def_id (object->document, object->id, NULL);
 			g_free (object->id);
 			object->id = g_strdup ((const char*)value);
 			sp_document_def_id (object->document, object->id, object);
+			g_free(object->_default_label);
+			object->_default_label = NULL;
 		} else {
-			g_warning ("ID of cloned object changed, so document is out of sync");
+			// This warning fires when the id is changed on the original of an SPUse, because the SPUse is updated from the same repr.
+			// The child of an SPUse has a cloned flag set - I have little idea of what it is used for.
+			// Anyway, the warning is useless, because the child has no repr of its own, and therefore no id conflict may ensue in XML.
+			//g_warning ("ID of cloned object changed, so document is out of sync");
+		}
+		break;
+	case SP_ATTR_INKSCAPE_LABEL:
+		g_free(object->_label);
+		if (value) {
+			object->_label = g_strdup(value);
+		} else {
+			object->_label = NULL;
+		}
+		g_free(object->_default_label);
+		object->_default_label = NULL;
+		break;
+	case SP_ATTR_INKSCAPE_COLLECT:
+		if ( value && !strcmp(value, "always") ) {
+			object->setCollectionPolicy(SPObject::ALWAYS_COLLECT);
+		} else {
+			object->setCollectionPolicy(SPObject::COLLECT_WITH_PARENT);
 		}
 		break;
 	case SP_ATTR_XML_SPACE:
@@ -569,7 +773,7 @@ sp_object_private_set (SPObject *object, unsigned int key, const gchar *value)
 			parent = object->parent;
 			object->xml_space.value = parent->xml_space.value;
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 		break;
 	default:
 		break;
@@ -590,8 +794,6 @@ sp_object_set (SPObject *object, unsigned int key, const gchar *value)
 void
 sp_object_read_attr (SPObject *object, const gchar *key)
 {
-	unsigned int keyid;
-
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (key != NULL);
@@ -601,11 +803,10 @@ sp_object_read_attr (SPObject *object, const gchar *key)
 	/* fixme: rething that cloning issue */
 	g_assert (SP_OBJECT_IS_CLONED (object) || object->id != NULL);
 
-	keyid = sp_attribute_lookup (key);
+	unsigned int keyid = sp_attribute_lookup (key);
 	if (keyid != SP_ATTR_INVALID) {
-		const gchar *value;
 		/* Retrieve the 'key' attribute from the object's XML representation */
-		value = sp_repr_attr (object->repr, key);
+		const gchar *value = sp_repr_attr (object->repr, key);
 
 		sp_object_set (object, keyid, value);
 	}
@@ -614,69 +815,60 @@ sp_object_read_attr (SPObject *object, const gchar *key)
 static unsigned int
 sp_object_repr_change_attr (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, gpointer data)
 {
-	SPObject * object;
-	gpointer defid;
+	SPObject *object = SP_OBJECT (data);
+	g_assert(SP_OBJECT_REPR(object) == repr);
 
-	object = SP_OBJECT (data);
-
-	if (strcmp ((const char*)key, "id") == 0) {
-		if (!newval) return FALSE;
-		defid = sp_document_lookup_id (object->document, newval);
-		if (defid == object) return TRUE;
-		if (defid) return FALSE;
+	if (SP_OBJECT_IS_CLONED(object)) {
+		// in general clones should not concern themselves with
+		// validating repr changes; that should be left to the
+		// "original" SPObject
+		return TRUE;
 	}
 
-	return TRUE;
+	if (strcmp ((const char*)key, "id") == 0) {
+		if (!newval) {
+			// this will get cleared up when we no longer force
+			// ids on reprs (and institute direct mapping between
+			// SPObjects and SPReprs)
+			// g_warning("Attempt to clear id of bound SPRepr (%p)", repr);
+			return FALSE;
+		}
+		SPObject *defid = object->document->getObjectById(newval);
+		// it's nominally ok if we don't collide with anyone
+		if ( defid && defid != object ) {
+			g_critical("The id '%s' is already claimed by SPObject %p", newval, defid);
+			return FALSE;
+		} else {
+			return TRUE;
+		}
+	} else {
+		return TRUE;
+	}
 }
 
 static void
 sp_object_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data)
 {
-	SPObject * object;
-
-	object = SP_OBJECT (data);
+	SPObject *object = SP_OBJECT (data);
 
 	sp_object_read_attr (object, key);
 
 	// manual changes to extension attributes require the normal
 	// attributes, which depend on them, to be updated immediately
 	if (is_interactive) {
-		sp_object_invoke_write (object, repr, 0);
+		object->updateRepr(repr, 0);
 	}
 }
 
 static void
 sp_object_repr_content_changed (SPRepr *repr, const gchar *oldcontent, const gchar *newcontent, gpointer data)
 {
-	SPObject * object;
-
-	object = SP_OBJECT (data);
+	SPObject * object = SP_OBJECT (data);
 
 	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->read_content)
 		(*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->read_content) (object);
 }
 
-void
-sp_object_invoke_forall (SPObject *object, SPObjectMethod func, gpointer data)
-{
-	SPRepr *repr, *child;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
-	g_return_if_fail (func != NULL);
-
-	func (object, data);
-
-	repr = SP_OBJECT_REPR (object);
-	for (child = repr->children; child != NULL; child = child->next) {
-		const gchar *id;
-		SPObject *cho;
-		id = sp_repr_attr (child, "id");
-		cho = sp_document_lookup_id (SP_OBJECT_DOCUMENT (object), id);
-		if (cho) sp_object_invoke_forall (cho, func, data);
-	}
-}
-
 static const gchar*
 sp_xml_get_space_string(unsigned int space)
 {
@@ -696,6 +888,9 @@ sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
 {
 	if (!repr && (flags & SP_OBJECT_WRITE_BUILD)) {
 		repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+		if (!( flags & SP_OBJECT_WRITE_EXT )) {
+			sp_repr_set_attr(repr, "inkscape:collect", NULL);
+		}
 	} else {
 		sp_repr_set_attr (repr, "id", object->id);
 
@@ -704,31 +899,53 @@ sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
 			xml_space = sp_xml_get_space_string(object->xml_space.value);
 			sp_repr_set_attr (repr, "xml:space", xml_space);
 		}
+
+		if ( flags & SP_OBJECT_WRITE_EXT &&
+		     object->collectionPolicy() == SPObject::ALWAYS_COLLECT )
+		{
+			sp_repr_set_attr(repr, "inkscape:collect", "always");
+		} else {
+			sp_repr_set_attr(repr, "inkscape:collect", NULL);
+		}
 	}
 	
 	return repr;
 }
 
-SPRepr *
-sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags)
-{
-	g_return_val_if_fail (object != NULL, NULL);
-	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
-
-	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->write) {
-		if (!(flags & SP_OBJECT_WRITE_BUILD) && !repr) {
-			repr = SP_OBJECT_REPR (object);
+SPRepr *SPObject::updateRepr(unsigned int flags) {
+	if (!SP_OBJECT_IS_CLONED(this)) {
+		SPRepr *repr=SP_OBJECT_REPR(this);
+		if (repr) {
+			return updateRepr(repr, flags);
+		} else {
+			g_critical("Attempt to update non-existent repr");
+			return NULL;
 		}
-		return ((SPObjectClass *) G_OBJECT_GET_CLASS(object))->write (object, repr, flags);
 	} else {
-		g_warning ("Class %s does not implement ::write", G_OBJECT_TYPE_NAME (object));
+		/* cloned objects have no repr */
+		return NULL;
+	}
+}
+
+SPRepr *SPObject::updateRepr(SPRepr *repr, unsigned int flags) {
+	if (SP_OBJECT_IS_CLONED(this)) {
+		/* cloned objects have no repr */
+		return NULL;
+	}
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS(this))->write) {
+		if (!(flags & SP_OBJECT_WRITE_BUILD) && !repr) {
+			repr = SP_OBJECT_REPR (this);
+		}
+		return ((SPObjectClass *) G_OBJECT_GET_CLASS(this))->write (this, repr, flags);
+	} else {
+		g_warning ("Class %s does not implement ::write", G_OBJECT_TYPE_NAME (this));
 		if (!repr) {
 			if (flags & SP_OBJECT_WRITE_BUILD) {
-				repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+				repr = sp_repr_duplicate (SP_OBJECT_REPR (this));
 			}
 			/* fixme: else probably error (Lauris) */
 		} else {
-			sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
+			sp_repr_merge (repr, SP_OBJECT_REPR (this), "id");
 		}
 		return repr;
 	}
@@ -736,123 +953,119 @@ sp_object_invoke_write (SPObject *object, SPRepr *repr, guint flags)
 
 /* Modification */
 
+/** Add \a flags to \a object's as dirtiness flags, and recursively add CHILD_MODIFIED flag to
+ *  parent and ancestors (as far up as necessary).
+ */
 void
-sp_object_request_update (SPObject *object, unsigned int flags)
+SPObject::requestDisplayUpdate(unsigned int flags)
 {
-	unsigned int propagate;
+	if (update_in_progress) {
+		g_print ("WARNING: Requested update while update in progress, counter = %d\n", update_in_progress);
+	}
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
 	g_return_if_fail (!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
 	g_return_if_fail ((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
 	g_return_if_fail (!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
 	/* Check for propagate before we set any flags */
-	/* Propagate means, that object is not passed through by modification request cascade yet */
-	propagate = (!(object->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+	/* Propagate means, that this is not passed through by modification request cascade yet */
+	unsigned int propagate = (!(this->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-	/* Just set object flags safe even if some have been set before */
-	object->uflags |= flags;
+	/* Just set this flags safe even if some have been set before */
+	this->uflags |= flags;
 
 	if (propagate) {
-		if (object->parent) {
-			sp_object_request_update (object->parent, SP_OBJECT_CHILD_MODIFIED_FLAG);
+		if (this->parent) {
+			this->parent->requestDisplayUpdate(SP_OBJECT_CHILD_MODIFIED_FLAG);
 		} else {
-			sp_document_request_modified (object->document);
+			sp_document_request_modified (this->document);
 		}
 	}
 }
 
 void
-sp_object_invoke_update (SPObject *object, SPCtx *ctx, unsigned int flags)
+SPObject::updateDisplay(SPCtx *ctx, unsigned int flags)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
 	g_return_if_fail (!(flags & ~SP_OBJECT_MODIFIED_CASCADE));
 
+	update_in_progress ++;
+
 #ifdef SP_OBJECT_DEBUG_CASCADE
-	g_print("Update %s:%s %x %x %x\n", g_type_name_from_instance ((GTypeInstance *) object), SP_OBJECT_ID (object), flags, object->uflags, object->mflags);
+	g_print("Update %s:%s %x %x %x\n", g_type_name_from_instance ((GTypeInstance *) this), SP_OBJECT_ID (this), flags, this->uflags, this->mflags);
 #endif
 
-	/* Get object flags */
-	flags |= object->uflags;
+	/* Get this flags */
+	flags |= this->uflags;
 	/* Copy flags to modified cascade for later processing */
-	object->mflags |= object->uflags;
+	this->mflags |= this->uflags;
 	/* We have to clear flags here to allow rescheduling update */
-	object->uflags = 0;
+	this->uflags = 0;
 
 	/* Merge style if we have good reasons to think that parent style is changed */
 	/* I am not sure, whether we should check only propagated flag */
 	/* We are currently assuming, that style parsing is done immediately */
 	/* I think this is correct (Lauris) */
 	if ((flags & SP_OBJECT_STYLE_MODIFIED_FLAG) && (flags & SP_OBJECT_PARENT_MODIFIED_FLAG)) {
-		if (object->style && object->parent) {
-			sp_style_merge_from_parent (object->style, object->parent->style);
+		if (this->style && this->parent) {
+			sp_style_merge_from_parent (this->style, this->parent->style);
 		}
-		/* attribute */
-		sp_object_read_attr (object, "xml:space");
 	}
 
-	if (((SPObjectClass *) G_OBJECT_GET_CLASS (object))->update)
-		((SPObjectClass *) G_OBJECT_GET_CLASS (object))->update (object, ctx, flags);
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS (this))->update)
+		((SPObjectClass *) G_OBJECT_GET_CLASS (this))->update (this, ctx, flags);
+
+	update_in_progress --;
 }
 
 void
-sp_object_request_modified (SPObject *object, unsigned int flags)
+SPObject::requestModified(unsigned int flags)
 {
-	unsigned int propagate;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
+	/* PARENT_MODIFIED is computed later on and is not intended to be
+	 * "manually" queued */
 	g_return_if_fail (!(flags & SP_OBJECT_PARENT_MODIFIED_FLAG));
+
+	/* we should be setting either MODIFIED or CHILD_MODIFIED... */
 	g_return_if_fail ((flags & SP_OBJECT_MODIFIED_FLAG) || (flags & SP_OBJECT_CHILD_MODIFIED_FLAG));
+
+	/* ...but not both */
 	g_return_if_fail (!((flags & SP_OBJECT_MODIFIED_FLAG) && (flags & SP_OBJECT_CHILD_MODIFIED_FLAG)));
 
-	/* Check for propagate before we set any flags */
-	/* Propagate means, that object is not passed through by modification request cascade yet */
-	propagate = (!(object->mflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG)));
+	unsigned int old_mflags=this->mflags;
+	this->mflags |= flags;
 
-	/* Just set object flags safe even if some have been set before */
-	object->mflags |= flags;
-
-	if (propagate) {
-		if (object->parent) {
-			sp_object_request_modified (object->parent, SP_OBJECT_CHILD_MODIFIED_FLAG);
+	/* If we already had MODIFIED or CHILD_MODIFIED queued, we will
+	 * have already queued CHILD_MODIFIED with our ancestors and
+	 * need not disturb them again.
+	 */
+	if (!( old_mflags & ( SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG ) )) {
+		SPObject *parent=SP_OBJECT_PARENT(this);
+		if (parent) {
+			parent->requestModified(SP_OBJECT_CHILD_MODIFIED_FLAG);
 		} else {
-			sp_document_request_modified (object->document);
+			sp_document_request_modified(SP_OBJECT_DOCUMENT(this));
 		}
 	}
 }
 
 void
-sp_object_invoke_modified (SPObject *object, unsigned int flags)
+SPObject::emitModified(unsigned int flags)
 {
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (SP_IS_OBJECT (object));
+	/* only the MODIFIED_CASCADE flag is legal here */
 	g_return_if_fail (!(flags & ~SP_OBJECT_MODIFIED_CASCADE));
 
 #ifdef SP_OBJECT_DEBUG_CASCADE
-	g_print("Modified %s:%s %x %x %x\n", g_type_name_from_instance ((GTypeInstance *) object), SP_OBJECT_ID (object), flags, object->uflags, object->mflags);
+	g_print("Modified %s:%s %x %x %x\n", g_type_name_from_instance ((GTypeInstance *) this), SP_OBJECT_ID (this), flags, this->uflags, this->mflags);
 #endif
 
-	/* Get object flags */
-	flags |= object->mflags;
-	/* We have to clear flags here to allow rescheduling modified */
-	object->mflags = 0;
+	flags |= this->mflags;
+	/* We have to clear mflags beforehand, as signal handlers may
+	 * make changes and therefore queue new modification notifications
+	 * themselves. */
+	this->mflags = 0;
 
-	g_object_ref (G_OBJECT (object));
-	g_signal_emit (G_OBJECT (object), object_signals[MODIFIED], 0, flags);
-	g_object_unref (G_OBJECT (object));
-}
-
-/* Sequence */
-gint
-sp_object_sequence (SPObject *object, gint seq)
-{
-	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->sequence)
-		return (*((SPObjectClass *) G_OBJECT_GET_CLASS(object))->sequence) (object, seq);
-
-	return seq + 1;
+	g_object_ref(G_OBJECT (this));
+	g_signal_emit(G_OBJECT (this), object_signals[MODIFIED], 0, flags);
+	g_object_unref(G_OBJECT (this));
 }
 
 /*
@@ -889,7 +1102,9 @@ const gchar *
 sp_object_tagName_get (const SPObject *object, SPException *ex)
 {
 	/* If exception is not clear, return */
-	if (!SP_EXCEPTION_IS_OK (ex)) return NULL;
+	if (!SP_EXCEPTION_IS_OK (ex)) {
+		return NULL;
+	}
 
 	/* fixme: Exception if object is NULL? */
 	return sp_repr_name (object->repr);
@@ -899,7 +1114,9 @@ const gchar *
 sp_object_getAttribute (const SPObject *object, const gchar *key, SPException *ex)
 {
 	/* If exception is not clear, return */
-	if (!SP_EXCEPTION_IS_OK (ex)) return NULL;
+	if (!SP_EXCEPTION_IS_OK (ex)) {
+		return NULL;
+	}
 
 	/* fixme: Exception if object is NULL? */
 	return (const gchar *) sp_repr_attr (object->repr, key);
@@ -935,32 +1152,27 @@ static gchar *
 sp_object_get_unique_id (SPObject * object, const gchar * id)
 {
 	static gint count = 0;
-	const gchar * name;
-	const gchar * local;
-	gchar * realid;
-	gchar * b;
-	gint len;
 
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (SP_IS_DOCUMENT (object->document));
 
 	count++;
 
-	name = sp_repr_name (object->repr);
+	const gchar *name = sp_repr_name (object->repr);
 	g_assert (name != NULL);
 
-	local = strchr (name, ':');
+	const gchar *local = strchr (name, ':');
 	if (local) {
 		name = local + 1;
 	}
 
-	len = strlen (name) + 17;
-	b = (gchar*) alloca (len);
+	gint len = strlen (name) + 17;
+	gchar *b = (gchar*) alloca (len);
 	g_assert (b != NULL);
-	realid = NULL;
+	gchar *realid = NULL;
 
 	if (id != NULL) {
-		if (sp_document_lookup_id (object->document, id) == NULL) {
+		if (object->document->getObjectById(id) == NULL) {
 			realid = g_strdup (id);
 			g_assert (realid != NULL);
 		}
@@ -968,7 +1180,7 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 
 	while (realid == NULL) {
 		g_snprintf (b, len, "%s%d", name, count);
-		if (sp_document_lookup_id (object->document, b) == NULL) {
+		if (object->document->getObjectById(b) == NULL) {
 			realid = g_strdup (b);
 			g_assert (realid != NULL);
 		} else {
@@ -984,19 +1196,14 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 const gchar *
 sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *def)
 {
-	const gchar *style;
-	const gchar *val;
-
 	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
-	style = sp_repr_attr (object->repr, "style");
+	const gchar *style = sp_repr_attr (object->repr, "style");
 	if (style) {
-		gchar *p;
-		gint len;
-		len = strlen (key);
-		for (p = strstr (style, key); p; p = strstr (style, key)) {
+		gint len = strlen (key);
+		for (gchar *p = strstr (style, key); p; p = strstr (style, key)) {
 			p += len;
 			while ((*p <= ' ') && *p) p++;
 			if (*p++ != ':') break;
@@ -1004,8 +1211,10 @@ sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *d
 			if (*p) return p;
 		}
 	}
-	val = sp_repr_attr (object->repr, key);
-	if (val) return val;
+	const gchar *val = sp_repr_attr (object->repr, key);
+	if (val) {
+		return val;
+	}
 	if (object->parent) {
 		return sp_object_get_style_property (object->parent, key, def);
 	}
