@@ -1,8 +1,10 @@
 #define __SP_OFFSET_C__
 
+/** \file
+ * Implementation of SPOffset, i.e. <path sodipodi:type="inkscape:offset">.
+ */
+
 /*
- * <inkscape:offset> implementation
- *
  * Authors (of the sp-spiral.c upon which this file was constructed):
  *   Mitsuru Oka <oka326@parkcity.ne.jp>
  *   Lauris Kaplinski <lauris@kaplinski.com>
@@ -23,7 +25,7 @@
 #include "svg/stringstream.h"
 #include "attributes.h"
 #include "document.h"
-#include "helper/bezier-utils.h"
+#include "display/curve.h"
 #include "dialogs/object-attributes.h"
 #include "helper/sp-intl.h"
 #include "xml/repr-private.h"
@@ -31,16 +33,21 @@
 #include "livarot/Path.h"
 #include "livarot/Shape.h"
 
+#include "enums.h"
+#include "prefs-utils.h"
 #include "sp-text.h"
 #include "sp-shape.h"
-
 #include "sp-offset.h"
+#include "sp-use-reference.h"
+#include "uri.h"
 
+#include "libnr/n-art-bpath.h"
 #include "libnr/nr-point.h"
 #include <libnr/nr-point-fns.h>
 #include <libnr/nr-point-ops.h>
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-fns.h>
+#include <libnr/nr-matrix-ops.h>
 
 #define noOFFSET_VERBOSE
 
@@ -65,6 +72,7 @@
 
 static void sp_offset_class_init (SPOffsetClass * klass);
 static void sp_offset_init (SPOffset * offset);
+static void sp_offset_finalize(GObject *obj);
 
 static void sp_offset_build (SPObject * object, SPDocument * document,
                              SPRepr * repr);
@@ -76,59 +84,20 @@ static void sp_offset_update (SPObject * object, SPCtx * ctx, guint flags);
 static void sp_offset_release (SPObject * object);
 
 static gchar *sp_offset_description (SPItem * item);
-static int sp_offset_snappoints(SPItem *item, NR::Point p[], int size);
+static void sp_offset_snappoints(SPItem const *item, SnapPointsIter p);
 static void sp_offset_set_shape (SPShape * shape);
 
-Path *bpath_to_liv_path (ArtBpath * bpath);
+Path *bpath_to_liv_path (NArtBpath * bpath);
 
 void   refresh_offset_source(SPOffset* offset);
 
-// pour recevoir les changements de l'objet source
-static void sp_offset_source_attr_changed (SPRepr * repr, const gchar * key,
-                                           const gchar * oldval,
-                                           const gchar * newval,
-					   bool is_interactive, void * data);
-static void sp_offset_source_destroy (SPRepr * repr, void *data);
-static void sp_offset_source_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, void * data);
-static void sp_offset_source_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, void * data);
-static void sp_offset_source_content_changed (SPRepr *repr, const gchar *oldcontent, const gchar *newcontent, void * data);
+static void sp_offset_start_listening(SPOffset *offset,SPObject* to);
+static void sp_offset_quit_listening(SPOffset *offset);
+static void sp_offset_href_changed(SPObject *old_ref, SPObject *ref, SPOffset *offset);
+static void sp_offset_move_compensate(NR::Matrix const *mp, SPItem *original, SPOffset *self);
+static void sp_offset_delete_self(SPObject *deleted, SPOffset *self);
+static void sp_offset_source_modified (SPObject *iSource, guint flags, SPItem *item);
 
-// the regular listener vector
-SPReprEventVector offset_source_event_vector = {
-  sp_offset_source_destroy,
-  NULL,				/* Add child */
-  sp_offset_source_child_added,
-  NULL,
-  sp_offset_source_child_removed,				/* Child removed */
-  NULL,
-  sp_offset_source_attr_changed,
-  NULL,				/* Change content */
-  sp_offset_source_content_changed,
-  NULL,				/* change_order */
-  NULL
-};
-
-// listener vector for sons of source object
-// it is intended to deal with text elements
-// indeed, text is mostly stored in tspans (or text itself), so we need to listen to these
-// the text object has listener attached that will track adding and removal of children,
-// and add this listener vector to them
-// theoritically, children should get the same listener vector as their parents, but that would require us to
-// track in the sp-offset what is the source and what is the childrens. having only one object with the
-// offset_source_event_vector makes that easy
-SPReprEventVector offset_source_child_event_vector = {
-  sp_offset_source_destroy,
-  NULL,				/* Add child */
-  NULL,
-  NULL,
-  NULL,				/* Child removed */
-  NULL,
-  NULL,
-  NULL,				/* Change content */
-  sp_offset_source_content_changed,
-  NULL,				/* change_order */
-  NULL
-};
 
 // slow= source path->polygon->offset of polygon->polygon->path
 // fast= source path->offset of source path->polygon->path
@@ -170,12 +139,15 @@ sp_offset_get_type (void)
 static void
 sp_offset_class_init (SPOffsetClass * klass)
 {
+	GObjectClass  *gobject_class = (GObjectClass *) klass;
   SPObjectClass *sp_object_class = (SPObjectClass *) klass;
-  SPItemClass *item_class = (SPItemClass *) klass;
-  SPShapeClass *shape_class = (SPShapeClass *) klass;
+  SPItemClass   *item_class = (SPItemClass *) klass;
+  SPShapeClass  *shape_class = (SPShapeClass *) klass;
   
   parent_class = (SPShapeClass *) g_type_class_ref (SP_TYPE_SHAPE);
   
+	gobject_class->finalize = sp_offset_finalize;
+
   sp_object_class->build = sp_offset_build;
   sp_object_class->write = sp_offset_write;
   sp_object_class->set = sp_offset_set;
@@ -195,9 +167,28 @@ sp_offset_init (SPOffset * offset)
   offset->original = NULL;
   offset->originalPath = NULL;
   offset->knotSet = false;
-  offset->sourceObject = NULL;
+	offset->sourceDirty=false;
+	offset->isUpdating=false;
+	// init various connections
+  offset->sourceHref = NULL;
   offset->sourceRepr = NULL;
-  offset->sourceDirty=false;
+  offset->sourceObject = NULL;
+	new (&offset->_delete_connection) sigc::connection();
+	new (&offset->_changed_connection) sigc::connection();
+	new (&offset->_transformed_connection) sigc::connection();
+	// set up the uri reference
+	offset->sourceRef = new SPUseReference(SP_OBJECT(offset));
+	offset->_changed_connection = offset->sourceRef->changedSignal().connect(sigc::bind(sigc::ptr_fun(sp_offset_href_changed), offset));
+}
+static void
+sp_offset_finalize(GObject *obj)
+{
+	SPOffset *offset = (SPOffset *) obj;
+	
+	delete offset->sourceRef;
+	offset->_delete_connection.~connection();
+	offset->_changed_connection.~connection();
+	offset->_transformed_connection.~connection();
 }
 
 static void
@@ -224,14 +215,21 @@ sp_offset_build (SPObject * object, SPDocument * document, SPRepr * repr)
     
     sp_object_read_attr (object, "inkscape:original");
   }
-  if ( sp_repr_attr (object->repr, "inkscape:href") ) {
-    sp_object_read_attr (object, "inkscape:href");
+  if ( sp_repr_attr (object->repr, "xlink:href") ) {
+    sp_object_read_attr (object, "xlink:href");
   } else {
-    const gchar* oldA=sp_repr_attr(object->repr,"xlink:href");
-    sp_repr_set_attr(object->repr,"inkscape:href",oldA);
-    sp_repr_set_attr(object->repr,"xlink:href",NULL);
-    
-    sp_object_read_attr (object, "inkscape:href");
+    const gchar* oldA=sp_repr_attr(object->repr,"inkscape:href");
+		if ( oldA ) {
+			int   lA=strlen(oldA);
+			char* nA=(char*)malloc((lA+1)*sizeof(char));
+			memcpy(nA+1,oldA,lA*sizeof(char));
+			nA[0]='#';
+			nA[lA+1]=0;
+			sp_repr_set_attr(object->repr,"xlink:href",nA);
+			free(nA);
+			sp_repr_set_attr(object->repr,"inkscape:href",NULL);
+    }
+    sp_object_read_attr (object, "xlink:href");
   }
 }
 
@@ -240,22 +238,28 @@ sp_offset_write (SPObject * object, SPRepr * repr, guint flags)
 {
   SPOffset *offset = SP_OFFSET (object);
   
-  if ((flags & SP_OBJECT_WRITE_BUILD) && !repr)
-  {
+  if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
     repr = sp_repr_new ("path");
   }
   
-  if (flags & SP_OBJECT_WRITE_EXT)
-  {
+  if (flags & SP_OBJECT_WRITE_EXT) {
     /* Fixme: we may replace these attributes by
-    * inkscape:offset="cx cy exp revo rad arg t0"
-    */
+		 * inkscape:offset="cx cy exp revo rad arg t0"
+		 */
     sp_repr_set_attr (repr, "sodipodi:type", "inkscape:offset");
     sp_repr_set_double (repr, "inkscape:radius", offset->rad);
     sp_repr_set_attr (repr, "inkscape:original", offset->original);
-    sp_repr_set_attr (repr, "inkscape:href", offset->sourceObject);
+    sp_repr_set_attr (repr, "inkscape:href", offset->sourceHref);
   }
-  
+
+
+  // Make sure the object has curve 
+  SPCurve *curve = sp_shape_get_curve (SP_SHAPE (offset));
+  if (curve == NULL) {
+    sp_offset_set_shape (SP_SHAPE (offset));
+  }
+
+  // write that curve to "d"
   char *d = sp_svg_write_path (((SPShape *) offset)->curve->bpath);
   sp_repr_set_attr (repr, "d", d);
   g_free (d);
@@ -272,27 +276,19 @@ sp_offset_release (SPObject * object)
 {
   SPOffset *offset = (SPOffset *) object;
   
-  if (offset->original)
-    free (offset->original);
-  if (offset->originalPath)
-    delete ((Path *) offset->originalPath);
-  
+  if (offset->original) free (offset->original);
+  if (offset->originalPath) delete ((Path *) offset->originalPath);
   offset->original = NULL;
   offset->originalPath = NULL;
   
-  if (offset->sourceObject)
-    free (offset->sourceObject);
-  offset->sourceObject = NULL;
-  
-  if (offset->sourceRepr)
-  {
-    sp_repr_remove_listener_by_data (offset->sourceRepr, offset);
-    for (SPRepr *child=offset->sourceRepr->children;child;child=child->next) sp_repr_remove_listener_by_data (child, offset);
-  }
-  offset->sourceRepr = NULL;
-  
-  if (((SPObjectClass *) parent_class)->release)
-  {
+	sp_offset_quit_listening(offset);
+	
+	offset->_changed_connection.disconnect();
+	g_free(offset->sourceHref);
+	offset->sourceHref = NULL;
+	offset->sourceRef->detach();
+    
+  if (((SPObjectClass *) parent_class)->release) {
     ((SPObjectClass *) parent_class)->release (object);
   }
   
@@ -311,103 +307,58 @@ sp_offset_set (SPObject * object, unsigned int key, const gchar * value)
   {
     case SP_ATTR_INKSCAPE_ORIGINAL:
     case SP_ATTR_SODIPODI_ORIGINAL:
-      if (value == NULL)
-      {
-      }
-      else
-      {
-        if (offset->original)
-        {
+      if (value == NULL) {
+      } else {
+        if (offset->original) {
           free (offset->original);
           delete ((Path *) offset->originalPath);
           offset->original = NULL;
           offset->originalPath = NULL;
         }
-        ArtBpath *bpath;
+        NArtBpath *bpath;
         SPCurve *curve;
         
         offset->original = strdup (value);
         
         bpath = sp_svg_read_path (offset->original);
         curve = sp_curve_new_from_bpath (bpath);	// curve se chargera de detruire bpath
+	g_assert (curve != NULL);
         offset->originalPath = bpath_to_liv_path (curve->bpath);
         sp_curve_unref (curve);
         
         offset->knotSet = false;
-        sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+        if ( offset->isUpdating == false ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
       }
       break;
     case SP_ATTR_INKSCAPE_RADIUS:
     case SP_ATTR_SODIPODI_RADIUS:
-	if (!sp_svg_length_read_computed_absolute (value, &offset->rad)) {
+			if (!sp_svg_length_read_computed_absolute (value, &offset->rad)) {
         if (fabs (offset->rad) < 0.01)
           offset->rad = (offset->rad < 0) ? -0.01 : 0.01;
         offset->knotSet = false; // knotset=false because it's not set from the context
       }
-      sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG);
+      if ( offset->isUpdating == false ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
       break;
     case SP_ATTR_INKSCAPE_HREF:
     case SP_ATTR_XLINK_HREF:
-      if (value)
-      {
-        if (offset->sourceObject)
-        {
-          if (strcmp (value, offset->sourceObject) == 0)
-            return;
-          free (offset->sourceObject);
-          offset->sourceObject = strdup (value);
-        }
-        else
-        {
-          offset->sourceObject = strdup (value);
-        }
-        if (offset->sourceRepr)
-        {
-          sp_repr_remove_listener_by_data (offset->sourceRepr, offset);
-        }
-        offset->sourceRepr = NULL;
-        SPObject *refobj;
-        refobj =
-          sp_document_lookup_id (SP_OBJECT (offset)->document,
-                                 offset->sourceObject);
-        if (refobj)
-          offset->sourceRepr = refobj->repr;
-        if (offset->sourceRepr)
-        {
-          sp_repr_add_listener (offset->sourceRepr,
-                                &offset_source_event_vector, offset);
-          // et les enfants s'il y en a (pour le texte)
-          SPRepr   *cur_child=offset->sourceRepr->children;
-          while ( cur_child ) {
-            sp_repr_add_listener (cur_child,
-                                  &offset_source_child_event_vector, offset);
-            cur_child=cur_child->next;
-          }
-        }
-        else
-        {
-          free (offset->sourceObject);
-          offset->sourceObject = NULL;
-        }
-        if (offset->sourceRepr)
-        {
-          refresh_offset_source(offset);
-        }
-      }
-      else
-      {
-        if (offset->sourceObject)
-        {
-          free (offset->sourceObject);
-          offset->sourceObject = NULL;
-          if (offset->sourceRepr)
-          {
-            sp_repr_remove_listener_by_data (offset->sourceRepr,
-                                             offset);
-          }
-          offset->sourceRepr = NULL;
-        }
-      }
+			if ( value == NULL ) {
+				sp_offset_quit_listening(offset);
+				if ( offset->sourceHref ) g_free(offset->sourceHref);
+				offset->sourceHref = NULL;
+				offset->sourceRef->detach();
+			} else {
+				if ( offset->sourceHref && ( strcmp(value, offset->sourceHref) == 0 ) ) {
+				} else {
+					if ( offset->sourceHref ) g_free(offset->sourceHref);
+					offset->sourceHref = g_strdup(value);
+					try {
+						offset->sourceRef->attach(Inkscape::URI(value));
+					} catch (Inkscape::BadURIException &e) {
+						g_warning("%s", e.what());
+						offset->sourceRef->detach();
+					}
+				}
+			}
       break;
     default:
       if (((SPObjectClass *) parent_class)->set)
@@ -421,13 +372,14 @@ static void
 sp_offset_update (SPObject * object, SPCtx * ctx, guint flags)
 {
   SPOffset* offset = SP_OFFSET(object);
+	offset->isUpdating=true; // prevent sp_offset_set from requesting updates
   if ( offset->sourceDirty ) refresh_offset_source(offset);
   if (flags &
       (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG |
-       SP_OBJECT_VIEWPORT_MODIFIED_FLAG))
-  {
+       SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
     sp_shape_set_shape ((SPShape *) object);
   }
+	offset->isUpdating=false;
   
   if (((SPObjectClass *) parent_class)->update)
     ((SPObjectClass *) parent_class)->update (object, ctx, flags);
@@ -436,25 +388,23 @@ sp_offset_update (SPObject * object, SPCtx * ctx, guint flags)
 static gchar *
 sp_offset_description (SPItem * item)
 {
-  SPOffset *offset = SP_OFFSET (item);
-  char tempSt[256];
-  if (offset->rad > 0)
-  {
-    sprintf (tempSt, "Path outset by %f pt", offset->rad);
-    return g_strdup (tempSt);
-  }
-  if (offset->rad < 0)
-  {
-    sprintf (tempSt, "Path inset by %f pt", -offset->rad);
-    return g_strdup (tempSt);
-  }
-  return g_strdup ("Offset by 0pt = original path");
+	SPOffset *offset = SP_OFFSET (item);
+
+	if ( offset->sourceHref ) {
+		// TRANSLATORS COMMENT: %s is either "outset" or "inset" depending on sign
+		return g_strdup_printf(_("<b>Linked offset</b>, %s by %f pt. Use <b>Shift+D</b> to look up original"),
+			 (offset->rad >= 0)? _("outset") : _("inset"), fabs (offset->rad));
+	} else {
+		// TRANSLATORS COMMENT: %s is either "outset" or "inset" depending on sign
+		return g_strdup_printf(_("<b>Dynamic offset</b>, %s by %f pt"),
+			 (offset->rad >= 0)? _("outset") : _("inset"), fabs (offset->rad));
+	}
 }
 
 // duplicate of splivarot
-// loads a ArtBpath (like the one stored in a SPCurve) into a livarot Path 
+// loads a NArtBpath (like the one stored in a SPCurve) into a livarot Path 
 Path *
-bpath_to_liv_path (ArtBpath * bpath)
+bpath_to_liv_path (NArtBpath * bpath)
 {
   if (bpath == NULL)
     return NULL;
@@ -467,11 +417,11 @@ bpath_to_liv_path (ArtBpath * bpath)
     float lastX = 0.0;
     float lastY = 0.0;
     
-    for (i = 0; bpath[i].code != ART_END; i++)
+    for (i = 0; bpath[i].code != NR_END; i++)
     {
       switch (bpath[i].code)
       {
-        case ART_LINETO:
+        case NR_LINETO:
           lastX = bpath[i].x3;
           lastY = bpath[i].y3;
           {
@@ -480,7 +430,7 @@ bpath_to_liv_path (ArtBpath * bpath)
           }
             break;
           
-        case ART_CURVETO:
+        case NR_CURVETO:
         {
           NR::Point  tmp(bpath[i].x3, bpath[i].y3);
           NR::Point  tms;
@@ -495,11 +445,11 @@ bpath_to_liv_path (ArtBpath * bpath)
           lastY = bpath[i].y3;
           break;
           
-        case ART_MOVETO_OPEN:
-        case ART_MOVETO:
+        case NR_MOVETO_OPEN:
+        case NR_MOVETO:
           if (closed)
             dest->Close ();
-          closed = (bpath[i].code == ART_MOVETO);
+          closed = (bpath[i].code == NR_MOVETO);
           lastX = bpath[i].x3;
           lastY = bpath[i].y3;
           {
@@ -518,68 +468,12 @@ bpath_to_liv_path (ArtBpath * bpath)
   return dest;
 }
 
-// write the livarot Path into a "d" element to be placed in the xml
-gchar *
-liv_svg_dump_path2 (Path * path)
-{
-  Inkscape::SVGOStringStream os; 
- 
-  for (int i = 0; i < path->descr_nb; i++)
-  {
-    Path::path_descr theD = path->descr_cmd[i];
-    int typ = theD.flags & descr_type_mask;
-    if (typ == descr_moveto)
-    {
-      Path::path_descr_moveto*  nData=(Path::path_descr_moveto*)(path->descr_data+theD.dStart);
-      os << "M " << nData->p[0] << " " << nData->p[1] << " ";
-    }
-    else if (typ == descr_lineto)
-    {
-      Path::path_descr_lineto*  nData=(Path::path_descr_lineto*)(path->descr_data+theD.dStart);
-      os << "L " << nData->p[0] << " " << nData->p[1] << " ";
-    }
-    else if (typ == descr_cubicto)
-    {
-      Path::path_descr_cubicto*  nData=(Path::path_descr_cubicto*)(path->descr_data+theD.dStart);
-      float lastX, lastY;
-      {
-        NR::Point tmp = path->PrevPoint (i - 1);
-        lastX=tmp[0];
-        lastY=tmp[1];
-      }
-		os << "C " 
-			<< lastX + nData->stD[0] / 3 << " "
-			<< lastY + nData->stD[1] / 3 << " "
-			<< nData->p[0] - nData->enD[0] / 3 << " "
-			<< nData->p[1] - nData->enD[1] / 3 << " "
-			<< nData->p[0] << " " << nData->p[1] << " ";
-    }
-    else if (typ == descr_arcto)
-    {
-      Path::path_descr_arcto*  nData=(Path::path_descr_arcto*)(path->descr_data+theD.dStart);
-		os << "A " << nData->rx << " " << nData->ry << " " << nData->angle << " "
-			<< ((nData->large) ? "1" : "0") << " " << ((nData->clockwise) ? "0" : "1") << " "
-			<< nData->p[0] << " " << nData->p[1] << " ";
-    }
-    else if (typ == descr_close)
-    {
-		os << "z ";
-    }
-    else
-    {
-    }
-  }
-  
-  return g_strdup(os.str().c_str());
-}
 
 static void
 sp_offset_set_shape (SPShape * shape)
 {
   SPOffset *offset = SP_OFFSET (shape);
   
-  sp_object_request_modified (SP_OBJECT (offset), SP_OBJECT_MODIFIED_FLAG);
-  
   if ( offset->originalPath == NULL ) {
     // oops : no path?! (the offset object should do harakiri)
     return;
@@ -596,8 +490,9 @@ sp_offset_set_shape (SPShape * shape)
     
     const char *res_d = sp_repr_attr(SP_OBJECT(shape)->repr,"inkscape:original");
     if ( res_d ) {
-      ArtBpath *bpath = sp_svg_read_path (res_d);
+      NArtBpath *bpath = sp_svg_read_path (res_d);
       SPCurve *c = sp_curve_new_from_bpath (bpath);
+      g_assert(c != NULL);
       sp_shape_set_curve_insync ((SPShape *) offset, c, TRUE);
       sp_curve_unref (c);
     }
@@ -648,11 +543,11 @@ sp_offset_set_shape (SPShape * shape)
   
   theRes->ConvertToForme (orig, 1, originaux);
 
-		SPItem *item = &(shape->item);
+		SPItem *item = shape;
 		NR::Rect bbox = sp_item_bbox_desktop (item);
-		if (bbox.topleft()[NR::X] < bbox.bottomright()[NR::X]) { // otherwise the bbox is invalid
+		if (!bbox.isEmpty()) {
 			gdouble size = L2(bbox.dimensions());
-			gdouble exp = NR::expansion(NR::Matrix(item->transform));
+			gdouble const exp = NR::expansion(item->transform);
 			if (exp != 0) 
 				size /= exp;
 			orig->Coalesce (size * 0.001);
@@ -831,21 +726,22 @@ sp_offset_set_shape (SPShape * shape)
   } 
   {
     char *res_d = NULL;
-    if (orig->descr_nb <= 1)
+    if (orig->descr_cmd.size() <= 1)
     {
       // aie.... plus rien
       res_d = strdup ("M 0 0 L 0 0 z");
-      printf("%s\n",res_d);
+      //printf("%s\n",res_d);
     }
     else
     {
       
-      res_d = liv_svg_dump_path2 (orig);
+      res_d = orig->svg_dump_path ();
     } 
     delete orig;
     
-    ArtBpath *bpath = sp_svg_read_path (res_d);
+    NArtBpath *bpath = sp_svg_read_path (res_d);
     SPCurve *c = sp_curve_new_from_bpath (bpath);
+    g_assert(c != NULL);
     sp_shape_set_curve_insync ((SPShape *) offset, c, TRUE);
     sp_curve_unref (c);
     
@@ -854,12 +750,11 @@ sp_offset_set_shape (SPShape * shape)
 }
 
 
-static int sp_offset_snappoints(SPItem *item, NR::Point p[], int size)
+static void sp_offset_snappoints(SPItem const *item, SnapPointsIter p)
 {
-  if (((SPItemClass *) parent_class)->snappoints)
-    return ((SPItemClass *) parent_class)->snappoints (item, p, size);
-  
-  return 0;
+  if (((SPItemClass *) parent_class)->snappoints) {
+    ((SPItemClass *) parent_class)->snappoints (item, p);
+  }
 }
 
 
@@ -923,7 +818,7 @@ double
 sp_offset_distance_to_original (SPOffset * offset, NR::Point px)
 {
   if (offset == NULL || offset->originalPath == NULL
-      || ((Path *) offset->originalPath)->descr_nb <= 1)
+      || ((Path *) offset->originalPath)->descr_cmd.size() <= 1)
     return 1.0;
   double dist = 1.0;
   Shape *theShape = new Shape;
@@ -939,7 +834,7 @@ sp_offset_distance_to_original (SPOffset * offset, NR::Point px)
   ((Path *) offset->originalPath)->Fill (theShape, 0);
   theRes->ConvertToShape (theShape, fill_oddEven);
   
-  if (theRes->nbAr <= 1)
+  if (theRes->numberOfEdges() <= 1)
   {
     
   }
@@ -950,39 +845,39 @@ sp_offset_distance_to_original (SPOffset * offset, NR::Point px)
     double arDist = -1.0;
     bool arSet = false;
     // first get the minimum distance to the points
-    for (int i = 0; i < theRes->nbPt; i++)
+    for (int i = 0; i < theRes->numberOfPoints(); i++)
     {
-      if (theRes->pts[i].dI + theRes->pts[i].dO > 0)
+      if (theRes->getPoint(i).totalDegree() > 0)
 	    {
-        NR::Point nx = theRes->pts[i].x;
+        NR::Point nx = theRes->getPoint(i).x;
         NR::Point nxpx = px-nx;
 	      double ndist = sqrt (dot(nxpx,nxpx));
 	      if (ptSet == false || fabs (ndist) < fabs (ptDist))
         {
           // we have a new minimum distance
           // now we need to wheck if px is inside or outside (for the sign)
-          nx = px - theRes->pts[i].x;
+          nx = px - theRes->getPoint(i).x;
           double nlen = sqrt (dot(nx , nx));
           nx /= nlen;
           int pb, cb, fb;
-          fb = theRes->pts[i].lastA;
-          pb = theRes->pts[i].lastA;
-          cb = theRes->pts[i].firstA;
+          fb = theRes->getPoint(i).incidentEdge[LAST];
+          pb = theRes->getPoint(i).incidentEdge[LAST];
+          cb = theRes->getPoint(i).incidentEdge[FIRST];
           do
           {
             // one angle
             NR::Point prx, nex;
-            prx = theRes->aretes[pb].dx;
+            prx = theRes->getEdge(pb).dx;
             nlen = sqrt (dot(prx, prx));
             prx /= nlen;
-            nex = theRes->aretes[cb].dx;
+            nex = theRes->getEdge(cb).dx;
             nlen = sqrt (dot(nex , nex));
             nex /= nlen;
-            if (theRes->aretes[pb].en == i)
+            if (theRes->getEdge(pb).en == i)
             {
               prx = -prx;
             }
-            if (theRes->aretes[cb].en == i)
+            if (theRes->getEdge(cb).en == i)
             {
               nex = -nex;
             }
@@ -990,7 +885,7 @@ sp_offset_distance_to_original (SPOffset * offset, NR::Point px)
             if (vectors_are_clockwise (nex, nx, prx))
             {
               // we're in that angle. set the sign, and exit that loop
-              if (theRes->aretes[cb].st == i)
+              if (theRes->getEdge(cb).st == i)
               {
                 ptDist = -ndist;
                 ptSet = true;
@@ -1010,10 +905,10 @@ sp_offset_distance_to_original (SPOffset * offset, NR::Point px)
 	    }
     }
     // loop over the edges to try to improve the distance
-    for (int i = 0; i < theRes->nbAr; i++)
+    for (int i = 0; i < theRes->numberOfEdges(); i++)
     {
-      NR::Point sx = theRes->pts[theRes->aretes[i].st].x;
-      NR::Point ex = theRes->pts[theRes->aretes[i].en].x;
+      NR::Point sx = theRes->getPoint(theRes->getEdge(i).st).x;
+      NR::Point ex = theRes->getPoint(theRes->getEdge(i).en).x;
       NR::Point nx = ex - sx;
       double len = sqrt (dot(nx,nx));
       if (len > 0.0001)
@@ -1088,10 +983,10 @@ sp_offset_top_point (SPOffset * offset, NR::Point *px)
   finalPath->Convert (1.0);
   finalPath->Fill (theShape, 0);
   
-  if (theShape->nbPt > 0)
+  if (theShape->hasPoints())
   {
     theShape->SortPoints ();
-    *px = theShape->pts[0].x;
+    *px = theShape->getPoint(0).x;
   }
   
   delete theShape;
@@ -1100,166 +995,98 @@ sp_offset_top_point (SPOffset * offset, NR::Point *px)
 }
 
 // the listening functions
-static void
-sp_offset_source_attr_changed (SPRepr * repr, const gchar * key,
-                               const gchar * /*oldval*/, const gchar * newval,
-                               bool is_interactive, void * data)
+static void sp_offset_start_listening(SPOffset *offset,SPObject* to)
 {
-  SPOffset *offset = (SPOffset *) data;
-  if ( offset == NULL ) return;
-  if (repr == NULL ||  repr != offset->sourceRepr ) {
-    return; 
-  }
-  
-  // deux dans lesquels on ne fera rien
-  if (strcmp ((const char *) key, "id") == 0)
-    return;
-  if (strcmp ((const char *) key, "transform") == 0)
-    return;
-  if (strcmp ((const char *) key, "inkscape:original") == 0)
-    return;
-  
-  Path *orig = NULL;
-  // le bon cas: il y a un attribut d
-  if (strcmp ((const char *) key, "d") == 0)
-  {
-    if (!newval)
-      return;
-    //    sp_repr_set_attr (SP_OBJECT(offset)->repr, "inkscape:original", newval);
-    
-    
-    ArtBpath *bpath;
-    SPCurve *curve;
-    
-    bpath = sp_svg_read_path (newval);
-    curve = sp_curve_new_from_bpath (bpath);	// curve se chargera de detruire bpath
-    orig = bpath_to_liv_path (curve->bpath);
-    sp_curve_unref (curve);
-  }
-  else
-  {
-    // le mauvais cas: pas d'attribut d => il faut verifier que c'est une SPShape puis prendre le contour
-    SPObject *refobj;
-    refobj =
-      sp_document_lookup_id (SP_OBJECT (offset)->document,
-                             offset->sourceObject);
-    SPItem *item = SP_ITEM (refobj);
-    
-    SPCurve *curve=NULL;
-    if (!SP_IS_SHAPE (item) && !SP_IS_TEXT (item))
-      return;
-    if (SP_IS_SHAPE (item))
-    {
-      curve = sp_shape_get_curve (SP_SHAPE (item));
-      if (curve == NULL)
-        return;
-    }
-    if (SP_IS_TEXT (item))
-    {
-      curve = sp_text_normalized_bpath (SP_TEXT (item));
-      if (curve == NULL)
-        return;
-    }
-    orig = bpath_to_liv_path (curve->bpath);
-    sp_curve_unref (curve);
-  }
-  
-  // finalisons
-  {
-    SPCSSAttr *css;
-    const gchar *val;
-    Shape *theShape = new Shape;
-    Shape *theRes = new Shape;
-    
-    orig->ConvertWithBackData (1.0);
-    orig->Fill (theShape, 0);
-    
-    css = sp_repr_css_attr (repr /*SP_OBJECT_REPR (repr) */ , "style");
-    val = sp_repr_css_property (css, "fill-rule", NULL);
-    if (val && strcmp (val, "nonzero") == 0)
-    {
-      theRes->ConvertToShape (theShape, fill_nonZero);
-    }
-    else if (val && strcmp (val, "evenodd") == 0)
-    {
-      theRes->ConvertToShape (theShape, fill_oddEven);
-    }
-    else
-    {
-      theRes->ConvertToShape (theShape, fill_nonZero);
-    }
-    
-    Path *originaux[1];
-    originaux[0] = orig;
-    Path *res = new Path;
-    theRes->ConvertToForme (res, 1, originaux);
-    
-    delete theShape;
-    delete theRes;
-    
-    char *res_d = liv_svg_dump_path2 (res);
-    delete res;
-    delete orig;
-    
-    sp_repr_set_attr (SP_OBJECT (offset)->repr, "inkscape:original", res_d);
-    
-    free (res_d);
-  }
-  
+	if ( to == NULL ) 
+		return;
+
+	offset->sourceObject = to;
+	offset->sourceRepr = SP_OBJECT_REPR(to);
+
+	offset->_delete_connection = SP_OBJECT(to)->connectDelete(sigc::bind(sigc::ptr_fun(&sp_offset_delete_self), offset));
+	offset->_transformed_connection = SP_ITEM(to)->connectTransformed(sigc::bind(sigc::ptr_fun(&sp_offset_move_compensate), offset));
+	offset->_modified_connection = g_signal_connect (G_OBJECT (to), "modified", G_CALLBACK (sp_offset_source_modified), offset);
+}
+static void sp_offset_quit_listening(SPOffset *offset)
+{
+	if ( offset->sourceObject == NULL )  
+		return;
+
+	g_signal_handler_disconnect (offset->sourceObject, offset->_modified_connection);
+	offset->_delete_connection.disconnect();
+	offset->_transformed_connection.disconnect();
+
+	offset->sourceRepr = NULL;
+	offset->sourceObject = NULL;
 }
 
 static void
-sp_offset_source_destroy (SPRepr * repr, void *data)
+sp_offset_href_changed(SPObject */*old_ref*/, SPObject */*ref*/, SPOffset *offset)
 {
-  SPOffset *offset = (SPOffset *) data;
-  if (offset == NULL)
-    return;
-  if ( repr == NULL || repr != offset->sourceRepr ) return;
-  if (offset->sourceObject)
-  {
-    free (offset->sourceObject);
-    offset->sourceObject = NULL;
-  }
-  if (offset->sourceRepr)
-  {
-    offset->sourceRepr = NULL;
-    // pas besoin d'enlever le listener
-  }
+//	SPItem *item = SP_ITEM(offset);
+	sp_offset_quit_listening(offset);
+	if (offset->sourceRef) {
+		SPItem *refobj = offset->sourceRef->getObject();
+		if (refobj) sp_offset_start_listening(offset,refobj);
+		offset->sourceDirty=true;
+		SP_OBJECT(offset)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
+	}
 }
-static void sp_offset_source_child_added (SPRepr *repr, SPRepr *child, SPRepr */*ref*/, void * data)
-{
-  SPOffset *offset = (SPOffset *) data;
-  if (offset == NULL)
-    return;
-  if ( child == NULL ) return;
-  if ( repr == NULL || offset->sourceRepr != repr ) return; // juste le premier niveau.
-  
-  sp_repr_add_listener (child,&offset_source_child_event_vector, offset);
-  
-  offset->sourceDirty=true;
-  sp_object_request_update (SP_OBJECT(offset), SP_OBJECT_MODIFIED_FLAG);
+
+static void
+sp_offset_move_compensate(NR::Matrix const *mp, SPItem *original, SPOffset *self)
+{	
+	guint mode = prefs_get_int_attribute("options.clonecompensation", "value", SP_CLONE_COMPENSATION_PARALLEL);
+	if (mode == SP_CLONE_COMPENSATION_NONE) return;
+
+	NR::Matrix m(*mp);
+	if (!(m.is_translation())) return;
+
+	// calculate the compensation matrix and the advertized movement matrix
+	SPItem *item = SP_ITEM(self);
+
+	NR::Matrix compensate;
+	NR::Matrix advertized_move;
+
+	if (mode == SP_CLONE_COMPENSATION_UNMOVED) {
+		compensate = NR::identity();
+		advertized_move.set_identity();
+	} else if (mode == SP_CLONE_COMPENSATION_PARALLEL) {
+		compensate = m;
+		advertized_move = m;
+	} else {
+		g_assert_not_reached();
+	}
+
+	item->transform *= compensate;
+
+	// commit the compensation
+	sp_item_write_transform(item, SP_OBJECT_REPR(item), item->transform, &advertized_move);
+	SP_OBJECT(item)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 }
-static void sp_offset_source_child_removed (SPRepr *repr, SPRepr *child, SPRepr */*ref*/, void * data)
+
+static void
+sp_offset_delete_self(SPObject */*deleted*/, SPOffset *offset)
 {
-  SPOffset *offset = (SPOffset *) data;
-  if (offset == NULL)
-    return;
-  if ( child == NULL ) return;
-  if ( repr == NULL || offset->sourceRepr != repr ) return; // juste le premier niveau.
-  
-  sp_repr_remove_listener_by_data (child, offset);
-  
-  offset->sourceDirty=true;
-  sp_object_request_update (SP_OBJECT(offset), SP_OBJECT_MODIFIED_FLAG);
+	guint const mode = prefs_get_int_attribute("options.cloneorphans", "value", SP_CLONE_ORPHANS_UNLINK);
+	
+	if (mode == SP_CLONE_ORPHANS_UNLINK) {
+		// leave it be. just forget about the source
+		sp_offset_quit_listening(offset);
+		if ( offset->sourceHref ) g_free(offset->sourceHref);
+		offset->sourceHref = NULL;
+		offset->sourceRef->detach();
+	} else if (mode == SP_CLONE_ORPHANS_DELETE) {
+		SP_OBJECT(offset)->deleteObject();
+	}
 }
-static void sp_offset_source_content_changed (SPRepr */*repr*/, const gchar */*oldcontent*/, const gchar */*newcontent*/, void * data)
+static void
+sp_offset_source_modified (SPObject *iSource, guint flags, SPItem *item)
 {
-  SPOffset *offset = (SPOffset *) data;
-  if (offset == NULL)
-    return;
-  
-  offset->sourceDirty=true;
-  sp_object_request_update (SP_OBJECT(offset), SP_OBJECT_MODIFIED_FLAG);
+	SPOffset *offset = SP_OFFSET(item);
+	offset->sourceDirty=true;
+	refresh_offset_source(offset);
+	sp_shape_set_shape ((SPShape *) offset);
 }
 
 void   refresh_offset_source(SPOffset* offset)
@@ -1269,24 +1096,18 @@ void   refresh_offset_source(SPOffset* offset)
   Path *orig = NULL;
   
   // le mauvais cas: pas d'attribut d => il faut verifier que c'est une SPShape puis prendre le contour
-  SPObject *refobj;
-  refobj = sp_document_lookup_id (SP_OBJECT (offset)->document,
-                                  offset->sourceObject);
+  SPObject *refobj=offset->sourceObject;
   if ( refobj == NULL ) return;
-  
   SPItem *item = SP_ITEM (refobj);
   
   SPCurve *curve=NULL;
-  if (!SP_IS_SHAPE (item) && !SP_IS_TEXT (item))
-  	return;
-  if (SP_IS_SHAPE (item))
-  {
+  if (!SP_IS_SHAPE (item) && !SP_IS_TEXT (item)) return;
+  if (SP_IS_SHAPE (item)) {
     curve = sp_shape_get_curve (SP_SHAPE (item));
     if (curve == NULL)
       return;
   }
-  if (SP_IS_TEXT (item))
- 	{
+  if (SP_IS_TEXT (item)) {
  	  curve = sp_text_normalized_bpath (SP_TEXT (item));
  	  if (curve == NULL)
 	    return;
@@ -1328,7 +1149,7 @@ void   refresh_offset_source(SPOffset* offset)
     delete theShape;
     delete theRes;
     
-    char *res_d = liv_svg_dump_path2 (res);
+    char *res_d = res->svg_dump_path ();
     delete res;
     delete orig;
     
@@ -1338,5 +1159,14 @@ void   refresh_offset_source(SPOffset* offset)
   }
 }
 
-
+SPItem *
+sp_offset_get_source (SPOffset *offset)
+{
+	if (offset && offset->sourceRef) {
+		SPItem *refobj = offset->sourceRef->getObject();
+		if (SP_IS_ITEM (refobj))
+			return (SPItem *) refobj;
+	}
+	return NULL;
+}
 
