@@ -11,12 +11,16 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
 #include "attributes.h"
 #include "sp-polyline.h"
+#include "display/curve.h"
 #include "helper/sp-intl.h"
+#include "xml/repr.h"
 
 static void sp_polyline_class_init (SPPolyLineClass *klass);
 static void sp_polyline_init (SPPolyLine *polyline);
@@ -112,12 +116,21 @@ sp_polyline_set (SPObject *object, unsigned int key, const gchar *value)
 		while (TRUE) {
 			gdouble x, y;
 
-			x = strtod (cptr, &eptr);
+			while (*cptr != '\0' && (*cptr == ',' || *cptr == '\x20' || *cptr == '\x9' || *cptr == '\xD' || *cptr == '\xA')) {
+				cptr++;
+			}
+			if (!*cptr) break;
+
+			x = g_ascii_strtod (cptr, &eptr);
 			if (eptr == cptr) break;
-			cptr = strchr (eptr, ',');
-			if (!cptr) break;
-			cptr++;
-			y = strtod (cptr, &eptr);
+			cptr = eptr;
+
+			while (*cptr != '\0' && (*cptr == ',' || *cptr == '\x20' || *cptr == '\x9' || *cptr == '\xD' || *cptr == '\xA')) {
+				cptr++;
+			}
+			if (!*cptr) break;
+
+			y = g_ascii_strtod (cptr, &eptr);
 			if (eptr == cptr) break;
 			cptr = eptr;
 			if (hascpt) {
@@ -163,6 +176,6 @@ sp_polyline_write (SPObject *object, SPRepr *repr, guint flags)
 static gchar *
 sp_polyline_description (SPItem * item)
 {
-	return g_strdup ("PolyLine");
+	return g_strdup ("<b>Polyline</b>");
 }
 
