@@ -12,6 +12,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "config.h"
+
 #include <string.h>
 #include "svg/svg.h"
 #include "display/nr-arena-group.h"
@@ -20,11 +22,14 @@
 #include "document.h"
 #include "desktop.h"
 #include "sp-defs.h"
-/* #include "sp-namedview.h" */
+#include "sp-item.h"
 #include "sp-root.h"
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-ops.h>
+#include <libnr/nr-matrix-translate-ops.h>
 #include <libnr/nr-scale-ops.h>
+#include <libnr/nr-translate-scale-ops.h>
+#include <xml/repr.h>
 
 static void sp_root_class_init (SPRootClass *klass);
 static void sp_root_init (SPRoot *root);
@@ -39,7 +44,7 @@ static void sp_root_modified (SPObject *object, guint flags);
 static SPRepr *sp_root_write (SPObject *object, SPRepr *repr, guint flags);
 
 static NRArenaItem *sp_root_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
-static void sp_root_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags);
+static void sp_root_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
 static void sp_root_print (SPItem *item, SPPrintContext *ctx);
 
 static SPGroupClass *parent_class;
@@ -117,8 +122,6 @@ sp_root_init (SPRoot *root)
 	root->version.inkscape = root->original.inkscape =
 	  root->version.sodipodi = root->original.sodipodi = zero_version;
 
-	root->group.mode = SP_GROUP_MODE_LAYER;
-
 	sp_svg_length_unset (&root->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&root->height, SP_SVG_UNIT_NONE, 0.0, 0.0);
 	sp_svg_length_unset (&root->width, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
@@ -163,8 +166,6 @@ sp_root_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	if (((SPObjectClass *) parent_class)->build)
 		(* ((SPObjectClass *) parent_class)->build) (object, document, repr);
 
-	group->mode = SP_GROUP_MODE_LAYER;
-
 	/* Search for first <defs> node */
 	for (SPObject *o = sp_object_first_child(SP_OBJECT(group)) ; o != NULL; o = SP_OBJECT_NEXT(o) ) {
 		if (SP_IS_DEFS (o)) {
@@ -172,6 +173,9 @@ sp_root_build (SPObject *object, SPDocument *document, SPRepr *repr)
 			break;
 		}
 	}
+
+	// clear transform, if any was read in - SVG does not allow transform= on <svg>
+	SP_ITEM(object)->transform = NR::identity();
 }
 
 /** sp_root_release(object)
@@ -221,7 +225,7 @@ sp_root_set (SPObject *object, unsigned int key, const gchar *value)
 			sp_svg_length_unset (&root->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		/* fixme: I am almost sure these do not require viewport flag (Lauris) */
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_Y:
 		if (!sp_svg_length_read_absolute (value, &root->y)) {
@@ -229,21 +233,21 @@ sp_root_set (SPObject *object, unsigned int key, const gchar *value)
 			sp_svg_length_unset (&root->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
 		}
 		/* fixme: I am almost sure these do not require viewport flag (Lauris) */
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_WIDTH:
 		if (!sp_svg_length_read_absolute (value, &root->width) || !(root->width.computed > 0.0)) {
 		    /* fixme: em, ex, % are probably valid, but require special treatment (Lauris) */
 			sp_svg_length_unset (&root->width, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_HEIGHT:
 		if (!sp_svg_length_read_absolute (value, &root->height) || !(root->height.computed > 0.0)) {
 		    /* fixme: em, ex, % are probably valid, but require special treatment (Lauris) */
 			sp_svg_length_unset (&root->height, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_VIEWBOX:
 		if (value) {
@@ -252,13 +256,13 @@ sp_root_set (SPObject *object, unsigned int key, const gchar *value)
 			/* fixme: We have to take original item affine into account */
 			/* fixme: Think (Lauris) */
 			eptr = (gchar *) value;
-			x = strtod (eptr, &eptr);
+			x = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			y = strtod (eptr, &eptr);
+			y = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			width = strtod (eptr, &eptr);
+			width = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			height = strtod (eptr, &eptr);
+			height = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
 			if ((width > 0) && (height > 0)) {
 				/* Set viewbox */
@@ -273,14 +277,14 @@ sp_root_set (SPObject *object, unsigned int key, const gchar *value)
 		} else {
 			root->viewBox_set = FALSE;
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_PRESERVEASPECTRATIO:
 		/* Do setup before, so we can use break to escape */
 		root->aspect_set = FALSE;
 		root->aspect_align = SP_ASPECT_XMID_YMID;
 		root->aspect_clip = SP_ASPECT_MEET;
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		if (value) {
 			int len;
 			gchar c[256];
@@ -359,7 +363,7 @@ sp_root_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 		(* ((SPObjectClass *) (parent_class))->child_added) (object, child, ref);
 
 	gchar const *id = sp_repr_attr(child, "id");
-	SPObject *co = sp_document_lookup_id(object->document, id);
+	SPObject *co = object->document->getObjectById(id);
 	g_assert (co != NULL);
 
 	if (SP_IS_DEFS (co)) {
@@ -647,15 +651,15 @@ sp_root_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags
 }
 
 static void
-sp_root_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags)
+sp_root_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags)
 {
-	SPRoot *root = SP_ROOT (item);
+	SPRoot const *root = SP_ROOT (item);
 
 	if (((SPItemClass *) (parent_class))->bbox) {
-		NRMatrix const product( root->c2p * NR::Matrix(transform) );
-		((SPItemClass *) (parent_class))->bbox (item, bbox, 
-							&product,
-							flags);
+		NR::Matrix const product( root->c2p * transform );
+		((SPItemClass *) (parent_class))->bbox(item, bbox,
+						       product,
+						       flags);
 	}
 }
 
