@@ -11,9 +11,13 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "config.h"
+
 #include <string.h>
+#include "libnr/nr-matrix-ops.h"
 #include "svg/svg.h"
 #include "display/nr-arena-group.h"
+#include "xml/repr.h"
 #include "attributes.h"
 #include "print.h"
 #include "document.h"
@@ -35,7 +39,7 @@ static SPRepr *sp_symbol_write (SPObject *object, SPRepr *repr, guint flags);
 
 static NRArenaItem *sp_symbol_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 static void sp_symbol_hide (SPItem *item, unsigned int key);
-static void sp_symbol_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags);
+static void sp_symbol_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
 static void sp_symbol_print (SPItem *item, SPPrintContext *ctx);
 
 static SPGroupClass *parent_class;
@@ -139,13 +143,13 @@ sp_symbol_set (SPObject *object, unsigned int key, const gchar *value)
 			/* fixme: We have to take original item affine into account */
 			/* fixme: Think (Lauris) */
 			eptr = (gchar *) value;
-			x = strtod (eptr, &eptr);
+			x = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			y = strtod (eptr, &eptr);
+			y = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			width = strtod (eptr, &eptr);
+			width = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
-			height = strtod (eptr, &eptr);
+			height = g_ascii_strtod (eptr, &eptr);
 			while (*eptr && ((*eptr == ',') || (*eptr == ' '))) eptr++;
 			if ((width > 0) && (height > 0)) {
 				/* Set viewbox */
@@ -160,14 +164,14 @@ sp_symbol_set (SPObject *object, unsigned int key, const gchar *value)
 		} else {
 			symbol->viewBox_set = FALSE;
 		}
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_PRESERVEASPECTRATIO:
 		/* Do setup before, so we can use break to escape */
 		symbol->aspect_set = FALSE;
 		symbol->aspect_align = SP_ASPECT_NONE;
 		symbol->aspect_clip = SP_ASPECT_MEET;
-		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
+		object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		if (value) {
 			int len;
 			gchar c[256];
@@ -436,20 +440,16 @@ sp_symbol_hide (SPItem *item, unsigned int key)
 }
 
 static void
-sp_symbol_bbox (SPItem *item, NRRect *bbox, const NRMatrix *transform, unsigned int flags)
+sp_symbol_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags)
 {
-	SPSymbol *symbol;
-	NRMatrix a;
-
-	symbol = SP_SYMBOL (item);
+	SPSymbol const *symbol = SP_SYMBOL(item);
 
 	if (SP_OBJECT_IS_CLONED (symbol)) {
 		/* Cloned <symbol> is actually renderable */
 
-		nr_matrix_multiply (&a, &symbol->c2p, transform);
-
 		if (((SPItemClass *) (parent_class))->bbox) {
-			((SPItemClass *) (parent_class))->bbox (item, bbox, &a, flags);
+			NR::Matrix const a( symbol->c2p * transform );
+			((SPItemClass *) (parent_class))->bbox(item, bbox, a, flags);
 		}
 	}
 }
