@@ -13,7 +13,8 @@
  * Released under GNU GPL
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -25,24 +26,27 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "macros.h"
-#include "helper/sp-canvas.h"
+#include "display/sp-canvas.h"
 #include "sp-spiral.h"
 #include "inkscape.h"
 #include "document.h"
 #include "selection.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
-#include "desktop-snap.h"
+#include "snap.h"
+#include "desktop.h"
+#include "desktop-style.h"
 #include "pixmaps/cursor-spiral.xpm"
 #include "spiral-context.h"
+#include "sp-desktop-widget.h"
 #include "sp-metrics.h"
 #include "helper/sp-intl.h"
 #include "object-edit.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/repr-private.h"
+#include "xml/sp-repr-event-vector.h"
 #include "prefs-utils.h"
-
+#include "widgets/spw-utilities.h"
 #include <libnr/nr-point-fns.h>
 
 static void sp_spiral_context_class_init (SPSpiralContextClass * klass);
@@ -53,8 +57,6 @@ static void sp_spiral_context_set (SPEventContext *ec, const gchar *key, const g
 
 static gint sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event);
 
-static GtkWidget *sp_spiral_context_config_widget (SPEventContext *ec);
-
 static void sp_spiral_drag (SPSpiralContext * sc, NR::Point p, guint state);
 static void sp_spiral_finish (SPSpiralContext * sc);
 
@@ -93,7 +95,6 @@ sp_spiral_context_class_init (SPSpiralContextClass * klass)
 	event_context_class->setup = sp_spiral_context_setup;
 	event_context_class->set = sp_spiral_context_set;
 	event_context_class->root_handler = sp_spiral_context_root_handler;
-	event_context_class->config_widget = sp_spiral_context_config_widget;
 }
 
 static void
@@ -115,6 +116,8 @@ sp_spiral_context_init (SPSpiralContext * spiral_context)
 	spiral_context->revo = 3.0;
 	spiral_context->exp = 1.0;
 	spiral_context->t0 = 0.0;
+
+	new (&spiral_context->sel_changed_connection) sigc::connection();
 }
 
 static void
@@ -124,6 +127,9 @@ sp_spiral_context_dispose (GObject *object)
 
     SPEventContext *ec = SP_EVENT_CONTEXT (object);
 
+    sc->sel_changed_connection.disconnect();
+    sc->sel_changed_connection.~connection();
+
 	/* fixme: This is necessary because we do not grab */
 	if (sc->item) sp_spiral_finish (sc);
 
@@ -138,19 +144,17 @@ sp_spiral_context_dispose (GObject *object)
         sc->repr = 0;
     }
 
-    if (SP_EVENT_CONTEXT_DESKTOP (sc)) {
-        sp_signal_disconnect_by_data (SP_DT_SELECTION (SP_EVENT_CONTEXT_DESKTOP (sc)), sc);
+    if (sc->_message_context) {
+        delete sc->_message_context;
     }
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+
+    G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value, const gchar * new_value, bool is_interactive, gpointer data)
 {
-    SPSpiralContext *sc;
-    SPEventContext *ec;
-
-    sc = SP_SPIRAL_CONTEXT (data);
-    ec = SP_EVENT_CONTEXT (sc);
+    SPSpiralContext *sc = SP_SPIRAL_CONTEXT (data);
+    SPEventContext *ec = SP_EVENT_CONTEXT (sc);
 
     if (sc->knot_holder) {
         sp_knot_holder_destroy (sc->knot_holder);
@@ -158,8 +162,7 @@ static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const g
     sc->knot_holder = NULL;
 
     SPDesktop *desktop = ec->desktop;
-
-    SPItem *item = sp_selection_item (SP_DT_SELECTION(desktop));
+    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
 
     if (item) {
         sc->knot_holder = sp_item_knot_holder (item, desktop);
@@ -167,7 +170,6 @@ static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const g
 }
 
 static SPReprEventVector shape_repr_events = {
-    NULL, /* destroy */
     NULL, /* add_child */
     NULL, /* child_added */
     NULL, /* remove_child */
@@ -187,11 +189,8 @@ destroys old and creates new knotholder
 void
 sp_spiral_context_selection_changed (SPSelection * selection, gpointer data)
 {
-    SPSpiralContext *sc;
-    SPEventContext *ec;
-
-    sc = SP_SPIRAL_CONTEXT (data);
-    ec = SP_EVENT_CONTEXT (sc);
+    SPSpiralContext *sc = SP_SPIRAL_CONTEXT (data);
+    SPEventContext *ec = SP_EVENT_CONTEXT (sc);
 
     if (sc->knot_holder) { // desktroy knotholder
         sp_knot_holder_destroy (sc->knot_holder);
@@ -204,7 +203,7 @@ sp_spiral_context_selection_changed (SPSelection * selection, gpointer data)
         sc->repr = 0;
     }
 
-    SPItem *item = sp_selection_item (selection);
+    SPItem *item = selection->singleItem();
     if (item) {
         sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
         SPRepr *repr = SP_OBJECT_REPR (item);
@@ -231,8 +230,10 @@ sp_spiral_context_setup (SPEventContext *ec)
 	sp_event_context_read (ec, "revolution");
 	sp_event_context_read (ec, "t0");
 
-    SPItem *item = sp_selection_item (SP_DT_SELECTION (ec->desktop));
-        if (item) {
+	SPSelection *selection = SP_DT_SELECTION(ec->desktop);
+
+	SPItem *item = selection->singleItem();
+      if (item) {
             sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
             SPRepr *repr = SP_OBJECT_REPR (item);
             if (repr) {
@@ -241,11 +242,16 @@ sp_spiral_context_setup (SPEventContext *ec)
                 sp_repr_add_listener (repr, &shape_repr_events, ec);
                 sp_repr_synthesize_events (repr, &shape_repr_events, ec);
             }
-        }
+      }
 
-        g_signal_connect (G_OBJECT (SP_DT_SELECTION (ec->desktop)),
-            "changed", G_CALLBACK (sp_spiral_context_selection_changed), sc);
+	sc->sel_changed_connection.disconnect();
+	sc->sel_changed_connection = selection->connectChanged(sigc::bind(sigc::ptr_fun(&sp_spiral_context_selection_changed), (gpointer)sc));
 
+	if (prefs_get_int_attribute("tools.shapes", "selcue", 0) != 0) {
+		ec->enableSelectionCue();
+	}
+
+	sc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
 static void
@@ -254,50 +260,17 @@ sp_spiral_context_set (SPEventContext *ec, const gchar *key, const gchar *val)
 	SPSpiralContext *sc = SP_SPIRAL_CONTEXT (ec);
 
 	if (!strcmp (key, "expansion")) {
-		sc->exp = (val) ? atof (val) : 1.0;
+		sc->exp = (val) ? g_ascii_strtod (val, NULL) : 1.0;
 		sc->exp = CLAMP (sc->exp, 0.0, 1000.0);
 	} else if (!strcmp (key, "revolution")) {
-		sc->revo = (val) ? atof (val) : 3.0;
+		sc->revo = (val) ? g_ascii_strtod (val, NULL) : 3.0;
 		sc->revo = CLAMP (sc->revo, 0.05, 40.0);
 	} else if (!strcmp (key, "t0")) {
-		sc->t0 = (val) ? atof (val) : 0.0;
+		sc->t0 = (val) ? g_ascii_strtod (val, NULL) : 0.0;
 		sc->t0 = CLAMP (sc->t0, 0.0, 0.999);
 	}
 }
 
-static gint
-sp_spiral_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
-{
-    SPDesktop *desktop = event_context->desktop;
-
-    gint ret = FALSE;
-
-    switch (event->type) {
-    case GDK_BUTTON_PRESS:
-        if (event->button.button == 1) {
-
-            // save drag origin
-            event_context->xp = (gint) event->button.x;
-            event_context->yp = (gint) event->button.y;
-            event_context->within_tolerance = true;
-
-            // remember clicked item, disregarding groups
-            event_context->item_to_select = sp_desktop_item_at_point (desktop, NR::Point(event->button.x, event->button.y), TRUE);
-
-            ret = TRUE;
-        }
-        break;
-        // motion and release are always on root (why?)
-    default:
-        break;
-    }
-
-    if (((SPEventContextClass *) parent_class)->item_handler)
-        ret = ((SPEventContextClass *) parent_class)->item_handler (event_context, item, event);
-
-    return ret;
-}
-
 static gint
 sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 {
@@ -325,7 +298,7 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 			/* Position center */
 			sc->center = sp_desktop_w2d_xy_point (event_context->desktop, NR::Point(event->button.x, event->button.y));
 			/* Snap center to nearest magnetic point */
-			sp_desktop_free_snap (event_context->desktop, sc->center);
+			namedview_free_snap (event_context->desktop->namedview, Snapper::SNAP_POINT, sc->center);
 			sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
 						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
 						NULL, event->button.time);
@@ -360,10 +333,10 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 			sp_spiral_finish (sc);
             } else if (event_context->item_to_select) {
                 // no dragging, select clicked item if any
-                sp_selection_set_item (SP_DT_SELECTION (desktop), event_context->item_to_select);
+                SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
             } else {
                 // click in an empty space
-                sp_selection_empty (SP_DT_SELECTION (desktop));
+                SP_DT_SELECTION(desktop)->clear();
             }
 
             event_context->item_to_select = NULL;
@@ -373,6 +346,18 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 		break;
 	case GDK_KEY_PRESS:
 		switch (event->key.keyval) {
+		case GDK_Alt_R:
+		case GDK_Control_L:
+		case GDK_Control_R:
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Meta_L:  // Meta is when you press Shift+Alt (at least on my machine)
+		case GDK_Meta_R:
+			sp_event_show_modifier_tip (event_context->defaultMessageContext(), event,
+												_("<b>Ctrl</b>: snap angle"),
+												NULL,
+												_("<b>Alt</b>: lock spiral radius"));
+			break;
 		case GDK_Up: 
 		case GDK_Down: 
 		case GDK_KP_Up: 
@@ -381,12 +366,39 @@ sp_spiral_context_root_handler (SPEventContext * event_context, GdkEvent * event
 			if (!MOD__CTRL_ONLY)
 				ret = TRUE;
 			break;
-        case GDK_Escape:
-            sp_selection_empty (SP_DT_SELECTION (desktop)); // deselect
-            //TODO: make dragging escapable by Esc
+ 		case GDK_x:
+ 		case GDK_X:
+ 			if (MOD__ALT_ONLY) {
+ 				gpointer hb = sp_search_by_data_recursive (desktop->owner->aux_toolbox, (gpointer) "altx-spiral");
+ 				if (hb && GTK_IS_WIDGET(hb)) {
+					gtk_widget_grab_focus (GTK_WIDGET (hb));
+				}
+ 				ret = TRUE;
+ 			}
+			break;
+		case GDK_Escape:
+			SP_DT_SELECTION(desktop)->clear();
+			//TODO: make dragging escapable by Esc
 		default:
 			break;
 		}
+		break;
+	case GDK_KEY_RELEASE:
+		switch (event->key.keyval) {
+		case GDK_Alt_L:
+		case GDK_Alt_R:
+		case GDK_Control_L:
+		case GDK_Control_R:
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Meta_L:  // Meta is when you press Shift+Alt
+		case GDK_Meta_R:
+			event_context->defaultMessageContext()->clear();
+			break;
+		default:
+			break;
+		}
+		break;
 	default:
 		break;
 	}
@@ -404,40 +416,47 @@ sp_spiral_drag (SPSpiralContext * sc, NR::Point p, guint state)
 {
 	SPDesktop *desktop = SP_EVENT_CONTEXT (sc)->desktop;
 
-	int snaps = prefs_get_int_attribute ("options.rotationsnapsperpi", "value", 12);
+	const int snaps = prefs_get_int_attribute ("options.rotationsnapsperpi", "value", 12);
 
 	if (!sc->item) {
+
+        SPItem *layer=SP_ITEM(desktop->currentLayer());
+        if ( !layer || desktop->itemIsHidden(layer)) {
+            sc->_message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is hidden</b>. Unhide it to be able to draw on it."));
+            return;
+        }
+        if ( !layer || layer->isLocked()) {
+            sc->_message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is locked</b>. Unlock it to be able to draw on it."));
+            return;
+        }
+
 		/* Create object */
 		SPRepr *repr = sp_repr_new ("path");
-                sp_repr_set_attr (repr, "sodipodi:type", "spiral");
-		/* Set style */
-		SPRepr *style = inkscape_get_repr (INKSCAPE, "tools.shapes.spiral");
-		if (style) {
-			SPCSSAttr *css = sp_repr_css_attr_inherited (style, "style");
-			sp_repr_css_set (repr, css, "style");
-			sp_repr_css_attr_unref (css);
-		}
-                else
-                  g_print ("sp_spiral_drag: cant get style\n");
+             sp_repr_set_attr (repr, "sodipodi:type", "spiral");
 
-		sc->item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), repr);
+		/* Set style */
+		sp_desktop_apply_style_tool (desktop, repr, "tools.shapes.spiral", false);
+
+		sc->item = (SPItem *) desktop->currentLayer()->appendChildRepr(repr);
 		sp_repr_unref (repr);
+		sc->item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
+		sc->item->updateRepr();
 	}
 
 	/* Free movement for corner point */
-	NR::Point p0 = sp_desktop_dt2root_xy_point (desktop, sc->center);
+	const NR::Point p0 = sp_desktop_dt2root_xy_point (desktop, sc->center);
 	NR::Point p1 = sp_desktop_dt2root_xy_point (desktop, p);
-	sp_desktop_free_snap (desktop, p1);
+	namedview_free_snap (desktop->namedview, Snapper::SNAP_POINT, p1);
 	
 	SPSpiral *spiral = SP_SPIRAL (sc->item);
 
-	NR::Point delta = p1 - p0;
-	gdouble rad = NR::L2 (delta);
+	const NR::Point delta = p1 - p0;
+	const gdouble rad = NR::L2 (delta);
 
 	gdouble arg = NR::atan2 (delta) - 2.0*M_PI*spiral->revo;
 
   	if (state & GDK_CONTROL_MASK) { 
-		arg = round (arg/(M_PI/snaps))*(M_PI/snaps);
+		arg = sp_round(arg, M_PI/snaps);
 	} 
 	
         /* Fixme: these parameters should be got from dialog box */
@@ -448,142 +467,27 @@ sp_spiral_drag (SPSpiralContext * sc, NR::Point p, guint state)
 		       /*t0*/ sc->t0);
 	
 	/* status text */
-	GString *xs, *ys;
-	gchar status[80];
-	xs = SP_PT_TO_METRIC_STRING (fabs(p0[NR::X]), SP_DEFAULT_METRIC);
-	ys = SP_PT_TO_METRIC_STRING (fabs(p0[NR::Y]), SP_DEFAULT_METRIC);
-	sprintf (status, _("Draw spiral at (%s,%s)"), xs->str, ys->str);
-	sp_view_set_status (SP_VIEW (desktop), status, FALSE);
-	g_string_free (xs, FALSE);
-	g_string_free (ys, FALSE);
+	GString *rads = SP_PT_TO_METRIC_STRING (rad, SP_DEFAULT_METRIC);
+	sc->_message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Spiral</b>: radius %s, angle %5g; with <b>Ctrl</b> to snap angle"), 
+														 rads->str, sp_round ((arg + 2.0*M_PI*spiral->revo)*180/M_PI, 0.0001));
+	g_string_free (rads, FALSE);
 }
 
 static void
 sp_spiral_finish (SPSpiralContext * sc)
 {
+	sc->_message_context->clear();
+
 	if (sc->item != NULL) {
 		SPDesktop *desktop = SP_EVENT_CONTEXT (sc)->desktop;
 		SPSpiral  *spiral = SP_SPIRAL (sc->item);
 
 		sp_shape_set_shape(SP_SHAPE(spiral));
-		sp_object_invoke_write (SP_OBJECT (spiral), NULL, SP_OBJECT_WRITE_EXT);
+		SP_OBJECT (spiral)->updateRepr(NULL, SP_OBJECT_WRITE_EXT);
 
-		sp_selection_set_item (SP_DT_SELECTION (desktop), sc->item);
+		SP_DT_SELECTION(desktop)->setItem(sc->item);
 		sp_document_done (SP_DT_DOCUMENT (desktop));
 
 		sc->item = NULL;
 	}
 }
-
-/* Gtk stuff */
-
-static void
-sp_sc_revolution_value_changed (GtkAdjustment *adj, SPSpiralContext *sc)
-{
-	sp_repr_set_int (SP_EVENT_CONTEXT_REPR (sc), "revolution", (gint) adj->value);
-}
-
-static void
-sp_sc_expansion_value_changed (GtkAdjustment *adj, SPSpiralContext *sc)
-{
-	sp_repr_set_double (SP_EVENT_CONTEXT_REPR (sc), "expansion", adj->value);
-}
-
-static void
-sp_sc_t0_value_changed (GtkAdjustment *adj, SPSpiralContext *sc)
-{
-	sp_repr_set_double (SP_EVENT_CONTEXT_REPR (sc), "t0", adj->value);
-}
-
-static void
-sp_sc_defaults (GtkWidget *widget, GtkObject *obj)
-{
-	GtkAdjustment *adj;
-
-	adj = (GtkAdjustment*)gtk_object_get_data (obj, "revolution");
-	gtk_adjustment_set_value (adj, 3.0);
-	adj = (GtkAdjustment*)gtk_object_get_data (obj, "expansion");
-	gtk_adjustment_set_value (adj, 1.0);
-	adj = (GtkAdjustment*)gtk_object_get_data (obj, "t0");
-	gtk_adjustment_set_value (adj, 0.0);
-}
-
-static GtkWidget *
-sp_spiral_context_config_widget (SPEventContext *ec)
-{
-	GtkWidget *tbl, *l, *sb, *b;
-	GtkObject *a;
-
-	SPSpiralContext *sc = SP_SPIRAL_CONTEXT (ec);
-
-	tbl = gtk_table_new (4, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (tbl), 4);
-	gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
-
-	/* Revolution */
-	l = gtk_label_new (_("Revolution:"));
-	gtk_widget_show (l);
-	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 0, 1, 
-			  (GtkAttachOptions)0, 
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	a = gtk_adjustment_new (sc->revo, 0.05, 20.0, 1.0, 1.0, 1.0);
-	gtk_object_set_data (GTK_OBJECT (tbl), "revolution", a);
-	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 2);
-	gtk_widget_show (sb);
-	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 0, 1, 
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)0, 
-			  0, 0);
-
-	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_sc_revolution_value_changed), sc);
-
-	/* Expansion */
-	l = gtk_label_new (_("Expansion:"));
-	gtk_widget_show (l);
-	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 1, 2, 
-			  (GtkAttachOptions)0, 
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	a = gtk_adjustment_new (sc->exp, 0.0, 1000.0, 0.1, 1.0, 1.0);
-	gtk_object_set_data (GTK_OBJECT (tbl), "expansion", a);
-	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
-	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 1, 2, 
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_sc_expansion_value_changed), sc);
-
-	/* T0 */
-	l = gtk_label_new (_("Inner radius:"));
-	gtk_widget_show (l);
-	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 2, 3,
-			  (GtkAttachOptions)0, 
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	a = gtk_adjustment_new (sc->t0, 0.0, 0.999, 0.1, 1.0, 1.0);
-	gtk_object_set_data (GTK_OBJECT (tbl), "t0", a);
-	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 2);
-	gtk_widget_show (sb);
-	gtk_table_attach (GTK_TABLE (tbl), sb, 1, 2, 2, 3, 
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	gtk_signal_connect (GTK_OBJECT (a), "value_changed", GTK_SIGNAL_FUNC (sp_sc_t0_value_changed), sc);
-
-	/* Reset */
-	b = gtk_button_new_with_label (_("Defaults"));
-	gtk_widget_show (b);
-	gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 3, 4,
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)0, 
-			  0, 0);
-	gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_sc_defaults), tbl);
-
-	return tbl;
-}
-
