@@ -14,9 +14,10 @@
  * Released under GNU GPL
  */
 
-#include "knot.h"
+#include <sigc++/sigc++.h>
 #include "event-context.h"
-#include "knotholder.h"
+#include "libnr/nr-point.h"
+struct SPKnotHolder;
 
 #define SP_TYPE_SPIRAL_CONTEXT            (sp_spiral_context_get_type ())
 #define SP_SPIRAL_CONTEXT(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_SPIRAL_CONTEXT, SPSpiralContext))
@@ -24,11 +25,10 @@
 #define SP_IS_SPIRAL_CONTEXT(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_SPIRAL_CONTEXT))
 #define SP_IS_SPIRAL_CONTEXT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_SPIRAL_CONTEXT))
 
-typedef struct _SPSpiralContext SPSpiralContext;
-typedef struct _SPSpiralContextClass SPSpiralContextClass;
+class SPSpiralContext;
+class SPSpiralContextClass;
 
-struct _SPSpiralContext {
-	SPEventContext event_context;
+struct SPSpiralContext : public SPEventContext {
 	SPItem * item;
 	NR::Point center;
 	gdouble revo;
@@ -38,9 +38,12 @@ struct _SPSpiralContext {
     SPKnotHolder *knot_holder;
     SPRepr *repr;
 
+    sigc::connection sel_changed_connection;
+
+    Inkscape::MessageContext *_message_context;
 };
 
-struct _SPSpiralContextClass {
+struct SPSpiralContextClass {
 	SPEventContextClass parent_class;
 };
 
