@@ -13,7 +13,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -26,25 +27,29 @@
 #include <gtk/gtkcheckbutton.h>
 
 #include "macros.h"
-#include "helper/sp-canvas.h"
+#include "display/sp-canvas.h"
 #include "sp-star.h"
 #include "inkscape.h"
 #include "document.h"
 #include "selection.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
-#include "desktop-snap.h"
+#include "snap.h"
+#include "desktop.h"
+#include "desktop-style.h"
 #include "pixmaps/cursor-star.xpm"
+#include "sp-desktop-widget.h"
 #include "sp-metrics.h"
 #include "helper/sp-intl.h"
 #include "prefs-utils.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/repr-private.h"
+#include "xml/sp-repr-event-vector.h"
 #include "object-edit.h"
+#include "widgets/spw-utilities.h"
+#include <libnr/nr-point-fns.h>
 
 #include "star-context.h"
-#include <libnr/nr-point-fns.h>
 
 static void sp_star_context_class_init (SPStarContextClass * klass);
 static void sp_star_context_init (SPStarContext * star_context);
@@ -53,7 +58,6 @@ static void sp_star_context_dispose (GObject *object);
 static void sp_star_context_setup (SPEventContext *ec);
 static void sp_star_context_set (SPEventContext *ec, const gchar *key, const gchar *val);
 static gint sp_star_context_root_handler (SPEventContext *ec, GdkEvent *event);
-static GtkWidget *sp_star_context_config_widget (SPEventContext *ec);
 
 static void sp_star_drag (SPStarContext * sc, NR::Point p, guint state);
 static void sp_star_finish (SPStarContext * sc);
@@ -93,7 +97,6 @@ sp_star_context_class_init (SPStarContextClass * klass)
     event_context_class->setup = sp_star_context_setup;
     event_context_class->set = sp_star_context_set;
     event_context_class->root_handler = sp_star_context_root_handler;
-    event_context_class->config_widget = sp_star_context_config_widget;
 }
 
 static void
@@ -115,6 +118,8 @@ sp_star_context_init (SPStarContext * star_context)
     star_context->magnitude = 5;
     star_context->proportion = 0.5;
     star_context->isflatsided = false;
+
+    new (&star_context->sel_changed_connection) sigc::connection();
 }
 
 static void
@@ -124,6 +129,9 @@ sp_star_context_dispose (GObject *object)
 
     SPStarContext *sc = SP_STAR_CONTEXT (object);
 
+    sc->sel_changed_connection.disconnect();
+    sc->sel_changed_connection.~connection();
+
     if (sc->knot_holder) {
         sp_knot_holder_destroy (sc->knot_holder);
         sc->knot_holder = NULL;
@@ -135,22 +143,20 @@ sp_star_context_dispose (GObject *object)
         sc->repr = 0;
     }
 
-    if (SP_EVENT_CONTEXT_DESKTOP (sc)) {
-        sp_signal_disconnect_by_data (SP_DT_SELECTION (SP_EVENT_CONTEXT_DESKTOP (sc)), sc);
-    }
     /* fixme: This is necessary because we do not grab */
     if (sc->item) sp_star_finish (sc);
 
+    if (sc->_message_context) {
+        delete sc->_message_context;
+    }
+
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value, const gchar * new_value, bool is_interactive, gpointer data)
 {
-    SPStarContext *sc;
-    SPEventContext *ec;
-
-    sc = SP_STAR_CONTEXT (data);
-    ec = SP_EVENT_CONTEXT (sc);
+    SPStarContext *sc = SP_STAR_CONTEXT(data);
+    SPEventContext *ec = SP_EVENT_CONTEXT(sc);
 
     if (sc->knot_holder) {
         sp_knot_holder_destroy (sc->knot_holder);
@@ -159,7 +165,7 @@ static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const g
 
     SPDesktop *desktop = ec->desktop;
 
-    SPItem *item = sp_selection_item (SP_DT_SELECTION(desktop));
+    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
 
     if (item) {
         sc->knot_holder = sp_item_knot_holder (item, desktop);
@@ -167,7 +173,6 @@ static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const g
 }
 
 static SPReprEventVector shape_repr_events = {
-    NULL, /* destroy */
     NULL, /* add_child */
     NULL, /* child_added */
     NULL, /* remove_child */
@@ -183,15 +188,15 @@ static SPReprEventVector shape_repr_events = {
 /**
 \brief  Callback that processes the "changed" signal on the selection;
 destroys old and creates new knotholder
+\param  selection Should not be NULL.
 */
 void
 sp_star_context_selection_changed (SPSelection * selection, gpointer data)
 {
-    SPStarContext *sc;
-    SPEventContext *ec;
+    g_assert (selection != NULL);
 
-    sc = SP_STAR_CONTEXT (data);
-    ec = SP_EVENT_CONTEXT (sc);
+    SPStarContext *sc = SP_STAR_CONTEXT (data);
+    SPEventContext *ec = SP_EVENT_CONTEXT (sc);
 
     if (sc->knot_holder) { // desktroy knotholder
         sp_knot_holder_destroy (sc->knot_holder);
@@ -204,7 +209,7 @@ sp_star_context_selection_changed (SPSelection * selection, gpointer data)
         sc->repr = 0;
     }
 
-    SPItem *item = sp_selection_item (selection);
+    SPItem *item = selection->singleItem();
     if (item) {
         sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
         SPRepr *repr = SP_OBJECT_REPR (item);
@@ -214,8 +219,6 @@ sp_star_context_selection_changed (SPSelection * selection, gpointer data)
             sp_repr_add_listener (repr, &shape_repr_events, ec);
             sp_repr_synthesize_events (repr, &shape_repr_events, ec);
         }
-
-
     }
 }
 
@@ -230,8 +233,12 @@ sp_star_context_setup (SPEventContext *ec)
     sp_event_context_read (ec, "magnitude");
     sp_event_context_read (ec, "proportion");
     sp_event_context_read (ec, "isflatsided");
+    sp_event_context_read (ec, "rounded");
+    sp_event_context_read (ec, "randomized");
 
-    SPItem *item = sp_selection_item (SP_DT_SELECTION (ec->desktop));
+    SPSelection *selection = SP_DT_SELECTION(ec->desktop);
+
+    SPItem *item = selection->singleItem();
         if (item) {
             sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
             SPRepr *repr = SP_OBJECT_REPR (item);
@@ -243,9 +250,14 @@ sp_star_context_setup (SPEventContext *ec)
             }
         }
 
-        g_signal_connect (G_OBJECT (SP_DT_SELECTION (ec->desktop)),
-            "changed", G_CALLBACK (sp_star_context_selection_changed), sc);
+    sc->sel_changed_connection.disconnect();
+    sc->sel_changed_connection = selection->connectChanged(sigc::bind(sigc::ptr_fun(&sp_star_context_selection_changed), (gpointer)sc));
 
+    if (prefs_get_int_attribute("tools.shapes", "selcue", 0) != 0) {
+        ec->enableSelectionCue();
+    }
+
+    sc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
 static void
@@ -254,51 +266,22 @@ sp_star_context_set (SPEventContext *ec, const gchar *key, const gchar *val)
     SPStarContext *sc = SP_STAR_CONTEXT (ec);
     if (!strcmp (key, "magnitude")) {
         sc->magnitude = (val) ? atoi (val) : 5;
-        sc->magnitude = CLAMP (sc->magnitude, 3, 32);
+        sc->magnitude = CLAMP (sc->magnitude, 3, 1024);
     } else if (!strcmp (key, "proportion")) {
-        sc->proportion = (val) ? atof (val) : 0.5;
+        sc->proportion = (val) ? g_ascii_strtod (val, NULL) : 0.5;
         sc->proportion = CLAMP (sc->proportion, 0.01, 2.0);
     } else if (!strcmp (key, "isflatsided")) {
-       if (val && !strcmp (val, "true"))
+        if (val && !strcmp(val, "true"))
             sc->isflatsided = true;
-       else sc->isflatsided = false;
-    }
+        else
+            sc->isflatsided = false;
+    } else if (!strcmp (key, "rounded")) {
+        sc->rounded = (val) ? g_ascii_strtod (val, NULL) : 0.0;
+    } else if (!strcmp (key, "randomized")) {
+        sc->randomized = (val) ? g_ascii_strtod (val, NULL) : 0.0;
+    }    
 }
 
-static gint
-sp_star_context_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
-{
-    SPDesktop *desktop = event_context->desktop;
-
-    gint ret = FALSE;
-
-    switch (event->type) {
-    case GDK_BUTTON_PRESS:
-        if (event->button.button == 1) {
-
-            // save drag origin
-            event_context->xp = (gint) event->button.x;
-            event_context->yp = (gint) event->button.y;
-            event_context->within_tolerance = true;
-
-            // remember clicked item, disregarding groups
-            event_context->item_to_select = sp_desktop_item_at_point (desktop, NR::Point(event->button.x, event->button.y), TRUE);
-
-            ret = TRUE;
-        }
-        break;
-        // motion and release are always on root (why?)
-    default:
-        break;
-    }
-
-    if (((SPEventContextClass *) parent_class)->item_handler)
-        ret = ((SPEventContextClass *) parent_class)->item_handler (event_context, item, event);
-
-    return ret;
-}
-
-
 static gint
 sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 {
@@ -327,7 +310,7 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
             /* Position center */
             sc->center = sp_desktop_w2d_xy_point (event_context->desktop, NR::Point(event->button.x, event->button.y));
             /* Snap center to nearest magnetic point */
-            sp_desktop_free_snap (event_context->desktop, sc->center);
+            namedview_free_snap (event_context->desktop->namedview, Snapper::SNAP_POINT, sc->center);
             sp_canvas_item_grab (SP_CANVAS_ITEM (desktop->acetate),
                                  GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
                                  NULL, event->button.time);
@@ -362,10 +345,10 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
             sp_star_finish (sc);
             } else if (event_context->item_to_select) {
                 // no dragging, select clicked item if any
-                sp_selection_set_item (SP_DT_SELECTION (desktop), event_context->item_to_select);
+                SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
             } else {
                 // click in an empty space
-                sp_selection_empty (SP_DT_SELECTION (desktop));
+                SP_DT_SELECTION(desktop)->clear();
             }
 
             event_context->item_to_select = NULL;
@@ -375,20 +358,59 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
         break;
     case GDK_KEY_PRESS:
         switch (event->key.keyval) {
-        case GDK_Up: 
-        case GDK_Down: 
-        case GDK_KP_Up: 
-        case GDK_KP_Down: 
+        case GDK_Alt_R:
+        case GDK_Control_L:
+        case GDK_Control_R:
+        case GDK_Shift_L:
+        case GDK_Shift_R:
+        case GDK_Meta_L:  // Meta is when you press Shift+Alt (at least on my machine)
+        case GDK_Meta_R:
+            sp_event_show_modifier_tip (event_context->defaultMessageContext(), event,
+                                        _("<b>Ctrl</b>: snap angle; keep rays radial"),
+                                        NULL,
+                                        NULL);
+            break;
+        case GDK_Up:
+        case GDK_Down:
+        case GDK_KP_Up:
+        case GDK_KP_Down:
             // prevent the zoom field from activation
             if (!MOD__CTRL_ONLY)
                 ret = TRUE;
             break;
+        case GDK_x:
+        case GDK_X:
+            if (MOD__ALT_ONLY) {
+                gpointer hb = sp_search_by_data_recursive (desktop->owner->aux_toolbox, (gpointer) "altx-star");
+                if (hb && GTK_IS_WIDGET(hb)) {
+                    gtk_widget_grab_focus (GTK_WIDGET (hb));
+                }
+                ret = TRUE;
+            }
+            break;
         case GDK_Escape:
-            sp_selection_empty (SP_DT_SELECTION (desktop)); // deselect
+            SP_DT_SELECTION(desktop)->clear();
             //TODO: make dragging escapable by Esc
         default:
             break;
         }
+        break;
+    case GDK_KEY_RELEASE:
+        switch (event->key.keyval) {
+        case GDK_Alt_L:
+        case GDK_Alt_R:
+        case GDK_Control_L:
+        case GDK_Control_R:
+        case GDK_Shift_L:
+        case GDK_Shift_R:
+        case GDK_Meta_L:  // Meta is when you press Shift+Alt
+        case GDK_Meta_R:
+            event_context->defaultMessageContext()->clear();
+            break;
+        default:
+            break;
+        }
+        break;
     default:
         break;
     }
@@ -402,31 +424,41 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
 }
 
 static void
-sp_star_drag (SPStarContext * sc, NR::Point p, guint state)
+sp_star_drag(SPStarContext *sc, NR::Point p, guint state)
 {
-    SPDesktop *desktop = SP_EVENT_CONTEXT (sc)->desktop;
+    SPDesktop *desktop = SP_EVENT_CONTEXT(sc)->desktop;
 
     int snaps = prefs_get_int_attribute ("options.rotationsnapsperpi", "value", 12);
 
     if (!sc->item) {
-        /* Create object */
-        SPRepr *repr = sp_repr_new ("polygon");
-        sp_repr_set_attr (repr, "sodipodi:type", "star");
-        /* Set style */
-        SPRepr *style = inkscape_get_repr (INKSCAPE, "tools.shapes.star");
-        if (style) {
-            SPCSSAttr *css = sp_repr_css_attr_inherited (style, "style");
-            sp_repr_css_set (repr, css, "style");
-            sp_repr_css_attr_unref (css);
+
+        SPItem *layer=SP_ITEM(desktop->currentLayer());
+        if ( !layer || desktop->itemIsHidden(layer)) {
+            sc->_message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is hidden</b>. Unhide it to be able to draw on it."));
+            return;
         }
-        sc->item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), repr);
+        if ( !layer || layer->isLocked()) {
+            sc->_message_context->set(Inkscape::ERROR_MESSAGE, _("<b>Current layer is locked</b>. Unlock it to be able to draw on it."));
+            return;
+        }
+
+        /* Create object */
+        SPRepr *repr = sp_repr_new ("path");
+        sp_repr_set_attr (repr, "sodipodi:type", "star");
+
+        /* Set style */
+        sp_desktop_apply_style_tool (desktop, repr, "tools.shapes.star", false);
+
+        sc->item = SP_ITEM(desktop->currentLayer()->appendChildRepr(repr));
         sp_repr_unref (repr);
+        sc->item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
+        sc->item->updateRepr();
     }
 
     /* Free movement for corner point */
     NR::Point p0 = sp_desktop_dt2root_xy_point (desktop, sc->center);
     NR::Point p1 = sp_desktop_dt2root_xy_point (desktop, p);
-    sp_desktop_free_snap (desktop, p1);
+    namedview_free_snap (desktop->namedview, Snapper::SNAP_POINT, p1);
 
     SPStar *star = SP_STAR(sc->item);
 
@@ -434,146 +466,43 @@ sp_star_drag (SPStarContext * sc, NR::Point p, guint state)
     NR::Point d = p1 - p0;
     gdouble r1 = NR::L2 (d);
     gdouble arg1 = atan2 (d);
-    if (state & GDK_CONTROL_MASK) { 
-        arg1 = round (arg1/(M_PI/snaps))*(M_PI/snaps);
-    } 
-	bool isflat = sc->isflatsided;
-	sp_star_position_set (star, sc->magnitude, p0, r1, r1 * sc->proportion, arg1, arg1 + M_PI / sides, isflat);
+    if (state & GDK_CONTROL_MASK) {
+        arg1 = sp_round(arg1, M_PI/snaps);
+    }
+
+    sp_star_position_set(star, sc->magnitude, p0, r1, r1 * sc->proportion,
+                         arg1, arg1 + M_PI / sides, sc->isflatsided, sc->rounded, sc->randomized);
 
     /* status text */
-    gchar status[80];
-    GString *xs = SP_PT_TO_METRIC_STRING (fabs(p0[NR::X]), SP_DEFAULT_METRIC);
-    GString *ys = SP_PT_TO_METRIC_STRING (fabs(p0[NR::Y]), SP_DEFAULT_METRIC);
-    if (isflat)
-         sprintf (status, _("Draw polygon at (%s,%s)"), xs->str, ys->str);
-    else sprintf (status, _("Draw star at (%s,%s)"), xs->str, ys->str);
-    sp_view_set_status (SP_VIEW (desktop), status, FALSE);
-    g_string_free (xs, FALSE);
-    g_string_free (ys, FALSE);
+    GString *rads = SP_PT_TO_METRIC_STRING (r1, SP_DEFAULT_METRIC);
+    sc->_message_context->setF(Inkscape::NORMAL_MESSAGE,
+                                      ( sc->isflatsided?
+                                      _("<b>Polygon</b>: radius %s, angle %5g; with <b>Ctrl</b> to snap angle")
+                                          : _("<b>Star</b>: radius %s, angle %5g; with <b>Ctrl</b> to snap angle") ),
+                                      rads->str, sp_round ((arg1)*180/M_PI, 0.0001));
+    g_string_free(rads, FALSE);
 }
 
 static void
 sp_star_finish (SPStarContext * sc)
 {
+    sc->_message_context->clear();
+
     if (sc->item != NULL) {
-        SPDesktop *desktop;
-        SPObject  *object;
+        SPDesktop *desktop = SP_EVENT_CONTEXT(sc)->desktop;
+        SPObject *object = SP_OBJECT(sc->item);
 
-        desktop = SP_EVENT_CONTEXT (sc)->desktop;
-        object  = SP_OBJECT(sc->item);
-        
-        sp_shape_set_shape (SP_SHAPE (sc->item));
+        sp_shape_set_shape(SP_SHAPE(sc->item));
 
-        sp_object_invoke_write (object, NULL, SP_OBJECT_WRITE_EXT);
+        object->updateRepr(NULL, SP_OBJECT_WRITE_EXT);
 
-        sp_selection_set_item (SP_DT_SELECTION (desktop), sc->item);
-        sp_document_done (SP_DT_DOCUMENT (desktop));
+        SP_DT_SELECTION(desktop)->setItem(sc->item);
+        sp_document_done(SP_DT_DOCUMENT(desktop));
 
         sc->item = NULL;
     }
 }
 
-/* Gtk stuff */
-
-static void
-sp_sc_magnitude_value_changed (GtkAdjustment *adj, SPStarContext *sc)
-{
-    sp_repr_set_int (SP_EVENT_CONTEXT_REPR (sc), "magnitude", (gint) adj->value);
-}
-
-static void
-sp_sc_proportion_value_changed (GtkAdjustment *adj, SPStarContext *sc)
-{
-    sp_repr_set_double (SP_EVENT_CONTEXT_REPR (sc), "proportion", adj->value);
-}
-
-
-static void
-sp_sc_sides_flat_state_changed (GtkWidget *widget, SPStarContext *sc)
-{
-sp_repr_set_boolean (SP_EVENT_CONTEXT_REPR (sc), "isflatsided", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
-    
-}
-
-static void
-sp_sc_defaults (GtkWidget *widget, GtkObject *obj)
-{
-    GtkAdjustment *adj = (GtkAdjustment*)gtk_object_get_data (obj, "magnitude");
-    gtk_adjustment_set_value (adj, 3);
-    adj = (GtkAdjustment*)gtk_object_get_data (obj, "proportion");
-    gtk_adjustment_set_value (adj, 0.5);
-}
-
-static GtkWidget *
-sp_star_context_config_widget (SPEventContext *ec)
-{
-    SPStarContext *sc = SP_STAR_CONTEXT (ec);
-
-    GtkWidget *tbl = gtk_table_new (4, 2, FALSE);
-    gtk_container_set_border_width (GTK_CONTAINER (tbl), 4);
-    gtk_table_set_row_spacings (GTK_TABLE (tbl), 4);
-
-    /* Magnitude */
-    GtkWidget *l = gtk_label_new (_("Corners:"));
-    gtk_widget_show (l);
-    gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-    gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 0, 1, 
-                      (GtkAttachOptions)0, 
-                      (GtkAttachOptions)0, 
-                      0, 0);
-    GtkObject *mag_adj = gtk_adjustment_new (sc->magnitude, 3, 32, 1, 1, 1);
-    gtk_object_set_data (GTK_OBJECT (tbl), "magnitude", mag_adj);
-    GtkWidget *sb1 = gtk_spin_button_new (GTK_ADJUSTMENT (mag_adj), 1, 0);
-    gtk_widget_show (sb1);
-    gtk_table_attach (GTK_TABLE (tbl), sb1, 1, 2, 0, 1, 
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      (GtkAttachOptions)0, 
-                      0, 0);
-    gtk_signal_connect (GTK_OBJECT (mag_adj), "value_changed", GTK_SIGNAL_FUNC (sp_sc_magnitude_value_changed), sc);
-
-    /* Proportion */
-    l = gtk_label_new (_("Proportion:"));
-    gtk_widget_show (l);
-    gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-    gtk_table_attach (GTK_TABLE (tbl), l, 0, 1, 1, 2, 
-                      (GtkAttachOptions)0, 
-                      (GtkAttachOptions)0, 
-                      0, 0);
-    GtkObject *proportion_adj = gtk_adjustment_new (sc->proportion, 0.01, 1.0, 0.01, 0.1, 0.1);
-    gtk_object_set_data (GTK_OBJECT (tbl), "proportion", proportion_adj);
-    GtkWidget *sb2 = gtk_spin_button_new (GTK_ADJUSTMENT (proportion_adj), 0.1, 2);
-    gtk_widget_show (sb2);
-    gtk_table_attach (GTK_TABLE (tbl), sb2, 1, 2, 1, 2, 
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      (GtkAttachOptions)0, 
-                      0, 0);
-    gtk_signal_connect (GTK_OBJECT (proportion_adj), "value_changed", GTK_SIGNAL_FUNC (sp_sc_proportion_value_changed), sc);
-    
-
-    /* Flat Sides checkbox */
-    GtkWidget *fscb = gtk_check_button_new_with_label (_("Flat Sided"));
-    gtk_widget_set_sensitive (GTK_WIDGET (fscb), TRUE);
-    gtk_widget_show (fscb);
-    gtk_table_attach (GTK_TABLE (tbl), fscb, 0, 2, 2, 3,
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      0, 0);
-
-    g_signal_connect (G_OBJECT(fscb), "toggled", GTK_SIGNAL_FUNC (sp_sc_sides_flat_state_changed ), sc);
-    
-    /* Reset */
-    GtkWidget *b = gtk_button_new_with_label (_("Defaults"));
-    gtk_widget_show (b);
-    gtk_table_attach (GTK_TABLE (tbl), b, 0, 2, 3, 4, 
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
-                      0, 0);
-    gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_sc_defaults), tbl);
-
-    return tbl;
-}
-
-
 /*
   Local Variables:
   mode:c++
