@@ -10,7 +10,7 @@
  *
  * This code is in public domain
  */
-#include <config.h>
+#include "config.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -56,7 +56,7 @@ sp_svg_number_read_f (const gchar *str, float *val)
 	float v;
 
 	if (!str) return 0;
-	v = strtod (str, &e);
+	v = g_ascii_strtod (str, &e);
 	if ((const gchar *) e == str) return 0;
 	*val = v;
 	return 1;
@@ -69,7 +69,7 @@ sp_svg_number_read_d (const gchar *str, double *val)
 	double v;
 
 	if (!str) return 0;
-	v = strtod (str, &e);
+	v = g_ascii_strtod (str, &e);
 	if ((const gchar *) e == str) return 0;
 	*val = v;
 	return 1;
@@ -160,7 +160,7 @@ sp_svg_number_write_de (gchar *buf, double val, unsigned int tprec, unsigned int
 unsigned int
 sp_svg_length_read (const gchar *str, SPSVGLength *length)
 {
-	unsigned long unit;
+	SPSVGLengthUnit unit;
 	float value, computed;
 
 	if (!str) return 0;
@@ -179,7 +179,7 @@ sp_svg_length_read (const gchar *str, SPSVGLength *length)
 unsigned int
 sp_svg_length_read_absolute (const gchar *str, SPSVGLength *length)
 {
-	unsigned long unit;
+	SPSVGLengthUnit unit;
 	float value, computed;
 
 	if (!str) return 0;
@@ -203,7 +203,7 @@ sp_svg_length_read_absolute (const gchar *str, SPSVGLength *length)
 unsigned int
 sp_svg_length_read_computed_absolute (const gchar *str, float *length)
 {
-	unsigned long unit;
+	SPSVGLengthUnit unit;
 	float computed;
 
 	if (!str) return 0;
@@ -224,7 +224,7 @@ sp_svg_length_read_computed_absolute (const gchar *str, float *length)
 GList *
 sp_svg_length_list_read (const gchar *str)
 {
-	unsigned long unit;
+	SPSVGLengthUnit unit;
 	float value, computed;
 	char *next = (char *) str;
 	GList *list = NULL;
@@ -255,13 +255,13 @@ sp_svg_length_list_read (const gchar *str)
 #define UVAL(a,b) (((unsigned int) (a) << 8) | (unsigned int) (b))
 
 unsigned int
-sp_svg_length_read_lff (const gchar *str, unsigned long *unit, float *val, float *computed, char **next)
+sp_svg_length_read_lff (const gchar *str, SPSVGLengthUnit *unit, float *val, float *computed, char **next)
 {
 	const gchar *e;
 	float v;
 
 	if (!str) return 0;
-	v = strtod (str, (char **) &e);
+	v = g_ascii_strtod (str, (char **) &e);
 	if (e == str) {
 		return 0;
 	}
@@ -272,13 +272,14 @@ sp_svg_length_read_lff (const gchar *str, unsigned long *unit, float *val, float
 		if (computed) *computed = v;
 		if (next) *next = NULL; // no more values
 		return 1;
-	} else if (!isalnum (e[0])) {
+	} else if (!g_ascii_isalnum (e[0])) {
 		/* Unitless or percent */
 		if (e[0] == '%') {
 			/* Percent */
-			if (e[1] && isalnum (e[1])) return 0;
+			if (e[1] && g_ascii_isalnum (e[1])) return 0;
 			if (unit) *unit = SP_SVG_UNIT_PERCENT;
 			if (val) *val = v * 0.01;
+			if (computed) *computed = v * 0.01;
 			if (next) *next = (char *) e + 1; 
 			return 1;
 		} else {
@@ -289,7 +290,7 @@ sp_svg_length_read_lff (const gchar *str, unsigned long *unit, float *val, float
 			if (next) *next = (char *) e; 
 			return 1;
 		}
-	} else if (e[1] && !isalnum (e[2])) {
+	} else if (e[1] && !g_ascii_isalnum (e[2])) {
 		unsigned int uval;
 		/* Units */
 		uval = UVAL (e[0], e[1]);
@@ -338,7 +339,7 @@ sp_svg_length_read_lff (const gchar *str, unsigned long *unit, float *val, float
 	return 0;
 }
 
-unsigned int sp_svg_length_read_ldd (const gchar *str, unsigned long *unit, double *value, double *computed) {
+unsigned int sp_svg_length_read_ldd (const gchar *str, SPSVGLengthUnit *unit, double *value, double *computed) {
 	float a, b;
 	unsigned int r = sp_svg_length_read_lff (str, unit, &a, &b, NULL);
 	if(value) *value = a;
@@ -347,7 +348,7 @@ unsigned int sp_svg_length_read_ldd (const gchar *str, unsigned long *unit, doub
 }
 
 void
-sp_svg_length_set (SPSVGLength *length, unsigned long unit, float value, float computed)
+sp_svg_length_set (SPSVGLength *length, SPSVGLengthUnit unit, float value, float computed)
 {
 	length->set = 1;
 	length->unit = unit;
@@ -356,7 +357,7 @@ sp_svg_length_set (SPSVGLength *length, unsigned long unit, float value, float c
 }
 
 void
-sp_svg_length_unset (SPSVGLength *length, unsigned long unit, float value, float computed)
+sp_svg_length_unset (SPSVGLength *length, SPSVGLengthUnit unit, float value, float computed)
 {
 	length->set = 0;
 	length->unit = unit;
@@ -384,7 +385,7 @@ sp_svg_read_percentage (const char * str, double def)
 
 	if (str == NULL) return def;
 
-	v = strtod (str, &u);
+	v = g_ascii_strtod (str, &u);
 	while (isspace (*u)) {
 		if (*u == '\0') return v;
 		u++;
