@@ -12,17 +12,22 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <math.h>
 
 #include <ctype.h>
-#include <libart_lgpl/art_affine.h>
 #include <glib-object.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
-#include <helper/sp-ctrlline.h>
-#include <helper/sodipodi-ctrlrect.h>
+#include <display/sp-ctrlline.h>
+#include <display/sodipodi-ctrlrect.h>
 #include <libnr/nr-matrix-ops.h>
 #include <gtk/gtkimmulticontext.h>
+
+#include <gtkmm.h>
+
 #include "macros.h"
 #include "sp-text.h"
 #include "inkscape.h"
@@ -30,14 +35,22 @@
 #include "style.h"
 #include "selection.h"
 #include "desktop.h"
+#include "desktop-style.h"
 #include "desktop-handles.h"
 #include "desktop-affine.h"
 #include "pixmaps/cursor-text.xpm"
-#include "text-context.h"
+#include "pixmaps/cursor-text-insert.xpm"
 #include "view.h"
+#include "helper/sp-intl.h"
+#include "xml/repr.h"
+#include "prefs-utils.h"
+
+#include "text-context.h"
+
 
 static void sp_text_context_class_init (SPTextContextClass * klass);
 static void sp_text_context_init (SPTextContext * text_context);
+static void sp_text_context_dispose(GObject *obj);
 
 static void sp_text_context_setup (SPEventContext *ec);
 static void sp_text_context_finish (SPEventContext *ec);
@@ -82,10 +95,13 @@ sp_text_context_get_type (void)
 static void
 sp_text_context_class_init (SPTextContextClass * klass)
 {
+	GObjectClass *object_class=(GObjectClass *)klass;
 	SPEventContextClass *event_context_class = (SPEventContextClass *) klass;
 
 	parent_class = (SPEventContextClass*)g_type_class_peek_parent (klass);
 
+	object_class->dispose = sp_text_context_dispose;
+
 	event_context_class->setup = sp_text_context_setup;
 	event_context_class->finish = sp_text_context_finish;
 	event_context_class->root_handler = sp_text_context_root_handler;
@@ -98,8 +114,8 @@ sp_text_context_init (SPTextContext *tc)
 	SPEventContext *event_context = SP_EVENT_CONTEXT (tc);
 
 	event_context->cursor_shape = cursor_text_xpm;
-	event_context->hot_x = 0;
-	event_context->hot_y = 0;
+	event_context->hot_x = 7;
+	event_context->hot_y = 7;
 
 	tc->imc = NULL;
 
@@ -117,6 +133,20 @@ sp_text_context_init (SPTextContext *tc)
 	tc->nascent_object = 0;
 
 	tc->preedit_string = NULL;
+
+	new (&tc->sel_changed_connection) sigc::connection();
+	new (&tc->sel_modified_connection) sigc::connection();
+}
+
+static void
+sp_text_context_dispose(GObject *obj)
+{
+	SPTextContext *tc=SP_TEXT_CONTEXT(obj);
+	tc->sel_changed_connection.~connection();
+	tc->sel_modified_connection.~connection();
+	if (G_OBJECT_CLASS(parent_class)->dispose) {
+		G_OBJECT_CLASS(parent_class)->dispose(obj);
+	}
 }
 
 static void
@@ -152,16 +182,23 @@ sp_text_context_setup (SPEventContext *ec)
 		if (GTK_WIDGET_HAS_FOCUS (canvas)) {
 			sptc_focus_in (canvas, NULL, tc);
 		}
-
 	}
 
 	if (((SPEventContextClass *) parent_class)->setup)
 		((SPEventContextClass *) parent_class)->setup (ec);
 
-	g_signal_connect (G_OBJECT (SP_DT_SELECTION (desktop)), "changed", G_CALLBACK (sp_text_context_selection_changed), tc);
-	g_signal_connect (G_OBJECT (SP_DT_SELECTION (desktop)), "modified", G_CALLBACK (sp_text_context_selection_modified), tc);
+	tc->sel_changed_connection = SP_DT_SELECTION(desktop)->connectChanged(
+		sigc::bind(sigc::ptr_fun(&sp_text_context_selection_changed), tc)
+	);
+	tc->sel_modified_connection = SP_DT_SELECTION(desktop)->connectModified(
+		sigc::bind(sigc::ptr_fun(&sp_text_context_selection_modified), tc)
+	);
 
 	sp_text_context_selection_changed (SP_DT_SELECTION (desktop), tc);
+
+	if (prefs_get_int_attribute("tools.text", "selcue", 0) != 0) {
+		ec->enableSelectionCue();
+	}
 }
 
 static void
@@ -169,6 +206,9 @@ sp_text_context_finish (SPEventContext *ec)
 {
 	SPTextContext *tc = SP_TEXT_CONTEXT (ec);
 
+	tc->sel_changed_connection.disconnect();
+	tc->sel_modified_connection.disconnect();
+
 	sp_text_context_forget_text (SP_TEXT_CONTEXT (ec));
 
 	if (tc->imc) {
@@ -193,7 +233,6 @@ sp_text_context_finish (SPEventContext *ec)
 
 	if (ec->desktop) {
   		sp_signal_disconnect_by_data (SP_DT_CANVAS (ec->desktop), tc);
-  		sp_signal_disconnect_by_data (SP_DT_SELECTION (ec->desktop), ec);
 	}
 }
 
@@ -212,7 +251,7 @@ sp_text_context_item_handler (SPEventContext *ec, SPItem *item, GdkEvent *event)
 			// find out clicked item, disregarding groups
 			item_ungrouped = sp_desktop_item_at_point (desktop, NR::Point(event->button.x, event->button.y), TRUE);
 			if (SP_IS_TEXT (item_ungrouped)) {
-				sp_selection_set_item (SP_DT_SELECTION (ec->desktop), item_ungrouped);
+				SP_DT_SELECTION(ec->desktop)->setItem(item_ungrouped);
 				if (tc->text) {
 					// find out click point in document coordinates
 					NR::Point p = sp_desktop_w2d_xy_point (ec->desktop, NR::Point(event->button.x, event->button.y));
@@ -233,8 +272,14 @@ sp_text_context_item_handler (SPEventContext *ec, SPItem *item, GdkEvent *event)
 			sp_item_bbox_desktop(item_ungrouped, &bbox);
 			sp_canvas_item_show (tc->indicator);
 			sp_ctrlrect_set_area (SP_CTRLRECT (tc->indicator), 
-					      bbox.x0, bbox.y0, 
-					      bbox.x1, bbox.y1);
+								bbox.x0, bbox.y0, 
+								bbox.x1, bbox.y1);
+
+			ec->cursor_shape = cursor_text_insert_xpm;
+			ec->hot_x = 7;
+			ec->hot_y = 10;
+			sp_event_context_update_cursor (ec);
+
 			ret = TRUE;
 		}
 		break;
@@ -260,18 +305,14 @@ sp_text_context_setup_text (SPTextContext *tc)
 	sp_repr_set_attr (rtext, "xml:space", "preserve"); // we preserve spaces in the text objects we create
 
 	/* Set style */
-	SPRepr *style = inkscape_get_repr (INKSCAPE, "tools.text");
-	if (style) {
-		SPCSSAttr *css;
-		css = sp_repr_css_attr_inherited (style, "style");
-		sp_repr_css_set (rtext, css, "style");
-		sp_repr_css_attr_unref (css);
-	}
+	sp_desktop_apply_style_tool (SP_EVENT_CONTEXT_DESKTOP(ec), rtext, "tools.text", true);
+
 	sp_repr_set_double (rtext, "x", tc->pdoc[NR::X]);
 	sp_repr_set_double (rtext, "y", tc->pdoc[NR::Y]);
 
 	/* Create <tspan> */
 	SPRepr *rtspan = sp_repr_new ("tspan");
+	sp_repr_set_attr (rtspan, "sodipodi:role", "line"); // otherwise, why bother creating the tspan?
 	sp_repr_add_child (rtext, rtspan, NULL);
 	sp_repr_unref (rtspan);
 
@@ -279,10 +320,13 @@ sp_text_context_setup_text (SPTextContext *tc)
 	SPRepr *rstring = sp_xml_document_createTextNode (sp_repr_document (rtext), "");
 	sp_repr_add_child (rtspan, rstring, NULL);
 	sp_repr_unref (rstring);
-	sp_document_add_repr (SP_DT_DOCUMENT (ec->desktop), rtext);
+	SPItem *text_item = SP_ITEM(ec->desktop->currentLayer()->appendChildRepr(rtext));
 	/* fixme: Is selection::changed really immediate? */
-	sp_selection_set_repr (SP_DT_SELECTION (ec->desktop), rtext);
+	/* yes, it's immediate .. why does it matter? */
+	SP_DT_SELECTION(ec->desktop)->setItem(text_item);
 	sp_repr_unref (rtext);
+	text_item->transform = SP_ITEM(ec->desktop->currentRoot())->getRelativeTransform(ec->desktop->currentLayer());
+	text_item->updateRepr();
 	sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 }
 
@@ -294,10 +338,28 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	sp_canvas_item_hide (tc->indicator);
 
 	switch (event->type) {
+	case GDK_MOTION_NOTIFY:
+		ec->cursor_shape = cursor_text_xpm;
+		ec->hot_x = 7;
+		ec->hot_y = 7;
+		sp_event_context_update_cursor (ec);
+		break;
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
+
+            SPDesktop *desktop = SP_EVENT_CONTEXT_DESKTOP(ec);
+            SPItem *layer=SP_ITEM(desktop->currentLayer());
+            if ( !layer || desktop->itemIsHidden(layer)) {
+                desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("<b>Current layer is hidden</b>. Unhide it to be able to add text."));
+                return TRUE;
+            }
+            if ( !layer || layer->isLocked()) {
+                desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("<b>Current layer is locked</b>. Unlock it to be able to add text."));
+                return TRUE;
+            }
+
 			/* Button 1, set X & Y & new item */
-			sp_selection_empty (SP_DT_SELECTION (ec->desktop));
+			SP_DT_SELECTION(ec->desktop)->clear();
 			NR::Point dtp = sp_desktop_w2d_xy_point (ec->desktop, NR::Point(event->button.x, event->button.y));
 			tc->pdoc = sp_desktop_dt2root_xy_point (ec->desktop, dtp);
 
@@ -307,11 +369,10 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 
 			/* Cursor */
 			sp_canvas_item_show (tc->cursor);
-			// the cursor right after click has height == 0.8 * 12px (corresponds to the default 12 px font in preferences-skeleton)
-			// It needs to be set articifically, for the text object does not exist yet.
-			// Properly, we need to look up the current default font size in tools.text, but this may be more trouble than it's worth
-			// (parsing, units conversion, etc...)
-			sp_ctrlline_set_coords (SP_CTRLLINE (tc->cursor), dtp, dtp + NR::Point(0, 9.6)); 
+			// Cursor height is defined by the new text object's font size; it needs to be set
+			// articifically here, for the text object does not exist yet:
+			double cursor_height = 0.8 * sp_desktop_get_font_size_tool (ec->desktop);
+			sp_ctrlline_set_coords (SP_CTRLLINE (tc->cursor), dtp, dtp + NR::Point(0, cursor_height)); 
 
 			/* Processed */
 			return TRUE;
@@ -322,36 +383,41 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		if (event->key.keyval == GDK_KP_Add || event->key.keyval == GDK_KP_Subtract)
 			break; // pass on keypad +/- so they can zoom
 
-		if (!sp_selection_is_empty (SP_DT_SELECTION (ec->desktop)) || (tc->nascent_object)) {
-			// there is a selection, or a new object was just created
+		if (MOD__CTRL && MOD__SHIFT) {
+			// some input methods (recently started to) gobble ctrl+shift keystrokes, but we need them for dialogs!
+			break;
+		}
 
-			tc->nascent_object = 0; // we don't need it anymore, because now sp_selection_is_empty != NULL
+		if ((tc->text) || (tc->nascent_object)) {
+			// there is an active text object in this context, or a new object was just created
 
 			if (tc->unimode || !tc->imc || !gtk_im_context_filter_keypress (tc->imc, (GdkEventKey*) event)) {
-				//IM did not consumed the key, or we're in unimode
+				//IM did not consume the key, or we're in unimode
 
-				if (!tc->text) sp_text_context_setup_text (tc);
-				else sp_text_context_preedit_reset (tc);
-				g_assert (tc->text != NULL);
-				SPStyle *style = SP_OBJECT_STYLE (tc->text);
+				if (tc->text) 
+					sp_text_context_preedit_reset (tc);
 
 				if (MOD__CTRL_ONLY) {
 					switch (event->key.keyval) {
 					case GDK_space:
 						/* No-break space */
+						if (!tc->text) { // printable key; create text if none (i.e. if nascent_object)
+							sp_text_context_setup_text (tc);
+							tc->nascent_object = 0; // we don't need it anymore, having created a real <text>
+						}
 						tc->ipos = sp_text_insert (SP_TEXT (tc->text), tc->ipos, "\302\240");
-						sp_view_set_statusf_flash (SP_VIEW(ec->desktop), "No-break space");
+						ec->desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("No-break space"));
 						sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 						return TRUE;
 					case GDK_U:
 					case GDK_u:
 						if (tc->unimode) {
 							tc->unimode = FALSE;
-							sp_view_clear_status (SP_VIEW(ec->desktop));
+							ec->defaultMessageContext()->clear();
 						} else {
 							tc->unimode = TRUE;
 							tc->unipos = 0;
-							sp_view_set_statusf (SP_VIEW(ec->desktop), "Unicode: ");
+							ec->defaultMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("Unicode: "));
 						}
 						if (tc->imc) {
 							gtk_im_context_reset (tc->imc);
@@ -364,8 +430,8 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 					if (tc->unimode) {
 						if (isxdigit ((guchar) event->key.keyval)) {
 							tc->uni[tc->unipos] = event->key.keyval;
-							sp_view_set_statusf (SP_VIEW(ec->desktop), 
-                                                        "Unicode: %c%c%c%c", 
+							ec->defaultMessageContext()->setF(Inkscape::NORMAL_MESSAGE,
+                                                        _("Unicode: %c%c%c%c"), 
                                                         tc->uni[0], 
                                                         tc->unipos > 0 ? tc->uni[1] : ' ', 
                                                         tc->unipos > 1 ? tc->uni[2] : ' ', 
@@ -378,8 +444,12 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 								u[len] = '\0';
 								tc->unipos = 0;
 								if (!g_unichar_isprint ((gunichar) uv)) {
-									sp_view_set_statusf_error (SP_VIEW(ec->desktop), "Non-printable character"); // this may be due to bad input, so it goes to statusbar
+								    ec->desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Non-printable character")); // this may be due to bad input, so it goes to statusbar
 								} else {
+									if (!tc->text) { // printable key; create text if none (i.e. if nascent_object)
+										sp_text_context_setup_text (tc);                                                 
+										tc->nascent_object = 0; // we don't need it anymore, having created a real <text>
+									}
 									tc->ipos = sp_text_insert (SP_TEXT (tc->text), tc->ipos, u);
 									sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 								}
@@ -388,10 +458,10 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 								tc->unipos += 1;
 								return TRUE;
 							}
-						} else { // non-hex-digit, canceling unimode
+						} else if (event->key.keyval != GDK_Shift_L && event->key.keyval != GDK_Shift_R) { // non-hex-digit, canceling unimode
 							tc->unimode = FALSE;
 							gtk_im_context_reset (tc->imc);
-							sp_view_clear_status (SP_VIEW(ec->desktop));
+							ec->defaultMessageContext()->clear();
 							return TRUE;
 						}
 					}
@@ -400,140 +470,167 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 					switch (event->key.keyval) {
 					case GDK_Return:
 					case GDK_KP_Enter:
-						sp_text_insert_line (SP_TEXT (tc->text), tc->ipos);
-						tc->ipos += 1;
+						if (!tc->text) { // printable key; create text if none (i.e. if nascent_object)
+							sp_text_context_setup_text (tc);                                                 
+							tc->nascent_object = 0; // we don't need it anymore, having created a real <text>
+						}
+						tc->ipos += sp_text_insert_line (SP_TEXT (tc->text), tc->ipos);
 						sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 						return TRUE;
 					case GDK_BackSpace:
-						tc->ipos = sp_text_delete (SP_TEXT (tc->text), MAX (tc->ipos - 1, 0), tc->ipos);
-						sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+						if (tc->text) { // if nascent_object, do nothing, but return TRUE; same for all other delete and move keys
+							tc->ipos = sp_text_delete (SP_TEXT (tc->text), MAX (tc->ipos - 1, 0), tc->ipos);
+							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+						}
 						return TRUE;
 					case GDK_Delete:
 					case GDK_KP_Delete:
-						tc->ipos = sp_text_delete (SP_TEXT (tc->text), tc->ipos, MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text))));
-						sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+						if (tc->text) {
+							tc->ipos = sp_text_delete (SP_TEXT (tc->text), tc->ipos, MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text))));
+							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+						}
 						return TRUE;
 					case GDK_Left:
 					case GDK_KP_Left:
 					case GDK_KP_4:
-						if (MOD__ALT) { 
-							if (MOD__SHIFT)
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(-10, 0));
-							else
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(-1, 0));
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-						} else {
-							if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-								tc->ipos = sp_text_down (SP_TEXT (tc->text), tc->ipos);
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__SHIFT)
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(-10, 0));
+								else
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(-1, 0));
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 							} else {
-								tc->ipos = MAX (tc->ipos - 1, 0);
+								SPStyle *style = SP_OBJECT_STYLE (tc->text);
+								if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
+									tc->ipos = sp_text_down (SP_TEXT (tc->text), tc->ipos);
+								} else {
+									tc->ipos = MAX (tc->ipos - 1, 0);
+								}
 							}
+							sp_text_context_update_cursor (tc);
 						}
-						sp_text_context_update_cursor (tc);
 						return TRUE;
 					case GDK_Right:
 					case GDK_KP_Right:
 					case GDK_KP_6:
-						if (MOD__ALT) { 
-							if (MOD__SHIFT)
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(10, 0));
-							else
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(1, 0));
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-						} else {
-							if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-								tc->ipos = sp_text_up (SP_TEXT (tc->text), tc->ipos);
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__SHIFT)
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(10, 0));
+								else
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(1, 0));
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 							} else {
-								tc->ipos = MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text)));
+								SPStyle *style = SP_OBJECT_STYLE (tc->text);
+								if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
+									tc->ipos = sp_text_up (SP_TEXT (tc->text), tc->ipos);
+								} else {
+									tc->ipos = MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text)));
+								}
 							}
+							sp_text_context_update_cursor (tc);
 						}
-						sp_text_context_update_cursor (tc);
 						return TRUE;
 					case GDK_Up:
 					case GDK_KP_Up:
 					case GDK_KP_8:
-						if (MOD__ALT) { 
-							if (MOD__SHIFT)
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, -10));
-							else 
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, -1));
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-						} else {
-							if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-								tc->ipos = MAX (tc->ipos - 1, 0);
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__SHIFT)
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, -10));
+								else 
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, -1));
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 							} else {
-								tc->ipos = sp_text_up (SP_TEXT (tc->text), tc->ipos);
+								SPStyle *style = SP_OBJECT_STYLE (tc->text);
+								if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
+									tc->ipos = MAX (tc->ipos - 1, 0);
+								} else {
+									tc->ipos = sp_text_up (SP_TEXT (tc->text), tc->ipos);
+								}
 							}
+							sp_text_context_update_cursor (tc);
 						}
-						sp_text_context_update_cursor (tc);
 						return TRUE;
 					case GDK_Down:
 					case GDK_KP_Down:
 					case GDK_KP_2:
-						if (MOD__ALT) { 
-							if (MOD__SHIFT)
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, 10));
-							else 
-								sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, 1));
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-						} else {
-							if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-								tc->ipos = MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text)));
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__SHIFT)
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, 10));
+								else 
+									sp_adjust_kerning_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, NR::Point(0, 1));
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
 							} else {
-								tc->ipos = sp_text_down (SP_TEXT (tc->text), tc->ipos);
+								SPStyle *style = SP_OBJECT_STYLE (tc->text);
+								if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
+									tc->ipos = MIN (tc->ipos + 1, sp_text_get_length (SP_TEXT (tc->text)));
+								} else {
+									tc->ipos = sp_text_down (SP_TEXT (tc->text), tc->ipos);
+								}
 							}
+							sp_text_context_update_cursor (tc);
 						}
-						sp_text_context_update_cursor (tc);
 						return TRUE;
 					case GDK_Home:
 					case GDK_KP_Home:
-						tc->ipos = sp_text_start_of_line (SP_TEXT (tc->text), tc->ipos);
-						sp_text_context_update_cursor (tc);
+						if (tc->text) {
+							tc->ipos = sp_text_start_of_line (SP_TEXT (tc->text), tc->ipos);
+							sp_text_context_update_cursor (tc);
+						}
 						return TRUE;
 					case GDK_End:
 					case GDK_KP_End:
-						tc->ipos = sp_text_end_of_line (SP_TEXT (tc->text), tc->ipos);
-						sp_text_context_update_cursor (tc);
+						if (tc->text) {
+							tc->ipos = sp_text_end_of_line (SP_TEXT (tc->text), tc->ipos);
+							sp_text_context_update_cursor (tc);
+						}
 						return TRUE;
 					case GDK_Escape:
-						sp_selection_empty (SP_DT_SELECTION (ec->desktop));
+						SP_DT_SELECTION(ec->desktop)->clear();
 						return TRUE;
 					case GDK_less:
 					case GDK_comma:
-						if (MOD__ALT) { 
-							if (MOD__CTRL) {
-								if (MOD__SHIFT)
-									sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, -10);
-								else 
-									sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, -1);
-							} else {
-								if (MOD__SHIFT)
-									sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, -10);
-								else 
-									sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, -1);
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__CTRL) {
+									if (MOD__SHIFT)
+										sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, -10);
+									else 
+										sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, -1);
+								} else {
+									if (MOD__SHIFT)
+										sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, -10);
+									else 
+										sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, -1);
+								}
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+								sp_text_context_update_cursor (tc);
+								return TRUE;
 							}
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-							sp_text_context_update_cursor (tc);
-							return TRUE;
 						}
 						break;
 					case GDK_greater:
 					case GDK_period:
-						if (MOD__ALT) { 
-							if (MOD__CTRL) {
-								if (MOD__SHIFT)
-									sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, 10);
-								else 
-									sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, 1);
-							} else {
-								if (MOD__SHIFT)
-									sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, 10);
-								else 
-									sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, 1);
+						if (tc->text) {
+							if (MOD__ALT) { 
+								if (MOD__CTRL) {
+									if (MOD__SHIFT)
+										sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, 10);
+									else 
+										sp_adjust_linespacing_screen (SP_TEXT (tc->text), ec->desktop, 1);
+								} else {
+									if (MOD__SHIFT)
+										sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, 10);
+									else 
+										sp_adjust_tspan_letterspacing_screen (SP_TEXT (tc->text), tc->ipos, ec->desktop, 1);
+								}
+								sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+								sp_text_context_update_cursor (tc);
+								return TRUE;
 							}
-							sp_document_done (SP_DT_DOCUMENT (ec->desktop));
-							sp_text_context_update_cursor (tc);
-							return TRUE;
 						}
 						break;
 					default:
@@ -543,8 +640,10 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 			} else return TRUE; // return the "I took care of it" value if it was consumed by the IM
 		} else { // do nothing if there's no object to type in - the key will be sent to parent context, 
 			    // except up/down that are swallowed to prevent the zoom field from activation
-			if ((event->key.keyval == GDK_Up || event->key.keyval == GDK_Down || 
-					 event->key.keyval == GDK_KP_Up || event->key.keyval == GDK_KP_Down) && !MOD__CTRL_ONLY) 
+			if ((event->key.keyval == GDK_Up || 
+				 event->key.keyval == GDK_Down ||
+				 event->key.keyval == GDK_KP_Up || 
+				 event->key.keyval == GDK_KP_Down) && !MOD__CTRL_ONLY) 
 				return TRUE;
 		}
 		break;
@@ -565,10 +664,49 @@ sp_text_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	}
 }
 
+/**
+ Attempts to paste system clipboard into the currently edited text, returns true on success
+ */
+bool
+sp_text_paste_inline(SPEventContext *ec)
+{
+	if (!SP_IS_TEXT_CONTEXT (ec))
+		return false;
+
+	SPTextContext *tc = SP_TEXT_CONTEXT (ec);
+
+	if ((tc->text) || (tc->nascent_object)) {
+		// there is an active text object in this context, or a new object was just created
+
+		Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
+		Glib::ustring const text = refClipboard->wait_for_text();
+
+		if (!text.empty()) {
+
+			if (!tc->text) { // create text if none (i.e. if nascent_object)
+				sp_text_context_setup_text (tc);
+				tc->nascent_object = 0; // we don't need it anymore, having created a real <text>
+			}
+
+			tc->ipos = sp_text_insert (SP_TEXT (tc->text), tc->ipos, text.c_str());
+			sp_document_done (SP_DT_DOCUMENT (ec->desktop));
+
+			return true;
+		}
+	} // FIXME: else create and select a new object under cursor!
+
+	return false;
+}
+
+/**
+ * \param selection Should not be NULL.
+ */
 static void
 sp_text_context_selection_changed (SPSelection *selection, SPTextContext *tc)
 {
-	SPItem *item = sp_selection_item (selection);
+	g_assert (selection != NULL);
+	
+	SPItem *item = selection->singleItem();
 
 	if (tc->text && (item != tc->text)) {
 		sp_text_context_forget_text (tc);
@@ -600,7 +738,12 @@ sp_text_context_update_cursor (SPTextContext *tc,  bool scroll_to_see)
 
 	if (tc->text) {
 		NR::Point p0, p1;
-		sp_text_get_cursor_coords (SP_TEXT (tc->text), tc->ipos, p0, p1);
+
+		// text length may have changed elsewhere without updating ipos, e.g. by undo;
+		// in that case we position at the end
+		guint pos = (tc->ipos >= sp_text_get_length (SP_TEXT(tc->text)))? sp_text_get_length (SP_TEXT(tc->text)) : tc->ipos;
+
+		sp_text_get_cursor_coords (SP_TEXT (tc->text), pos, p0, p1);
 		NR::Point const d0 = p0 * sp_item_i2d_affine(SP_ITEM(tc->text));
 		NR::Point const d1 = p1 * sp_item_i2d_affine(SP_ITEM(tc->text));
 
@@ -658,7 +801,7 @@ sp_text_context_forget_text (SPTextContext *tc)
 	/* We have to set it to zero,
 	 * or selection changed signal messes everything up */
 	tc->text = NULL;
-	if (sp_text_is_empty(SP_TEXT (ti))) {
+	if (SP_IS_TEXT(ti) && sp_text_is_empty(SP_TEXT (ti))) {
 		SPRepr *text_repr=SP_OBJECT_REPR(ti);
 		// the repr may already have been unparented
 		// if we were called e.g. as the result of
@@ -684,10 +827,13 @@ sptc_focus_out (GtkWidget *widget, GdkEventFocus *event, SPTextContext *tc)
 	return FALSE;
 }
 
-void
+static void
 sptc_commit (GtkIMContext *imc, gchar *string, SPTextContext *tc)
 {
-	if (!tc->text) sp_text_context_setup_text (tc);
+	if (!tc->text) {
+		sp_text_context_setup_text (tc);                                                 
+		tc->nascent_object = 0; // we don't need it anymore, having created a real <text>
+	}
 
 	if (!tc->preedit_string ) sp_text_context_preedit_reset (tc);
 
