@@ -17,6 +17,7 @@
 #include <ctype.h>
 #include "document.h"
 #include "sp-object.h"
+#include "uri.h"
 #include "uri-references.h"
 
 static gchar *uri_to_id(SPDocument *document, const gchar *uri);
@@ -36,13 +37,8 @@ URIReference::~URIReference() {
 
 void URIReference::attach(const URI &uri) throw(BadURIException)
 {
-	SPDocument *document;
-	const gchar *fragment;
-	gchar *id;
-
-	document = SP_OBJECT_DOCUMENT(_owner);
-
-	fragment = uri.getFragment();
+	SPDocument *document = SP_OBJECT_DOCUMENT(_owner);
+	gchar const *fragment = uri.getFragment();
 	if ( !uri.isRelative() || uri.getQuery() || !fragment ) {
 		throw UnsupportedURIException();
 	}
@@ -51,11 +47,15 @@ void URIReference::attach(const URI &uri) throw(BadURIException)
 	/* for now this handles the minimal xpointer form that SVG 1.0
 	 * requires of us
 	 */
+	gchar *id;
 	if (!strncmp(fragment, "xpointer(", 9)) {
 		/* FIXME !!! this is wasteful */
+		/* FIXME: It looks as though this is including "))" in the id.  I suggest moving
+		   the strlen calculation and validity testing to before strdup, and copying just
+		   the id without the "))".  -- pjrm */
 		if (!strncmp(fragment, "xpointer(id(", 12)) {
 			id = g_strdup(fragment+12);
-			int len=strlen(id);
+			size_t const len = strlen(id);
 			if ( len < 3 || strcmp(id+len-2, "))") ) {
 				g_free(id);
 				throw MalformedURIException();
@@ -75,8 +75,8 @@ void URIReference::attach(const URI &uri) throw(BadURIException)
 	_uri = new URI(uri);
 
 	_connection.disconnect();
-	_setObject(sp_document_lookup_id(document, id));
-	_connection = sp_document_id_changed_connect(document, id, SigC::slot(*this, &URIReference::_setObject));
+	_setObject(document->getObjectById(id));
+	_connection = sp_document_id_changed_connect(document, id, sigc::mem_fun(*this, &URIReference::_setObject));
 
 	g_free(id);
 }
@@ -162,7 +162,7 @@ sp_uri_reference_resolve (SPDocument *document, const gchar *uri)
 	if (!id) return NULL;
 
 	SPObject *ref;
-	ref = sp_document_lookup_id (document, id);
+	ref = document->getObjectById(id);
 	g_free(id);
 	return ref;
 }
