@@ -12,9 +12,6 @@
  * This code is in public domain
  */
 
-typedef struct _SPButton SPButton;
-typedef struct _SPButtonClass SPButtonClass;
-
 #define SP_TYPE_BUTTON (sp_button_get_type ())
 #define SP_BUTTON(o) (GTK_CHECK_CAST ((o), SP_TYPE_BUTTON, SPButton))
 #define SP_IS_BUTTON(o) (GTK_CHECK_TYPE ((o), SP_TYPE_BUTTON))
@@ -34,19 +31,20 @@ typedef enum {
 	SP_BUTTON_TYPE_TOGGLE
 } SPButtonType;
 
-struct _SPBChoiceData {
+struct SPBChoiceData {
 	guchar *px;
 };
 
-struct _SPButton {
+struct SPButton {
 	GtkToggleButton widget;
 	SPButtonType type;
 	unsigned int size;
 	SPAction *action;
+	SPAction *doubleclick_action;
 	GtkTooltips *tooltips;
 };
 
-struct _SPButtonClass {
+struct SPButtonClass {
 	GtkToggleButtonClass parent_class;
 };
 
@@ -54,7 +52,7 @@ struct _SPButtonClass {
 
 GType sp_button_get_type (void);
 
-GtkWidget *sp_button_new (unsigned int size, SPButtonType type, SPAction *action, GtkTooltips *tooltips);
+GtkWidget *sp_button_new (unsigned int size, SPButtonType type, SPAction *action, SPAction *doubleclick_action, GtkTooltips *tooltips);
 
 void sp_button_toggle_set_down (SPButton *button, gboolean down);
 
