@@ -9,8 +9,10 @@
  * Copyright (C) 2001-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
  * Copyright (C) 2004 Monash University
+ * Copyright (C) 2004 David Turner
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
+ *
  */
 
 #include <config.h>
@@ -32,7 +34,6 @@
 #include "../widgets/gradient-image.h"
 #include "../inkscape.h"
 #include "../document-private.h"
-#include "../sp-gradient.h"
 #include "../gradient-chemistry.h"
 #include "gradient-vector.h"
 #include "../helper/window.h"
@@ -42,6 +43,8 @@
 #include "../verbs.h"
 #include "../interface.h"
 #include "../svg/stringstream.h"
+#include "sp-gradient-vector.h"
+#include "sp-stop.h"
 
 enum {
 	VECTOR_SET,
@@ -177,7 +180,7 @@ sp_gradient_vector_selector_set_gradient (SPGradientVectorSelector *gvs, SPDocum
 		/* Disconnect signals */
 		if (gvs->gr) {
   			sp_signal_disconnect_by_data (gvs->gr, gvs);
-		g_signal_handlers_disconnect_matched (G_OBJECT(gvs->gr), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gvs);
+			g_signal_handlers_disconnect_matched (G_OBJECT(gvs->gr), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gvs);
 			gvs->gr = NULL;
 		}
 		if (gvs->doc) {
@@ -229,25 +232,20 @@ sp_gradient_vector_selector_get_gradient (SPGradientVectorSelector *gvs)
 static void
 sp_gvs_rebuild_gui_full (SPGradientVectorSelector *gvs)
 {
-	GtkWidget *m;
-	GSList *gl;
-	gint pos, idx;
-
 	/* Clear old menu, if there is any */
 	if (gtk_option_menu_get_menu (GTK_OPTION_MENU (gvs->menu))) {
 		gtk_option_menu_remove_menu (GTK_OPTION_MENU (gvs->menu));
 	}
 
 	/* Create new menu widget */
-	m = gtk_menu_new ();
+	GtkWidget *m = gtk_menu_new ();
 	gtk_widget_show (m);
 
 	/* Pick up all gradients with vectors */
-	gl = NULL;
+	GSList *gl = NULL;
 	if (gvs->gr) {
-		const GSList *gradients, *l;
-		gradients = sp_document_get_resource_list (SP_OBJECT_DOCUMENT (gvs->gr), "gradient");
-		for (l = gradients; l != NULL; l = l->next) {
+		const GSList *gradients = sp_document_get_resource_list (SP_OBJECT_DOCUMENT (gvs->gr), "gradient");
+		for (const GSList *l = gradients; l != NULL; l = l->next) {
 			if (SP_GRADIENT_HAS_STOPS (l->data)) {
 				gl = g_slist_prepend (gl, l->data);
 			}
@@ -255,7 +253,8 @@ sp_gvs_rebuild_gui_full (SPGradientVectorSelector *gvs)
 	}
 	gl = g_slist_reverse (gl);
 
-	pos = idx = 0;
+	gint pos = 0;
+	gint idx = 0;
 
 	if (!gvs->doc) {
 		GtkWidget *i;
@@ -330,11 +329,11 @@ sp_gvs_gradient_activate (GtkMenuItem *mi, SPGradientVectorSelector *gvs)
 	/* Namely - menuitems etc. will be fucked up */
 	/* Hmmm - probably we can just re-set it as menuitem data (Lauris) */
 
-	g_print ("SPGradientVectorSelector: gradient %s activated\n", SP_OBJECT_ID (gr));
+	//g_print ("SPGradientVectorSelector: gradient %s activated\n", SP_OBJECT_ID (gr));
 
 	norm = sp_gradient_ensure_vector_normalized (gr);
 	if (norm != gr) {
-		g_print ("SPGradientVectorSelector: become %s after normalization\n", SP_OBJECT_ID (norm));
+		//g_print ("SPGradientVectorSelector: become %s after normalization\n", SP_OBJECT_ID (norm));
 		/* But be careful that we do not have gradient saved anywhere else */
 		g_object_set_data (G_OBJECT (mi), "gradient", norm);
 	}
@@ -365,9 +364,14 @@ sp_gvs_gradient_activate (GtkMenuItem *mi, SPGradientVectorSelector *gvs)
 static void
 sp_gvs_gradient_release (SPGradient *gr, SPGradientVectorSelector *gvs)
 {
-	/* fixme: (Lauris) */
-	/* fixme: Not sure, what to do here (Lauris) */
-	gvs->gr = NULL;
+	/* Disconnect gradient */
+	if (gvs->gr) {
+  		sp_signal_disconnect_by_data (gvs->gr, gvs);
+		gvs->gr = NULL;
+	}
+
+	/* Rebuild GUI */
+	sp_gvs_rebuild_gui_full (gvs);
 }
 
 static void
@@ -387,9 +391,9 @@ sp_gvs_defs_release (SPObject *defs, SPGradientVectorSelector *gvs)
 static void
 sp_gvs_defs_modified (SPObject *defs, guint flags, SPGradientVectorSelector *gvs)
 {
-	/* fixme: (Lauris) */
 	/* fixme: We probably have to check some flags here (Lauris) */
-	/* fixme: Not exactly sure, what we have to do here (Lauris) */
+
+	sp_gvs_rebuild_gui_full (gvs);
 }
 
 /*##################################################################
@@ -403,6 +407,7 @@ sp_gvs_defs_modified (SPObject *defs, guint flags, SPGradientVectorSelector *gvs
 #include "../desktop-handles.h"
 #include "../selection.h"
 #include "../xml/repr-private.h"
+#include "../xml/sp-repr-event-vector.h"
 #include "../svg/svg.h"
 
 
@@ -433,7 +438,6 @@ static void grad_edit_dia_stopremoved (SPRepr *repr, SPRepr *child, SPRepr *ref,
 
 static SPReprEventVector grad_edit_dia_repr_events =
 {
-    NULL, /* destroy */
     NULL, /* add_child */
     NULL, /* child_added */
     NULL, /* remove_child */
@@ -451,12 +455,14 @@ verify_grad(SPGradient *gradient)
 {
 	int i = 0;
 	SPStop *stop = NULL;
+	/* count stops */
 	for ( SPObject *ochild = sp_object_first_child(SP_OBJECT(gradient)) ; ochild != NULL ; ochild = SP_OBJECT_NEXT(ochild) ) {
 		if (SP_IS_STOP (ochild)) {
 			i++;
 			stop = SP_STOP(ochild);
 		}
 	}
+
 	if (i < 1) {
 		gchar c[64];
 		sp_svg_write_color (c, 64, 0x00000000);
@@ -607,12 +613,12 @@ sp_grad_edit_select (GtkOptionMenu *mnu,  GtkWidget *tbl)
 	GtkAdjustment *adj = (GtkAdjustment*)gtk_object_get_data (GTK_OBJECT (tbl), "offset");
 	SPStop *prev = NULL;
 	prev = sp_prev_stop(stop, gradient);
-	if (prev != NULL )  adj->lower = prev->offset+.01;
+	if (prev != NULL )  adj->lower = prev->offset;
 	else adj->lower = 0;
 
 	SPStop *next = NULL;
 	next = sp_next_stop(stop);
-	if (next != NULL )  adj->upper = next->offset-.01;
+	if (next != NULL )  adj->upper = next->offset;
 	else adj->upper = 1.0;
 
 	sp_repr_set_double (SP_OBJECT_REPR (stop), "offset", stop->offset);
@@ -682,7 +688,7 @@ sp_grd_ed_add_stop (GtkWidget *widget,  GtkWidget *vb)
 		sp_repr_add_child (SP_OBJECT_REPR(gradient), new_stop_repr, SP_OBJECT_REPR(sp_prev_stop(stop, gradient)));
 	}
 
-	SPStop *newstop = (SPStop *) sp_document_lookup_id (SP_OBJECT_DOCUMENT(gradient), sp_repr_attr (new_stop_repr, "id"));
+	SPStop *newstop = (SPStop *) SP_OBJECT_DOCUMENT(gradient)->getObjectByRepr(new_stop_repr);
    
 	newstop->offset = (stop->offset + next->offset) * 0.5 ;
 
@@ -754,6 +760,7 @@ sp_gradient_vector_widget_new (SPGradient *gradient)
 
 	/* Add and Remove buttons */
 	GtkWidget *hb = gtk_hbox_new (FALSE, 1);
+	// TRANSLATORS: "Stop" means: a "phase" of a gradient
 	GtkWidget *b = gtk_button_new_with_label (_("Add stop"));
 	gtk_widget_show (b);
 	gtk_container_add (GTK_CONTAINER (hb), b);
@@ -817,6 +824,7 @@ sp_gradient_vector_widget_new (SPGradient *gradient)
 	gtk_widget_show (hb);
 	gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, PAD);
 
+	// TRANSLATORS: "Stop" means: a "phase" of a gradient
 	f = gtk_frame_new (_("Stop Color"));
 	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, TRUE, TRUE, PAD);
@@ -951,6 +959,9 @@ sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient
 
 	GtkWidget *mnu = static_cast<GtkWidget *>(g_object_get_data(G_OBJECT(widget), "stopmenu"));
 	update_stop_list (GTK_WIDGET(mnu), gradient, NULL);
+
+	// Once the user edits a gradient, it stops being auto-collectable
+	sp_repr_set_attr (SP_OBJECT_REPR(gradient), "inkscape:collect", NULL);
 }
 
 static void
