@@ -2,13 +2,7 @@
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkcolorsel.h>
+#include <gtk/gtk.h>
 #include "../color.h"
 #include "../helper/sp-intl.h"
 #include "../dialogs/dialog-events.h"
@@ -144,6 +138,8 @@ void ColorScales::init()
 	_updating = FALSE;
 	_dragging = FALSE;
 
+	_tt = gtk_tooltips_new();
+
 	t = gtk_table_new (5, 3, FALSE);
 	gtk_widget_show (t);
 	gtk_box_pack_start (GTK_BOX (_csel), t, TRUE, TRUE, 0);
@@ -151,20 +147,21 @@ void ColorScales::init()
 	/* Create components */
 	for (i = 0; i < static_cast< gint > (G_N_ELEMENTS(_a)) ; i++) {
 		/* Label */
-		_l[i] = gtk_label_new ("");
+		_l[i] = gtk_label_new("");
 		gtk_misc_set_alignment (GTK_MISC (_l[i]), 1.0, 0.5);
 		gtk_widget_show (_l[i]);
 		gtk_table_attach (GTK_TABLE (t), _l[i], 0, 1, i, i + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
 		/* Adjustment */
-		_a[i] = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 1.0, 0.01, 0.1, 0.1);
+		_a[i] = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 255.0, 1.0, 10.0, 10.0);
 		/* Slider */
 		_s[i] = sp_color_slider_new (_a[i]);
 		gtk_widget_show (_s[i]);
 		gtk_table_attach (GTK_TABLE (t), _s[i], 1, 2, i, i + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)GTK_FILL, XPAD, YPAD);
 
 		/* Spinbutton */
-		_b[i] = gtk_spin_button_new (GTK_ADJUSTMENT (_a[i]), 0.01, 2);
+		_b[i] = gtk_spin_button_new (GTK_ADJUSTMENT (_a[i]), 1.0, 0);
 		sp_dialog_defocus_on_enter (_b[i]);
+		gtk_label_set_mnemonic_widget (GTK_LABEL(_l[i]), _b[i]);
 		gtk_widget_show (_b[i]);
 		gtk_table_attach (GTK_TABLE (t), _b[i], 2, 3, i, i + 1, (GtkAttachOptions)0, (GtkAttachOptions)0, XPAD, YPAD);
 
@@ -247,6 +244,15 @@ void ColorScales::_recalcColor( gboolean changing )
 }
 
 /* Helpers for setting color value */
+gfloat get1( const GtkAdjustment *a )
+{
+	return (a->value)/255;
+}
+
+void set255( GtkAdjustment *a, gfloat v)
+{
+	gtk_adjustment_set_value  (a, v*255.0);
+}
 
 void ColorScales::_colorChanged( const SPColor& color, gfloat alpha )
 {
@@ -261,7 +267,7 @@ void ColorScales::_colorChanged( const SPColor& color, gfloat alpha )
 		break;
 	case SP_COLOR_SCALES_MODE_HSV:
 		sp_color_get_rgb_floatv (&color, tmp);
-		c[0] = _a[0]->value;
+		c[0] = get1 (_a[0]);
 		sp_color_rgb_to_hsv_floatv (c, tmp[0], tmp[1], tmp[2]);
 		c[3] = alpha;
 		c[4] = 0.0;
@@ -276,11 +282,11 @@ void ColorScales::_colorChanged( const SPColor& color, gfloat alpha )
 	}
 
 	_updating = TRUE;
-	gtk_adjustment_set_value (_a[0], c[0]);
-	gtk_adjustment_set_value (_a[1], c[1]);
-	gtk_adjustment_set_value (_a[2], c[2]);
-	gtk_adjustment_set_value (_a[3], c[3]);
-	gtk_adjustment_set_value (_a[4], c[4]);
+	set255 (_a[0], c[0]);
+	set255 (_a[1], c[1]);
+	set255 (_a[2], c[2]);
+	set255 (_a[3], c[3]);
+	set255 (_a[4], c[4]);
 	_updateSliders( CSC_CHANNELS_ALL );
 	_updating = FALSE;
 }
@@ -291,18 +297,18 @@ void ColorScales::_getRgbaFloatv( gfloat *rgba )
 
 	switch (_mode) {
 	case SP_COLOR_SCALES_MODE_RGB:
-		rgba[0] = _a[0]->value;
-		rgba[1] = _a[1]->value;
-		rgba[2] = _a[2]->value;
-		rgba[3] = _a[3]->value;
+		rgba[0] = get1(_a[0]);
+		rgba[1] = get1(_a[1]);
+		rgba[2] = get1(_a[2]);
+		rgba[3] = get1(_a[3]);
 		break;
 	case SP_COLOR_SCALES_MODE_HSV:
-		sp_color_hsv_to_rgb_floatv (rgba, _a[0]->value, _a[1]->value, _a[2]->value);
-		rgba[3] = _a[3]->value;
+		sp_color_hsv_to_rgb_floatv (rgba, get1(_a[0]), get1(_a[1]), get1(_a[2]));
+		rgba[3] = get1(_a[3]);
 		break;
 	case SP_COLOR_SCALES_MODE_CMYK:
-		sp_color_cmyk_to_rgb_floatv (rgba, _a[0]->value, _a[1]->value, _a[2]->value, _a[3]->value);
-		rgba[3] = _a[4]->value;
+		sp_color_cmyk_to_rgb_floatv (rgba, get1(_a[0]), get1(_a[1]), get1(_a[2]), get1(_a[3]));
+		rgba[3] = get1(_a[4]);
 		break;
 	default:
 		g_warning ("file %s: line %d: Illegal color selector mode", __FILE__, __LINE__);
@@ -318,20 +324,20 @@ void ColorScales::_getCmykaFloatv( gfloat *cmyka )
 
 	switch (_mode) {
 	case SP_COLOR_SCALES_MODE_RGB:
-		sp_color_rgb_to_cmyk_floatv (cmyka, _a[0]->value, _a[1]->value, _a[2]->value);
-		cmyka[4] = _a[3]->value;
+		sp_color_rgb_to_cmyk_floatv (cmyka, get1(_a[0]), get1(_a[1]), get1(_a[2]));
+		cmyka[4] = get1(_a[3]);
 		break;
 	case SP_COLOR_SCALES_MODE_HSV:
-		sp_color_hsv_to_rgb_floatv (rgb, _a[0]->value, _a[1]->value, _a[2]->value);
+		sp_color_hsv_to_rgb_floatv (rgb, get1(_a[0]), get1(_a[1]), get1(_a[2]));
 		sp_color_rgb_to_cmyk_floatv (cmyka, rgb[0], rgb[1], rgb[2]);
-		cmyka[4] = _a[3]->value;
+		cmyka[4] = get1(_a[3]);
 		break;
 	case SP_COLOR_SCALES_MODE_CMYK:
-		cmyka[0] = _a[0]->value;
-		cmyka[1] = _a[1]->value;
-		cmyka[2] = _a[2]->value;
-		cmyka[3] = _a[3]->value;
-		cmyka[4] = _a[4]->value;
+		cmyka[0] = get1(_a[0]);
+		cmyka[1] = get1(_a[1]);
+		cmyka[2] = get1(_a[2]);
+		cmyka[3] = get1(_a[3]);
+		cmyka[4] = get1(_a[4]);
 		break;
 	default:
 		g_warning ("file %s: line %d: Illegal color selector mode", __FILE__, __LINE__);
@@ -370,27 +376,43 @@ void ColorScales::setMode(SPColorScalesMode mode)
 
 	switch (mode) {
 	case SP_COLOR_SCALES_MODE_RGB:
-		gtk_label_set_text (GTK_LABEL (_l[0]), _("Red:"));
-		gtk_label_set_text (GTK_LABEL (_l[1]), _("Green:"));
-		gtk_label_set_text (GTK_LABEL (_l[2]), _("Blue:"));
-		gtk_label_set_text (GTK_LABEL (_l[3]), _("Alpha:"));
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[0]), _("_R"));
+		gtk_tooltips_set_tip (_tt, _s[0], _("Red"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[0], _("Red"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[1]), _("_G"));
+		gtk_tooltips_set_tip (_tt, _s[1], _("Green"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[1], _("Green"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[2]), _("_B"));
+		gtk_tooltips_set_tip (_tt, _s[2], _("Blue"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[2], _("Blue"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[3]), _("_A"));
+		gtk_tooltips_set_tip (_tt, _s[3], _("Alpha (opacity)"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[3], _("Alpha (opacity)"), NULL);
 		sp_color_slider_set_map (SP_COLOR_SLIDER (_s[0]), NULL);
 		gtk_widget_hide (_l[4]);
 		gtk_widget_hide (_s[4]);
 		gtk_widget_hide (_b[4]);
 		_updating = TRUE;
-		gtk_adjustment_set_value (_a[0], rgba[0]);
-		gtk_adjustment_set_value (_a[1], rgba[1]);
-		gtk_adjustment_set_value (_a[2], rgba[2]);
-		gtk_adjustment_set_value (_a[3], rgba[3]);
+		set255 (_a[0], rgba[0]);
+		set255 (_a[1], rgba[1]);
+		set255 (_a[2], rgba[2]);
+		set255 (_a[3], rgba[3]);
 		_updating = FALSE;
 		_updateSliders( CSC_CHANNELS_ALL );
 		break;
 	case SP_COLOR_SCALES_MODE_HSV:
-		gtk_label_set_text (GTK_LABEL (_l[0]), _("Hue:"));
-		gtk_label_set_text (GTK_LABEL (_l[1]), _("Saturation:"));
-		gtk_label_set_text (GTK_LABEL (_l[2]), _("Value:"));
-		gtk_label_set_text (GTK_LABEL (_l[3]), _("Alpha:"));
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[0]), _("_H"));
+		gtk_tooltips_set_tip (_tt, _s[0], _("Hue"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[0], _("Hue"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[1]), _("_S"));
+		gtk_tooltips_set_tip (_tt, _s[1], _("Saturation"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[1], _("Saturation"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[2]), _("_V"));
+		gtk_tooltips_set_tip (_tt, _s[2], _("Value (brightness)"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[2], _("Value (brightness)"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[3]), _("_A"));
+		gtk_tooltips_set_tip (_tt, _s[3], _("Alpha (opacity)"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[3], _("Alpha (opacity)"), NULL);
 		sp_color_slider_set_map (SP_COLOR_SLIDER (_s[0]), (guchar*)sp_color_scales_hue_map ());
 		gtk_widget_hide (_l[4]);
 		gtk_widget_hide (_s[4]);
@@ -398,30 +420,40 @@ void ColorScales::setMode(SPColorScalesMode mode)
 		_updating = TRUE;
 		c[0] = 0.0;
 		sp_color_rgb_to_hsv_floatv (c, rgba[0], rgba[1], rgba[2]);
-		gtk_adjustment_set_value (_a[0], c[0]);
-		gtk_adjustment_set_value (_a[1], c[1]);
-		gtk_adjustment_set_value (_a[2], c[2]);
-		gtk_adjustment_set_value (_a[3], rgba[3]);
+		set255 (_a[0], c[0]);
+		set255 (_a[1], c[1]);
+		set255 (_a[2], c[2]);
+		set255 (_a[3], rgba[3]);
 		_updating = FALSE;
 		_updateSliders( CSC_CHANNELS_ALL );
 		break;
 	case SP_COLOR_SCALES_MODE_CMYK:
-		gtk_label_set_text (GTK_LABEL (_l[0]), _("Cyan:"));
-		gtk_label_set_text (GTK_LABEL (_l[1]), _("Magenta:"));
-		gtk_label_set_text (GTK_LABEL (_l[2]), _("Yellow:"));
-		gtk_label_set_text (GTK_LABEL (_l[3]), _("Black:"));
-		gtk_label_set_text (GTK_LABEL (_l[4]), _("Alpha:"));
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[0]), _("_C"));
+		gtk_tooltips_set_tip (_tt, _s[0], _("Cyan"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[0], _("Cyan"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[1]), _("_M"));
+		gtk_tooltips_set_tip (_tt, _s[1], _("Magenta"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[1], _("Magenta"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[2]), _("_Y"));
+		gtk_tooltips_set_tip (_tt, _s[2], _("Yellow"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[2], _("Yellow"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[3]), _("_K"));
+		gtk_tooltips_set_tip (_tt, _s[3], _("Black"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[3], _("Black"), NULL);
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (_l[4]), _("_A"));
+		gtk_tooltips_set_tip (_tt, _s[4], _("Alpha (opacity)"), NULL);
+		gtk_tooltips_set_tip (_tt, _b[4], _("Alpha (opacity)"), NULL);
 		sp_color_slider_set_map (SP_COLOR_SLIDER (_s[0]), NULL);
 		gtk_widget_show (_l[4]);
 		gtk_widget_show (_s[4]);
 		gtk_widget_show (_b[4]);
 		_updating = TRUE;
 		sp_color_rgb_to_cmyk_floatv (c, rgba[0], rgba[1], rgba[2]);
-		gtk_adjustment_set_value (_a[0], c[0]);
-		gtk_adjustment_set_value (_a[1], c[1]);
-		gtk_adjustment_set_value (_a[2], c[2]);
-		gtk_adjustment_set_value (_a[3], c[3]);
-		gtk_adjustment_set_value (_a[4], rgba[3]);
+		set255 (_a[0], c[0]);
+		set255 (_a[1], c[1]);
+		set255 (_a[2], c[2]);
+		set255 (_a[3], c[3]);
+		set255 (_a[4], rgba[3]);
 		_updating = FALSE;
 		_updateSliders( CSC_CHANNELS_ALL );
 		break;
@@ -479,9 +511,12 @@ guint ColorScales::getSubmode() const
 
 void ColorScales::_adjustmentAnyChanged( GtkAdjustment *adjustment, SPColorScales *cs )
 {
-	gint channel;
+	// if a value is entered between 0 and 1 exclusive, normalize it to (int) 0..255 
+	if (adjustment->value > 0.0 && adjustment->value < 1.0) {
+		gtk_adjustment_set_value  (adjustment, floor ((adjustment->value)*255.0 + 0.5));
+	} 
 
-	channel = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (adjustment), "channel"));
+	gint channel = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (adjustment), "channel"));
 
 	_adjustmentChanged(cs, channel);
 }
@@ -540,49 +575,49 @@ void ColorScales::_updateSliders( guint channels )
 		if ((channels != CSC_CHANNEL_R) && (channels != CSC_CHANNEL_A)) {
 			/* Update red */
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[0]),
-							SP_RGBA32_F_COMPOSE (0.0, _a[1]->value, _a[2]->value, 1.0),
-							SP_RGBA32_F_COMPOSE (1.0, _a[1]->value, _a[2]->value, 1.0));
+							SP_RGBA32_F_COMPOSE (0.0, get1(_a[1]), get1(_a[2]), 1.0),
+							SP_RGBA32_F_COMPOSE (1.0, get1(_a[1]), get1(_a[2]), 1.0));
 		}
 		if ((channels != CSC_CHANNEL_G) && (channels != CSC_CHANNEL_A)) {
 			/* Update green */
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[1]),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, 0.0, _a[2]->value, 1.0),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, 1.0, _a[2]->value, 1.0));
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), 0.0, get1(_a[2]), 1.0),
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), 1.0, get1(_a[2]), 1.0));
 		}
 		if ((channels != CSC_CHANNEL_B) && (channels != CSC_CHANNEL_A)) {
 			/* Update blue */
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[2]),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, _a[1]->value, 0.0, 1.0),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, _a[1]->value, 1.0, 1.0));
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), get1(_a[1]), 0.0, 1.0),
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), get1(_a[1]), 1.0, 1.0));
 		}
 		if (channels != CSC_CHANNEL_A) {
 			/* Update alpha */
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[3]),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, _a[1]->value, _a[2]->value, 0.0),
-							SP_RGBA32_F_COMPOSE (_a[0]->value, _a[1]->value, _a[2]->value, 1.0));
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), get1(_a[1]), get1(_a[2]), 0.0),
+							SP_RGBA32_F_COMPOSE (get1(_a[0]), get1(_a[1]), get1(_a[2]), 1.0));
 		}
 		break;
 	case SP_COLOR_SCALES_MODE_HSV:
 		/* Hue is never updated */
 		if ((channels != CSC_CHANNEL_S) && (channels != CSC_CHANNEL_A)) {
 			/* Update saturation */
-			sp_color_hsv_to_rgb_floatv (rgb0, _a[0]->value, 0.0, _a[2]->value);
-			sp_color_hsv_to_rgb_floatv (rgb1, _a[0]->value, 1.0, _a[2]->value);
+			sp_color_hsv_to_rgb_floatv (rgb0, get1(_a[0]), 0.0, get1(_a[2]));
+			sp_color_hsv_to_rgb_floatv (rgb1, get1(_a[0]), 1.0, get1(_a[2]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[1]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if ((channels != CSC_CHANNEL_V) && (channels != CSC_CHANNEL_A)) {
 			/* Update value */
-			sp_color_hsv_to_rgb_floatv (rgb0, _a[0]->value, _a[1]->value, 0.0);
-			sp_color_hsv_to_rgb_floatv (rgb1, _a[0]->value, _a[1]->value, 1.0);
+			sp_color_hsv_to_rgb_floatv (rgb0, get1(_a[0]), get1(_a[1]), 0.0);
+			sp_color_hsv_to_rgb_floatv (rgb1, get1(_a[0]), get1(_a[1]), 1.0);
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[2]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if (channels != CSC_CHANNEL_A) {
 			/* Update alpha */
-			sp_color_hsv_to_rgb_floatv (rgb0, _a[0]->value, _a[1]->value, _a[2]->value);
+			sp_color_hsv_to_rgb_floatv (rgb0, get1(_a[0]), get1(_a[1]), get1(_a[2]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[3]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 0.0),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0));
@@ -591,39 +626,39 @@ void ColorScales::_updateSliders( guint channels )
 	case SP_COLOR_SCALES_MODE_CMYK:
 		if ((channels != CSC_CHANNEL_C) && (channels != CSC_CHANNEL_CMYKA)) {
 			/* Update saturation */
-			sp_color_cmyk_to_rgb_floatv (rgb0, 0.0, _a[1]->value, _a[2]->value, _a[3]->value);
-			sp_color_cmyk_to_rgb_floatv (rgb1, 1.0, _a[1]->value, _a[2]->value, _a[3]->value);
+			sp_color_cmyk_to_rgb_floatv (rgb0, 0.0, get1(_a[1]), get1(_a[2]), get1(_a[3]));
+			sp_color_cmyk_to_rgb_floatv (rgb1, 1.0, get1(_a[1]), get1(_a[2]), get1(_a[3]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[0]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if ((channels != CSC_CHANNEL_M) && (channels != CSC_CHANNEL_CMYKA)) {
 			/* Update saturation */
-			sp_color_cmyk_to_rgb_floatv (rgb0, _a[0]->value, 0.0, _a[2]->value, _a[3]->value);
-			sp_color_cmyk_to_rgb_floatv (rgb1, _a[0]->value, 1.0, _a[2]->value, _a[3]->value);
+			sp_color_cmyk_to_rgb_floatv (rgb0, get1(_a[0]), 0.0, get1(_a[2]), get1(_a[3]));
+			sp_color_cmyk_to_rgb_floatv (rgb1, get1(_a[0]), 1.0, get1(_a[2]), get1(_a[3]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[1]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if ((channels != CSC_CHANNEL_Y) && (channels != CSC_CHANNEL_CMYKA)) {
 			/* Update saturation */
-			sp_color_cmyk_to_rgb_floatv (rgb0, _a[0]->value, _a[1]->value, 0.0, _a[3]->value);
-			sp_color_cmyk_to_rgb_floatv (rgb1, _a[0]->value, _a[1]->value, 1.0, _a[3]->value);
+			sp_color_cmyk_to_rgb_floatv (rgb0, get1(_a[0]), get1(_a[1]), 0.0, get1(_a[3]));
+			sp_color_cmyk_to_rgb_floatv (rgb1, get1(_a[0]), get1(_a[1]), 1.0, get1(_a[3]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[2]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if ((channels != CSC_CHANNEL_K) && (channels != CSC_CHANNEL_CMYKA)) {
 			/* Update saturation */
-			sp_color_cmyk_to_rgb_floatv (rgb0, _a[0]->value, _a[1]->value, _a[2]->value, 0.0);
-			sp_color_cmyk_to_rgb_floatv (rgb1, _a[0]->value, _a[1]->value, _a[2]->value, 1.0);
+			sp_color_cmyk_to_rgb_floatv (rgb0, get1(_a[0]), get1(_a[1]), get1(_a[2]), 0.0);
+			sp_color_cmyk_to_rgb_floatv (rgb1, get1(_a[0]), get1(_a[1]), get1(_a[2]), 1.0);
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[3]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0),
 							SP_RGBA32_F_COMPOSE (rgb1[0], rgb1[1], rgb1[2], 1.0));
 		}
 		if (channels != CSC_CHANNEL_CMYKA) {
 			/* Update saturation */
-			sp_color_cmyk_to_rgb_floatv (rgb0, _a[0]->value, _a[1]->value, _a[2]->value, _a[3]->value);
+			sp_color_cmyk_to_rgb_floatv (rgb0, get1(_a[0]), get1(_a[1]), get1(_a[2]), get1(_a[3]));
 			sp_color_slider_set_colors (SP_COLOR_SLIDER (_s[4]),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 0.0),
 							SP_RGBA32_F_COMPOSE (rgb0[0], rgb0[1], rgb0[2], 1.0));
