@@ -1,18 +1,19 @@
 #ifndef __SP_COLOR_WHEEL_SELECTOR_H__
 #define __SP_COLOR_WHEEL_SELECTOR_H__
 
+#include <glib/gtypes.h>
+#include <gtk/gtktooltips.h>
 #include <gtk/gtkvbox.h>
+
 #include "../color.h"
 #include "sp-color-wheel.h"
 #include "sp-color-slider.h"
 #include "sp-color-selector.h"
 
-#include <glib.h>
 
 
-
-typedef struct _SPColorWheelSelector SPColorWheelSelector;
-typedef struct _SPColorWheelSelectorClass SPColorWheelSelectorClass;
+struct SPColorWheelSelector;
+struct SPColorWheelSelectorClass;
 
 
 class ColorWheelSelector: public ColorSelector
@@ -45,6 +46,8 @@ protected:
     GtkWidget* _sbtn; /* Spinbutton */
     GtkWidget* _label; /* Label */
 
+    GtkTooltips* _tt; /* tooltip container */
+
 private:
     // By default, disallow copy constructor and assignment operator
     ColorWheelSelector( const ColorWheelSelector& obj );
@@ -65,11 +68,11 @@ namespace Inkscape {
     }
 }
 
-struct _SPColorWheelSelector {
+struct SPColorWheelSelector {
     SPColorSelector parent;
 };
 
-struct _SPColorWheelSelectorClass {
+struct SPColorWheelSelectorClass {
     SPColorSelectorClass parent_class;
 };
 
