@@ -14,7 +14,7 @@
 
 #include <gtk/gtksignal.h>
 #include "macros.h"
-#include "../xml/repr-private.h"
+#include "../xml/sp-repr-event-vector.h"
 #include "../inkscape.h"
 #include "../desktop.h"
 #include "../desktop-handles.h"
@@ -50,12 +50,6 @@ static void sp_widget_set_selection (Inkscape::Application *inkscape, SPSelectio
 static GtkBinClass *parent_class;
 static guint signals[LAST_SIGNAL] = {0};
 
-static void
-spw_repr_destroy (SPRepr *repr, gpointer data)
-{
-	g_warning ("Oops! Repr destroyed while SPWidget still present");
-}
-
 static void
 spw_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data)
 {
@@ -69,7 +63,6 @@ spw_repr_content_changed (SPRepr *repr, const gchar *oldval, const gchar *newval
 }
 
 SPReprEventVector spw_event_vector = {
-	spw_repr_destroy,
 	NULL, /* Add child */
 	NULL, /* Child added */
 	NULL, /* Remove child */
@@ -403,7 +396,7 @@ sp_widget_get_item_list (SPWidget *spw)
 	g_return_val_if_fail (SP_IS_WIDGET (spw), NULL);
 
 	if (spw->inkscape) {
-		return sp_selection_item_list (SP_DT_SELECTION (SP_ACTIVE_DESKTOP));
+		return SP_DT_SELECTION(SP_ACTIVE_DESKTOP)->itemList();
 	}
 
 	return NULL;
