@@ -11,6 +11,9 @@
  * Released under the GNU GPL; see COPYING for details
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <string.h>
 #include <glib.h>
 #include <gtk/gtkmain.h>
@@ -18,9 +21,10 @@
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtklist.h>
 #include <gtk/gtkadjustment.h>
+#include "helper/sp-marshal.h"
 #include "helper/sp-intl.h"
 #include "../xml/repr.h"
-#include "../xml/repr-private.h"
+#include "../xml/sp-repr-event-vector.h"
 #include "sp-xmlview-attr-list.h"
 
 static void sp_xmlview_attr_list_class_init (SPXMLViewAttrListClass * klass);
@@ -32,7 +36,6 @@ static void event_attr_changed (SPRepr * repr, const gchar * name, const gchar *
 static GtkCListClass * parent_class = NULL;
 
 static SPReprEventVector repr_events = {
-	NULL, /* destroy */
 	NULL, /* add_child */
 	NULL, /* child_added */
 	NULL, /* remove_child */
@@ -115,6 +118,15 @@ sp_xmlview_attr_list_class_init (SPXMLViewAttrListClass * klass)
 	object_class->destroy = sp_xmlview_attr_list_destroy;
 
 	parent_class = (GtkCListClass*)gtk_type_class (GTK_TYPE_CLIST);
+
+        g_signal_new (  "row-value-changed",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			G_STRUCT_OFFSET (SPXMLViewAttrListClass, row_changed),
+			NULL, NULL,
+			sp_marshal_NONE__UINT,
+			G_TYPE_NONE, 1,
+			G_TYPE_UINT);
 }
 
 void
@@ -157,20 +169,16 @@ event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value,
 		gtktext = NULL;
 	}
 
-	if (old_value) {
-		row = gtk_clist_find_row_from_data (GTK_CLIST (list), GINT_TO_POINTER (g_quark_from_string (name)));
-		g_assert (row != -1);
-
+	row = gtk_clist_find_row_from_data (GTK_CLIST (list), GINT_TO_POINTER (g_quark_from_string (name)));
+	if (row != -1) {
 		if (new_value) {
 			gtk_clist_set_text (GTK_CLIST (list), row, 1, gtktext);
 		} else {
 			gtk_clist_remove (GTK_CLIST (list), row);
 		}
-	} else {
+	} else if (new_value != NULL) {
 		const gchar * text[2];
 
-		g_assert (new_value != NULL);
-
 		text[0] = name;
 		text[1] = gtktext;
 
@@ -179,5 +187,8 @@ event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value,
 	}
 
 	gtk_clist_thaw (GTK_CLIST (list));
+
+	// send a "changed" signal so widget owners will know I've updated
+	g_signal_emit_by_name(G_OBJECT (list), "row-value-changed", row );
 }
 
