@@ -1,13 +1,14 @@
 #define TOOL_BUTTON_SIZE 28
 
 #define AUX_BUTTON_SIZE 20
-#define AUX_BETWEEN_BUTTON_GROUPS 5
-#define AUX_BETWEEN_SPINBUTTONS 2
+#define AUX_SPACING 3
+#define AUX_BETWEEN_BUTTON_GROUPS 7
+#define AUX_BETWEEN_SPINBUTTONS 0
 #define AUX_FONT_SIZE 8000
 #define AUX_SPINBUTTON_WIDTH 57
-#define AUX_SPINBUTTON_HEIGHT 22
+#define AUX_SPINBUTTON_HEIGHT 20
 #define AUX_OPTION_MENU_WIDTH 55
-#define AUX_OPTION_MENU_HEIGHT 24
+#define AUX_OPTION_MENU_HEIGHT 20
 #define AUX_MENU_ITEM_WIDTH 32
 #define AUX_MENU_ITEM_HEIGHT 18
 
@@ -26,6 +27,8 @@
 #define STATUS_COORD_SKIP 1
 #define STATUS_COORD_WIDTH 84
 
+#define STATUS_LAYER_FONT_SIZE 7700
+
 #define RULER_WIDTH           12
 #define RULER_HEIGHT          12
 #define RULER_FONT_SIZE 7000
