@@ -1,8 +1,10 @@
 #define __SP_REPR_UTIL_C__
 
+/** \file
+ * Miscellaneous helpers for reprs.
+ */
+
 /*
- * Miscellaneous helpers for reprs
- *
  * Authors:
  *   Lauris Kaplinski <lauris@ximian.com>
  *
@@ -29,8 +31,9 @@
 
 #include <glib.h>
 
-#include "repr-private.h"
-#include "../svg/stringstream.h"
+#include <svg/stringstream.h>
+#include <xml/repr-private.h>
+#include <xml/sp-repr-attr.h>
 
 /*#####################
 # DEFINITIONS
@@ -70,7 +73,7 @@ sp_xml_dtoa (gchar *buf, double val, unsigned int tprec, unsigned int fprec, uns
     i = 0;
     if (val < 0.0) {
         buf[i++] = '-';
-        val = fabs (val);
+        val = -val;
     }
     /* Determine number of integral digits */
     if (val >= 1.0) {
@@ -133,10 +136,14 @@ sp_xml_dtoa (gchar *buf, double val, unsigned int tprec, unsigned int fprec, uns
 
 static SPXMLNs *namespaces=NULL;
 
+/*
+ * There are the default XML namespaces to use for the URIs defined
+ * in repr.h
+ */
 static void
 sp_xml_ns_register_defaults ()
 {
-    static SPXMLNs defaults[5];
+    static SPXMLNs defaults[7];
 
     defaults[0].uri = g_quark_from_static_string (SP_SODIPODI_NS_URI);
     defaults[0].prefix = g_quark_from_static_string ("sodipodi");
@@ -156,7 +163,15 @@ sp_xml_ns_register_defaults ()
 
     defaults[4].uri = g_quark_from_static_string (SP_RDF_NS_URI);
     defaults[4].prefix = g_quark_from_static_string ("rdf");
-    defaults[4].next = NULL;
+    defaults[4].next = &defaults[5];
+
+    defaults[5].uri = g_quark_from_static_string (SP_CC_NS_URI);
+    defaults[5].prefix = g_quark_from_static_string ("cc");
+    defaults[5].next = &defaults[6];
+
+    defaults[6].uri = g_quark_from_static_string (SP_DC_NS_URI);
+    defaults[6].prefix = g_quark_from_static_string ("dc");
+    defaults[6].next = NULL;
 
     namespaces = &defaults[0];
 }
@@ -191,7 +206,6 @@ sp_xml_ns_auto_prefix (const char *uri)
 const gchar *
 sp_xml_ns_uri_prefix (const gchar *uri, const gchar *suggested)
 {
-    unsigned int key;
     SPXMLNs *iter;
     const char *prefix;
 
@@ -201,7 +215,7 @@ sp_xml_ns_uri_prefix (const gchar *uri, const gchar *suggested)
         sp_xml_ns_register_defaults ();
     }
 
-    key = g_quark_from_string (uri);
+    GQuark const key = g_quark_from_string (uri);
     prefix = NULL;
     for ( iter = namespaces ; iter ; iter = iter->next ) {
         if ( iter->uri == key ) {
@@ -235,7 +249,6 @@ sp_xml_ns_uri_prefix (const gchar *uri, const gchar *suggested)
 const gchar *
 sp_xml_ns_prefix_uri (const gchar *prefix)
 {
-    unsigned int key;
     SPXMLNs *iter;
     const char *uri;
 
@@ -245,7 +258,7 @@ sp_xml_ns_prefix_uri (const gchar *prefix)
         sp_xml_ns_register_defaults ();
     }
 
-    key = g_quark_from_string (prefix);
+    GQuark const key = g_quark_from_string(prefix);
     uri = NULL;
     for ( iter = namespaces ; iter ; iter = iter->next ) {
         if ( iter->prefix == key ) {
@@ -296,19 +309,39 @@ sp_xml_node_get_Document (SPXMLNode *node)
 
 /* SPXMLElement */
 
+/** Returns the first child of \a repr, or NULL if \a repr has no children (or if repr is itself
+ *  NULL).
+ *
+ * \see sp_repr_next
+ */
 SPRepr *
 sp_repr_children (SPRepr *repr)
 {
-    g_return_val_if_fail (repr != NULL, NULL);
+    //This is not worth a warning
+    // child of null is null
+    //g_return_val_if_fail (repr != NULL, NULL);
 
+    if (!repr)
+        return NULL;
     return repr->children;
 }
 
+/** Returns the next sibling of \a repr, or NULL if \a repr is the last sibling (or if repr is
+ *  NULL).
+ *
+ *  \see sp_repr_prev
+ *  \see sp_repr_parent
+ *  \see sp_repr_children
+ */
 SPRepr *
 sp_repr_next (SPRepr *repr)
 {
-    g_return_val_if_fail (repr != NULL, NULL);
+    //This is not worth a warning
+    // next of null is null
+    //g_return_val_if_fail (repr != NULL, NULL);
 
+    if (!repr)
+        return NULL;
     return repr->next;
 }
 
@@ -332,7 +365,7 @@ double sp_repr_get_double_attribute (SPRepr * repr, const char * key, double def
 
     if (result == NULL) return def;
 
-    return atof (result);
+    return g_ascii_strtod (result, NULL);
 }
 
 int sp_repr_get_int_attribute (SPRepr * repr, const char * key, int def)
@@ -365,67 +398,74 @@ sp_repr_doc_attr (SPRepr * repr, const char * key)
 }
 
 int
-sp_repr_compare_position (SPRepr * first, SPRepr * second)
+sp_repr_compare_position(SPRepr *first, SPRepr *second)
 {
-    SPRepr * parent;
-    int p1, p2;
+    g_assert( sp_repr_parent(first) == sp_repr_parent(second) );
 
-    parent = sp_repr_parent (first);
-    g_assert (parent == sp_repr_parent (second));
-
-    p1 = sp_repr_position (first);
-    p2 = sp_repr_position (second);
+    int const p1 = sp_repr_position(first);
+    int const p2 = sp_repr_position(second);
 
     if (p1 > p2) return 1;
     if (p1 < p2) return -1;
     return 0;
+
+    /* effic: Assuming that the parent--child relationship is consistent (i.e. that the parent
+       really does contain first and second among its list of children), it should be equivalent to
+       walk along the children and see which we encounter first (returning 0 iff first == second).
+
+       Given that this function is used solely for sorting, we can use a similar approach to do the
+       sort: gather the things to be sorted, into an STL vector (to allow random access and faster
+       traversals).  Do a single pass of the parent's children; for each child, do a pass on
+       whatever items in the vector we haven't yet encountered.  If the child is found, then swap
+       it to the beginning of the yet-unencountered elements of the vector.  Continue until no more
+       than one remains unencountered.  -- pjrm */
 }
 
+
+/** Returns the position of \a repr among its parent's children (starting with 0 for the first
+ *  child).
+ *
+ *  \pre repr != NULL.
+ *  \pre sp_repr_parent(repr) != NULL.
+ *  \pre sp_repr_parent(repr)'s list of children includes \a repr.
+ */
 int
-sp_repr_position (SPRepr * repr)
+sp_repr_position(SPRepr const *repr)
 {
-    SPRepr * parent;
-    SPRepr * sibling;
-    int pos;
+    g_assert(repr != NULL);
+    SPRepr const *parent = sp_repr_parent(repr);
+    g_assert(parent != NULL);
 
-    g_assert (repr != NULL);
-    parent = sp_repr_parent (repr);
-    g_assert (parent != NULL);
-
-    pos = 0;
-    for (sibling = parent->children; sibling != NULL; sibling = sibling->next) {
-        if (repr == sibling) return pos;
-        pos += 1;
+    if (!parent->_child_counts_complete) {
+        unsigned n_remaining=( parent->children->_n_siblings - 1 );
+        for ( SPRepr *sibling=parent->children->next ;
+              sibling ;
+              sibling = sibling->next )
+        {
+            sibling->_n_siblings = n_remaining;
+            n_remaining--;
+        }
+        g_assert(n_remaining == 0);
+        parent->_child_counts_complete = true;
     }
-    
-    g_assert_not_reached ();
-
-    return -1;
-}
-
-void
-sp_repr_set_position_relative (SPRepr * repr, int pos)
-{
-    int cpos;
-
-    g_assert (repr != NULL);
-
-    cpos = sp_repr_position (repr);
-    sp_repr_set_position_absolute (repr, cpos + pos);
+    return (int)parent->children->_n_siblings - (int)repr->_n_siblings;
 }
 
 int
-sp_repr_n_children (SPRepr * repr)
+sp_repr_n_children(SPRepr *repr)
 {
-    SPRepr * child;
-    int n;
+    return ( repr->children ? repr->children->_n_siblings : 0 );
+}
 
-    g_assert (repr != NULL);
+SPRepr *sp_repr_nth_child(SPRepr *repr, int n) {
+    g_assert(repr != NULL);
 
-    n = 0;
-    for (child = repr->children; child != NULL; child = child->next) n++;
+    SPRepr *child=repr->children;
+    for ( ; n > 0 && child ; child = child->next ) {
+        n--;
+    }
 
-    return n;
+    return child;
 }
 
 void
@@ -505,34 +545,27 @@ sp_repr_set_attr_recursive (SPRepr *repr, const gchar *key, const gchar *value)
 }
 
 /**
- * sp_repr_lookup_child:
- * @repr: 
- * @key: 
- * @value: 
- *
- * lookup child by (@key, @value)
+ * lookup child by \a key, \a value.
  */
 SPRepr *
 sp_repr_lookup_child (SPRepr       *repr,
                       const gchar *key,
                       const gchar *value)
 {
-    SPRepr *child;
-    SPReprAttr *attr;
-    unsigned int quark;
+    g_return_val_if_fail(repr != NULL, NULL);
+    g_return_val_if_fail(key != NULL, NULL);
+    g_return_val_if_fail(value != NULL, NULL);
 
-    g_return_val_if_fail (repr != NULL, NULL);
-    g_return_val_if_fail (key != NULL, NULL);
-    g_return_val_if_fail (value != NULL, NULL);
-
-    quark = g_quark_from_string (key);
+    GQuark const quark = g_quark_from_string (key);
 
     /* Fixme: we should use hash table for faster lookup? */
     
-    for (child = repr->children; child != NULL; child = child->next) {
-        for (attr = child->attributes; attr != NULL; attr = attr->next) {
-            if ( (static_cast< unsigned int > (attr->key) == quark) && !strcmp (attr->value, value))
+    for (SPRepr *child = repr->children; child != NULL; child = child->next) {
+        for (SPReprAttr *attr = child->attributes; attr != NULL; attr = attr->next) {
+            if ( ( attr->key == quark )
+                 && !strcmp(attr->value, value) ) {
                 return child;
+            }
         }
     }
 
@@ -540,7 +573,39 @@ sp_repr_lookup_child (SPRepr       *repr,
 }
 
 /**
-Parses the boolean value of an attribute "key" in repr and sets val accordingly, or to FALSE if the attr is not set. Returns TRUE if the attr was set, FALSE otherwise.
+ *  \brief   Recursively find the SPRepr matching the given XML name.
+ *  \return  A pointer to the matching SPRepr
+ *  \param   repr    The SPRepr to start from
+ *  \param   name    The desired XML name
+ *  
+ */
+SPRepr *
+sp_repr_lookup_name ( SPRepr *repr, gchar const *name, gint maxdepth )
+{
+    g_return_val_if_fail (repr != NULL, NULL);
+    g_return_val_if_fail (name != NULL, NULL);
+
+    GQuark const quark = g_quark_from_string (name);
+
+    if ( (unsigned)repr->name == quark ) return repr;
+    if ( maxdepth == 0 ) return NULL;
+
+    // maxdepth == -1 means unlimited
+    if ( maxdepth == -1 ) maxdepth = 0;
+
+    SPRepr * found = NULL;
+    for (SPRepr *child = repr->children; child && !found; child = child->next) {
+        found = sp_repr_lookup_name ( child, name, maxdepth-1 );
+    }
+
+    return found;
+}
+
+/**
+ * Parses the boolean value of an attribute "key" in repr and sets val accordingly, or to FALSE if
+ * the attr is not set.
+ *
+ * \return TRUE if the attr was set, FALSE otherwise.
  */
 unsigned int
 sp_repr_get_boolean (SPRepr *repr, const gchar *key, unsigned int *val)
@@ -600,7 +665,7 @@ sp_repr_get_double (SPRepr *repr, const gchar *key, double *val)
     v = sp_repr_attr (repr, key);
 
     if (v != NULL) {
-        *val = atof (v);
+        *val = g_ascii_strtod (v, NULL);
         return TRUE;
     }
 
